Package: chimeraviz
Type: Package
Title: Visualization tools for gene fusions
Version: 1.36.0
Authors@R: c(
  person(
    given = "Stian",
    family = "Lågstad",
    email = "stianlagstad@gmail.com",
    role = c("aut", "cre")),
  person(
    given = "Sen",
    family = "Zhao",
    email = "Sen.Zhao@rr-research.no",
    role = "ctb"),
  person(
    given = "Andreas M.",
    family = "Hoff",
    email = "andreas.m.hoff@rr-research.no",
    role = "ctb"),
  person(
    given = "Bjarne",
    family = "Johannessen",
    email = "Bjarne.Johannessen@rr-research.no",
    role = "ctb"),
  person(
    given = "Ole Christian",
    family = "Lingjærde",
    email = "ole@ifi.uio.no",
    role = "ctb"),
  person(
    given = "Rolf",
    family = "Skotheim",
    email = "Rolf.I.Skotheim@rr-research.no",
    role = "ctb"))
Description: chimeraviz manages data from fusion gene finders and provides
    useful visualization tools.
License: Artistic-2.0
Imports: methods, grid, Rsamtools, GenomeInfoDb, GenomicAlignments,
        RColorBrewer, graphics, AnnotationDbi, RCircos, org.Hs.eg.db,
        org.Mm.eg.db, rmarkdown, graph, Rgraphviz, DT, plyr, dplyr,
        BiocStyle, checkmate, gtools, magick
Depends: Biostrings, GenomicRanges, IRanges, Gviz, S4Vectors,
        ensembldb, AnnotationFilter, data.table
Suggests: testthat, roxygen2, devtools, knitr, lintr
SystemRequirements: bowtie, samtools, and egrep are required for some
        functionalities
RoxygenNote: 7.1.1
VignetteBuilder: knitr
biocViews: Infrastructure, Alignment
Encoding: UTF-8
URL: https://github.com/stianlagstad/chimeraviz
BugReports: https://github.com/stianlagstad/chimeraviz/issues
git_url: https://git.bioconductor.org/packages/chimeraviz
git_branch: RELEASE_3_22
git_last_commit: 36b8261
git_last_commit_date: 2025-10-29
Repository: Bioconductor 3.22
Date/Publication: 2025-10-29
NeedsCompilation: no
Packaged: 2025-10-30 03:08:46 UTC; biocbuild
Author: Stian Lågstad [aut, cre],
  Sen Zhao [ctb],
  Andreas M. Hoff [ctb],
  Bjarne Johannessen [ctb],
  Ole Christian Lingjærde [ctb],
  Rolf Skotheim [ctb]
Maintainer: Stian Lågstad <stianlagstad@gmail.com>

# Generated by roxygen2: do not edit by hand

export("downstream_partner_gene<-")
export("partner_gene_ensembl_id<-")
export("upstream_partner_gene<-")
export(add_fusion_reads_alignment)
export(create_fusion_report)
export(decide_transcript_category)
export(down_shift)
export(downstream_partner_gene)
export(fetch_reads_from_fastq)
export(fusion_spanning_reads_count)
export(fusion_split_reads_count)
export(fusion_to_data_frame)
export(get_ensembl_ids)
export(get_fusion_by_chromosome)
export(get_fusion_by_gene_name)
export(get_fusion_by_id)
export(get_transcripts_ensembl_db)
export(import_aeron)
export(import_chimpipe)
export(import_defuse)
export(import_ericscript)
export(import_fusioncatcher)
export(import_fusionmap)
export(import_infusion)
export(import_jaffa)
export(import_oncofuse)
export(import_prada)
export(import_soapfuse)
export(import_squid)
export(import_starfusion)
export(partner_gene_ensembl_id)
export(partner_gene_junction_sequence)
export(plot_circle)
export(plot_fusion)
export(plot_fusion_reads)
export(plot_fusion_separate)
export(plot_fusion_together)
export(plot_fusion_transcript)
export(plot_fusion_transcript_with_protein_domain)
export(plot_fusion_transcripts_graph)
export(plot_transcripts)
export(select_transcript)
export(split_on_utr_and_add_feature)
export(upstream_partner_gene)
export(write_fusion_reference)
exportClasses(Fusion)
exportClasses(PartnerGene)
import(BiocStyle)
import(Biostrings)
import(GenomicRanges)
import(Gviz)
import(IRanges)
import(RCircos)
import(graphics)
import(methods)
import(org.Hs.eg.db)
import(org.Mm.eg.db)
importFrom(AnnotationFilter,GeneIdFilter)
importFrom(DT,datatable)
importFrom(S4Vectors,mcols)
importFrom(data.table,fread)
importFrom(ensembldb,EnsDb)
importFrom(ensembldb,exonsBy)
importFrom(plyr,ldply)

#' Import results from a PRADA run into a list of Fusion objects.
#'
#' A function that imports the results from a PRADA run into a list of Fusion
#' objects.
#'
#' @param filename Filename for the PRADA results file.
#' @param genome_version Which genome was used in mapping (hg19, hg38, etc.).
#' @param limit A limit on how many lines to read.
#'
#' @return A list of Fusion objects.
#'
#' @examples
#' pradaData <- system.file(
#'   "extdata",
#'   "PRADA.acc.fusion.fq.TAF.tsv",
#'   package = "chimeraviz")
#' fusions <- import_prada(pradaData, "hg19", 3)
#' # This should import a list of 3 fusions described in Fusion objects.
#'
#' @importFrom data.table fread
#'
#' @export
import_prada <- function(filename, genome_version, limit) {

  # Is the genome version valid?
  valid_genomes <- c("hg19", "hg38", "mm10")
  if (is.na(match(tolower(genome_version), tolower(valid_genomes)))) {
    stop("Invalid genome version given")
  }

  # If the limit is set, is the value valid?
  if (missing(limit) == FALSE) {
    if (is.numeric(limit) == FALSE || limit <= 0) {
      stop("limit must be a numeric value bigger than 0")
    }
  }

  # Try to read the fusion report
  report <- withCallingHandlers({
      col_types <- c(
        "sample" = "character",
        "A_strand" = "integer",
        "B_strand" = "integer",
        "Discordant_n" = "integer",
        "JSR_n" = "character",
        "Junction" = "character"
      )
      if (missing(limit)) {
        # Read all lines
        data.table::fread(
          input = filename,
          colClasses = col_types,
          showProgress = FALSE
        )
      } else {
        # Only read up to the limit
        data.table::fread(
          input = filename,
          colClasses = col_types,
          showProgress = FALSE,
          nrows = limit
        )
      }
    },
    error = function(cond) {
      message(paste0("Reading ", filename, " caused an error: ", cond[[1]]))
      stop(cond)
    },
    warning = function(cond) {
      # Begin Exclude Linting
      message(paste0("Reading ", filename, " caused a warning: ", cond[[1]]))
      # End Exclude Linting
    }
  )

  # Set variables
  id                   <- NA
  inframe              <- NA
  fusion_tool          <- "prada"
  spanning_reads_count <- NA
  split_reads_count    <- NA

  # List to hold all Fusion objects
  fusion_list <- vector("list", dim(report)[1])

  # Iterate through each line in the .tsv file
  for (i in 1:dim(report)[1]) {

    # Import prada-specific fields
    fusion_tool_specific_data <- list()

    # id for this fusion
    id <- as.character(i)

    # The format of report[[i, "Junction"]] should be:
    # "RBM14:11:66386032_BBS1:11:66283011,6". Split into two first:
    junction_data_gene_upstream <-
      strsplit(report[[i, "Junction"]], split = "_")[[1]][1]
    junction_data_gene_downstream <-
      strsplit(report[[i, "Junction"]], split = "_")[[1]][2]
    # The format of junctionDataGeneB should now be: "BBS1:11:66283011,6". So
    # remove everything after comma
    junction_data_gene_downstream <- substr(
      junction_data_gene_downstream,
      1,
      unlist(gregexpr(pattern = ",", junction_data_gene_downstream)) - 1)
    # Now split both on :
    junction_data_gene_upstream <-
      unlist(strsplit(junction_data_gene_upstream, split = ":"))
    junction_data_gene_downstream <-
      unlist(strsplit(junction_data_gene_downstream, split = ":"))

    # Chromosome names
    chromosome_upstream <-
      paste("chr", junction_data_gene_upstream[[2]], sep = "")
    chromosome_downstream <-
      paste("chr", junction_data_gene_downstream[[2]], sep = "")

    # Breakpoints
    breakpoint_upstream <- as.numeric(junction_data_gene_upstream[[3]])
    breakpoint_downstream <- as.numeric(junction_data_gene_downstream[[3]])

    # Strand
    strand_upstream <- if (report[[i, "A_strand"]] == 1) "+" else "-"
    strand_downstream <- if (report[[i, "B_strand"]] == 1) "+" else "-"

    # Number of supporting reads
    split_reads_count <- as.numeric(report[[i, "JSR_n"]])
    spanning_reads_count <- as.numeric(report[[i, "Discordant_n"]])

    # PRADA doesn't provide the fusion sequence
    junction_sequence_upstream <- Biostrings::DNAString()
    junction_sequence_downstream <- Biostrings::DNAString()

    # Gene names
    name_upstream <- junction_data_gene_upstream[[1]]
    name_downstream <- junction_data_gene_downstream[[1]]

    # Ensembl ids
    ensembl_id_upstream <- NA_character_
    ensembl_id_downstream <- NA_character_

    # PartnerGene objects
    gene_upstream <- new(
      Class = "PartnerGene",
      name = name_upstream,
      ensembl_id = ensembl_id_upstream,
      chromosome = chromosome_upstream,
      breakpoint = breakpoint_upstream,
      strand = strand_upstream,
      junction_sequence = junction_sequence_upstream,
      transcripts = GenomicRanges::GRangesList()
    )
    gene_downstream <- new(
      Class = "PartnerGene",
      name = name_downstream,
      ensembl_id = ensembl_id_downstream,
      chromosome = chromosome_downstream,
      breakpoint = breakpoint_downstream,
      strand = strand_downstream,
      junction_sequence = junction_sequence_downstream,
      transcripts = GenomicRanges::GRangesList()
    )

    fusion_list[[i]] <- new(
      Class = "Fusion",
      id = id,
      fusion_tool = fusion_tool,
      genome_version = genome_version,
      spanning_reads_count = spanning_reads_count,
      split_reads_count = split_reads_count,
      fusion_reads_alignment = Gviz::AlignmentsTrack(),
      gene_upstream = gene_upstream,
      gene_downstream = gene_downstream,
      inframe = inframe,
      fusion_tool_specific_data = fusion_tool_specific_data
    )
  }

  # Return the list of Fusion objects
  fusion_list
}

context("Test importing PRADA data into a list of Fusion objects")

prada_data <- system.file(
  "extdata",
  "PRADA.acc.fusion.fq.TAF.tsv",
  package = "chimeraviz")
fusions <- import_prada(prada_data, "hg19", 3)
fusion <- fusions[[1]]

test_that("import_prada imports data into Fusion objects", {
  expect_equal(length(fusions), 3)
  expect_equal(fusion@id, "1")
  expect_equal(fusion@gene_upstream@name, "TBCD")
  expect_equal(fusion@gene_upstream@chromosome, "chr17")
  expect_equal(fusion@gene_upstream@strand, "+")
})

test_that("import_prada fails to import data", {
  expect_error(import_prada("Non-existent-file", "hg18"))
  expect_error(import_prada(prada_data, "Invalid-genome"))
  expect_error(import_prada(prada_data, "hg19", "invalid limit"))
})

test_that("import_prada limits work", {
  expect_equal(length(import_prada(prada_data, "hg19", 2)), 2)
})

#' Import results from a Fusioncatcher run into a list of Fusion objects.
#'
#' A function that imports the results from a Fusioncatcher run, typically from
#' a final-list-candidate-fusion-genes.txt file, into a list of Fusion objects.
#'
#' @param filename Filename for the Fusioncatcher
#' final-list-candidate-fusion-genes.txt results file.
#' @param genome_version Which genome was used in mapping (hg19, hg38, etc.).
#' @param limit A limit on how many lines to read.
#'
#' @return A list of Fusion objects.
#'
#' @examples
#' fusioncatcher833ke <- system.file(
#'   "extdata",
#'   "fusioncatcher_833ke_final-list-candidate-fusion-genes.txt",
#'   package = "chimeraviz")
#' fusions <- import_fusioncatcher(fusioncatcher833ke, "hg38", 3)
#' # This should import a list of 3 fusions described in Fusion objects.
#'
#' @importFrom data.table fread
#'
#' @export
import_fusioncatcher <- function(filename, genome_version, limit) {

  # Is the genome version valid?
  valid_genomes <- c("hg19", "hg38", "mm10")
  if (is.na(match(tolower(genome_version), tolower(valid_genomes)))) {
    stop("Invalid genome version given")
  }

  # If the limit is set, is the value valid?
  if (missing(limit) == FALSE) {
    if (is.numeric(limit) == FALSE || limit <= 0) {
      stop("limit must be a numeric value bigger than 0")
    }
  }

  # Try to read the fusion report
  report <- withCallingHandlers({
      col_types <- c(
        "Gene_1_symbol(5end_fusion_partner)" = "character",
        "Gene_2_symbol(3end_fusion_partner)" = "character",
        "Spanning_pairs" = "integer",
        "Spanning_unique_reads" = "integer",
        "Fusion_point_for_gene_1(5end_fusion_partner)" = "character",
        "Fusion_point_for_gene_2(3end_fusion_partner)" = "character",
        "Gene_1_id(5end_fusion_partner)" = "character",
        "Gene_2_id(3end_fusion_partner)" = "character",
        "Fusion_sequence" = "character",
        "Predicted_effect" = "character"
      )
      if (missing(limit)) {
        # Read all lines
        data.table::fread(
          input = filename,
          colClasses = col_types,
          showProgress = FALSE
        )
      } else {
        # Only read up to the limit
        data.table::fread(
          input = filename,
          colClasses = col_types,
          showProgress = FALSE,
          nrows = limit
        )
      }
    },
    error = function(cond) {
      message(paste0("Reading ", filename, " caused an error: ", cond[[1]]))
      stop(cond)
    },
    warning = function(cond) {
      # Begin Exclude Linting
      message(paste0("Reading ", filename, " caused a warning: ", cond[[1]]))
      # End Exclude Linting
    }
  )

  # Set variables
  id                   <- NA
  inframe              <- NA
  fusion_tool          <- "fusioncatcher"
  spanning_reads_count <- NA
  split_reads_count    <- NA
  junction_sequence    <- NA

  # List to hold all Fusion objects
  fusion_list <- vector("list", dim(report)[1])

  # Iterate through each line in the .tsv file
  for (i in 1:dim(report)[1]) {

    # Import fusioncatcher-specific fields
    fusion_tool_specific_data <- list()

    # Set id for this fusion. Fusioncatcher does not have any unique identifier
    # for each row of the results file, so just use our counter.
    id <- as.character(i)

    # Is the downstream fusion partner in-frame?
    if (report[[i, "Predicted_effect"]] == "in-frame") {
      inframe <- TRUE
    } else {
      inframe <- FALSE
    }

    gene_upstream_position <-
      unlist(
        strsplit(
          report[[i, "Fusion_point_for_gene_1(5end_fusion_partner)"]],
          split = ":"
        )
      )
    gene_downstream_position <-
      unlist(
        strsplit(
          report[[i, "Fusion_point_for_gene_2(3end_fusion_partner)"]],
          split = ":"
        )
      )

    # Chromosome names
    chromosome_upstream <- paste("chr", gene_upstream_position[1], sep = "")
    chromosome_downstream <- paste("chr", gene_downstream_position[1], sep = "")

    # Breakpoints
    breakpoint_upstream <- as.numeric(gene_upstream_position[2])
    breakpoint_downstream <- as.numeric(gene_downstream_position[2])

    # Strand
    strand_upstream <- gene_upstream_position[3]
    strand_downstream <- gene_downstream_position[3]

    # Number of supporting reads
    split_reads_count <- report[[i, "Spanning_pairs"]]
    spanning_reads_count <- report[[i, "Spanning_unique_reads"]]

    # Get the fusion sequence. Split it into the part from gene1 and gene2
    junction_sequence <- strsplit(report[[i, "Fusion_sequence"]], split = "\\*")
    junction_sequence_upstream <-
      Biostrings::DNAString(junction_sequence[[1]][1])
    junction_sequence_downstream <-
      Biostrings::DNAString(junction_sequence[[1]][2])

    # Gene names
    name_upstream <- report[[i, "Gene_1_symbol(5end_fusion_partner)"]]
    name_downstream <- report[[i, "Gene_2_symbol(3end_fusion_partner)"]]

    # Ensembl ids
    ensembl_id_upstream <- report[[i, "Gene_1_id(5end_fusion_partner)"]]
    ensembl_id_downstream <- report[[i, "Gene_2_id(3end_fusion_partner)"]]

    # PartnerGene objects
    gene_upstream <- new(
      Class = "PartnerGene",
      name = name_upstream,
      ensembl_id = ensembl_id_upstream,
      chromosome = chromosome_upstream,
      breakpoint = breakpoint_upstream,
      strand = strand_upstream,
      junction_sequence = junction_sequence_upstream,
      transcripts = GenomicRanges::GRangesList()
    )
    gene_downstream <- new(
      Class = "PartnerGene",
      name = name_downstream,
      ensembl_id = ensembl_id_downstream,
      chromosome = chromosome_downstream,
      breakpoint = breakpoint_downstream,
      strand = strand_downstream,
      junction_sequence = junction_sequence_downstream,
      transcripts = GenomicRanges::GRangesList()
    )

    fusion_list[[i]] <- new(
      Class = "Fusion",
      id = id,
      fusion_tool = fusion_tool,
      genome_version = genome_version,
      spanning_reads_count = spanning_reads_count,
      split_reads_count = split_reads_count,
      fusion_reads_alignment = Gviz::AlignmentsTrack(),
      gene_upstream = gene_upstream,
      gene_downstream = gene_downstream,
      inframe = inframe,
      fusion_tool_specific_data = fusion_tool_specific_data
    )
  }

  # Return the list of Fusion objects
  fusion_list
}

context("Test importing fusioncatcher data into a list of Fusion objects")

fusioncatcher833ke <- system.file(
  "extdata",
  "fusioncatcher_833ke_final-list-candidate-fusion-genes.txt",
  package = "chimeraviz")
fusions <- import_fusioncatcher(fusioncatcher833ke, "hg38", 3)
fusion <- fusions[[1]]

test_that("import_fusioncatcher imports data into Fusion objects", {
  expect_equal(length(fusions), 3)
  expect_equal(fusion@id, "1")
  expect_equal(fusion@gene_upstream@name, "CLEC6A")
  expect_equal(fusion@gene_upstream@chromosome, "chr12")
})

test_that("import_fusioncatcher fails to import data", {
  expect_error(import_fusioncatcher("Non-existent-file", "hg38"))
  expect_error(import_fusioncatcher(fusioncatcher833ke, "Invalid-genome"))
  expect_error(
    import_fusioncatcher(fusioncatcher833ke, "hg38", "invalid limit"))
})

test_that("import_fusioncatcher limits work", {
  expect_equal(length(import_fusioncatcher(fusioncatcher833ke, "hg38", 2)), 2)
})

#' Plot transcripts for each partner gene in a fusion event.
#'
#' This function takes a fusion object and an ensembldb object and plots
#' transcripts for both genes, showing which parts of each genes are included
#' in the fusion event. If the bamfile parameter is set, then the coverage is
#' plotted beneath the transcripts.
#'
#' @param fusion The Fusion object to plot.
#' @param edb The edb object that will be used to fetch data.
#' @param bamfile The bamfile with RNA-seq data.
#' @param which_transcripts This character vector decides which transcripts are
#' to be plotted. Can be "exonBoundary", "withinExon", "withinIntron",
#' "intergenic", or a character vector with specific transcript ids. Default
#' value is "exonBoundary".
#' @param ylim Limits for the coverage y-axis.
#' @param non_ucsc Boolean indicating whether or not the bamfile used has UCSC-
#' styled chromosome names (i.e. with the "chr" prefix). Setting this to true
#' lets you use a bamfile with chromosome names like "1" and "X", instead of
#' "chr1" and "chrX".
#' @param reduce_transcripts Boolean indicating whether or not to reduce all
#' transcripts into a single transcript for each partner gene.
#' @param bedgraphfile A bedGraph file to use instead of the bamfile to plot
#' coverage.
#'
#' @return Creates a fusion transcripts plot.
#'
#' @examples
#' # Load data and example fusion event
#' defuse833ke <- system.file(
#'   "extdata",
#'   "defuse_833ke_results.filtered.tsv",
#'   package="chimeraviz")
#' fusions <- import_defuse(defuse833ke, "hg19", 1)
#' fusion <- get_fusion_by_id(fusions, 5267)
#' # Load edb
#' edbSqliteFile <- system.file(
#'   "extdata",
#'   "Homo_sapiens.GRCh37.74.sqlite",
#'   package="chimeraviz")
#' edb <- ensembldb::EnsDb(edbSqliteFile)
#' # bamfile with reads in the regions of this fusion event
#' bamfile5267 <- system.file(
#'   "extdata",
#'   "fusion5267and11759reads.bam",
#'   package="chimeraviz")
#' # Temporary file to store the plot
#' pngFilename <- tempfile(
#'   pattern = "fusionPlot",
#'   fileext = ".png",
#'   tmpdir = tempdir())
#' # Open device
#' png(pngFilename, width = 500, height = 500)
#' # Plot!
#' plot_transcripts(
#'   fusion = fusion,
#'   edb = edb,
#'   bamfile = bamfile5267,
#'   non_ucsc = TRUE)
#' # Close device
#' dev.off()
#'
#' # Example using a .bedGraph file instead of a .bam file:
#' # Load data and example fusion event
#' defuse833ke <- system.file(
#'   "extdata",
#'   "defuse_833ke_results.filtered.tsv",
#'   package="chimeraviz")
#' fusions <- import_defuse(defuse833ke, "hg19", 1)
#' fusion <- get_fusion_by_id(fusions, 5267)
#' # Load edb
#' edbSqliteFile <- system.file(
#'   "extdata",
#'   "Homo_sapiens.GRCh37.74.sqlite",
#'   package="chimeraviz")
#' edb <- ensembldb::EnsDb(edbSqliteFile)
#' # bedgraphfile with coverage data from the regions of this fusion event
#' bedgraphfile <- system.file(
#'   "extdata",
#'   "fusion5267and11759reads.bedGraph",
#'   package="chimeraviz")
#' # Temporary file to store the plot
#' pngFilename <- tempfile(
#'   pattern = "fusionPlot",
#'   fileext = ".png",
#'   tmpdir = tempdir())
#' # Open device
#' png(pngFilename, width = 500, height = 500)
#' # Plot!
#' plot_transcripts(
#'   fusion = fusion,
#'   edb = edb,
#'   bedgraphfile = bedgraphfile,
#'   non_ucsc = TRUE)
#' # Close device
#' dev.off()
#'
#' @importFrom ensembldb EnsDb
#'
#' @export
plot_transcripts <- function(
  fusion,
  edb = NULL,
  bamfile = NULL,
  which_transcripts = "exonBoundary",
  non_ucsc = TRUE,
  ylim = c(0, 1000),
  reduce_transcripts = FALSE,
  bedgraphfile = NULL) {

  .validate_plot_fusion_transcripts_params(
    fusion,
    edb,
    bamfile,
    which_transcripts,
    non_ucsc,
    ylim,
    reduce_transcripts,
    bedgraphfile)

  fusion <- .get_transcripts_if_not_there(fusion, edb)

  # Select which transcripts to use
  transcripts_upstream <-
    select_transcript(fusion@gene_upstream, which_transcripts)
  transcripts_downstream <-
    select_transcript(fusion@gene_downstream, which_transcripts)

  # Set seqlevels to UCSC
  GenomeInfoDb::seqlevelsStyle(transcripts_upstream) <- "UCSC"
  GenomeInfoDb::seqlevelsStyle(transcripts_downstream) <- "UCSC"

  if (reduce_transcripts) {
    reduced_transcripts_upstream <-
      GenomicRanges::reduce(unlist(transcripts_upstream))
    reduced_transcripts_downstream <-
      GenomicRanges::reduce(unlist(transcripts_downstream))
    # Add transcript metadata column to make Gviz group correctly
    mcols(reduced_transcripts_upstream)$transcript <-
      fusion@gene_upstream@name
    mcols(reduced_transcripts_downstream)$transcript <-
      fusion@gene_downstream@name
    # Create the tracks
    tr_upstream_track <- Gviz::GeneRegionTrack(
      reduced_transcripts_upstream,
      chromosome = fusion@gene_upstream@chromosome
    )
    tr_downstream_track <- Gviz::GeneRegionTrack(
      reduced_transcripts_downstream,
      chromosome = fusion@gene_downstream@chromosome
    )
    # Set symbol names on the tracks
    symbol(tr_upstream_track) <- fusion@gene_upstream@name
    symbol(tr_downstream_track) <- fusion@gene_downstream@name
  } else {
    # Create the tracks
    tr_upstream_track <- Gviz::GeneRegionTrack(
      data.frame(transcripts_upstream),
      chromosome = fusion@gene_upstream@chromosome)
    tr_downstream_track <- Gviz::GeneRegionTrack(
      data.frame(transcripts_downstream),
      chromosome = fusion@gene_downstream@chromosome)
  }

  # Set display parameters
  transcript_display_params <- list(
    background.title = "transparent", # hide left panel
    showId = TRUE, # show transcript id
    col = "transparent",
    cex.group = 0.8,
    fontcolor.group = "black",
    just.group = "below",
    min.distance = 0,
    thinBoxFeature = c(
      "5utr_excludedA",
      "5utr_includedA",
      "3utr_excludedA",
      "3utr_includedA",

      "5utr_excludedB",
      "5utr_includedB",
      "3utr_excludedB",
      "3utr_includedB"
    ))
  Gviz::displayPars(tr_upstream_track) <- transcript_display_params
  Gviz::displayPars(tr_downstream_track) <- transcript_display_params

  # Create axis track
  axis_track <- Gviz::GenomeAxisTrack()
  Gviz::displayPars(axis_track) <- list(
    labelPos = "below",
    cex = 1,
    col = "black", # line color
    fontcolor = "black",
    lwd = 1.5
  )

  # Helper variables
  gene_upstream_at_minus_strand <- fusion@gene_upstream@strand == "-"
  gene_downstream_at_minus_strand <- fusion@gene_downstream@strand == "-"

  # Color options for exons included/excluded in fusion
  cols <- RColorBrewer::brewer.pal(4, "Paired")
  interestcolor <- list(
    "5utr_excludedA" = cols[1],
    "protein_coding_excludedA" = cols[1],
    "3utr_excludedA" = cols[1],

    "5utr_includedA" = cols[2],
    "protein_coding_includedA" = cols[2],
    "3utr_includedA" = cols[2],

    "5utr_excludedB" = cols[3],
    "protein_coding_excludedB" = cols[3],
    "3utr_excludedB" = cols[3],

    "5utr_includedB" = cols[4],
    "protein_coding_includedB" = cols[4],
    "3utr_includedB" = cols[4])

  # Set all colors to exluded
  if (reduce_transcripts) {
    Gviz::feature(tr_upstream_track) <- "protein_coding_excludedA"
    Gviz::feature(tr_downstream_track) <- "protein_coding_excludedB"
  } else {
    Gviz::feature(tr_upstream_track)[
      Gviz::feature(tr_upstream_track) == "5utr"
    ] <- "5utr_excludedA"
    Gviz::feature(tr_upstream_track)[
      Gviz::feature(tr_upstream_track) == "protein_coding"
    ] <- "protein_coding_excludedA"
    Gviz::feature(tr_upstream_track)[
      Gviz::feature(tr_upstream_track) == "3utr"
    ] <- "3utr_excludedA"

    Gviz::feature(tr_downstream_track)[
      Gviz::feature(tr_downstream_track) == "5utr"
    ] <- "5utr_excludedB"
    Gviz::feature(tr_downstream_track)[
      Gviz::feature(tr_downstream_track) == "protein_coding"
    ] <- "protein_coding_excludedB"
    Gviz::feature(tr_downstream_track)[
      Gviz::feature(tr_downstream_track) == "3utr"
    ] <- "3utr_excludedB"
  }

  if (gene_upstream_at_minus_strand) {
    message(paste("As ",
                  fusion@gene_upstream@name,
                  " is on the minus strand, the plot for this gene will be ",
                  "reversed",
                  sep = ""))
    highlight_start_upstream <- fusion@gene_upstream@breakpoint
    highlight_end_upstream <-
      max(start(tr_upstream_track) + width(tr_upstream_track) - 1)

    # Color the exons included/not included in the fusion
    Gviz::feature(tr_upstream_track)[
      start(tr_upstream_track) <= highlight_end_upstream &
        end(tr_upstream_track) >= highlight_start_upstream &
        Gviz::feature(tr_upstream_track) == "5utr_excludedA"
    ] <- "5utr_includedA"

    Gviz::feature(tr_upstream_track)[
      start(tr_upstream_track) <= highlight_end_upstream &
        end(tr_upstream_track) >= highlight_start_upstream &
        Gviz::feature(tr_upstream_track) == "protein_coding_excludedA"
    ] <- "protein_coding_includedA"

    Gviz::feature(tr_upstream_track)[
      start(tr_upstream_track) <= highlight_end_upstream &
        end(tr_upstream_track) >= highlight_start_upstream &
        Gviz::feature(tr_upstream_track) == "3utr_excludedA"
    ] <- "3utr_includedA"

  } else {
    highlight_start_upstream <- min(start(tr_upstream_track))
    highlight_end_upstream <- fusion@gene_upstream@breakpoint

    # Color the exons included/not included in the fusion
    Gviz::feature(tr_upstream_track)[
      start(tr_upstream_track) >= highlight_start_upstream &
      end(tr_upstream_track) <= highlight_end_upstream &
      Gviz::feature(tr_upstream_track) == "5utr_excludedA"
    ] <- "5utr_includedA"

    Gviz::feature(tr_upstream_track)[
      start(tr_upstream_track) >= highlight_start_upstream &
      end(tr_upstream_track) <= highlight_end_upstream &
      Gviz::feature(tr_upstream_track) == "protein_coding_excludedA"
    ] <- "protein_coding_includedA"

    Gviz::feature(tr_upstream_track)[
      start(tr_upstream_track) >= highlight_start_upstream &
      end(tr_upstream_track) <= highlight_end_upstream &
      Gviz::feature(tr_upstream_track) == "3utr_excludedA"
    ] <- "3utr_includedA"

  }

  if (gene_downstream_at_minus_strand) {
    message(paste("As ",
                  fusion@gene_downstream@name,
                  " is on the minus strand, the plot for this gene will be ",
                  "reversed",
                  sep = ""))
    highlight_start_downstream <- min(start(tr_downstream_track))
    highlight_end_downstream <- fusion@gene_downstream@breakpoint

    # Color the exons included/not included in the fusion
    Gviz::feature(tr_downstream_track)[
      start(tr_downstream_track) <= highlight_end_downstream &
      end(tr_downstream_track) >= highlight_start_downstream &
      Gviz::feature(tr_downstream_track) == "5utr_excludedB"
    ] <- "5utr_includedB"

    Gviz::feature(tr_downstream_track)[
      start(tr_downstream_track) <= highlight_end_downstream &
      end(tr_downstream_track) >= highlight_start_downstream &
      Gviz::feature(tr_downstream_track) == "protein_coding_excludedB"
    ] <- "protein_coding_includedB"

    Gviz::feature(tr_downstream_track)[
      start(tr_downstream_track) <= highlight_end_downstream &
      end(tr_downstream_track) >= highlight_start_downstream &
      Gviz::feature(tr_downstream_track) == "3utr_excludedB"
    ] <- "3utr_includedB"

  } else {
    highlight_start_downstream <- fusion@gene_downstream@breakpoint
    highlight_end_downstream <-
      max(start(tr_downstream_track) + width(tr_downstream_track) - 1)

    # Color the exons included/not included in the fusion
    Gviz::feature(tr_downstream_track)[
      start(tr_downstream_track) >= highlight_start_downstream &
      end(tr_downstream_track) <= highlight_end_downstream &
      Gviz::feature(tr_downstream_track) == "5utr_excludedB"
    ] <- "5utr_includedB"

    Gviz::feature(tr_downstream_track)[
      start(tr_downstream_track) >= highlight_start_downstream &
      end(tr_downstream_track) <= highlight_end_downstream &
      Gviz::feature(tr_downstream_track) == "protein_coding_excludedB"
    ] <- "protein_coding_includedB"

    Gviz::feature(tr_downstream_track)[
      start(tr_downstream_track) >= highlight_start_downstream &
      end(tr_downstream_track) <= highlight_end_downstream &
      Gviz::feature(tr_downstream_track) == "3utr_excludedB"
    ] <- "3utr_includedB"

  }

  # Update transcript track with colors
  Gviz::displayPars(tr_upstream_track) <- interestcolor
  Gviz::displayPars(tr_downstream_track) <- interestcolor

  # Highlight transcript tracks

  # Display parameters for highlight tracks
  display_pars_highlight_tracks <- list(
    fill = "lightgrey",
    col = "lightgrey"
  )

  # Only plot highlight tracks if we actually have transcripts that has the
  # fusion breakpoint within them

  if (any(start(tr_upstream_track) < fusion@gene_upstream@breakpoint) &&
      any(fusion@gene_upstream@breakpoint < end(tr_upstream_track))) {

    gr_track_highlight_upstream <- Gviz::HighlightTrack(
      trackList = tr_upstream_track,
      start = highlight_start_upstream,
      end = highlight_end_upstream,
      chromosome = fusion@gene_upstream@chromosome)
    Gviz::displayPars(gr_track_highlight_upstream) <-
      display_pars_highlight_tracks
  }

  if (any(start(tr_downstream_track) < fusion@gene_downstream@breakpoint) &&
      any(fusion@gene_downstream@breakpoint < end(tr_downstream_track))) {

    gr_track_highlight_downstream <- Gviz::HighlightTrack(
      trackList = tr_downstream_track,
      start = highlight_start_downstream,
      end = highlight_end_downstream,
      chromosome = fusion@gene_downstream@chromosome)
    Gviz::displayPars(gr_track_highlight_downstream) <-
      display_pars_highlight_tracks
  }

  # Plot coverage?
  if (!is.null(bamfile) || !is.null(bedgraphfile)) {
    # Create alignment track
    if (!is.null(bamfile)) {
      # We're getting coverage data from a bam file
      if (non_ucsc) {
        # If the bam file has non-ucsc chromosome names, i.e. "1" instead of
        # "chr1", then we need to use a custom import function that adds "chr"
        # to the chromosome names, to keep Gviz happy. Gviz strongly prefers
        # having the "chr" prefix.
        al_track <- Gviz::AlignmentsTrack(
          bamfile,
          isPaired = TRUE,
          genome = fusion@genome_version,
          ylim = ylim,
          importFunction = import_function_non_ucsc)
      } else {
        al_track <- Gviz::AlignmentsTrack(
          bamfile,
          isPaired = TRUE,
          genome = fusion@genome_version,
          ylim = ylim)
      }
      # Set display paramters
      Gviz::displayPars(al_track) <- list(
        showTitle = FALSE, # hide name of track
        background.panel = "transparent", # bg color of the content panel
        background.title = "transparent", # bg color for the title panels
        col.axis = "black",
        col.coverage = "black",
        col.title = "black",
        fill.coverage = "orange",
        fontsize = 15,
        lwd.coverage = 0.4,
        type = "coverage",
        cex.title = .8,
        cex.axis = .6,
        show
      )
    } else {
      # We're getting coverage data from a bedGraph file
      al_track <- DataTrack(
        range = bedgraphfile,
        ylim = ylim,
        genome = "hg19",
        name = "Coverage",
        type = "h",
        col = "orange",
        fill = "orange")
      # Set display parameters
      Gviz::displayPars(al_track) <- list(
        showTitle = FALSE, # hide name of track
        background.panel = "transparent", # bg color of the content panel
        background.title = "transparent", # bg color for the title panels
        cex.axis = .6,
        cex.title = .6,
        col.axis = "black",
        col.coverage = "black",
        col.title = "black",
        coverageHeight = 0.08,
        fill.coverage = "orange",
        fontsize = 15,
        lty = 1,
        lty.coverage = 1,
        lwd = 0.5
      )
    }

    # Highlight coverage tracks

    # Only plot highlight tracks if we actually have transcripts that has the
    # fusion breakpoint within them
    if (any(start(tr_upstream_track) < fusion@gene_upstream@breakpoint) &&
        any(fusion@gene_upstream@breakpoint < end(tr_upstream_track))) {

      al_track_highlight_upstream <- Gviz::HighlightTrack(
        trackList = al_track,
        start = highlight_start_upstream,
        end = highlight_end_upstream,
        chromosome = fusion@gene_upstream@chromosome)
      Gviz::displayPars(al_track_highlight_upstream) <-
        display_pars_highlight_tracks
    }

    if (any(start(tr_downstream_track) < fusion@gene_downstream@breakpoint) &&
        any(fusion@gene_downstream@breakpoint < end(tr_downstream_track))) {

      al_track_highlight_downstream <- Gviz::HighlightTrack(
        trackList = al_track,
        start = highlight_start_downstream,
        end = highlight_end_downstream,
        chromosome = fusion@gene_downstream@chromosome)
      Gviz::displayPars(al_track_highlight_downstream) <-
        display_pars_highlight_tracks
    }
  }

  # Do some plotting

  # Calculate heights in the plot grid according to how many transcripts there
  # are in each partnergene
  if (reduce_transcripts) {
    number_of_transcripts_upstream <- 1
    number_of_transcripts_downstream <- 1
  } else {
    number_of_transcripts_upstream <- length(transcripts_upstream)
    number_of_transcripts_downstream <- length(transcripts_downstream)
  }

  # Create the grid we're gonna use for the plot
  nf <- grid::grid.layout(
    nrow = 2,
    ncol = 1,
    heights = grid::unit(
      c(
        number_of_transcripts_upstream,
        number_of_transcripts_downstream
      ),
      "null"
    )
  )

  # Open a new page to plot it
  grid::grid.newpage()

  # Divide the page according to the grid we created
  grid::pushViewport(grid::viewport(layout = nf))

  # Open row 1, column 1
  grid::pushViewport(grid::viewport(layout.pos.row = 1, layout.pos.col = 1))
  # Plot transcriptsA and coverage with highlight
  # Plot coverage?
  if (!is.null(bamfile) || !is.null(bedgraphfile)) {

    # Only plot highlight tracks if we actually have transcripts that has the
    # fusion breakpoint within them
    if (any(start(tr_upstream_track) < fusion@gene_upstream@breakpoint) &&
        any(fusion@gene_upstream@breakpoint < end(tr_upstream_track))) {

      Gviz::plotTracks(
        # without this gviz create cluster_X entries in the GeneRegionTrack
        collapse = FALSE,
        list(gr_track_highlight_upstream, al_track_highlight_upstream),
        from = min(start(tr_upstream_track)),
        to = max(end(tr_upstream_track)),
        sizes = c(5, 2),
        add = TRUE,
        margin = 3,
        innerMargin = 0,
        # Plot reverse if gene is at minus strand
        reverseStrand = gene_upstream_at_minus_strand)
    } else {
      Gviz::plotTracks(
        # without this gviz create cluster_X entries in the GeneRegionTrack
        collapse = FALSE,
        list(tr_upstream_track, al_track),
        from = min(start(tr_upstream_track)),
        to = max(end(tr_upstream_track)),
        sizes = c(5, 2),
        add = TRUE,
        margin = 3,
        innerMargin = 0,
        # Plot reverse if gene is at minus strand
        reverseStrand = gene_upstream_at_minus_strand)
    }
  } else {
    # Only plot highlight tracks if we actually have transcripts that has the
    # fusion breakpoint within them
    if (any(start(tr_upstream_track) < fusion@gene_upstream@breakpoint) &&
        any(fusion@gene_upstream@breakpoint < end(tr_upstream_track))) {

      Gviz::plotTracks(
        # without this gviz create cluster_X entries in the GeneRegionTrack
        collapse = FALSE,
        gr_track_highlight_upstream,
        from = min(start(tr_upstream_track)),
        to = max(end(tr_upstream_track)),
        add = TRUE,
        margin = 3,
        innerMargin = 0,
        # Plot reverse if gene is at minus strand
        reverseStrand = gene_upstream_at_minus_strand)
    } else {
      Gviz::plotTracks(
        # without this gviz create cluster_X entries in the GeneRegionTrack
        collapse = FALSE,
        tr_upstream_track,
        from = min(start(tr_upstream_track)),
        to = max(end(tr_upstream_track)),
        add = TRUE,
        margin = 3,
        innerMargin = 0,
        # Plot reverse if gene is at minus strand
        reverseStrand = gene_upstream_at_minus_strand)
    }
  }
  # Close row 1, column 1
  grid::popViewport(1)

  # Open row 2, column 1
  grid::pushViewport(grid::viewport(layout.pos.row = 2, layout.pos.col = 1))
  # Plot transcriptsB and coverage with highlight
  # Plot coverage?
  if (!is.null(bamfile) || !is.null(bedgraphfile)) {
    # Only plot highlight tracks if we actually have transcripts that has the
    # fusion breakpoint within them
    if (any(start(tr_downstream_track) < fusion@gene_downstream@breakpoint) &&
        any(fusion@gene_downstream@breakpoint < end(tr_downstream_track))) {

      Gviz::plotTracks(
        # without this gviz create cluster_X entries in the GeneRegionTrack
        collapse = FALSE,
        list(gr_track_highlight_downstream, al_track_highlight_downstream),
        from = min(start(tr_downstream_track)),
        to = max(end(tr_downstream_track)),
        sizes = c(5, 2),
        add = TRUE,
        margin = 3,
        innerMargin = 0,
        # Plot reverse if gene is at minus strand
        reverseStrand = gene_downstream_at_minus_strand)
    } else {
      Gviz::plotTracks(
        # without this gviz create cluster_X entries in the GeneRegionTrack
        collapse = FALSE,
        list(tr_downstream_track, al_track),
        from = min(start(tr_downstream_track)),
        to = max(end(tr_downstream_track)),
        sizes = c(5, 2),
        add = TRUE,
        margin = 3,
        innerMargin = 0,
        # Plot reverse if gene is at minus strand
        reverseStrand = gene_downstream_at_minus_strand)
    }
  } else {
    # Only plot highlight tracks if we actually have transcripts that has the
    # fusion breakpoint within them
    if (any(start(tr_downstream_track) < fusion@gene_downstream@breakpoint) &&
        any(fusion@gene_downstream@breakpoint < end(tr_downstream_track))) {

      Gviz::plotTracks(
        # without this gviz create cluster_X entries in the GeneRegionTrack
        collapse = FALSE,
        gr_track_highlight_downstream,
        from = min(start(tr_downstream_track)),
        to = max(end(tr_downstream_track)),
        add = TRUE,
        margin = 3,
        innerMargin = 0,
        # Plot reverse if gene is at minus strand
        reverseStrand = gene_downstream_at_minus_strand)
    } else {
      Gviz::plotTracks(
        # without this gviz create cluster_X entries in the GeneRegionTrack
        collapse = FALSE,
        tr_downstream_track,
        from = min(start(tr_downstream_track)),
        to = max(end(tr_downstream_track)),
        add = TRUE,
        margin = 3,
        innerMargin = 0,
        # Plot reverse if gene is at minus strand
        reverseStrand = gene_downstream_at_minus_strand)
    }
  }
  # Close row 1, column 1
  grid::popViewport(1)
}

.validate_plot_fusion_transcripts_params <- function(
  fusion,
  edb,
  bamfile,
  which_transcripts,
  non_ucsc,
  ylim,
  reduce_transcripts,
  bedgraphfile
) {
  # Establish a new 'AssertCollection' object
  argument_checker <- checkmate::makeAssertCollection()

  # Check parameters
  .is_fusion_valid(argument_checker, fusion)
  .is_edb_valid(argument_checker, edb, fusion)
  # Either bamfile or bedgraphfile can be given, not both
  bamfile_given <- !is.null(bamfile)
  bedgraphfile_given <- !is.null(bedgraphfile)
  if (bamfile_given && bedgraphfile_given) {
    argument_checker$push(
      "Either 'bamfile' or 'bedgraphfile' must be given, not both."
    )
  }
  if (bamfile_given) {
    .is_bamfile_valid(argument_checker, bamfile)
  }
  if (bedgraphfile_given) {
    .is_bedgraphfile_valid(argument_checker, bedgraphfile)
  }
  .is_which_transcripts_valid(
    argument_checker,
    which_transcripts,
    fusion)
  .is_ylim_valid(argument_checker, ylim)
  .is_parameter_boolean(
    argument_checker,
    non_ucsc,
    "non_ucsc")
  .is_parameter_boolean(
    argument_checker,
    reduce_transcripts,
    "reduce_transcripts")

  # Return errors and warnings (if any)
  checkmate::reportAssertions(argument_checker)
}

#' Import results from a deFuse run into a list of Fusion objects.
#'
#' A function that imports the results from a deFuse run, typically from a
#' results.filtered.tsv file, into a list of Fusion objects.
#'
#' @param filename Filename for the deFuse results .tsv file.
#' @param genome_version Which genome was used in mapping (hg19, hg38, etc.).
#' @param limit A limit on how many lines to read.
#'
#' @return A list of Fusion objects.
#'
#' @examples
#' defuse833ke <- system.file(
#'   "extdata",
#'   "defuse_833ke_results.filtered.tsv",
#'   package="chimeraviz")
#' fusions <- import_defuse(defuse833ke, "hg19", 3)
#' # This should import a list of 3 fusions described in Fusion objects.

#' @importFrom data.table fread
#'
#' @export
import_defuse <- function(filename, genome_version, limit) {

  # Is the genome version valid?
  valid_genomes <- c("hg19", "hg38", "mm10")
  if (is.na(match(tolower(genome_version), tolower(valid_genomes)))) {
    stop("Invalid genome version given")
  }

  # If the limit is set, is the value valid?
  if (missing(limit) == FALSE) {
    if (is.numeric(limit) == FALSE || limit <= 0) {
      stop("limit must be a numeric value bigger than 0")
    }
  }

  # Try to read the fusion report
  report <- withCallingHandlers({
      col_types <- c(
        "cluster_id" = "character",
        "splitr_sequence" = "character",
        "splitr_count" = "integer",
        "gene1" = "character",
        "gene2" = "character",
        "gene_chromosome1" = "character",
        "gene_chromosome2" = "character",
        "gene_name1" = "character",
        "gene_name2" = "character",
        "gene_strand1" = "character",
        "gene_strand2" = "character",
        "genomic_break_pos1" = "integer",
        "genomic_break_pos2" = "integer",
        "span_count" = "integer",
        "orf" = "character",
        "probability" = "numeric"
      )
      if (missing(limit)) {
        # Read all lines
        data.table::fread(
          input = filename,
          colClasses = col_types,
          showProgress = FALSE
        )
      } else {
          # Only read up to the limit
          data.table::fread(
            input = filename,
            colClasses = col_types,
            showProgress = FALSE,
            nrows = limit
          )
      }
    },
    error = function(cond) {
      message(paste0("Reading ", filename, " caused an error: ", cond[[1]]))
      stop(cond)
    },
    warning = function(cond) {
      # Begin Exclude Linting
      message(paste0("Reading ", filename, " caused a warning: ", cond[[1]]))
      # End Exclude Linting
    }
  )

  # Set variables
  id                   <- NA
  inframe              <- NA
  fusion_tool          <- "defuse"
  spanning_reads_count <- NA
  split_reads_count    <- NA
  junction_sequence    <- NA

  # List to hold all Fusion objects
  fusion_list <- vector("list", dim(report)[1])

  # Iterate through each line in the .tsv file
  for (i in 1:dim(report)[1]) {

    # Import defuse-specific fields
    fusion_tool_specific_data <- list()
    fusion_tool_specific_data[["probability"]] <- report[[i, "probability"]]

    # Cluster id
    id <- report[[i, "cluster_id"]]

    # Is the downstream fusion partner in-frame?
    if (report[[i, "orf"]] == "Y") {
      inframe <- TRUE
    } else {
      inframe <- FALSE
    }

    # Strand
    strand_upstream <- report[[i, "gene_strand1"]]
    strand_downstream <- report[[i, "gene_strand2"]]

    # Number of supporting reads
    split_reads_count <- report[[i, "splitr_count"]]
    spanning_reads_count <- report[[i, "span_count"]]

    # Get the fusion sequence. Split it into the part from gene1 and gene2
    junction_sequence <- strsplit(report[[i, "splitr_sequence"]], "\\|")
    junction_sequence_upstream <-
      Biostrings::DNAString(junction_sequence[[1]][1])
    junction_sequence_downstream <-
      Biostrings::DNAString(junction_sequence[[1]][2])

    # Breakpoints
    breakpoint_upstream <- report[[i, "genomic_break_pos1"]]
    breakpoint_downstream <- report[[i, "genomic_break_pos2"]]

    # Chromosome names
    chromosome_upstream <-
      paste("chr", report[[i, "gene_chromosome1"]], sep = "")
    chromosome_downstream <-
      paste("chr", report[[i, "gene_chromosome2"]], sep = "")

    # Gene names
    name_upstream <- report[[i, "gene_name1"]]
    name_downstream <- report[[i, "gene_name2"]]

    # Ensembl ids
    ensembl_id_upstream <- report[[i, "gene1"]]
    ensembl_id_downstream <- report[[i, "gene2"]]

    # PartnerGene objects
    gene_upstream <- new(
      Class = "PartnerGene",
      name = name_upstream,
      ensembl_id = ensembl_id_upstream,
      chromosome = chromosome_upstream,
      breakpoint = breakpoint_upstream,
      strand = strand_upstream,
      junction_sequence = junction_sequence_upstream,
      transcripts = GenomicRanges::GRangesList()
    )
    gene_downstream <- new(
      Class = "PartnerGene",
      name = name_downstream,
      ensembl_id = ensembl_id_downstream,
      chromosome = chromosome_downstream,
      breakpoint = breakpoint_downstream,
      strand = strand_downstream,
      junction_sequence = junction_sequence_downstream,
      transcripts = GenomicRanges::GRangesList()
    )

    fusion_list[[i]] <- new(
      Class = "Fusion",
      id = id,
      fusion_tool = fusion_tool,
      genome_version = genome_version,
      spanning_reads_count = spanning_reads_count,
      split_reads_count = split_reads_count,
      fusion_reads_alignment = Gviz::AlignmentsTrack(),
      gene_upstream = gene_upstream,
      gene_downstream = gene_downstream,
      inframe = inframe,
      fusion_tool_specific_data = fusion_tool_specific_data
    )
  }

  # Return the list of Fusion objects
  fusion_list
}

context("Test plot_fusion_reads()")

# Load data
defuse833ke <- system.file(
  "extdata",
  "defuse_833ke_results.filtered.tsv",
  package = "chimeraviz")
fusions <- import_defuse(defuse833ke, "hg19", 1)
# Find the specific fusion we have aligned reads for
fusion <- get_fusion_by_id(fusions, 5267)
bamfile <- system.file(
  "extdata",
  "5267readsAligned.bam",
  package = "chimeraviz")
# Temporary file to store the plot
png_filename <- tempfile(
  pattern = "fusionPlot",
  fileext = ".png",
  tmpdir = tempdir())

test_that("plot_fusion_reads produces a png file", {
  # Expect error when we haven't added the fusion reads bam file to the fusion
  # object yet
  expect_error(plot_fusion_reads(fusion, png_filename))

  # Add the bam file of aligned fusion reads to the fusion object
  fusion <- add_fusion_reads_alignment(fusion, bamfile)
  # Calculate image size based on supporting reads and lenght of junction
  # sequence.
  image_width <- (nchar(fusion@gene_upstream@junction_sequence) +
                    nchar(fusion@gene_downstream@junction_sequence)) * 10
  image_height <- (fusion@split_reads_count + fusion@spanning_reads_count) * 20
  # Open device
  png(png_filename, width = image_width, height = image_height)
  # Now we can plot
  plot_fusion_reads(fusion)
  # Close device
  dev.off()
  # See if we actually produced a file with size > 0
  expect_equal(file.info(png_filename)$size > 0, TRUE)
})

test_that("plot_fusion_reads with show_all_nucleotides=FALSE produces a png file", {
  # Expect error when we haven't added the fusion reads bam file to the fusion
  # object yet
  expect_error(plot_fusion_reads(fusion, png_filename))

  # Add the bam file of aligned fusion reads to the fusion object
  fusion <- add_fusion_reads_alignment(fusion, bamfile)
  # Calculate image size based on supporting reads and lenght of junction
  # sequence.
  image_width <- (nchar(fusion@gene_upstream@junction_sequence) +
    nchar(fusion@gene_downstream@junction_sequence)) * 10
  image_height <- (fusion@split_reads_count + fusion@spanning_reads_count) * 20
  # Open device
  png(png_filename, width = image_width, height = image_height)
  # Now we can plot
  plot_fusion_reads(fusion, show_all_nucleotides = FALSE)
  # Close device
  dev.off()
  # See if we actually produced a file with size > 0
  expect_equal(file.info(png_filename)$size > 0, TRUE)
})

context("Test plot_fusion_transcript_with_protein_domain()")

# Load data and example fusion event
defuse833ke <- system.file(
  "extdata",
  "defuse_833ke_results.filtered.tsv",
  package = "chimeraviz")
fusions <- import_defuse(defuse833ke, "hg19", 1)
fusion <- get_fusion_by_id(fusions, 5267)
# Select transcripts
gene_upstream_transcript <- "ENST00000434290"
gene_downstream_transcript <- "ENST00000370031"
# Load edb
edb_sqlite_file <- system.file(
  "extdata",
  "Homo_sapiens.GRCh37.74.sqlite",
  package = "chimeraviz")
edb <- ensembldb::EnsDb(edb_sqlite_file)
# bamfile with reads in the regions of this fusion event
bamfile5267 <- system.file(
  "extdata",
  "fusion5267and11759reads.bam",
  package = "chimeraviz")
# bedfile with protein domains for the transcripts in this example
bedfile <- system.file(
  "extdata",
  "protein_domains_5267.bed",
  package = "chimeraviz")
# Temporary file to store the plot
png_filename <- tempfile(
  pattern = "fusionPlot",
  fileext = ".png",
  tmpdir = tempdir())

test_that("plot_fusion_transcript_with_protein_domain produces a png file when bamfile is not given", {
  # Open device
  png(png_filename, width = 500, height = 500)
  # Plot and expect certain message output
  expect_message(
    plot_fusion_transcript_with_protein_domain(
      fusion = fusion,
      edb = edb,
      bamfile = NULL,
      bedfile = bedfile,
      gene_upstream_transcript = gene_upstream_transcript,
      gene_downstream_transcript = gene_downstream_transcript,
      plot_downstream_protein_domains_if_fusion_is_out_of_frame = TRUE
    ),
    paste0(
      "No protein domains are retained in the downstream gene, because the ",
      "fusion is not in frame. But since ",
      "plot_downstream_protein_domains_if_fusion_is_out_of_frame = TRUE, ",
      "we'll plot the protein domains in the downstream gene anyway.."
    )
  )
  # Close device
  dev.off()
  # See if we actually produced a file with size > 0
  expect_equal(file.info(png_filename)$size > 0, TRUE)
})

context("Test plot_circle()")

# Load data
defuse833ke <- system.file(
  "extdata",
  "defuse_833ke_results.filtered.tsv",
  package = "chimeraviz")
fusions <- import_defuse(defuse833ke, "hg19", 3)
# Temporary file to store the plot
png_filename <- tempfile(
  pattern = "circlePlot",
  fileext = ".png",
  tmpdir = tempdir())

test_that("plot_circle produces a png file", {
  # Open device
  png(png_filename, width = 1000, height = 750)
  # Plot and expect certain message output
  plot_circle(fusions)
  # Close device
  dev.off()
  # See if we actually produced a file with size > 0
  expect_equal(file.info(png_filename)$size > 0, TRUE)
})

test_that("plot_circle() fails with invalid input", {
  # List without fusion objects?
  expect_error(plot_circle(list()))
  # List with fusion objects and something else?
  expect_error(plot_circle(c(fusions, "test")))
  # List with fusion objects where the genomes differ
  fusions[[1]]@genome_version <- "test"
  expect_error(plot_circle(fusions))
  # Inputs that are not lists?
  expect_error(plot_circle(42))
  expect_error(plot_circle("hey this should fail"))
})

context(
  paste0(
    "Test creating link data that RCircos will accept from a list of Fusion ",
    "object"
  )
)

defuse833ke <- system.file(
  "extdata",
  "defuse_833ke_results.filtered.tsv",
  package = "chimeraviz")
fusions <- import_defuse(defuse833ke, "hg19", 3)
link_data <- .fusions_to_link_data(fusions)

test_that(".fusions_to_link_data works", {
  expect_equal(length(link_data$chromosome), length(fusions))
  expect_equal(
    as.character(link_data$chromosome[1]),
    fusions[[1]]@gene_upstream@chromosome
  )
})

context("Test plot_fusion_transcript()")

# Load data and example fusion event
defuse833ke <- system.file(
  "extdata",
  "defuse_833ke_results.filtered.tsv",
  package = "chimeraviz")
fusions <- import_defuse(defuse833ke, "hg19", 1)
fusion <- get_fusion_by_id(fusions, 5267)
# Load edb
edb_sqlite_file <- system.file(
  "extdata",
  "Homo_sapiens.GRCh37.74.sqlite",
  package = "chimeraviz")
edb <- ensembldb::EnsDb(edb_sqlite_file)
# Temporary file to store the plot
png_filename <- tempfile(
  pattern = "fusionPlot",
  fileext = ".png",
  tmpdir = tempdir())

test_that("plot_fusion_transcript produces a png file", {
  # Open device
  png(png_filename, width = 500, height = 500)
  # Plot and expect certain message output
  expect_message(
    plot_fusion_transcript(
      fusion = fusion,
      edb = edb),
    "Fetching transcripts for gene partners..")
  # Close device
  dev.off()
  # See if we actually produced a file with size > 0
  expect_equal(file.info(png_filename)$size > 0, TRUE)
})

test_that("plot_fusion_transcript with bamfile produces a png file", {
  bamfile5267 <- system.file(
    "extdata",
    "fusion5267and11759reads.bam",
    package = "chimeraviz")
  # Open device
  png(png_filename, width = 500, height = 500)
  # Plot and expect certain message output
  expect_message(
    plot_fusion_transcript(
      fusion = fusion,
      bamfile = bamfile5267,
      edb = edb),
    "Fetching transcripts for gene partners..")
  # Close device
  dev.off()
  # See if we actually produced a file with size > 0
  expect_equal(file.info(png_filename)$size > 0, TRUE)
})

test_that("plot_fusion_transcript with bedgraphfile produces a png file", {
  bedgraphfile <- system.file(
    "extdata",
    "fusion5267and11759reads.bedGraph",
    package = "chimeraviz"
  )
  # Open device
  png(png_filename, width = 500, height = 500)
  # Plot and expect certain message output
  expect_message(
    plot_fusion_transcript(
      fusion = fusion,
      bedgraphfile = bedgraphfile,
      edb = edb),
    "Fetching transcripts for gene partners..")
  # Close device
  dev.off()
  # See if we actually produced a file with size > 0
  expect_equal(file.info(png_filename)$size > 0, TRUE)
})

context("Test importing defuse data into a list of Fusion objects")

defuse833ke <- system.file("extdata",
                           "defuse_833ke_results.filtered.tsv",
                           package = "chimeraviz")
fusions <- import_defuse(defuse833ke, "hg19", 3)
fusion <- fusions[[1]]

test_that("import_defuse imports data into Fusion objects", {
  expect_equal(length(fusions), 3)
  expect_equal(fusion@id, "5267")
  expect_equal(fusion@gene_upstream@name, "RCC1")
  expect_equal(fusion@gene_upstream@chromosome, "chr1")
})

test_that("import_defuse fails to import data", {
  expect_error(import_defuse("Non-existent-file", "hg18"))
  expect_error(import_defuse(defuse833ke, "Invalid-genome"))
  expect_error(import_defuse(defuse833ke, "hg19", "invalid limit"))
})

test_that("import_defuse limits work", {
  expect_equal(length(import_defuse(defuse833ke, "hg19", 2)), 2)
})

context(
  paste0(
    "Test creating gene label data that RCircos will accept from a list of ",
    "Fusion object"
  )
)

defuse833ke <- system.file(
  "extdata",
  "defuse_833ke_results.filtered.tsv",
  package = "chimeraviz")
fusions <- import_defuse(defuse833ke, "hg19", 3)
label_data <- .fusions_to_gene_label_data(fusions)

test_that(".fusions_to_gene_label_data works", {
  expect_equal(length(label_data$chromosome), length(fusions) * 2)
  expect_equal(
    as.character(label_data$chromosome[1]),
    fusions[[1]]@gene_upstream@chromosome
  )
})

context("Test creating Fusion reports")

test_that("create_fusion_report() produces a html file", {
  # Load data
  defuse833ke <- system.file(
    "extdata",
    "defuse_833ke_results.filtered.tsv",
    package = "chimeraviz")
  fusions <- import_defuse(defuse833ke, "hg19", 5)
  # Temporary file to store the report
  random_filename <- paste0(
    paste0(sample(LETTERS, 5, replace = TRUE), collapse = ""),
    ".png"
  )
  create_fusion_report(fusions, random_filename)
  # See if we actually produced a file with size > 0
  expect_equal(file.info(random_filename)$size > 0, TRUE)
  # Delete the temporary file
  file.remove(random_filename)
})

test_that("create_fusion_report() fails with invalid input", {
  # List without fusion objects?
  expect_error(create_fusion_report(list(42)))
})

#' chimeraviz: A package for working with and visualizing fusion genes.
#'
#' chimeraviz manages data from fusion gene finders and provides useful
#' visualization tools.
#'
#' @docType package
#' @name chimeraviz
#'
#' @import Biostrings GenomicRanges IRanges RCircos methods Gviz graphics
#' BiocStyle
NULL
#> NULL

#' Plot a specific fusion transcript with protein domain annotations
#'
#' This function takes a fusion object, an ensembldb object, a bedfile with
#' protein domain data and two specific transcript ids. The function plots the
#' specific fusion transcript along with annotations of protein domains. If a
#' bamfile is specified, the fusion transcript will be plotted with coverage
#' information.
#'
#' Note that the transcript database used (the edb object) must have the same
#' seqnames as any bamfile used. Otherwise the coverage data will be wrong.
#'
#' @param fusion The Fusion object to plot.
#' @param edb The edb object that will be used to fetch data.
#' @param bamfile The bamfile with RNA-seq data.
#' @param bedfile The bedfile with protein domain data.
#' @param gene_upstream_transcript The transcript id for the upstream gene.
#' @param gene_downstream_transcript The transcript id for the downstream gene.
#' @param plot_downstream_protein_domains_if_fusion_is_out_of_frame Setting this to true
#' makes the function plot protein domains in the downstream gene even though
#' the fusion is out of frame.
#'
#' @return Creates a fusion transcript plot with annotations of protein domains.
#'
#' @examples
#' # Load data and example fusion event
#' defuse833ke <- system.file(
#'   "extdata",
#'   "defuse_833ke_results.filtered.tsv",
#'   package="chimeraviz")
#' fusions <- import_defuse(defuse833ke, "hg19", 1)
#' fusion <- get_fusion_by_id(fusions, 5267)
#' # Select transcripts
#' gene_upstream_transcript <- "ENST00000434290"
#' gene_downstream_transcript <- "ENST00000370031"
#' # Load edb
#' edbSqliteFile <- system.file(
#'   "extdata",
#'   "Homo_sapiens.GRCh37.74.sqlite",
#'   package="chimeraviz")
#' edb <- ensembldb::EnsDb(edbSqliteFile)
#' # bamfile with reads in the regions of this fusion event
#' bamfile5267 <- system.file(
#'   "extdata",
#'   "fusion5267and11759reads.bam",
#'   package="chimeraviz")
#' # bedfile with protein domains for the transcripts in this example
#' bedfile <- system.file(
#'   "extdata",
#'   "protein_domains_5267.bed",
#'   package="chimeraviz")
#' # Temporary file to store the plot
#' pngFilename <- tempfile(
#'   pattern = "fusionPlot",
#'   fileext = ".png",
#'   tmpdir = tempdir())
#' # Open device
#' png(pngFilename, width = 500, height = 500)
#' # Plot!
#' plot_fusion_transcript_with_protein_domain(
#'   fusion = fusion,
#'   edb = edb,
#'   bamfile = bamfile5267,
#'   bedfile = bedfile,
#'   gene_upstream_transcript = gene_upstream_transcript,
#'   gene_downstream_transcript = gene_downstream_transcript,
#'   plot_downstream_protein_domains_if_fusion_is_out_of_frame = TRUE)
#' # Close device
#' dev.off()
#'
#' @importFrom data.table fread
#'
#' @export
#'
plot_fusion_transcript_with_protein_domain <- function(
  fusion,
  edb = NULL,
  bamfile = NULL,
  bedfile = NULL,
  gene_upstream_transcript = "",
  gene_downstream_transcript = "",
  plot_downstream_protein_domains_if_fusion_is_out_of_frame = FALSE) {

  # Validate that we're given a bedfile that exists
  if (class(bedfile) != "character") {
    stop("bedfile argument must be a character")
  }
  if (file.exists(bedfile) == FALSE) {
    stop("Invalid bedfile: File doesn't exist.")
  }
  # Validate geneAtranscript and geneBtranscript
  if (class(gene_upstream_transcript) != "character") {
    stop("gene_upstream_transcript argument must be a character")
  }
  if (class(gene_downstream_transcript) != "character") {
    stop("gene_downstream_transcript argument must be a character")
  }
  if (gene_upstream_transcript == "") {
    stop("gene_upstream_transcript argument must not be blank")
  }
  if (gene_downstream_transcript == "") {
    stop("gene_downstream_transcript argument must not be blank")
  }
  # Validate parameters and fetch transcripts from database
  .validate_plot_fusion_transcript_with_protein_domain_params(
    fusion,
    edb,
    bamfile,
    bedfile,
    gene_upstream_transcript,
    gene_downstream_transcript,
    plot_downstream_protein_domains_if_fusion_is_out_of_frame)
  fusion <- .get_transcripts_if_not_there(fusion, edb)
  # Validate that the transcript ids are found in the fusion
  if (
    length(
      fusion@gene_upstream@transcripts[
        names(fusion@gene_upstream@transcripts) == gene_upstream_transcript
      ]) == 0) {
    stop(
      paste0(
        gene_upstream_transcript,
        " was not found in among the transcripts for the upstream gene ",
        "partner."
      )
    )
  }
  if (
    length(
      fusion@gene_downstream@transcripts[
        names(fusion@gene_downstream@transcripts) == gene_downstream_transcript
      ]) == 0) {
    stop(
      paste0(
        gene_downstream_transcript,
        " was not found in among the transcripts for the downstream gene ",
        "partner."
      )
    )
  }
  # Extract the transcripts
  transcript_upstream <- unlist(
    fusion@gene_upstream@transcripts[
      names(fusion@gene_upstream@transcripts) == gene_upstream_transcript
    ]
  )
  transcript_downstream <- unlist(
    fusion@gene_downstream@transcripts[
      names(fusion@gene_downstream@transcripts) == gene_downstream_transcript
    ]
  )
  # Check that there's at least one transcript that has the fusion breakpoint
  # within the transcript.
  .check_that_breakpoints_are_within_transcripts(
    fusion,
    transcript_upstream,
    transcript_downstream
  )
  # Check that the transcripts have a breakpoint exon
  .check_that_transcripts_have_breakpoint_exons(
    fusion,
    transcript_upstream,
    transcript_downstream
  )
  # Add transcript metadata column to make Gviz group correctly
  mcols(transcript_upstream)$transcript <- fusion@gene_upstream@name
  mcols(transcript_downstream)$transcript <- fusion@gene_downstream@name
  # Add symbol metadata column to make Gviz show ids
  mcols(transcript_upstream)$symbol <- fusion@gene_upstream@name
  mcols(transcript_downstream)$symbol <- fusion@gene_downstream@name

  # Downshift transcripts (remove introns from the GRanges)
  transcript_upstream_downshifted <- down_shift(transcript_upstream)
  transcript_downstream_downshifted <- down_shift(transcript_downstream)

  # Figure out which exons from geneA and geneB are part of the fusion
  # transcript.
  #
  # Also, "rediscover" where the fusion breakpoints are in the downshifted
  # versions of the transcripts.
  #
  # 1: geneA is on the + strand
  #    - Exons part of the fusion transcript are those with start/end positions
  #      before breakpoint
  #    - Fusion breakpoint in the downshifted transcript is at the end
  #      coordinate of exon_id_at_fusion_breakpoint_a
  # 2: geneA is on the - strand
  #    - Exons part of the fusion transcript are those with start/end positions
  #      after breakpoint
  #    - Fusion breakpoint in the downshifted transcript is at the start
  #      coordinate of exon_id_at_fusion_breakpoint_a
  # 3: geneB is on the + strand
  #    - Exons part of the fusion transcript are those with start/end positions
  #      after breakpoint
  #    - Fusion breakpoint in the downshifted transcript is at the start
  #      coordinate of exon_id_at_fusion_breakpoint_b
  # 4: geneB is on the - strand
  #    - Exons part of the fusion transcript are those with start/end positions
  #      before breakpoint
  #    - Fusion breakpoint in the downshifted transcript is at the end
  #      coordinate of exon_id_at_fusion_breakpoint_b
  #
  if (fusion@gene_upstream@strand == "+") {

    exon_ids_part_of_fusion_from_transcript_a <- mcols(
      transcript_upstream[
        start(ranges(transcript_upstream)) < fusion@gene_upstream@breakpoint &
          end(ranges(transcript_upstream)) <= fusion@gene_upstream@breakpoint
      ]
    )$exon_id
    exons_part_of_fusion_a <- transcript_upstream[
      start(ranges(transcript_upstream)) < fusion@gene_upstream@breakpoint &
        end(ranges(transcript_upstream)) <= fusion@gene_upstream@breakpoint
    ]
    exon_id_at_fusion_breakpoint_a <- mcols(
      transcript_upstream[
        end(ranges(transcript_upstream)) == fusion@gene_upstream@breakpoint
      ]
    )$exon_id

    fusion_breakpoint_in_downshifted_transcript_a <-
      max(
        end(
          transcript_upstream_downshifted[
            mcols(transcript_upstream_downshifted)$exon_id ==
              exon_id_at_fusion_breakpoint_a
          ]
        )
      )
    exons_part_of_fusion_downshifted_a <-
      transcript_upstream_downshifted[
        start(ranges(transcript_upstream_downshifted)) <
          fusion_breakpoint_in_downshifted_transcript_a &
        end(ranges(transcript_upstream_downshifted)) <=
          fusion_breakpoint_in_downshifted_transcript_a
      ]

  } else {

    exon_ids_part_of_fusion_from_transcript_a <-
      mcols(
        transcript_upstream[
          start(ranges(transcript_upstream)) >=
            fusion@gene_upstream@breakpoint &
          end(ranges(transcript_upstream)) >
            fusion@gene_upstream@breakpoint
        ]
      )$exon_id
    exons_part_of_fusion_a <- transcript_upstream[
      start(ranges(transcript_upstream)) >=
        fusion@gene_upstream@breakpoint &
      end(ranges(transcript_upstream)) >
        fusion@gene_upstream@breakpoint
    ]
    exon_id_at_fusion_breakpoint_a <-
      mcols(
        transcript_upstream[
          start(ranges(transcript_upstream)) == fusion@gene_upstream@breakpoint
        ]
      )$exon_id

    fusion_breakpoint_in_downshifted_transcript_a <-
      min(
        start(
          transcript_upstream_downshifted[
            mcols(transcript_upstream_downshifted)$exon_id ==
              exon_id_at_fusion_breakpoint_a
          ]
        )
      )
    exons_part_of_fusion_downshifted_a <-
      transcript_upstream_downshifted[
        start(ranges(transcript_upstream_downshifted)) >=
          fusion_breakpoint_in_downshifted_transcript_a &
        end(ranges(transcript_upstream_downshifted)) >
          fusion_breakpoint_in_downshifted_transcript_a
      ]

  }
  if (fusion@gene_downstream@strand == "+") {

    exon_ids_part_of_fusion_from_transcript_b <- mcols(
      transcript_downstream[
        start(ranges(transcript_downstream)) >=
          fusion@gene_downstream@breakpoint &
        end(ranges(transcript_downstream)) >
          fusion@gene_downstream@breakpoint
      ]
    )$exon_id
    exons_part_of_fusion_b <- transcript_downstream[
      start(ranges(transcript_downstream)) >=
        fusion@gene_downstream@breakpoint &
      end(ranges(transcript_downstream)) >
        fusion@gene_downstream@breakpoint
    ]
    exon_id_at_fusion_breakpoint_b <- mcols(
      transcript_downstream[
        start(ranges(transcript_downstream)) ==
          fusion@gene_downstream@breakpoint
      ]
    )$exon_id

    fusion_breakpoint_in_downshifted_transcript_b <- min(
      start(
        transcript_downstream_downshifted[
          mcols(transcript_downstream_downshifted)$exon_id ==
            exon_id_at_fusion_breakpoint_b
        ]
      )
    )
    exons_part_of_fusion_downshifted_b <- transcript_downstream_downshifted[
      start(ranges(transcript_downstream_downshifted)) >=
        fusion_breakpoint_in_downshifted_transcript_b &
      end(ranges(transcript_downstream_downshifted)) >
        fusion_breakpoint_in_downshifted_transcript_b
    ]

  } else {

    exon_ids_part_of_fusion_from_transcript_b <- mcols(
      transcript_downstream[
        start(ranges(transcript_downstream)) <
          fusion@gene_downstream@breakpoint &
        end(ranges(transcript_downstream)) <=
          fusion@gene_downstream@breakpoint
      ]
    )$exon_id
    exons_part_of_fusion_b <- transcript_downstream[
      start(ranges(transcript_downstream)) <
        fusion@gene_downstream@breakpoint &
      end(ranges(transcript_downstream)) <=
        fusion@gene_downstream@breakpoint
    ]
    exon_id_at_fusion_breakpoint_b <- mcols(
      transcript_downstream[
        end(ranges(transcript_downstream)) == fusion@gene_downstream@breakpoint
      ]
    )$exon_id

    fusion_breakpoint_in_downshifted_transcript_b <- max(
      end(
        transcript_downstream_downshifted[
          mcols(transcript_downstream_downshifted)$exon_id ==
            exon_id_at_fusion_breakpoint_b
        ]
      )
    )
    exons_part_of_fusion_downshifted_b <- transcript_downstream_downshifted[
      start(ranges(transcript_downstream_downshifted)) <
        fusion_breakpoint_in_downshifted_transcript_b &
      end(ranges(transcript_downstream_downshifted)) <=
        fusion_breakpoint_in_downshifted_transcript_b
    ]

  }

  # Load protein domain data
  # Specify which columns from the bedfile we're going to read
  col_types_protein_data <- c(
    "Transcript_id" = "character",
    "Pfam_id" = "character",
    "Start" = "integer",
    "End" = "integer",
    "Domain_name_abbreviation" = "character",
    "Domain_name_full" = "character"
  )
  # Read the bedfile
  protein_data <- data.table::fread(
    input = bedfile,
    colClasses = col_types_protein_data,
    showProgress = FALSE
  )
  # Validate that the transcript ids are found in the bedfile
  if (
    nrow(
      protein_data[protein_data$Transcript_id == gene_upstream_transcript, ]
    ) == 0
  ) {
    stop(
      paste0(
        "Found no transcripts matching ",
        gene_upstream_transcript,
        " in the edb database."
      )
    )
  }
  if (
    nrow(
      protein_data[protein_data$Transcript_id == gene_downstream_transcript, ]
    ) == 0
  ) {
    stop(
      paste0(
        "Found no transcripts matching ",
        gene_downstream_transcript,
        " in the edb database."
      )
    )
  }
  # Extract the rows we need
  protein_data_gene_upstream <-
    protein_data[protein_data$Transcript_id == gene_upstream_transcript, ]
  protein_data_gene_downstream <-
    protein_data[protein_data$Transcript_id == gene_downstream_transcript, ]

  # For each protein domain we have (for each row in proteinDataGeneA and
  # proteinDataGeneb), decide which of the three categories the protein domain
  # is in: inside, outside, insideButBroken.
  #
  # The categories are explained as follows:
  # - inside:       Within the exons that are part of the fusion transcript.
  # - outside:      Not within the exons that are part of the fusion transcript
  # - insideButBroken: Partly within the exons that are part of the fusion
  #                 transcript (runs over one of the edges...)
  #
  # If a protein domain is inside, then the protein domain is retained in the
  # fusion transcript, and we can plot the annotation without problems.
  #
  # If a protein domain is outside, then it is not retained in the fusion
  # transcript and we can ignore it.
  #
  # If a protein domain is insideButBroken, then it is worth noting and we will
  # include it in the plot, but colored so that it's apparent that it is not
  # retained in the fusion transcript.
  #
  # Begin Exclude Linting
  # 1: geneA is on the + strand
  #    - The protein domain is "inside" if:
  #        protein_start_coordinate >=
  #          min(start(exons_part_of_fusion_downshifted_a)) &
  #        protein_end_coordinate <=
  #          max(end(exons_part_of_fusion_downshifted_a))
  #    - The protein domain is "insideButBroken" if:
  #        protein_start_coordinate >=
  #          min(start(exons_part_of_fusion_downshifted_a)) &
  #        protein_start_coordinate <
  #          max(end(exons_part_of_fusion_downshifted_a))
  #
  # 2: geneA is on the - strand
  #    - The protein domain is "inside" if:
  #        protein_start_coordinate <=
  #          max(end(exons_part_of_fusion_downshifted_a)) &
  #        protein_end_coordinate >=
  #          min(start(exons_part_of_fusion_downshifted_a))
  #    - The protein domain is "insideButBroken" if:
  #        protein_start_coordinate <=
  #          max(end(exons_part_of_fusion_downshifted_a)) &
  #        protein_start_coordinate >
  #          min(start(exons_part_of_fusion_downshifted_a))
  #
  # 3: geneB is on the + strand
  #    - The protein domain is "inside" if:
  #        protein_start_coordinate >=
  #          min(start(exons_part_of_fusion_downshifted_b)) &
  #        protein_end_coordinate <=
  #          max(end(exons_part_of_fusion_downshifted_b))
  #    - The protein domain is "insideButBroken" if:
  #        protein_start_coordinate >=
  #          min(start(exons_part_of_fusion_downshifted_b)) &
  #        protein_start_coordinate <
  #          max(end(exons_part_of_fusion_downshifted_b))
  #
  # 4: geneB is on the - strand
  #    - The protein domain is "inside" if:
  #        protein_start_coordinate <=
  #          max(end(exons_part_of_fusion_downshifted_b)) &
  #        protein_end_coordinate >=
  #          min(start(exons_part_of_fusion_downshifted_b))
  #    - The protein domain is "insideButBroken" if:
  #        protein_start_coordinate <=
  #          max(end(exons_part_of_fusion_downshifted_b)) &
  #        protein_start_coordinate >
  #          min(start(exons_part_of_fusion_downshifted_b))
  # End Exclude Linting

  # Go through all protein domains to classify them.
  # First extend the proteinDataGeneA and proteinDataGeneB data frames:
  protein_data_gene_upstream$protein_domain_location <- NA_character_
  protein_data_gene_upstream$plot_start <- NA_integer_
  protein_data_gene_upstream$plot_end <- NA_integer_
  protein_data_gene_downstream$protein_domain_location <- NA_character_
  protein_data_gene_downstream$plot_start <- NA_integer_
  protein_data_gene_downstream$plot_end <- NA_integer_
  # a)
  for (i in 1:nrow(protein_data_gene_upstream)) {
    if (
      protein_data_gene_upstream[i, ]$Start >=
        min(start(exons_part_of_fusion_downshifted_a)) &
      protein_data_gene_upstream[i, ]$End <=
        max(end(exons_part_of_fusion_downshifted_a))
    ) {
      # Is it "inside"?
      protein_data_gene_upstream[i, ]$protein_domain_location <- "inside"
      protein_data_gene_upstream[i, ]$plot_start <-
        protein_data_gene_upstream[i, ]$Start
      protein_data_gene_upstream[i, ]$plot_end <-
        protein_data_gene_upstream[i, ]$End
      message(
        paste0(
          "The protein domain ",
          protein_data_gene_upstream[i, ]$Domain_name_abbreviation,
          " from the upstream gene (",
          fusion@gene_upstream@name,
          ") is retained in the fusion transcript!"
        )
      )
    } else if (
      protein_data_gene_upstream[i, ]$Start >=
        min(start(exons_part_of_fusion_downshifted_a)) &
      protein_data_gene_upstream[i, ]$Start <
        max(end(exons_part_of_fusion_downshifted_a))
    ) {
      # Is it partly inside?
      protein_data_gene_upstream[i, ]$protein_domain_location <-
        "insideButBroken"
      protein_data_gene_upstream[i, ]$plot_start <-
        protein_data_gene_upstream[i, ]$Start
      protein_data_gene_upstream[i, ]$plot_end <-
        fusion_breakpoint_in_downshifted_transcript_a
    } else {
      protein_data_gene_upstream[i, ]$protein_domain_location <- "outside"
    }
  }
  if (
    nrow(
      dplyr::filter(
        protein_data_gene_upstream,
        protein_domain_location == "inside"
      )
    ) == 0
  ) {
    message("No protein domains are retained in the upstream gene.")
  }
  # b)
  if (fusion@inframe ||
      plot_downstream_protein_domains_if_fusion_is_out_of_frame
    ) {
    # We only consider downstream protein domains if the reading frame is kept
    # in the fusion, or if the user specifically asks for it.
    if (!fusion@inframe &&
        plot_downstream_protein_domains_if_fusion_is_out_of_frame
      ) {
      message(
        paste0(
          "No protein domains are retained in the downstream gene, because ",
          "the fusion is not in frame. But since ",
          "plot_downstream_protein_domains_if_fusion_is_out_of_frame = TRUE,",
          " we'll plot the protein domains in the downstream gene anyway.."
        )
      )
    }
    for (i in 1:nrow(protein_data_gene_downstream)) {
      if (
        protein_data_gene_downstream[i, ]$Start >=
          min(start(exons_part_of_fusion_downshifted_b)) &
        protein_data_gene_downstream[i, ]$End <=
          max(end(exons_part_of_fusion_downshifted_b))
      ) {
        # Is it "inside"?
        protein_data_gene_downstream[i, ]$protein_domain_location <- "inside"
        # When we connect the exons from geneA that are part of the fusion with
        # the exons from geneB that are part of the fusion, then the coordinates
        # we'll plot the protein domain on will shift to the right (as many
        # steps as the total length of the exons that are included from geneA).
        # That is why we add sum(width(exons_part_of_fusion_downshifted_a)) -
        # the total length of the exons that are included from geneA - below:
        protein_data_gene_downstream[i, ]$plot_start <-
          protein_data_gene_downstream[i, ]$Start +
            sum(width(exons_part_of_fusion_downshifted_a))
        protein_data_gene_downstream[i, ]$plot_end <-
          protein_data_gene_downstream[i, ]$End +
            sum(width(exons_part_of_fusion_downshifted_a))
        message(
          paste0(
            "The protein domain ",
            protein_data_gene_downstream[i, ]$Domain_name_abbreviation,
            " from the downstream gene (",
            fusion@gene_downstream@name,
            ") is retained in the fusion transcript!"
          )
        )
      } else if (
        protein_data_gene_downstream[i, ]$Start >=
          min(start(exons_part_of_fusion_downshifted_b)) &
        protein_data_gene_downstream[i, ]$Start <
          max(end(exons_part_of_fusion_downshifted_b))
      ) {
        # Is it partly inside?
        protein_data_gene_downstream[i, ]$protein_domain_location <-
          "insideButBroken"
        protein_data_gene_downstream[i, ]$plot_start <-
          protein_data_gene_downstream[i, ]$Start +
            sum(width(exons_part_of_fusion_downshifted_a))
        protein_data_gene_downstream[i, ]$plot_end <-
          fusion_breakpoint_in_downshifted_transcript_b
      } else {
        protein_data_gene_downstream[i, ]$protein_domain_location <- "outside"
      }
    }
    if (
      nrow(
        dplyr::filter(
          protein_data_gene_downstream,
          protein_domain_location == "inside"
        )
      ) == 0
    ) {
      message("No protein domains are retained in the downstream gene.")
    }
  } else {
    message(paste0("The fusion is out of frame, so no protein domains are ",
                   "retained in the downstream gene."))
  }

  # Reverse order of exons if on minus strand
  if (fusion@gene_upstream@strand == "-") {
    fusion_exons_upstream <- rev(exons_part_of_fusion_a)
  } else {
    fusion_exons_upstream <- exons_part_of_fusion_a
  }
  if (fusion@gene_downstream@strand == "-") {
    fusion_exons_downstream <- rev(exons_part_of_fusion_b)
  } else {
    fusion_exons_downstream <- exons_part_of_fusion_b
  }

  # Build fusion transcript
  fusion_transcript <- append(
    fusion_exons_upstream,
    fusion_exons_downstream
  )

  # If we've got a bamfile, calculate coverage
  if (!is.null(bamfile)) {
    # Get coverage
    cov <- coverage(
      bamfile,
      param = Rsamtools::ScanBamParam(
        which = fusion_transcript
      )
    )    ####  This must be unshifted!
    # Coverage only for my transcript
    cov <- cov[fusion_transcript]
    # Unlist
    cov <- suppressWarnings(unlist(cov))
    # Turn into numeric vector
    cov <- as.numeric(cov)

    # Create coverage track
    d_track <- DataTrack(
      start = seq_along(cov),
      width = 1,
      chromosome = "chrNA",
      genome = "hg19",
      name = "Coverage",
      type = "h",
      col = "orange",
      fill = "orange",
      data = cov)
    # Set display parameters
    Gviz::displayPars(d_track) <- list(
      showTitle = FALSE, # hide name of track
      background.panel = "transparent", # background color of the content panel
      background.title = "transparent", # background color for the title panels
      cex.axis = .6,
      cex.title = .6,
      col.axis = "black",
      col.coverage = "black",
      col.title = "black",
      coverageHeight = 0.08,
      fill.coverage = "orange",
      fontsize = 15,
      lty = 1,
      lty.coverage = 1,
      lwd = 0.5
    )
  }

  # Downshift the fusion transcript (remove introns)
  fusion_transcript <- down_shift(fusion_transcript)

  # Create new GRanges so that we can set the right seqname
  gr <- GRanges(seqnames = "chrNA", ranges = ranges(fusion_transcript))
  mcols(gr)$symbol <- mcols(fusion_transcript)$symbol

  # Create transcript track
  tr_track <- Gviz::GeneRegionTrack(
    gr,
    chromosome = "chrNA")
  # Set display parameters
  Gviz::displayPars(tr_track) <- list(
    background.panel = "transparent", # background color of the content panel
    background.title = "transparent", # background color for the title panels
    col.title = "black",
    showTitle = TRUE, # hide left panel
    showId = FALSE, # show transcript id
    col = "lightgray", # border around exons
    showTitle = FALSE, # hide title
    thinBoxFeature = c(
      "5utr_excludedA",
      "5utr_includedA",
      "3utr_excludedA",
      "3utr_includedA",

      "5utr_excludedB",
      "5utr_includedB",
      "3utr_excludedB",
      "3utr_includedB"
    )
  )

  # Color exons
  cols <- RColorBrewer::brewer.pal(4, "Paired")
  interestcolor <- list(
    "5utr_excludedA" = cols[1],
    "protein_coding_excludedA" = cols[1],
    "3utr_excludedA" = cols[1],

    "5utr_includedA" = cols[2],
    "protein_coding_includedA" = cols[2],
    "3utr_includedA" = cols[2],

    "5utr_excludedB" = cols[3],
    "protein_coding_excludedB" = cols[3],
    "3utr_excludedB" = cols[3],

    "5utr_includedB" = cols[4],
    "protein_coding_includedB" = cols[4],
    "3utr_includedB" = cols[4])
  Gviz::feature(tr_track)[
    Gviz::symbol(tr_track) == fusion@gene_upstream@name
  ] <- "protein_coding_includedA"
  Gviz::feature(tr_track)[
    Gviz::symbol(tr_track) == fusion@gene_upstream@name &
      mcols(fusion_transcript)$feature == "5utr"
  ] <- "5utr_includedA"
  Gviz::feature(tr_track)[
    Gviz::symbol(tr_track) == fusion@gene_upstream@name &
      mcols(fusion_transcript)$feature == "3utr"
  ] <- "3utr_includedA"
  Gviz::feature(tr_track)[
    Gviz::symbol(tr_track) == fusion@gene_downstream@name
  ] <- "protein_coding_includedB"
  Gviz::feature(tr_track)[
    Gviz::symbol(tr_track) == fusion@gene_downstream@name &
      mcols(fusion_transcript)$feature == "5utr"
  ] <- "5utr_includedB"
  Gviz::feature(tr_track)[
    Gviz::symbol(tr_track) == fusion@gene_downstream@name &
      mcols(fusion_transcript)$feature == "3utr"
  ] <- "3utr_includedB"
  Gviz::displayPars(tr_track) <- interestcolor

  # Create axis track
  axis_track <- Gviz::GenomeAxisTrack()
  Gviz::displayPars(axis_track) <- list(
    col = "black", # line color
    fontcolor = "black",
    fontsize =  25,
    lwd = 1.5,
    showId = TRUE
  )

  # What are the start and end positions of the protein domain annotations?
  protein_domain_annotations_start <- c(
    dplyr::filter(
      protein_data_gene_upstream,
      protein_domain_location == "inside"
    )$plot_start,
    dplyr::filter(
      protein_data_gene_downstream,
      protein_domain_location == "inside"
    )$plot_start
  )
  protein_domain_annotations_end <- c(
    dplyr::filter(
      protein_data_gene_upstream,
      protein_domain_location == "inside"
    )$plot_end,
    dplyr::filter(
      protein_data_gene_downstream,
      protein_domain_location == "inside"
    )$plot_end
  )
  protein_domain_annotations_names <- c(
    dplyr::filter(
      protein_data_gene_upstream,
      protein_domain_location == "inside"
    )$Domain_name_abbreviation,
    dplyr::filter(
      protein_data_gene_downstream,
      protein_domain_location == "inside"
    )$Domain_name_abbreviation
  )
  # Create protein domain track
  annotation_track <- Gviz::AnnotationTrack(
    start = protein_domain_annotations_start,
    end = protein_domain_annotations_end,
    chromosome = "chrNA",
    id = protein_domain_annotations_names,
    genome = "hg19")
  # Set display parameters
  Gviz::displayPars(annotation_track) <- list(
    background.panel = "transparent", # background color of the content panel
    background.title = "transparent", # background color for the title panels
    col.title = "black",
    showTitle = FALSE, # show title
    showId = FALSE, # show ids
    col = "lightgray", # border color
    showTitle = FALSE, # hide left title
    featureAnnotation = "id",
    fontcolor.feature = "black"
  )

  # If we've got a bamfile, plot with coverage
  if (!is.null(bamfile)) {
    Gviz::plotTracks(
      list(d_track, tr_track, annotation_track, axis_track),
      main = paste(
        fusion@gene_upstream@name,
        fusion@gene_downstream@name,
        sep = ":"
      ),
      chromosome = "chrNA")
  } else {
    Gviz::plotTracks(
      list(tr_track, annotation_track, axis_track),
      main = paste(
        fusion@gene_upstream@name,
        fusion@gene_downstream@name,
        sep = ":"
      ),
      chromosome = "chrNA")
  }

}

.validate_plot_fusion_transcript_with_protein_domain_params <- function(
  fusion,
  edb,
  bamfile,
  bedfile,
  gene_upstream_transcript,
  gene_downstream_transcript,
  plot_downstream_protein_domains_if_fusion_is_out_of_frame
) {
  # Establish a new 'AssertCollection' object
  argument_checker <- checkmate::makeAssertCollection()

  # Check parameters
  .is_fusion_valid(argument_checker, fusion)
  .is_edb_valid(argument_checker, edb, fusion)
  .is_bamfile_valid_allow_null(argument_checker, bamfile)
  .is_bedfile_valid(argument_checker, bedfile)
  .is_character_parameter_valid(
    argument_checker,
    gene_upstream_transcript,
    "gene_upstream_transcript")
  .is_character_parameter_valid(
    argument_checker,
    gene_downstream_transcript,
    "gene_downstream_transcript")
  .is_parameter_boolean(
    argument_checker,
    plot_downstream_protein_domains_if_fusion_is_out_of_frame,
    "plot_downstream_protein_domains_if_fusion_is_out_of_frame")

  # Return errors and warnings (if any)
  checkmate::reportAssertions(argument_checker)
}

#' Import results from a SQUID run into a list of Fusion objects.
#'
#' A function that imports the results from a SQUID run into a list of Fusion
#' objects.
#'
#' @param filename Filename for the SQUID results.
#' @param genome_version Which genome was used in mapping (hg19, hg38, etc.).
#' @param limit A limit on how many lines to read.
#'
#' @return A list of Fusion objects.
#'
#' @examples
#' squidfile <- system.file(
#'   "extdata",
#'   "squid_hcc1954_sv.txt",
#'   package="chimeraviz")
#' fusions <- import_squid(squidfile, "hg19", 3)
#' # This should import a list of 3 fusions described in Fusion objects.

#' @importFrom data.table fread
#'
#' @export
import_squid <- function(filename, genome_version, limit) {

  # Is the genome version valid?
  valid_genomes <- c("hg19", "hg38", "mm10")
  if (is.na(match(tolower(genome_version), tolower(valid_genomes)))) {
    stop("Invalid genome version given")
  }

  # If the limit is set, is the value valid?
  if (missing(limit) == FALSE) {
    if (is.numeric(limit) == FALSE || limit <= 0) {
      stop("limit must be a numeric value bigger than 0")
    }
  }

  # Try to read the fusion report
  report <- withCallingHandlers({
      col_types <- c(
        "chrom1" = "character",
        "start1" = "integer",
        "end1" = "integer",
        "chrom2" = "character",
        "start2" = "integer",
        "end2" = "integer",
        "score" = "integer",
        "strand1" = "character",
        "strand2" = "character",
        "num_concordantfrag_bp1" = "integer",
        "num_concordantfrag_bp2" = "integer"
      )
      if (missing(limit)) {
        # Read all lines
        data.table::fread(
          input = filename,
          colClasses = col_types,
          showProgress = FALSE
        )
      } else {
          # Only read up to the limit
          data.table::fread(
            input = filename,
            colClasses = col_types,
            showProgress = FALSE,
            nrows = limit
          )
      }
    },
    error = function(cond) {
      message(paste0("Reading ", filename, " caused an error: ", cond[[1]]))
      stop(cond)
    },
    warning = function(cond) {
      # Begin Exclude Linting
      message(paste0("Reading ", filename, " caused a warning: ", cond[[1]]))
      # End Exclude Linting
    }
  )

  # Set variables
  id                   <- NA
  inframe              <- NA
  fusion_tool          <- "SQUID"
  spanning_reads_count <- NA
  split_reads_count    <- NA

  # List to hold all Fusion objects
  fusion_list <- vector("list", dim(report)[1])

  # Iterate through each line in the .tsv file
  for (i in 1:dim(report)[1]) {

    # Import defuse-specific fields
    fusion_tool_specific_data <- list()
    fusion_tool_specific_data[["score"]] <- report[[i, "score"]]
    fusion_tool_specific_data[["score"]] <-
      report[[i, "num_concordantfrag_bp1"]]
    fusion_tool_specific_data[["score"]] <-
      report[[i, "num_concordantfrag_bp2"]]

    # Fusion id
    id <- as.character(i)

    # Is the downstream fusion partner in-frame?
    inframe <- NA

    # Strand
    strand_upstream <- report[[i, "strand1"]]
    strand_downstream <- report[[i, "strand2"]]

    # Number of supporting reads
    split_reads_count <- 0
    spanning_reads_count <- 0

    # No junction sequence is given
    junction_sequence_upstream <-
      Biostrings::DNAString()
    junction_sequence_downstream <-
      Biostrings::DNAString()

    # Breakpoints
    if (strand_upstream == "+") {
      breakpoint_upstream <- report[[i, "end1"]]
    } else {
      breakpoint_upstream <- report[[i, "start1"]]
    }
    if (strand_downstream == "+") {
      breakpoint_downstream <- report[[i, "start2"]]
    } else {
      breakpoint_downstream <- report[[i, "end2"]]
    }

    # Chromosome names
    chromosome_upstream <- report[[i, "chrom1"]]
    chromosome_downstream <- report[[i, "chrom2"]]

    # Gene names
    name_upstream <- NA_character_
    name_downstream <- NA_character_

    # Ensembl ids
    ensembl_id_upstream <- NA_character_
    ensembl_id_downstream <- NA_character_

    # PartnerGene objects
    gene_upstream <- new(
      Class = "PartnerGene",
      name = name_upstream,
      ensembl_id = ensembl_id_upstream,
      chromosome = chromosome_upstream,
      breakpoint = breakpoint_upstream,
      strand = strand_upstream,
      junction_sequence = junction_sequence_upstream,
      transcripts = GenomicRanges::GRangesList()
    )
    gene_downstream <- new(
      Class = "PartnerGene",
      name = name_downstream,
      ensembl_id = ensembl_id_downstream,
      chromosome = chromosome_downstream,
      breakpoint = breakpoint_downstream,
      strand = strand_downstream,
      junction_sequence = junction_sequence_downstream,
      transcripts = GenomicRanges::GRangesList()
    )

    fusion_list[[i]] <- new(
      Class = "Fusion",
      id = id,
      fusion_tool = fusion_tool,
      genome_version = genome_version,
      spanning_reads_count = spanning_reads_count,
      split_reads_count = split_reads_count,
      fusion_reads_alignment = Gviz::AlignmentsTrack(),
      gene_upstream = gene_upstream,
      gene_downstream = gene_downstream,
      inframe = inframe,
      fusion_tool_specific_data = fusion_tool_specific_data
    )
  }

  # Return the list of Fusion objects
  fusion_list
}

context("Test importing SQUID data into a list of Fusion objects")

squidfile <- system.file(
  "extdata",
  "squid_hcc1954_sv.txt",
  package = "chimeraviz")
fusions <- import_squid(squidfile, "hg19", 3)
fusion <- fusions[[1]]

test_that("import_squid imports data into Fusion objects", {
  expect_equal(length(fusions), 3)
  expect_equal(fusion@id, "1")
  expect_equal(fusion@gene_upstream@chromosome, "17")
})

test_that("import_squid fails to import data", {
  expect_error(import_squid("Non-existent-file", "hg18"))
  expect_error(import_squid(squidfile, "Invalid-genome"))
  expect_error(import_squid(squidfile, "hg19", "invalid limit"))
})

test_that("import_squid limits work", {
  expect_equal(length(import_squid(squidfile, "hg19", 2)), 2)
})

#' Import results from a STAR-Fusion run into a list of Fusion objects.
#'
#' A function that imports the results from a STAR-Fusion run, typically from
#' a star-fusion.fusion_candidates.final.abridged file, into a list of Fusion
#' objects.
#'
#' @param filename Filename for the STAR-Fusion
#' star-fusion.fusion_candidates.final.abridged results file.
#' @param genome_version Which genome was used in mapping (hg19, hg38, etc.).
#' @param limit A limit on how many lines to read.
#'
#' @return A list of Fusion objects.
#'
#' @examples
#' starfusionData <- system.file(
#'   "extdata",
#'   "star-fusion.fusion_candidates.final.abridged.txt",
#'   package = "chimeraviz")
#' fusions <- import_starfusion(starfusionData, "hg19", 3)
#' # This should import a list of 3 fusions described in Fusion objects.
#'
#' @importFrom data.table fread
#'
#' @export
import_starfusion <- function(filename, genome_version, limit) {

  # Is the genome version valid?
  valid_genomes <- c("hg19", "hg38", "mm10")
  if (is.na(match(tolower(genome_version), tolower(valid_genomes)))) {
    stop("Invalid genome version given")
  }

  # If the limit is set, is the value valid?
  if (missing(limit) == FALSE) {
    if (is.numeric(limit) == FALSE || limit <= 0) {
      stop("limit must be a numeric value bigger than 0")
    }
  }

  # Try to read the fusion report
  report <- withCallingHandlers({
      col_types <- c(
        "JunctionReadCount" = "integer",
        "SpanningFragCount" = "integer",
        "LeftGene" = "character",
        "LeftBreakpoint" = "character",
        "RightGene" = "character",
        "RightBreakpoint" = "character",
        "LargeAnchorSupport" = "character",
        "LeftBreakDinuc" = "character",
        "LeftBreakEntropy" = "numeric",
        "RightBreakDinuc" = "character",
        "RightBreakEntropy" = "numeric",
        "FFPM" = "numeric"
      )
      if (missing(limit)) {
        # Read all lines
        data.table::fread(
          input = filename,
          colClasses = col_types,
          showProgress = FALSE
        )
      } else {
        # Only read up to the limit
        data.table::fread(
          input = filename,
          colClasses = col_types,
          showProgress = FALSE,
          nrows = limit
        )
      }
    },
    error = function(cond) {
      message(paste0("Reading ", filename, " caused an error: ", cond[[1]]))
      stop(cond)
    },
    warning = function(cond) {
      # Begin Exclude Linting
      message(paste0("Reading ", filename, " caused a warning: ", cond[[1]]))
      # End Exclude Linting
    }
  )

  # Set variables
  id                   <- NA
  inframe              <- NA
  fusion_tool          <- "starfusion"
  spanning_reads_count <- NA
  split_reads_count    <- NA

  # List to hold all Fusion objects
  fusion_list <- vector("list", dim(report)[1])

  # Iterate through each line in the .tsv file
  for (i in 1:dim(report)[1]) {

    # Import starfusion-specific fields
    fusion_tool_specific_data <- list()
    fusion_tool_specific_data[["LargeAnchorSupport"]] <-
      report[[i, "LargeAnchorSupport"]]
    fusion_tool_specific_data[["LeftBreakDinuc"]] <-
      report[[i, "LeftBreakDinuc"]]
    fusion_tool_specific_data[["LeftBreakEntropy"]] <-
      report[[i, "LeftBreakEntropy"]]
    fusion_tool_specific_data[["RightBreakDinuc"]] <-
      report[[i, "RightBreakDinuc"]]
    fusion_tool_specific_data[["RightBreakEntropy"]] <-
      report[[i, "RightBreakEntropy"]]
    fusion_tool_specific_data[["FFPM"]] <-
      report[[i, "FFPM"]]

    # Optional extra fields:
    if (!is.null(report[[i, "PROT_FUSION_TYPE"]])) {
      fusion_tool_specific_data[["PROT_FUSION_TYPE"]] <-
        report[[i, "PROT_FUSION_TYPE"]]
      if (report[[i, "PROT_FUSION_TYPE"]] == "INFRAME") {
        inframe <- TRUE
      } else if (report[[i, "PROT_FUSION_TYPE"]] == "FRAMESHIFT") {
        inframe <- FALSE
      }
    }
    if (!is.null(report[[i, "annots"]])) {
      fusion_tool_specific_data[["annots"]] <- report[[i, "annots"]]
    }

    # id for this fusion
    id <- as.character(i)

    left_breakpoint <-
      unlist(strsplit(report[[i, "LeftBreakpoint"]], split = ":"))
    righ_breakpoint <-
      unlist(strsplit(report[[i, "RightBreakpoint"]], split = ":"))

    # Chromosome names
    chromosome_upstream <- left_breakpoint[1]
    chromosome_downstream <- righ_breakpoint[1]

    # Breakpoints
    breakpoint_upstream <- as.numeric(left_breakpoint[2])
    breakpoint_downstream <- as.numeric(righ_breakpoint[2])

    # Strand
    strand_upstream <- left_breakpoint[3]
    strand_downstream <- righ_breakpoint[3]

    # Number of supporting reads
    split_reads_count <- report[[i, "JunctionReadCount"]]
    spanning_reads_count <- report[[i, "SpanningFragCount"]]

    # Fusion sequence. Ref https://github.com/stianlagstad/chimeraviz/pull/23
    if (!is.null(report[[i, "FUSION_CDS"]])) {
      junction_sequence_upstream <- Biostrings::DNAString(
        gsub("[A-Z]+$", "", report[[i, "FUSION_CDS"]])
      )
      junction_sequence_downstream <- Biostrings::DNAString(
        gsub("^[a-z]+", "", report[[i, "FUSION_CDS"]])
      )
    } else {
      junction_sequence_upstream <- Biostrings::DNAString()
      junction_sequence_downstream <- Biostrings::DNAString()
    }

    # Gene names
    gene_names_1 <- unlist(strsplit(report[[i, "LeftGene"]], split = "\\^"))
    gene_names_2 <- unlist(strsplit(report[[i, "RightGene"]], split = "\\^"))
    name_upstream <- gene_names_1[1]
    name_downstream <- gene_names_2[1]

    # Ensembl ids
    ensembl_id_upstream <- strsplit(gene_names_1[2], "\\.")[[1]][[1]]
    ensembl_id_downstream <- strsplit(gene_names_2[2], "\\.")[[1]][[1]]

    # PartnerGene objects
    gene_upstream <- new(
      Class = "PartnerGene",
      name = name_upstream,
      ensembl_id = ensembl_id_upstream,
      chromosome = chromosome_upstream,
      breakpoint = breakpoint_upstream,
      strand = strand_upstream,
      junction_sequence = junction_sequence_upstream,
      transcripts = GenomicRanges::GRangesList()
    )
    gene_downstream <- new(
      Class = "PartnerGene",
      name = name_downstream,
      ensembl_id = ensembl_id_downstream,
      chromosome = chromosome_downstream,
      breakpoint = breakpoint_downstream,
      strand = strand_downstream,
      junction_sequence = junction_sequence_downstream,
      transcripts = GenomicRanges::GRangesList()
    )

    fusion_list[[i]] <- new(
      Class = "Fusion",
      id = id,
      fusion_tool = fusion_tool,
      genome_version = genome_version,
      spanning_reads_count = spanning_reads_count,
      split_reads_count = split_reads_count,
      fusion_reads_alignment = Gviz::AlignmentsTrack(),
      gene_upstream = gene_upstream,
      gene_downstream = gene_downstream,
      inframe = inframe,
      fusion_tool_specific_data = fusion_tool_specific_data
    )
  }

  # Return the list of Fusion objects
  fusion_list
}

context("Test importing STAR-Fusion data into a list of Fusion objects")

starfusion_data <- system.file(
  "extdata",
  "star-fusion.fusion_candidates.final.abridged.txt",
  package = "chimeraviz")
fusions <- import_starfusion(starfusion_data, "hg19", 3)
fusion <- fusions[[1]]

test_that("import_starfusion imports data into Fusion objects", {
  expect_equal(length(fusions), 3)
  expect_equal(fusion@id, "1")
  expect_equal(fusion@gene_upstream@name, "THRA")
  expect_equal(fusion@gene_upstream@chromosome, "chr17")
  expect_equal(fusion@gene_upstream@ensembl_id, "ENSG00000126351")
})

test_that("import_starfusion fails to import data", {
  expect_error(import_starfusion("Non-existent-file", "hg18"))
  expect_error(import_starfusion(starfusion_data, "Invalid-genome"))
  expect_error(import_starfusion(starfusion_data, "hg19", "invalid limit"))
})

test_that("import_starfusion limits work", {
  expect_equal(length(import_starfusion(starfusion_data, "hg19", 2)), 2)
})

context("Test importing STAR-Fusion data into a list of Fusion objects")

starfusion_data <- system.file(
  "extdata",
  "star-fusion.fusion_predictions.abridged.annotated.coding_effect.tsv",
  package = "chimeraviz")
fusions <- import_starfusion(starfusion_data, "hg19", 3)
fusion <- fusions[[1]]

test_that("import_starfusion imports data into Fusion objects", {
  expect_equal(length(fusions), 3)
  expect_equal(fusion@id, "1")
  expect_equal(fusion@gene_upstream@name, "RP11-9L18.2")
  expect_equal(fusion@gene_upstream@chromosome, "chr1")
  expect_equal(fusion@gene_upstream@ensembl_id, "ENSG00000215835")
})

test_that("import_starfusion fails to import data", {
  expect_error(import_starfusion("Non-existent-file", "hg18"))
  expect_error(import_starfusion(starfusion_data, "Invalid-genome"))
  expect_error(import_starfusion(starfusion_data, "hg19", "invalid limit"))
})

test_that("import_starfusion limits work", {
  expect_equal(length(import_starfusion(starfusion_data, "hg19", 2)), 2)
})

test_that("import_starfusion imports additional columns when available", {
  expect_equal(fusions[[3]]@gene_upstream@junction_sequence@length, 1431)
  expect_true(!is.null(fusion@fusion_tool_specific_data$annots))
  expect_true(!fusions[[3]]@inframe)
})

#' Import results from an InFusion run into a list of Fusion objects.
#'
#' A function that imports the results from an InFusion run nto a list of Fusion
#' objects.
#'
#' @param filename Filename for the jaffa_results.csv file.
#' @param genome_version Which genome was used in mapping (hg19, hg38, etc.).
#' @param limit A limit on how many lines to read.
#'
#' @return A list of Fusion objects.
#'
#' @examples
#' infusionData <- system.file(
#'   "extdata",
#'   "infusion_fusions.txt",
#'   package = "chimeraviz")
#' fusions <- import_infusion(infusionData, "hg19", 3)
#' # This should import a list of 3 fusions described in Fusion objects.
#'
#' @importFrom data.table fread
#'
#' @export
import_infusion <- function(filename, genome_version, limit) {

  # Is the genome version valid?
  valid_genomes <- c("hg19", "hg38", "mm10")
  if (is.na(match(tolower(genome_version), tolower(valid_genomes)))) {
    stop("Invalid genome version given")
  }

  # If the limit is set, is the value valid?
  if (missing(limit) == FALSE) {
    if (is.numeric(limit) == FALSE || limit <= 0) {
      stop("limit must be a numeric value bigger than 0")
    }
  }

  # Try to read the fusion report
  report <- withCallingHandlers({
      col_types <- c(
        "#id" = "integer",
        "ref1" = "character",
        "break_pos1" = "integer",
        "ref2" = "character",
        "break_pos2" = "integer",
        "num_span" = "integer",
        "num_paired" = "integer",
        "genes_1" = "character",
        "genes_2" = "character"
      )
      if (missing(limit)) {
        # Read all lines
        data.table::fread(
          input = filename,
          colClasses = col_types,
          showProgress = FALSE
        )
      } else {
        # Only read up to the limit
        data.table::fread(
          input = filename,
          colClasses = col_types,
          showProgress = FALSE,
          nrows = limit
        )
      }
    },
    error = function(cond) {
      message(paste0("Reading ", filename, " caused an error: ", cond[[1]]))
      stop(cond)
    },
    warning = function(cond) {
      # Begin Exclude Linting
      message(paste0("Reading ", filename, " caused a warning: ", cond[[1]]))
      # End Exclude Linting
    }
  )

  # Set variables
  id                   <- NA
  inframe              <- NA
  fusion_tool          <- "infusion"
  spanning_reads_count <- NA
  split_reads_count    <- NA

  # List to hold all Fusion objects
  fusion_list <- vector("list", dim(report)[1])

  # Iterate through each line in the .tsv file
  for (i in 1:dim(report)[1]) {

    # Import infusion-specific fields
    fusion_tool_specific_data <- list()

    # id for this fusion
    id <- as.character(report[[i, "#id"]])

    # Chromosome names
    chromosome_upstream <- paste("chr", report[[i, "ref1"]], sep = "")
    chromosome_downstream <- paste("chr", report[[i, "ref2"]], sep = "")

    # Breakpoints
    breakpoint_upstream <- report[[i, "break_pos1"]]
    breakpoint_downstream <- report[[i, "break_pos2"]]

    # Strand
    strand_upstream <- "*"
    strand_downstream <- "*"

    # Number of supporting reads
    split_reads_count <- as.numeric(report[[i, "num_span"]])
    spanning_reads_count <- as.numeric(report[[i, "num_paired"]])
    # Note: the terminology used for spanning and split reads varies. According
    # to the source code of InFusion (the file fusion.py), the "num_paired"
    # field is what the author calls "bridge reads", which is what we are
    # calling spanning reads.

    # InFusiondoesn't provide the fusion sequence
    junction_sequence_upstream <- Biostrings::DNAString()
    junction_sequence_downstream <- Biostrings::DNAString()

    # Gene names
    name_upstream <- report[[i, "genes_1"]]
    name_downstream <- report[[i, "genes_2"]]

    # Ensembl ids
    ensembl_id_upstream <- NA_character_
    ensembl_id_downstream <- NA_character_

    # PartnerGene objects
    gene_upstream <- new(
      Class = "PartnerGene",
      name = name_upstream,
      ensembl_id = ensembl_id_upstream,
      chromosome = chromosome_upstream,
      breakpoint = breakpoint_upstream,
      strand = strand_upstream,
      junction_sequence = junction_sequence_upstream,
      transcripts = GenomicRanges::GRangesList()
    )
    gene_downstream <- new(
      Class = "PartnerGene",
      name = name_downstream,
      ensembl_id = ensembl_id_downstream,
      chromosome = chromosome_downstream,
      breakpoint = breakpoint_downstream,
      strand = strand_downstream,
      junction_sequence = junction_sequence_downstream,
      transcripts = GenomicRanges::GRangesList()
    )

    fusion_list[[i]] <- new(
      Class = "Fusion",
      id = id,
      fusion_tool = fusion_tool,
      genome_version = genome_version,
      spanning_reads_count = spanning_reads_count,
      split_reads_count = split_reads_count,
      fusion_reads_alignment = Gviz::AlignmentsTrack(),
      gene_upstream = gene_upstream,
      gene_downstream = gene_downstream,
      inframe = inframe,
      fusion_tool_specific_data = fusion_tool_specific_data
    )
  }

  # Return the list of Fusion objects
  fusion_list
}

context("Test importing InFusion data into a list of Fusion objects")

infusion_data <- system.file(
  "extdata",
  "infusion_fusions.txt",
  package = "chimeraviz")
fusions <- import_infusion(infusion_data, "hg19", 3)
fusion <- fusions[[1]]

test_that("import_infusion imports data into Fusion objects", {
  expect_equal(length(fusions), 3)
  expect_equal(fusion@id, "283813")
  expect_equal(fusion@gene_upstream@name, "TMPRSS2")
  expect_equal(fusion@gene_upstream@chromosome, "chr21")
})

test_that("import_infusion fails to import data", {
  expect_error(import_infusion("Non-existent-file", "hg18"))
  expect_error(import_infusion(infusion_data, "Invalid-genome"))
  expect_error(import_infusion(infusion_data, "hg19", "invalid limit"))
})

test_that("import_infusion limits work", {
  expect_equal(length(import_infusion(infusion_data, "hg19", 2)), 2)
})

#' Import results from a oncofuse run into a list of Fusion objects.
#'
#' A function that imports the results from a oncofuse run, typically from a
#' results.filtered.tsv file, into a list of Fusion objects.
#'
#' This import function was contributed by Lavinia G, ref
#' https://github.com/stianlagstad/chimeraviz/issues/47#issuecomment-409773158
#'
#' @param filename Filename for the oncofuse results .tsv file.
#' @param genome_version Which genome was used in mapping (hg19, hg38, etc.).
#' @param limit A limit on how many lines to read.
#'
#' @return A list of Fusion objects.
#'
#' @examples
#' oncofuse833ke <- system.file(
#'   "extdata",
#'   "oncofuse.outfile",
#'   package="chimeraviz")
#' fusions <- import_oncofuse(oncofuse833ke, "hg19", 3)
#' # This should import a list of 3 fusions described in Fusion objects.
#'
#' @export
import_oncofuse <- function(filename, genome_version, limit) {

  # Is the genome version valid?
  valid_genomes <- c("hg19", "hg38", "mm10")
  if (is.na(match(tolower(genome_version), tolower(valid_genomes)))) {
    stop("Invalid genome version given")
  }

  # If the limit is set, is the value valid?
  if (missing(limit) == FALSE) {
    if (is.numeric(limit) == FALSE || limit <= 0) {
      stop("limit must be a numeric value bigger than 0")
    }
  }

  # Try to read the fusion report
  report <- withCallingHandlers({
      col_types <- c(
        "FUSION_ID" = "character",
        "SPANNING_READS" = "integer",
        "ENCOMPASSING_READS" = "integer",
        "GENOMIC" = "character",
        "5_FPG_GENE_NAME" = "character",
        "3_FPG_GENE_NAME" = "character",
        "FPG_FRAME_DIFFERENCE" = "integer",
        "DRIVER_PROB" = "numeric"
      )
      if (missing(limit)) {
        # Read all lines
        data.table::fread(
          input = filename,
          colClasses = col_types,
          showProgress = FALSE
        )
      } else {
        # Only read up to the limit
        data.table::fread(
          input = filename,
          colClasses = col_types,
          showProgress = FALSE,
          nrows = limit
        )
      }
    },
    error = function(cond) {
      message(paste0("Reading ", filename, " caused an error: ", cond[[1]]))
      stop(cond)
    },
    warning = function(cond) {
      # Begin Exclude Linting
      message(paste0("Reading ", filename, " caused a warning: ", cond[[1]]))
      # End Exclude Linting
    }
  )


  # Set variables
  id                   <- NA
  inframe              <- NA
  fusion_tool          <- "oncofuse"
  spanning_reads_count <- NA
  split_reads_count    <- NA

  # List to hold all Fusion objects
  fusion_list <- vector("list", dim(report)[1])

  # Iterate through each line in the .tsv file
  for (i in 1:dim(report)[1]) {

    # Import oncofuse-specific fields
    fusion_tool_specific_data <- list()
    fusion_tool_specific_data[["DRIVER_PROB"]] <- report[[i, "DRIVER_PROB"]]

    # Cluster id
    id <- report[[i, "FUSION_ID"]]

    # Is the downstream fusion partner in-frame?
    if (report[[i, "FPG_FRAME_DIFFERENCE"]] == "0") {
      inframe <- TRUE
    } else {
      inframe <- FALSE
    }

    # Number of supporting reads
    split_reads_count <- report[[i, "ENCOMPASSING_READS"]]
    spanning_reads_count <- report[[i, "SPANNING_READS"]]

    # Breakpoints
    genomics <- report[[i, "GENOMIC"]]
    genomic_vals <- unlist(strsplit(genomics, ">"))
    chromosome_upstream <- unlist(strsplit(genomic_vals[1], ":"))[1]
    breakpoint_upstream <- as.numeric(unlist(strsplit(genomic_vals[1], ":"))[2])
    chromosome_downstream <- unlist(strsplit(genomic_vals[2], ":"))[1]
    breakpoint_downstream <- as.numeric(
      unlist(strsplit(genomic_vals[2], ":"))[2]
    )

    # Gene names
    name_upstream <- report[[i, "5_FPG_GENE_NAME"]]
    name_downstream <- report[[i, "3_FPG_GENE_NAME"]]

    # Ensembl ids
    ensembl_id_upstream <- NA_character_
    ensembl_id_downstream <- NA_character_

    # oncofuse doesn't provide the fusion sequence
    junction_sequence_upstream <- Biostrings::DNAString()
    junction_sequence_downstream <- Biostrings::DNAString()

    # oncofuse doesn't provide the strands
    strand_upstream <- NA_character_
    strand_downstream <- NA_character_

    # PartnerGene objects
    gene_upstream <- new(
      Class = "PartnerGene",
      name = name_upstream,
      ensembl_id = ensembl_id_upstream,
      chromosome = chromosome_upstream,
      breakpoint = breakpoint_upstream,
      strand = strand_upstream,
      junction_sequence = junction_sequence_upstream,
      transcripts = GenomicRanges::GRangesList()
    )
    gene_downstream <- new(
      Class = "PartnerGene",
      name = name_downstream,
      ensembl_id = ensembl_id_downstream,
      chromosome = chromosome_downstream,
      breakpoint = breakpoint_downstream,
      strand = strand_downstream,
      junction_sequence = junction_sequence_downstream,
      transcripts = GenomicRanges::GRangesList()
    )

    fusion_list[[i]] <- new(
      Class = "Fusion",
      id = id,
      fusion_tool = fusion_tool,
      genome_version = genome_version,
      spanning_reads_count = spanning_reads_count,
      split_reads_count = split_reads_count,
      fusion_reads_alignment = Gviz::AlignmentsTrack(),
      gene_upstream = gene_upstream,
      gene_downstream = gene_downstream,
      inframe = inframe,
      fusion_tool_specific_data = fusion_tool_specific_data
    )
  }

  # Return the list of Fusion objects
  fusion_list
}

context("Test importing oncofuse data into a list of Fusion objects")

oncofuse_file <- system.file(
  "extdata",
  "oncofuse.outfile",
  package = "chimeraviz")
fusions <- import_oncofuse(oncofuse_file, "hg19", 3)
fusion <- fusions[[1]]

test_that("import_oncofuse imports data into Fusion objects", {
  expect_equal(length(fusions), 3)
  expect_equal(fusion@id, "1")
  expect_equal(fusion@gene_upstream@name, "PGC")
  expect_equal(fusion@gene_upstream@chromosome, "chr6")
})

test_that("import_oncofuse fails to import data", {
  expect_error(import_oncofuse("Non-existent-file", "hg18"))
  expect_error(import_oncofuse(oncofuse_file, "Invalid-genome"))
  expect_error(import_oncofuse(oncofuse_file, "hg19", "invalid limit"))
})

test_that("import_oncofuse limits work", {
  expect_equal(length(import_oncofuse(oncofuse_file, "hg19", 2)), 2)
})

#' Create a reference index using Rsubread.
#'
#' This functions builds an index from a reference sequence using Rsubread.
#'
#' See the plot-fusion-reads vignette for a complete example of how to use the
#' Rsubread functions.
#'
#' @seealso rsubread_align
#'
#' @param reference_fasta The path to the reference sequence to build index
#' from.
#'
#' @examples
#' \dontrun{
#' rsubread_index(
#'   reference_fasta = "reference.fa")
#' }
#'
#' @importFrom Rsubread buildindex
#'
#' @export
rsubread_index <- function(reference_fasta) {

  # Check if the input paths actually exist
  if (file.exists(reference_fasta) == FALSE) {
    stop("Invalid referenceFasta filename. File doesn't exist.")
  }

  Rsubread::buildindex(
    basename = reference_fasta,
    reference = reference_fasta)

  # Did we actually get an output?
  this_file_should_exist <- paste(reference_fasta, ".00.b.array", sep = "")
  if (file.exists(this_file_should_exist) == FALSE) {
    warning("It doesn't look like rsubread_index() worked.")
  }
}

#' Align reads to fusion sequence using Rsubread.
#'
#' This function aligns paired end reads to the given reference, producing .bam
#' and .bai files. Note that this function also uses Rsamtools to index and sort
#' the .bamfile that Rsubread::align() creates.
#'
#' See the plot-fusion-reads vignette for a complete example of how to use the
#' Rsubread functions.
#'
#' @seealso rsubread_index
#'
#' @param reference_name Name of the reference file.
#' @param fastq1 The first fastq file.
#' @param fastq2 The second fastq file.
#' @param output_bam_filename Filename for the resulting .bam file.
#'
#' @examples
#' \dontrun{
#' rsubread_align(
#'   reference_name = "reference.fa",
#'   fastq1 = "reads1.fq",
#'   fastq2 = "reads2.fq",
#'   output_bam_filename = "outputbam")
#' }
#'
#' @importFrom Rsamtools sortBam indexBam
#' @importFrom Rsubread align
#'
#' @export
rsubread_align <- function(
  reference_name,
  fastq1,
  fastq2,
  output_bam_filename) {

  # Check if the input paths actually exist
  if (file.exists(reference_name) == FALSE) {
    stop("Invalid referenceName filename. File doesn't exist.")
  }
  if (file.exists(fastq1) == FALSE || file.exists(fastq2) == FALSE) {
    stop("Invalid fastq filenames. File(s) doesn't exist.")
  }

  # Align
  Rsubread::align(
    index = reference_name,
    readfile1 = fastq1,
    readfile2 = fastq2,
    output_file = paste(output_bam_filename, ".bam", sep = ""))

  # Sort
  Rsamtools::sortBam(
    file = paste(output_bam_filename, ".bam", sep = ""),
    destination = output_bam_filename
  )

  # Index
  Rsamtools::indexBam(files = paste(output_bam_filename, ".bam", sep = ""))

  # Did we actually get an output?
  if (file.exists(paste(output_bam_filename, ".bam", sep = "")) == FALSE) {
    warning("It doesn't look like rsubreadAlign() worked.")
  }
}

#' oncofuse data
#'
#' Documentation for the oncofuse example data.
#'
#' @name raw_oncofuse
#'
#' @section oncofuse.outfile:
#'
#' The example output from oncofuse was kindly provided by Lavinia G here:
#' https://github.com/stianlagstad/chimeraviz/issues/47#issuecomment-409773158
NULL

#' Fusioncatcher data
#'
#' Documentation for the Fusioncatcher example data.
#'
#' @name raw_fusioncatcher
#'
#' @section fusioncatcher_833ke_final-list-candidate-fusion-genes.txt:
#'
#' This file has the results from a run of Fusioncatcher-0.99.3e on the 833ke
#' cell line. The program was ran with the standard configuration file and with
#' the parameters "-p 8 -z --keep-preliminary".
NULL

#' InFusion data
#'
#' Documentation for the InFusion example data.
#'
#' @name raw_infusion
#'
#' @section infusion_fusions.txt:
#'
#' This is example data from the InFusion getting started page located at
#' https://bitbucket.org/kokonech/infusion/wiki/Getting%20Started .
NULL

#' PRADA data
#'
#' Documentation for the PRADA example data.
#'
#' @name raw_prada
#'
#' @section PRADA.acc.fusion.fq.TAF.tsv:
#'
#' This is example data thankfully provided by PRADA authors Siyuan Zheng and
#' Roeland Verhaak.
NULL

#' JAFFA data
#'
#' Documentation for the JAFFA example data.
#'
#' @name raw_jaffa
#'
#' @section jaffa_results.csv:
#'
#' This is example data from the described JAFFA example run documented at
#' https://github.com/Oshlack/JAFFA/wiki/Example .
NULL

#' EricScript data
#'
#' Documentation for the EricScript example data.
#'
#' @name raw_ericscript
#'
#' @section ericscript_SRR1657556.results.total.tsv:
#'
#' This is example data thankfully provided by EricScript author Matteo Benelli.
NULL

#' FusionMap data
#'
#' Documentation for the FusionMap example data.
#'
#' @name raw_fusionmap
#'
#' @section FusionMap_01_TestDataset_InputFastq.FusionReport.txt:
#'
#' This is example data provided with the FusionMap version released 2015-03-31.
NULL

#' STAR-Fusion data
#'
#' Documentation for the STAR-Fusion example data.
#'
#' @name raw_starfusion
#'
#' @section star-fusion.fusion_candidates.final.abridged.txt:
#'
#' This example data was retrieved from the STAR-Fusion github page June 2.nd
#' 2017.
NULL

#' SOAPfuse data
#'
#' Documentation for the SOAPfuse example data.
#'
#' @name raw_soapfuse
#'
#' @section soapfuse_833ke_final.Fusion.specific.for.genes:
#'
#' This file has the results from a run of soapfuse-1.26 on the 833ke cell line.
#' The program was ran with the standard configuration file.
NULL

#' deFuse data
#'
#' Documentation for the deFuse example data.
#'
#' @name raw_defuse
#'
#' @section defuse_833ke_results.filtered.tsv:
#'
#' This file has the results from a run of deFuse-0.7.0 on the 833ke cell line.
#' The program was ran with the standard configuration, but with the parameter
#' span_count_threshold=5 instead of the standard 3. The resulting
#' results.filtered.tsv file was then manually filtered to only include 17
#' fusion events in the interest of saving computing time for tests and
#' examples. The original results contained 171 fusion events.
#'
#' @section reads_supporting_defuse_fusion_5267.*.fq:
#'
#' These two files, reads_supporting_defuse_fusion_5267.1.fq and
#' reads_supporting_defuse_fusion_5267.2.fq, contains the reads that support the
#' fusion event with cluster_id 5267.
#'
#' @section 5267readsAligned.bam:
#'
#' The bamfile 5267readsAligned.bam and the 5267readsAligned.bam.bai index file
#' contains the reads supporting the fusion event with cluster_id 5267 aligned
#' to the fusion sequence. It is used with plot_fusion_reads().
#'
NULL

#' Homo_sapiens.GRCh37.74_subset.gtf
#'
#' @name raw_Homo_sapiens.GRCh37.74
#'
#' @section Homo_sapiens.GRCh37.74_subset.gtf:
#'
#' The Homo_sapiens.GRCh37.74.gtf file is a subset version of the Ensembl
#' Homo_sapiens.GRCh37.74.gtf file, located here:
#' \url{ftp://ftp.ensembl.org/pub/release-74/gtf/homo_sapiens}. This gtf file
#' contains transcripts for the partner genes in two of the fusion transcripts
#' from the deFuse example data provided with this package: The fusion
#' transcript with cluster_id=5267, and the fusion transcript with
#' cluster_id=11759.
#'
#' The file is the result of running this command:
#'
#' # grep "ENST00000373831\|ENST00000373832\|ENST00000373833\|ENST00000398958\|ENST00000411533\|ENST00000419074\|ENST00000427469\|ENST00000429051\|ENST00000430407\|ENST00000434290\|ENST00000478232\|ENST00000486790\|ENST00000370031\|ENST00000370032\|ENST00000402983\|ENST00000420055\|ENST00000483729\|ENST00000493676\|ENST00000382073\|ENST00000299665\|ENST00000382064" Homo_sapiens.GRCh37.74.gtf > Homo_sapiens.GRCh37.74_subset.gtf
#'
#' The transcript names given in the command above are all transcripts available
#' for the genes CLEC6A, CLEC4D, HENMT1, and RCC1 in Ensembl version 74.
#'
#' @section Homo_sapiens.GRCh37.74.sqlite:
#'
#' The Homo_sapiens.GRCh37.74.sqlite file is the sqlite database that the
#' Ensembldb package creates from the corresponding gtf file. It was created
#' using this command:
#'
#' # ensDbFromGtf(
#' #   gtf = "Homo_sapiens.GRCh37.74_subset.gtf",
#' #   organism = "Homo_sapiens",
#' #   genomeVersion = "GRCh37",
#' #   version = 74)
NULL

#' Fusion5267and11759 bamfile
#'
#' Documentation for the fusion5267and11759reads.bam file containing reads
#' mapped to the region where the genes in the fusions with cluster_id=5267 and
#' cluster_id=11759 is.
#'
#' @name raw_fusion5267reads
#'
#' @section fusion5267and11759reads.bam:
#'
#' This file is the result of running these commands:
#'
#' samtools view -b original_bamfile.bam "1:28831455-28866812" "1:109189912-109205148" "12:8608225-8677832" > fusion5267and11759reads.bam
#' samtools index fusion5267and11759reads.bam fusion5267and11759reads.bam.bai
#'
#' where we extract the reads mapping to the region where we know the fusions
#' with cluster_id=5267 and cluster_id=11759 from the deFuse example data is.
#'
#' The original_bamfile.bam is from a study of the 833KE cell line by Andreas M. Hoff et al., documented in the paper [Identification of Novel Fusion Genes in Testicular Germ Cell Tumors](http://cancerres.aacrjournals.org/content/76/1/108.full).
NULL

#' protein_domains_5267 bed file
#'
#' Documentation for the protein_domains_5267.bed file containing protein
#' domains for the genes in the fusion with cluster_id=5267.
#'
#' @name raw_fusion5267proteindomains
#'
#' @section protein_domains_5267.bed:
#'
#' This file is an excerpt from a larger file that we created by:
#' - downloading domain name annotation from Pfam database (PfamA version 31)
#'   and domain region annotation from Ensembl database through BioMart API
#' - switching the domain coordinates in the protein level to these in
#'   transcript level.
NULL

#' Fusion5267and11759 bedGraph file
#'
#' Documentation for the fusion5267and11759reads.bedGraph file containing read
#' count data from the regions of the fusion event with cluster_id=5267.
#'
#' @name raw_fusion5267readsBedGraph
#'
#' @section fusion5267and11759reads.bedGraph:
#'
#' This file is the result of running this command:
#'
#' bedtools genomecov -ibam fusion5267and11759reads.bam -bg > fusion5267and11759reads.bam.bedGraph
#'
#' fusion5267and11759reads.bam has its own documentation entry for how it was
#' created.
NULL

#' Cytoband information HG19
#'
#' Cytoband information for the HG19 assembly from UCSC. Downloaded from
#' http://hgdownload.cse.ucsc.edu/goldenpath/hg19/database/
#'
#' @name raw_cytobandhg19
#'
#' @section UCSC.HG19.Human.CytoBandIdeogram.txt:
#'
#' This data is used with RCircos in plot_circle().
NULL

#' Cytoband information HG138
#'
#' Cytoband information for the HG38 assembly from UCSC. Downloaded from
#' http://hgdownload.cse.ucsc.edu/goldenpath/hg38database/
#'
#' All _alt or _random entries has been manually removed, as has the chrM entry.
#'
#' @name raw_cytobandhg38
#'
#' @section UCSC.HG38.Human.CytoBandIdeogram.txt:
#'
#' This data is used with RCircos in plot_circle().
NULL

#' ChimPipe data
#'
#' Documentation for the ChimPipe example data.
#'
#' @name raw_chimpipe
#'
#' @section chimericJunctions_MCF-7.txt:
#'
#' This is example data from the ChimPipe tutorial all-in-one package located at
#' https://chimpipe.readthedocs.io/en/latest/tutorial.html#download-all-in-one-package
#' It was downloaded March 10th 2020
NULL

#' SQUID data
#'
#' Documentation for the SQUID example data.
#'
#' @name raw_chimpipe
#'
#' @section squid_hcc1954_sv.txt:
#'
#' This is example data for SQUID provided on GitHub here:
#' https://github.com/Kingsford-Group/squid/issues/20#issuecomment-598888472
#' It was downloaded March 17th 2020
NULL

#' Aeron fusion data
#'
#' Documentation for the Aeron fusion support example data.
#'
#' @name raw_aeron_fusion_support
#'
#' @section aeron_fusion_support.txt:
#'
#' This is example data for Aeron provided on GitHub here:
#' https://github.com/SchulzLab/Aeron/issues/5#issuecomment-603447735
#' It was downloaded March 25th 2020. The original filename was
#' aeron_fusion_support_301018_K562_mRNA_PCS109_HumanTranscriptsReduced_HumanUpdated38V5.txt
NULL

#' Aeron transcript data
#'
#' Documentation for the Aeron fusion transcript example data.
#'
#' @name raw_aeron_fusion_transcript
#'
#' @section aeron_fusion_transcript.fa:
#'
#' This is example data for Aeron provided on GitHub here:
#' https://github.com/SchulzLab/Aeron/issues/5#issuecomment-603447735
#' It was downloaded March 25th 2020. The original filename was
#' fusion_transcripts_301018_K562_mRNA_PCS109_HumanTranscriptsReduced_HumanUpdated38V5.fa
NULL

#' Create a plot of the reads supporting the given fusion.
#'
#' This function takes a Fusion object and plots the reads supporting the fusion
#' on top of the fusion sequence (fusion@junction_sequence), provided that
#' add_fusion_reads_alignment() has been run earlier in order to add fusion reads
#' alignment data to the fusion object.
#'
#' Note that the package used for plotting, Gviz, is strict on chromosome names.
#' If the plot produced doesn't show the reads, the problem might be solved by
#' naming the fusion sequence "chrNA".
#'
#' @param fusion The Fusion object to plot.
#' @param show_all_nucleotides Boolean indicating whether or not to show all
#' nucleotides. If FALSE, then only nucleotide_amount amount of nucleotides will
#' be shown on each end of the fusion junction. If TRUE, then the whole fusion
#' junction sequence will be shown.
#' @param nucleotide_amount The number of nucleotides to show on each end of the
#' fusion junction sequence. Defaults to 10. Only applicable if
#' show_all_nucleotides is set to TRUE.
#'
#' @return Creates a fusion reads plot.
#'
#' @seealso add_fusion_reads_alignment
#'
#' @examples
#' # Load data
#' defuseData <- system.file(
#'   "extdata",
#'   "defuse_833ke_results.filtered.tsv",
#'   package="chimeraviz")
#' fusions <- import_defuse(defuseData, "hg19", 1)
#' # Find the specific fusion we have aligned reads for
#' fusion <- get_fusion_by_id(fusions, 5267)
#' bamfile <- system.file(
#'   "extdata",
#'   "5267readsAligned.bam",
#'   package="chimeraviz")
#' # Add the bam file of aligned fusion reads to the fusion object
#' fusion <- add_fusion_reads_alignment(fusion, bamfile)
#' # Temporary file to store the plot
#' pngFilename <- tempfile(
#'   pattern = "fusionPlot",
#'   fileext = ".png",
#'   tmpdir = tempdir())
#' # Calculate image size based on supporting reads and lenght of junction
#' # sequence.
#' imageWidth <- (nchar(partner_gene_junction_sequence(upstream_partner_gene(fusion))) +
#'   nchar(partner_gene_junction_sequence(downstream_partner_gene(fusion)))) * 15
#' imageHeight <- (fusion_split_reads_count(fusion)+fusion_spanning_reads_count(fusion)) * 20
#' # Open device
#' png(pngFilename, width = imageWidth, height = imageHeight)
#' # Now we can plot
#' plot_fusion_reads(fusion)
#' # Close device
#' dev.off()
#'
#' @export
plot_fusion_reads <- function(
  fusion,
  show_all_nucleotides = TRUE,
  nucleotide_amount = 10) {

  .validate_plot_fusion_reads_params(
    fusion,
    show_all_nucleotides,
    nucleotide_amount
  )

  # Create Biostrings::DNAStringSet of fusion sequence
  fusion_sequence <- Biostrings::DNAStringSet(
    x = c(fusion@gene_upstream@junction_sequence,
          fusion@gene_downstream@junction_sequence))
  names(fusion_sequence) <- "chrNA"
  # And also of a "zoomed-in" version only displaying some nucleotides on each
  # end of the fusion junction
  fusion_sequence_short <- Biostrings::DNAStringSet(
    x = c(
      substr(
        fusion@gene_upstream@junction_sequence,
        nchar(fusion@gene_upstream@junction_sequence) - nucleotide_amount + 1,
        nchar(fusion@gene_upstream@junction_sequence)),
      substr(
        fusion@gene_downstream@junction_sequence,
        nchar(fusion@gene_downstream@junction_sequence) -
          nucleotide_amount + 1 - 1,
        # Added -1 again on the line above because Gviz does not show the last
        # nucleotide. Ref:
        # http://permalink.gmane.org/gmane.science.biology.informatics.conductor.devel/5726 # Exclude Linting
        # Or if that link is down, google "Gviz doesn't provide enough
        # horizontial space"
        nchar(fusion@gene_downstream@junction_sequence))))
  names(fusion_sequence_short) <- "chrNA"

  # Load fusion sequences to tracks
  ref_chimera_short <- Gviz::SequenceTrack(
    fusion_sequence_short,
    genome = fusion@genome_version,
    name = "Fusion Sequence")
  ref_chimera <- Gviz::SequenceTrack(
    fusion_sequence,
    genome = fusion@genome_version,
    name = "Fusion Sequence")

  # DISPLAY PARAMETERS

  # Set display parameters for the fusion sequence
  display_parameters <- list(
    showTitle = FALSE,
    # The color of the line when no indiviual letters can be plotted due to
    # size limitations.
    col = "blue"
  )
  Gviz::displayPars(ref_chimera) <- display_parameters
  Gviz::displayPars(ref_chimera_short) <- display_parameters

  # Set display parameters for the fusion reads
  Gviz::displayPars(fusion@fusion_reads_alignment) <- list(
    showTitle = FALSE,
    showMismatches = FALSE # Show mismatched reads?
  )

  # Do some plotting

  if (show_all_nucleotides) {
    # Create the grid we're gonna use for the plot
    nf <- grid::grid.layout(
      nrow = 1,
      ncol = 1)

    # Open a new page to plot it
    grid::grid.newpage()

    # Divide the page according to the grid we created
    grid::pushViewport(grid::viewport(layout = nf))

    # Open row 1, column 1
    grid::pushViewport(grid::viewport(layout.pos.row = 1, layout.pos.col = 1))
    # Plot the fusion read alignment
    Gviz::plotTracks(
      c(fusion@fusion_reads_alignment, ref_chimera),
      from = 1,
      to = nchar(fusion_sequence),
      chromosome = fusion@fusion_reads_alignment@chromosome,
      type = "pileup",
      sizes = c(6, 1),
      add = TRUE
    )
    # Calculate where in the alignment the breakpoint is
    breakpoint <-
      nchar(fusion@gene_upstream@junction_sequence) / nchar(fusion_sequence)
    # Plot line indicating fusion breakpoint
    grid::grid.lines(x = grid::unit(breakpoint, "npc"),
               y = grid::unit(c(0, 1), "npc"),
               default.units = "npc",
               arrow = NULL,
               name = NULL,
               gp = grid::gpar(
                 col = "black",
                 lty = "dotted"),
               draw = TRUE,
               vp = NULL)
    # Close row 1, column 1
    grid::popViewport(1)
  } else {
    # Create the grid we're gonna use for the plot
    nf <- grid::grid.layout(
      nrow = 2,
      ncol = 1,
      heights = grid::unit(c(6, 1), "null"),
      widths = grid::unit(c(1, 1), "null"))

    # Open a new page to plot it
    grid::grid.newpage()

    # Divide the page according to the grid we created
    grid::pushViewport(grid::viewport(layout = nf))

    # Open row 1, column 1
    grid::pushViewport(grid::viewport(layout.pos.row = 1, layout.pos.col = 1))
    # Plot the fusion read alignment
    Gviz::plotTracks(
      fusion@fusion_reads_alignment,
      from = 1,
      to = nchar(fusion_sequence),
      chromosome = fusion@fusion_reads_alignment@chromosome,
      type = "pileup",
      add = TRUE
    )
    # Calculate where in the alignment the breakpoint is
    breakpoint <-
      nchar(fusion@gene_upstream@junction_sequence) / nchar(fusion_sequence)
    # Plot line indicating fusion breakpoint
    grid::grid.lines(x = grid::unit(breakpoint, "npc"),
               y = grid::unit(c(0, 1), "npc"),
               default.units = "npc",
               arrow = NULL,
               name = NULL,
               gp = grid::gpar(
                 col = "black",
                 lty = "dotted"),
               draw = TRUE,
               vp = NULL)
    # Close row 1, column 1
    grid::popViewport(1)

    # Open row 2, column 1
    grid::pushViewport(grid::viewport(layout.pos.row = 2, layout.pos.col = 1))
    # Plot a part of the fusion junction sequence
    Gviz::plotTracks(
      ref_chimera_short,
      from = 1,
      to = nchar(fusion_sequence_short),
      chromosome = fusion@fusion_reads_alignment@chromosome,
      add = TRUE
    )
    # Plot line indicating fusion breakpoint
    grid::grid.lines(x = grid::unit(0.5, "npc"),
               y = grid::unit(c(0, 1), "npc"),
               default.units = "npc",
               arrow = NULL,
               name = NULL,
               gp = grid::gpar(
                 col = "black",
                 lty = "dotted"),
               draw = TRUE,
               vp = NULL)
    # Plot line between lines
    grid::grid.lines(x = grid::unit(c(0.5, breakpoint), "npc"),
               y = grid::unit(1, "npc"),
               default.units = "npc",
               arrow = NULL,
               name = NULL,
               gp = grid::gpar(
                 col = "black",
                 lty = "dotted"),
               draw = TRUE,
               vp = NULL)
    # Close row 1, column 2
    grid::popViewport(1)
  }

}

.validate_plot_fusion_reads_params <- function(
  fusion,
  show_all_nucleotides,
  nucleotide_amount
) {
  # Establish a new 'checkmate' object
  argument_checker <- checkmate::makeAssertCollection()

  .is_fusion_valid(argument_checker, fusion)

  # Check if the Gviz::AlignmentsTrack object in
  # fusion@fusion_reads_alignment actually holds data, and is not just the empty
  # object it is after loading the initial fusion data. In other words, make
  # sure add_fusion_reads_alignment() has been run.
  if (class(fusion@fusion_reads_alignment) != "ReferenceAlignmentsTrack") {
    argument_checker$push(
      paste("fusion@fusion_reads_alignment should hold a",
                  "Gviz::ReferenceAlignmentsTrack object.",
                  "See add_fusion_reads_alignment().")
    )
  }

  .is_parameter_boolean(
    argument_checker,
    show_all_nucleotides,
    "show_all_nucleotides")

  .is_nucleotide_amount_valid(
    argument_checker,
    nucleotide_amount,
    fusion
  )

  # Return errors and warnings (if any)
  checkmate::reportAssertions(argument_checker)
}

## ----message = FALSE----------------------------------------------------------
# Load chimeraviz
library(chimeraviz)

# Get reference to results file from deFuse
defuse833ke <- system.file(
"extdata",
"defuse_833ke_results.filtered.tsv",
package="chimeraviz")

# Load the results file into a list of fusion objects
fusions <- import_defuse(defuse833ke, "hg19")

## ----message = FALSE----------------------------------------------------------
length(fusions)

## ----message = FALSE----------------------------------------------------------
# Find a specific fusion event
fusion <- get_fusion_by_id(fusions, 5267)

# Show information about this fusion event
fusion

# Show information about the upstream fusion partner
upstream_partner_gene(fusion)

# Show information about the downstream fusion partner
downstream_partner_gene(fusion)

## ----echo = FALSE, message = FALSE, fig.height = 8, fig.width = 8, dev='png'----
# Load SOAPfuse data
soapfuse833ke <- system.file(
  "extdata",
  "soapfuse_833ke_final.Fusion.specific.for.genes",
  package = "chimeraviz")
fusions <- import_soapfuse(soapfuse833ke, "hg38", 10)
# Plot!
plot_circle(fusions)

## -----------------------------------------------------------------------------
# Load SOAPfuse data
if(!exists("soapfuse833ke"))
  soapfuse833ke <- system.file(
    "extdata",
    "soapfuse_833ke_final.Fusion.specific.for.genes",
    package = "chimeraviz")
fusions <- import_soapfuse(soapfuse833ke, "hg38", 10)

## ----message = FALSE, fig.height = 8, fig.width = 8, dev='png'----------------
plot_circle(fusions)

## ----echo = FALSE, message = FALSE, fig.height = 3, fig.width = 50, dev='png'----
# Load data
if(!exists("defuse833ke"))
  defuse833ke <- system.file(
    "extdata",
    "defuse_833ke_results.filtered.tsv",
    package="chimeraviz")
fusions <- import_defuse(defuse833ke, "hg19", 1)
# Find the specific fusion we have aligned reads for
fusion <- get_fusion_by_id(fusions, 5267)
if(!exists("bamfile5267"))
  bamfile5267 <- system.file(
    "extdata",
    "5267readsAligned.bam",
    package="chimeraviz")
# Add the bam file of aligned fusion reads to the fusion object
fusion <- add_fusion_reads_alignment(fusion, bamfile5267)
# Plot!
plot_fusion_reads(fusion)

## -----------------------------------------------------------------------------
# Load deFuse data
if(!exists("defuse833ke"))
  defuse833ke <- system.file(
    "extdata",
    "defuse_833ke_results.filtered.tsv",
    package = "chimeraviz")
fusions <- import_defuse(defuse833ke, "hg19", 1)
# Choose example fusion event
fusion <- get_fusion_by_id(fusions, 5267)

## -----------------------------------------------------------------------------
fastq1 <- system.file(
  "extdata",
  "reads_supporting_defuse_fusion_5267.1.fq",
  package = "chimeraviz")
fastq2 <- system.file(
  "extdata",
  "reads_supporting_defuse_fusion_5267.2.fq",
  package = "chimeraviz")

## -----------------------------------------------------------------------------
referenceFilename <- "reference.fa"
write_fusion_reference(fusion = fusion, filename = referenceFilename)

## ----eval=FALSE---------------------------------------------------------------
# # First load the bowtie functions
# source(system.file(
#   "scripts",
#   "bowtie.R",
#   package="chimeraviz"))
# # Then create index
# bowtie_index(
#   bowtie_build_location = "/path/to/bowtie-build",
#   reference_fasta = reference_filename)
# # And align
# bowtie_align(
#   bowtie_location = "/path/to/bowtie",
#   reference_name = reference_filename,
#   fastq1 = fastq1,
#   fastq2 = fastq2,
#   output_bam_filename = "fusion_alignment")

## ----eval=FALSE---------------------------------------------------------------
# # First load the rsubread functions
# source(system.file(
#   "scripts",
#   "rsubread.R",
#   package="chimeraviz"))
# # Then create index
# rsubread_index(reference_fasta = reference_filename)
# # And align
# rsubread_align(
#   reference_name = reference_filename,
#   fastq1 = fastq1,
#   fastq2 = fastq2,
#   output_bam_filename = "fusion_alignment")

## -----------------------------------------------------------------------------
if(!exists("bamfile5267"))
  bamfile5267 <- system.file(
    "extdata",
    "5267readsAligned.bam",
    package="chimeraviz")

## -----------------------------------------------------------------------------
# Add bamfile of fusion reads to the fusion oject
fusion <- add_fusion_reads_alignment(fusion, bamfile5267)

## ----message = FALSE, fig.height = 3, fig.width = 50, dev='png'---------------
plot_fusion_reads(fusion)

## ----echo = FALSE, message = FALSE, fig.height = 7, fig.width = 10, dev='png'----
# Load deFuse data
if(!exists("defuse833ke"))
  defuse833ke <- system.file(
    "extdata",
    "defuse_833ke_results.filtered.tsv",
    package = "chimeraviz")
fusions <- import_defuse(defuse833ke, "hg19", 1)
# Choose a fusion object
fusion <- get_fusion_by_id(fusions, 5267)
# Load edb
if(!exists("edbSqliteFile"))
  edbSqliteFile <- system.file(
    "extdata",
    "Homo_sapiens.GRCh37.74.sqlite",
    package="chimeraviz")
edb <- ensembldb::EnsDb(edbSqliteFile)
# bamfile with reads in the regions of this fusion event
if(!exists("fusion5267and11759reads"))
  fusion5267and11759reads <- system.file(
   "extdata",
   "fusion5267and11759reads.bam",
   package = "chimeraviz")
# Plot!
plot_fusion(
 fusion = fusion,
 bamfile = fusion5267and11759reads,
 edb = edb,
 non_ucsc = TRUE)

## ----echo = FALSE, message = FALSE, fig.height = 5, fig.width = 10, dev='png'----
# Plot!
plot_fusion(
 fusion = fusion,
 bamfile = fusion5267and11759reads,
 edb = edb,
 non_ucsc = TRUE,
 reduce_transcripts = TRUE)

## -----------------------------------------------------------------------------
# First find the example data
if(!exists("defuse833ke"))
  defuse833ke <- system.file(
    "extdata",
    "defuse_833ke_results.filtered.tsv",
    package = "chimeraviz")
# Then load the fusion events
fusions <- import_defuse(defuse833ke, "hg19", 1)

## -----------------------------------------------------------------------------
# See if we can find any fusions involving RCC1
get_fusion_by_gene_name(fusions, "RCC1")

## -----------------------------------------------------------------------------
# Extract the specific fusion
fusion <- get_fusion_by_id(fusions, 5267)

## -----------------------------------------------------------------------------
# First find our example EnsDb file
if(!exists("edbSqliteFile"))
  edbSqliteFile <- system.file(
    "extdata",
    "Homo_sapiens.GRCh37.74.sqlite",
    package="chimeraviz")
# Then load it
edb <- ensembldb::EnsDb(edbSqliteFile)

## ----eval = FALSE-------------------------------------------------------------
# # Create EnsDb from a downloaded .gtf file
# edbSqliteFile <- ensDbFromGtf(gtf = "Homo_sapiens.GRCh37.74.gtf")
# # The function above create a .sqlite file, like the one that is included with
# # chimeraviz. The path to the file is stored in the edbSqliteFile variable. To
# # load the database, do this:
# edb <- ensembldb::EnsDb(edbSqliteFile)

## ----eval = FALSE-------------------------------------------------------------
# # Create an edb object directly from the .sqlite file
# edb <- ensembldb::EnsDb("Homo_sapiens.GRCh37.74.sqlite")

## -----------------------------------------------------------------------------
if(!exists("fusion5267and11759reads"))
  fusion5267and11759reads <- system.file(
    "extdata",
    "fusion5267and11759reads.bam",
    package = "chimeraviz")

## ----message = FALSE, fig.height = 7, fig.width = 10, dev='png'---------------
plot_fusion(
 fusion = fusion,
 bamfile = fusion5267and11759reads,
 edb = edb,
 non_ucsc = TRUE)

## ----message = FALSE, fig.height = 5, fig.width = 10, dev='png'---------------
plot_fusion(
 fusion = fusion,
 bamfile = fusion5267and11759reads,
 edb = edb,
 non_ucsc = TRUE,
 reduce_transcripts = TRUE)

## ----message = FALSE, fig.height = 5, fig.width = 10, dev='png'---------------
# Load deFuse data
if(!exists("defuse833ke"))
  defuse833ke <- system.file(
    "extdata",
    "defuse_833ke_results.filtered.tsv",
    package = "chimeraviz")
fusions <- import_defuse(defuse833ke, "hg19", 1)
# Choose a fusion object
fusion <- get_fusion_by_id(fusions, 5267)
# Load edb
if(!exists("edbSqliteFile"))
  edbSqliteFile <- system.file(
    "extdata",
    "Homo_sapiens.GRCh37.74.sqlite",
    package="chimeraviz")
edb <- ensembldb::EnsDb(edbSqliteFile)
# Plot!
plot_transcripts(
  fusion = fusion,
  edb = edb)

## ----message = FALSE, fig.height = 5, fig.width = 10, dev='png'---------------
# Choose a fusion object
fusion <- get_fusion_by_id(fusions, 5267)
# Load edb
if(!exists("edbSqliteFile"))
  edbSqliteFile <- system.file(
    "extdata",
    "Homo_sapiens.GRCh37.74.sqlite",
    package="chimeraviz")
edb <- ensembldb::EnsDb(edbSqliteFile)
# Get reference to the .BAM file
if(!exists("fusion5267and11759reads"))
  fusion5267and11759reads <- system.file(
    "extdata",
    "fusion5267and11759reads.bam",
    package="chimeraviz")
# Plot!
plot_transcripts(
  fusion = fusion,
  edb = edb,
  bamfile = fusion5267and11759reads,
  non_ucsc = TRUE)

## ----message = FALSE, fig.height = 5, fig.width = 10, dev='png'---------------
# Load deFuse data
if(!exists("defuse833ke"))
  defuse833ke <- system.file(
  "extdata",
  "defuse_833ke_results.filtered.tsv",
  package="chimeraviz")
fusions <- import_defuse(defuse833ke, "hg19", 1)
# Choose a fusion object
fusion <- get_fusion_by_id(fusions, 5267)
# Load edb
if(!exists("edbSqliteFile"))
  edbSqliteFile <- system.file(
    "extdata",
    "Homo_sapiens.GRCh37.74.sqlite",
    package="chimeraviz")
edb <- ensembldb::EnsDb(edbSqliteFile)
# Get reference to the .BAM file
if(!exists("fusion5267and11759reads"))
  fusion5267and11759reads <- system.file(
    "extdata",
    "fusion5267and11759reads.bam",
    package="chimeraviz")
# Plot!
plot_transcripts(
  fusion = fusion,
  edb = edb,
  bamfile = fusion5267and11759reads,
  non_ucsc = TRUE,
  reduce_transcripts = TRUE,
  ylim = c(0,1000))

## ----message = FALSE, fig.height = 2, fig.width = 10, dev='png'---------------
# Load deFuse data
if(!exists("defuse833ke"))
  defuse833ke <- system.file(
    "extdata",
    "defuse_833ke_results.filtered.tsv",
    package="chimeraviz")
fusions <- import_defuse(defuse833ke, "hg19", 1)
# Choose a fusion object
fusion <- get_fusion_by_id(fusions, 5267)
# Load edb
if(!exists("edbSqliteFile"))
  edbSqliteFile <- system.file(
    "extdata",
    "Homo_sapiens.GRCh37.74.sqlite",
    package="chimeraviz")
edb <- ensembldb::EnsDb(edbSqliteFile)
# Plot!
plot_fusion_transcript(
  fusion,
  edb)

## ----message = FALSE, fig.height = 4, fig.width = 10, dev='png'---------------
# Load deFuse data
if(!exists("defuse833ke"))
  defuse833ke <- system.file(
    "extdata",
    "defuse_833ke_results.filtered.tsv",
    package="chimeraviz")
fusions <- import_defuse(defuse833ke, "hg19", 1)
# Choose a fusion object
fusion <- get_fusion_by_id(fusions, 5267)
# Load edb
if(!exists("edbSqliteFile"))
  edbSqliteFile <- system.file(
    "extdata",
    "Homo_sapiens.GRCh37.74.sqlite",
    package="chimeraviz")
edb <- ensembldb::EnsDb(edbSqliteFile)
# Get reference to the .BAM file
if(!exists("fusion5267and11759reads"))
  fusion5267and11759reads <- system.file(
    "extdata",
    "fusion5267and11759reads.bam",
    package="chimeraviz")
# Plot!
plot_fusion_transcript(
  fusion,
  edb,
  fusion5267and11759reads)

## ----message = FALSE, fig.height = 2, fig.width = 10, dev='png'---------------
# Load deFuse data
if(!exists("defuse833ke"))
  defuse833ke <- system.file(
    "extdata",
    "defuse_833ke_results.filtered.tsv",
    package="chimeraviz")
fusions <- import_defuse(defuse833ke, "hg19", 1)
# Choose a fusion object
fusion <- get_fusion_by_id(fusions, 5267)
# Select transcripts
gene_upstream_transcript <- "ENST00000434290"
gene_downstream_transcript <- "ENST00000370031"
# Load edb
if(!exists("edbSqliteFile"))
  edbSqliteFile <- system.file(
    "extdata",
    "Homo_sapiens.GRCh37.74.sqlite",
    package="chimeraviz")
edb <- ensembldb::EnsDb(edbSqliteFile)
# bamfile with reads in the regions of this fusion event
if(!exists("fusion5267and11759reads"))
  fusion5267and11759reads <- system.file(
    "extdata",
    "fusion5267and11759reads.bam",
    package="chimeraviz")
# bedfile with protein domains for the transcripts in this example
if(!exists("bedfile"))
  bedfile <- system.file(
    "extdata",
    "protein_domains_5267.bed",
    package="chimeraviz")
# Plot!
plot_fusion_transcript_with_protein_domain(
      fusion = fusion,
      edb = edb,
      bamfile = fusion5267and11759reads,
      bedfile = bedfile,
      gene_upstream_transcript = gene_upstream_transcript,
      gene_downstream_transcript = gene_downstream_transcript,
      plot_downstream_protein_domains_if_fusion_is_out_of_frame = TRUE)

## ----echo = FALSE, message = FALSE, results = 'asis'--------------------------
bedfile <- system.file(
  "extdata",
  "protein_domains_5267.bed",
  package="chimeraviz")
bedfile_contents <- data.table::fread(
  input = bedfile,
  showProgress = FALSE
)
knitr::kable(bedfile_contents)

## ----message = FALSE, fig.height = 5, fig.width = 10, dev='png'---------------
# Load deFuse data
if(!exists("defuse833ke"))
  defuse833ke <- system.file(
    "extdata",
    "defuse_833ke_results.filtered.tsv",
    package="chimeraviz")
fusions <- import_defuse(defuse833ke, "hg19", 1)
# Choose a fusion object
fusion <- get_fusion_by_id(fusions, 5267)
# Plot!
plot_fusion_transcripts_graph(
  fusion,
  edb)

## ----message = FALSE----------------------------------------------------------
# Get reference to results file from deFuse
if(!exists("defuse833ke"))
  defuse833ke <- system.file(
    "extdata",
    "defuse_833ke_results.filtered.tsv",
    package="chimeraviz")

# Load the results file into a list of fusion objects
fusions <- import_defuse(defuse833ke, "hg19")

## ----message = FALSE----------------------------------------------------------
# Get a specific fusion object by id
get_fusion_by_id(fusions, 5267)

# Get all fusions with a matching gene name
length(get_fusion_by_gene_name(fusions, "RCC1"))

# Get all fusions on a specific chromosome
length(get_fusion_by_chromosome(fusions, "chr1"))

## ----eval = FALSE, echo = TRUE, message = FALSE-------------------------------
# # Load SOAPfuse data
# if(!exists("soapfuse833ke"))
#   soapfuse833ke <- system.file(
#     "extdata",
#     "soapfuse_833ke_final.Fusion.specific.for.genes",
#     package = "chimeraviz")
# fusions <- import_soapfuse(soapfuse833ke, "hg38", 10)
# # Create report!
# create_fusion_report(fusions, "output.html")

## ----echo = FALSE, message = FALSE, fig.height = 7, fig.width = 7, fig.align='center', dev='png'----
# Load SOAPfuse data
if(!exists("defuse833ke"))
  defuse833ke <- system.file(
    "extdata",
    "defuse_833ke_results.filtered.tsv",
    package="chimeraviz")
fusions <- import_defuse(defuse833ke, "hg19")
# Plot!
plot_circle(fusions)

## ----echo = FALSE-------------------------------------------------------------
# with the as.data.frame.Fusion function above, we can use ldply() from the plyr package to create a data frame of our fusion objects:
dfFusions <- plyr::ldply(fusions, fusion_to_data_frame)

# with this data frame, we can use datatable() from the DT package to create an html sortable table:
DT::datatable(dfFusions, filter = 'top')

## -----------------------------------------------------------------------------
sessionInfo()

#' Plot possible fusion transcripts based on annotation.
#'
#' This function takes a fusion object and an ensembldb object and plots the
#' reduced version of the fusion transcript. This transcript consist of the
#' "mashed together" version of all possible fusion transcripts based on known
#' annotations. If a bamfile is specified, the fusion transcript will be
#' plotted with coverage information.
#'
#' Note that the transcript database used (the edb object) must have the same
#' seqnames as any bamfile used. Otherwise the coverage data will be wrong.
#'
#' @param fusion The Fusion object to plot.
#' @param edb The edb object that will be used to fetch data.
#' @param bamfile The bamfile with RNA-seq data.
#' @param which_transcripts This character vector decides which transcripts are
#' to be plotted. Can be "exonBoundary", "withinExon", "withinIntron",
#' "intergenic", or a character vector with specific transcript ids. Default
#' value is "exonBoundary".
#' @param bedgraphfile A bedGraph file to use instead of the bamfile to plot
#' coverage.
#'
#' @return Creates a fusion transcript plot.
#'
#' @examples
#' # Load data and example fusion event
#' defuse833ke <- system.file(
#'   "extdata",
#'   "defuse_833ke_results.filtered.tsv",
#'   package="chimeraviz")
#' fusions <- import_defuse(defuse833ke, "hg19", 1)
#' fusion <- get_fusion_by_id(fusions, 5267)
#' # Load edb
#' edbSqliteFile <- system.file(
#'   "extdata",
#'   "Homo_sapiens.GRCh37.74.sqlite",
#'   package="chimeraviz")
#' edb <- ensembldb::EnsDb(edbSqliteFile)
#' # bamfile with reads in the regions of this fusion event
#' bamfile5267 <- system.file(
#'   "extdata",
#'   "fusion5267and11759reads.bam",
#'   package="chimeraviz")
#' # Temporary file to store the plot
#' pngFilename <- tempfile(
#'   pattern = "fusionPlot",
#'   fileext = ".png",
#'   tmpdir = tempdir())
#' # Open device
#' png(pngFilename, width = 500, height = 500)
#' # Plot!
#' plot_fusion_transcript(
#'   fusion = fusion,
#'   bamfile = bamfile5267,
#'   edb = edb)
#' # Close device
#' dev.off()
#'
#' # Example using a .bedGraph file instead of a .bam file:
#' # Load data and example fusion event
#' defuse833ke <- system.file(
#'   "extdata",
#'   "defuse_833ke_results.filtered.tsv",
#'   package="chimeraviz")
#' fusions <- import_defuse(defuse833ke, "hg19", 1)
#' fusion <- get_fusion_by_id(fusions, 5267)
#' # Load edb
#' edbSqliteFile <- system.file(
#'   "extdata",
#'   "Homo_sapiens.GRCh37.74.sqlite",
#'   package="chimeraviz")
#' edb <- ensembldb::EnsDb(edbSqliteFile)
#' # bedgraphfile with coverage data from the regions of this fusion event
#' bedgraphfile <- system.file(
#'   "extdata",
#'   "fusion5267and11759reads.bedGraph",
#'   package="chimeraviz")
#' # Temporary file to store the plot
#' pngFilename <- tempfile(
#'   pattern = "fusionPlot",
#'   fileext = ".png",
#'   tmpdir = tempdir())
#' # Open device
#' png(pngFilename, width = 500, height = 500)
#' # Plot!
#' plot_fusion_transcript(
#'   fusion = fusion,
#'   bamfile = bamfile5267,
#'   edb = edb)
#' # Close device
#' dev.off()
#'
#' @export
plot_fusion_transcript <- function(
  fusion,
  edb = NULL,
  bamfile = NULL,
  which_transcripts = "exonBoundary",
  bedgraphfile = NULL) {

  .validate_plot_fusion_transcript_params(
    fusion,
    edb,
    bamfile,
    which_transcripts,
    bedgraphfile
  )
  fusion <- .get_transcripts_if_not_there(fusion, edb)

  # Select which transcripts to use
  transcripts_upstream <-
    select_transcript(fusion@gene_upstream, which_transcripts)
  transcripts_downstream <-
    select_transcript(fusion@gene_downstream, which_transcripts)

  # Check that there's at least one transcript that has the fusion breakpoint
  # within the transcript.
  .check_that_breakpoints_are_within_transcripts(
    fusion,
    transcripts_upstream,
    transcripts_downstream
  )

  # Reduce transcript into one for each gene
  transcripts_upstream <- reduce(unlist(transcripts_upstream))
  transcripts_downstream <- reduce(unlist(transcripts_downstream))
  # Add transcript metadata column to make Gviz group correctly
  mcols(transcripts_upstream)$transcript <- fusion@gene_upstream@name
  mcols(transcripts_downstream)$transcript <- fusion@gene_downstream@name
  # Add symbol metadata column to make Gviz show ids
  mcols(transcripts_upstream)$symbol <- fusion@gene_upstream@name
  mcols(transcripts_downstream)$symbol <- fusion@gene_downstream@name

  # Select the exons that are part of a possible fusion transcript. I.e. these:
  #
  # 1: geneA is on the + strand
  #    Exons with start/end positions before breakpoint
  # 2: geneA is on the - strand
  #    Exons with start/end positions after breakpoint
  # 3: geneB is on the + strand
  #    Exons with start/end positions after breakpoint
  # 4: geneB is on the - strand
  #    Exons with start/end positions before breakpoint

  if (fusion@gene_upstream@strand == "+") {
    fusion_exons_upstream <-
      transcripts_upstream[
        start(ranges(transcripts_upstream)) <
          fusion@gene_upstream@breakpoint &
        end(ranges(transcripts_upstream)) <=
          fusion@gene_upstream@breakpoint
      ]
  } else {
    fusion_exons_upstream <-
      transcripts_upstream[
        start(ranges(transcripts_upstream)) >=
          fusion@gene_upstream@breakpoint &
        end(ranges(transcripts_upstream)) >
          fusion@gene_upstream@breakpoint
      ]
  }
  if (fusion@gene_downstream@strand == "+") {
    fusion_exons_downstream <-
      transcripts_downstream[
        start(ranges(transcripts_downstream)) >=
          fusion@gene_downstream@breakpoint &
        end(ranges(transcripts_downstream)) >
          fusion@gene_downstream@breakpoint
      ]
  } else {
    fusion_exons_downstream <-
      transcripts_downstream[
        start(ranges(transcripts_downstream)) <
          fusion@gene_downstream@breakpoint &
        end(ranges(transcripts_downstream)) <=
          fusion@gene_downstream@breakpoint
      ]
  }

  # Reverse order of exons if on minus strand
  if (fusion@gene_upstream@strand == "-") {
    fusion_exons_upstream <- rev(fusion_exons_upstream)
  }
  if (fusion@gene_downstream@strand == "-") {
    fusion_exons_downstream <- rev(fusion_exons_downstream)
  }

  # Build fusion transcript
  fusion_transcript <- append(
    fusion_exons_upstream,
    fusion_exons_downstream
  )

  # If we've got a bamfile, calculate coverage
  if (!is.null(bamfile)) {
    # Get coverage
    cov <- GenomicAlignments::coverage(
      bamfile,
      param = Rsamtools::ScanBamParam(which = fusion_transcript))
    # Coverage only for my transcript
    cov <- cov[fusion_transcript]
    # Unlist
    cov <- suppressWarnings(unlist(cov))
    # Turn into numeric vector
    cov <- as.numeric(cov)

    # Create coverage track
    d_track <- DataTrack(
      start = seq_along(cov),
      width = 1,
      chromosome = "chrNA",
      genome = "hg19",
      name = "Coverage",
      type = "h",
      col = "orange",
      fill = "orange",
      data = cov)
    # Set display parameters
    Gviz::displayPars(d_track) <- list(
      showTitle = FALSE, # hide name of track
      background.panel = "transparent", # background color of the content panel
      background.title = "transparent", # background color for the title panels
      cex.axis = .6,
      cex.title = .6,
      col.axis = "black",
      col.coverage = "black",
      col.title = "black",
      coverageHeight = 0.08,
      fill.coverage = "orange",
      fontsize = 15,
      lty = 1,
      lty.coverage = 1,
      lwd = 0.5
    )
  } else if (!is.null(bedgraphfile)) {
    # We're getting coverage data from a bedGraph file
    d_track <- DataTrack(
      range = bedgraphfile,
      genome = "hg19",
      chromosome = "chrNA",
      name = "Coverage",
      type = "h",
      col = "orange",
      fill = "orange")
    # Set display parameters
    Gviz::displayPars(d_track) <- list(
      showTitle = FALSE, # hide name of track
      background.panel = "transparent", # background color of the content panel
      background.title = "transparent", # background color for the title panels
      cex.axis = .6,
      cex.title = .6,
      col.axis = "black",
      col.coverage = "black",
      col.title = "black",
      coverageHeight = 0.08,
      fill.coverage = "orange",
      fontsize = 15,
      lty = 1,
      lty.coverage = 1,
      lwd = 0.5
    )
  }

  # Move the first exon down to position 1, and then remove the spaces between
  # all the others. Effecticely pushing all exons to the leftmost edge.
  for (i in seq_along(fusion_transcript)) {
    if (i == 1) {
      fusion_transcript[i] <-
        IRanges::shift(
          fusion_transcript[i],
          shift = 1 - start(fusion_transcript[i])
        )
    } else {
      shift_down_by <-
        - (start(fusion_transcript[i]) - end(fusion_transcript[i - 1])) + 1
      fusion_transcript[i] <-
        IRanges::shift(
          fusion_transcript[i],
          shift = shift_down_by
        )
    }
  }

  # Create new GRanges so that we can set the right seqname
  gr <- GRanges(seqnames = "chrNA", ranges = ranges(fusion_transcript))
  mcols(gr)$symbol <- mcols(fusion_transcript)$symbol

  # Create transcript track
  tr_track <- Gviz::GeneRegionTrack(
    gr,
    chromosome = "chrNA")
  # Set display parameters
  Gviz::displayPars(tr_track) <- list(
    background.panel = "transparent", # background color of the content panel
    background.title = "transparent", # background color for the title panels
    col.title = "black",
    showTitle = TRUE, # hide left panel
    showId = FALSE, # show transcript id
    col = "lightgray", # border around exons
    showTitle = FALSE # hide title
  )

  # Color exons
  cols <- RColorBrewer::brewer.pal(4, "Paired")
  interestcolor <- list(
    "excludedA" = cols[1],
    "includedA" = cols[2],
    "excludedB" = cols[3],
    "includedB" = cols[4])
  Gviz::feature(tr_track)[
    Gviz::symbol(tr_track) == fusion@gene_upstream@name
  ] <- "includedA"
  Gviz::feature(tr_track)[
    Gviz::symbol(tr_track) == fusion@gene_downstream@name
  ] <- "includedB"
  Gviz::displayPars(tr_track) <- interestcolor

  # Create axis track
  axis_track <- Gviz::GenomeAxisTrack()
  Gviz::displayPars(axis_track) <- list(
    col = "black", # line color
    fontcolor = "black",
    fontsize = 8,
    lwd = 1.5
  )

  # If we've got a bamfile, plot with coverage
  if (!is.null(bamfile) || !is.null(bedgraphfile)) {
    Gviz::plotTracks(
      list(d_track, tr_track, axis_track),
      main = paste(
        fusion@gene_upstream@name,
        fusion@gene_downstream@name,
        sep = ":"
      ),
      chromosome = "chrNA")
  } else {
    Gviz::plotTracks(
      list(tr_track, axis_track),
      main = paste(
        fusion@gene_upstream@name,
        fusion@gene_downstream@name,
        sep = ":"
      ),
      chromosome = "chrNA")
  }

}

.validate_plot_fusion_transcript_params <- function(
  fusion,
  edb,
  bamfile,
  which_transcripts,
  bedgraphfile
) {
  # Establish a new 'checkmate' object
  argument_checker <- checkmate::makeAssertCollection()

  # Check parameters
  .is_fusion_valid(argument_checker, fusion)
  .is_edb_valid(argument_checker, edb, fusion)
  # Either bamfile or bedgraphfile can be given, not both
  bamfile_given <- !is.null(bamfile)
  bedgraphfile_given <- !is.null(bedgraphfile)
  if (bamfile_given && bedgraphfile_given) {
    argument_checker$push(
      "Either 'bamfile' or 'bedgraphfile' must be given, not both."
    )
  }
  if (bamfile_given) {
    .is_bamfile_valid(argument_checker, bamfile)
  }
  if (bedgraphfile_given) {
    .is_bedgraphfile_valid(argument_checker, bedgraphfile)
  }
  .is_which_transcripts_valid(
    argument_checker,
    which_transcripts,
    fusion)

  # Return errors and warnings (if any)
  checkmate::reportAssertions(argument_checker)
}

#' Import results from an Aeron run into a list of Fusion objects.
#'
#' A function that imports the results from an Aeron run into a list of Fusion
#' objects.
#'
#' Note that the strands and breakpoint positions are not included in the
#' result files from Aeron. These have to be retrieved manually, using the
#' ensembl identifiers (which are included in the result files, and will be
#' available in the Fusion objects after importing).
#'
#' @param filename_fusion_support Filename for the Aeron result file
#' fusion_support..txt.
#' @param filename_fusion_transcript Filename for the Aeron result file
#' fusion_transcripts..txt.
#' @param genome_version Which genome was used in mapping (hg19, hg38, etc.).
#' @param limit A limit on how many lines to read.
#'
#' @return A list of Fusion objects.
#'
#' @examples
#' aeronfusionsupportfile <- system.file(
#'   "extdata",
#'   "aeron_fusion_support.txt",
#'   package="chimeraviz")
#' aeronfusiontranscriptfile <- system.file(
#'   "extdata",
#'   "aeron_fusion_transcripts.fa",
#'   package="chimeraviz")
#' fusions <- import_aeron(
#'   aeronfusionsupportfile,
#'   aeronfusiontranscriptfile,
#'   "hg19",
#'   3)
#' # This should import a list of 3 fusions described in Fusion objects.

#' @importFrom data.table fread
#'
#' @export
import_aeron <- function(
    filename_fusion_support,
    filename_fusion_transcript,
    genome_version,
    limit
  ) {

  # Is the genome version valid?
  valid_genomes <- c("hg19", "hg38", "mm10")
  if (is.na(match(tolower(genome_version), tolower(valid_genomes)))) {
    stop("Invalid genome version given")
  }

  # If the limit is set, is the value valid?
  if (missing(limit) == FALSE) {
    if (is.numeric(limit) == FALSE || limit <= 0) {
      stop("limit must be a numeric value bigger than 0")
    }
  }

  # Try to read the fusion support report
  report <- withCallingHandlers({
      col_types <- c(
        "character",
        "integer"
      )
      if (missing(limit)) {
        # Read all lines
        data.table::fread(
          input = filename_fusion_support,
          colClasses = col_types,
          showProgress = FALSE,
          header = FALSE
        )
      } else {
          # Only read up to the limit
          data.table::fread(
            input = filename_fusion_support,
            colClasses = col_types,
            showProgress = FALSE,
            nrows = limit,
            header = FALSE
          )
      }
    },
    error = function(cond) {
      message(paste0("Reading ", filename_fusion_support, " caused an error: ", cond[[1]]))
      stop(cond)
    },
    warning = function(cond) {
      # Begin Exclude Linting
      message(paste0("Reading ", filename_fusion_support, " caused a warning: ", cond[[1]]))
      # End Exclude Linting
    }
  )

  # Read and parse the transcripts file
  fusion_transcript_map <-
    .import_aeron_transcript_data(filename_fusion_transcript)

  # Set variables
  id                   <- NA
  inframe              <- NA
  fusion_tool          <- "Aeron"
  spanning_reads_count <- NA
  split_reads_count    <- NA

  # List to hold all Fusion objects
  fusion_list <- vector("list", dim(report)[1])

  # Iterate through each line in the .tsv file
  for (i in 1:dim(report)[1]) {

    # Import defuse-specific fields
    fusion_tool_specific_data <- list()

    parsed_data <- .aeron_parse_data(report[[i, 1]])

    # Fusion id
    id <- parsed_data[1]

    # Get the data from the transcripts file
    transcript_data <- get(as.character(id), fusion_transcript_map)

    # Is the downstream fusion partner in-frame?
    inframe <- NA

    # Strand
    strand_upstream <- NA_character_
    strand_downstream <- NA_character_

    # Number of supporting reads
    split_reads_count <- NA_integer_
    spanning_reads_count <- tryCatch(
      as.numeric(parsed_data[4]),
      warning = function(w) {
        NA_integer_
      }
    )

    #Junction sequence
    junction_sequence_upstream <-
      Biostrings::DNAString(transcript_data[2])
    junction_sequence_downstream <-
      Biostrings::DNAString(transcript_data[3])

    # Breakpoints
    breakpoint_upstream <- -1
    breakpoint_downstream <- -1

    # Chromosome names
    chromosome_upstream <- NA_character_
    chromosome_downstream <- NA_character_

    # Gene names
    name_upstream <- NA_character_
    name_downstream <- NA_character_

    # Ensembl ids
    ensembl_id_upstream <- parsed_data[2]
    ensembl_id_downstream <- parsed_data[3]

    # PartnerGene objects
    gene_upstream <- new(
      Class = "PartnerGene",
      name = name_upstream,
      ensembl_id = ensembl_id_upstream,
      chromosome = chromosome_upstream,
      breakpoint = breakpoint_upstream,
      strand = strand_upstream,
      junction_sequence = junction_sequence_upstream,
      transcripts = GenomicRanges::GRangesList()
    )
    gene_downstream <- new(
      Class = "PartnerGene",
      name = name_downstream,
      ensembl_id = ensembl_id_downstream,
      chromosome = chromosome_downstream,
      breakpoint = breakpoint_downstream,
      strand = strand_downstream,
      junction_sequence = junction_sequence_downstream,
      transcripts = GenomicRanges::GRangesList()
    )

    fusion_list[[i]] <- new(
      Class = "Fusion",
      id = id,
      fusion_tool = fusion_tool,
      genome_version = genome_version,
      spanning_reads_count = spanning_reads_count,
      split_reads_count = split_reads_count,
      fusion_reads_alignment = Gviz::AlignmentsTrack(),
      gene_upstream = gene_upstream,
      gene_downstream = gene_downstream,
      inframe = inframe,
      fusion_tool_specific_data = fusion_tool_specific_data
    )
  }

  # Return the list of Fusion objects
  fusion_list
}

.aeron_parse_data <- function(unparsed) {
  if (class(unparsed) != "character") {
    stop("'unparsed' must be a character")
  }

  # Example input: fusion_115_ENSG00000212907.2_291bp_ENSG00000198886.2_1340bp_50reads

  parts <- unlist(strsplit(unparsed, split = "_"))

  if (length(parts) != 7) {
    stop(paste0("Unable to parse 'unparsed':", unparsed))
  }

  fusion_id <- parts[2]
  upstream_id <- parts[3]
  downstream_id <- parts[5]
  spanning_reads <- gsub("[^0-9.-]", "", parts[7])

  c(fusion_id, upstream_id, downstream_id, spanning_reads)
}

.import_aeron_transcript_data <- function(
  filename_fusion_transcript
) {
  # Hashmap to hold all transcript data
  fusion_transcript_data_map <- new.env(hash = T, parent = emptyenv())

  # Read the .fa file
  dna = Biostrings::readDNAStringSet(filename_fusion_transcript)

  # Traverse the DNAStringSet
  for (i in 1:length(dna)) {
    entry <- dna[i]

    # Get the name
    fusion_name <- names(entry)
    # Split the fusion_name to get the fusion id
    fusion_name_parts <- unlist(strsplit(fusion_name, split = "_"))
    fusion_id <- fusion_name_parts[2]

    # Get the actual sequence
    actual_sequence <- toString(entry)

    # Split each on the "N" nucleotide
    actual_sequence_parts <- unlist(strsplit(actual_sequence, split = "N"))
    upstream_sequence <- actual_sequence_parts[1]
    downstream_sequence <- actual_sequence_parts[2]

    # Add this entry to our hashmap:
    assign(
      as.character(fusion_id),
      c(fusion_id, upstream_sequence, downstream_sequence),
      fusion_transcript_data_map
    )
  }

  fusion_transcript_data_map
}

context("Test importing Aeron data into a list of Fusion objects")

aeronfusionsupportfile <- system.file(
 "extdata",
 "aeron_fusion_support.txt",
 package = "chimeraviz")
aeronfusiontranscriptfile <- system.file(
 "extdata",
 "aeron_fusion_transcripts.fa",
 package = "chimeraviz")
fusions <- import_aeron(
  aeronfusionsupportfile,
  aeronfusiontranscriptfile,
  "hg19",
  3)
fusion <- fusions[[1]]

test_that("import_aeron imports data into Fusion objects", {
  expect_equal(length(fusions), 3)
  expect_equal(fusion@id, "115")
  expect_equal(fusion@gene_upstream@name, NA_character_)
  expect_equal(fusion@gene_upstream@chromosome, NA_character_)
})

test_that("import_aeron fails to import data", {
  expect_error(import_aeron("Non-existent-file", "hg18"))
  expect_error(
    import_aeron(
      aeronfusionsupportfile,
      aeronfusiontranscriptfile,
      "Invalid-genome"
    )
  )
  expect_error(
    import_aeron(
      aeronfusionsupportfile,
      aeronfusiontranscriptfile,
      "hg19",
      "invalid limit"
    )
  )
})

test_that("import_aeron limits work", {
  expect_equal(
    length(
      import_aeron(
        aeronfusionsupportfile,
        aeronfusiontranscriptfile,
        "hg19",
        2
      )
    ),
    2
  )
})

test_that(".aeron_parse_data works", {
  expect_equal(length(.aeron_parse_data("fusion_115_ENSG00000212907.2_291bp_ENSG00000198886.2_1340bp_50reads")), 4)
})

test_that(".import_aeron_transcript_data works", {
  expect_equal(length(.import_aeron_transcript_data(aeronfusiontranscriptfile)), 224)
})

#' Create a reference index using bowtie-build
#'
#' This functions builds an index from a reference sequence using bowtie.
#'
#' Both bowtie and bowtie2 can be used.
#'
#' From the bowtie manual: "The command should print many lines of output then
#' quit. When the command completes, the current directory will contain four new
#' files that all start with `referenceFasta` and end with `.1.bt2`, `.2.bt2`,
#' `.3.bt2`, `.4.bt2`, `.rev.1.bt2`, and `.rev.2.bt2`. These files constitute
#' the index."
#'
#' See the plot-fusion-reads vignette for a complete example of how to use the
#' Bowtie functions.
#'
#' @seealso bowtie_align
#'
#' @param bowtie_build_location The path to bowtie-build on your system.
#' @param reference_fasta The path to the reference sequence to build index
#' from.
#'
#' @examples
#' \dontrun{
#' bowtie_index(
#'   bowtie_build_location = "path/to/bowtie-build",
#'   reference_fasta = "reference.fa")
#' }
#'
#' @export
bowtie_index <- function(bowtie_build_location, reference_fasta) {

  # Check if the input paths actually exist
  if (file.exists(bowtie_build_location) == FALSE) {
    stop("Invalid bowtie_build_location File doesn't exist.")
  }
  if (file.exists(reference_fasta) == FALSE) {
    stop("Invalid reference_fasta filename. File doesn't exist.")
  }

  create_index_command <- paste(
    shQuote(bowtie_build_location),
    shQuote(reference_fasta),
    shQuote(reference_fasta),
    sep = " "
  )

  # This should create a command like this:
  # "bowtie2-build referenceFasta reference_fasta" # Exclude Linting

  # From the bowtie manual: "The command should print many lines of output then
  # quit. When the command completes, the current directory will contain four
  # new files that all start with `reference_fasta` and end with `.1.bt2`,
  # `.2.bt2`, `.3.bt2`, `.4.bt2`, `.rev.1.bt2`, and `.rev.2.bt2`. These files
  # constitute the index."

  # Run the command and wait for it to complete
  system(create_index_command, wait = TRUE)

  # Did we actually get an output?
  this_file_should_exist <- paste(reference_fasta, ".1.bt2", sep = "")
  if (file.exists(this_file_should_exist) == FALSE) {
    warning(paste("It doesn't look like bowtie_index() worked. Command tried:",
                  create_index_command))
  }
}

#' Align reads to fusion sequence using bowtie
#'
#' This function aligns paired end reads to the given reference, producing .bam
#' and .bai files. Note that this function also uses Rsamtools to index and sort
#' the .sam file that bowtie creates.
#'
#' Both bowtie and bowtie2 can be used.
#'
#' See the plot-fusion-reads vignette for a complete example of how to use the
#' Bowtie functions.
#'
#' @seealso bowtie_index
#'
#' @param bowtie_location The path to bowtie on your system.
#' @param reference_name Name of the reference file.
#' @param fastq1 The first fastq file.
#' @param fastq2 The second fastq file.
#' @param output_bam_filename Filename for the resulting .bam file.
#' @param p Number of alignment threads to launch (default: 1). Given to -p (or
#' --threads).
#'
#' @examples
#' \dontrun{
#' bowtie_align(
#'   bowtie_location = "path/to/bowtie",
#'   reference_name = "reference.fa",
#'   fastq1 = "reads1.fq",
#'   fastq2 = "reads2.fq",
#'   output_bam_filename = "outputbam",
#'   p = 2)
#' }
#'
#' @export
bowtie_align <- function(
  bowtie_location,
  reference_name,
  fastq1,
  fastq2,
  output_bam_filename,
  p = 1
) {

  # Check if the input paths actually exist
  if (file.exists(bowtie_location) == FALSE) {
    stop("Invalid bowtie_location File doesn't exist.")
  }
  if (file.exists(reference_name) == FALSE) {
    stop("Invalid reference_name filename. File doesn't exist.")
  }
  if (file.exists(fastq1) == FALSE || file.exists(fastq2) == FALSE) {
    stop("Invalid fastq filenames. File(s) doesn't exist.")
  }
  if (!any(class(p) == c("integer", "numeric"))) {
    stop("Invalid p. Must be integer.")
  }

  align_command <- paste(
    bowtie_location,
    "-x", shQuote(reference_name),
    "-1", shQuote(fastq1),
    "-2", shQuote(fastq2),
    "--very-sensitive", "-p", p,
    "-S",
    paste(output_bam_filename, ".sam", sep = ""),
    sep = " "
  )

  # This should create a command like this:
  # "bowtie2 -x ref.fa -1 fastq1 -2 fastq2 --very-sensitive -p 1

  # Run the command and wait for it to complete
  system(align_command, wait = TRUE)

  # Sort and index
  Rsamtools::asBam(
    file = paste(output_bam_filename, ".sam", sep = ""),
    destination = output_bam_filename)

  # Did we actually get an output?
  if (file.exists(paste(output_bam_filename, ".bam", sep = "")) == FALSE) {
    warning(paste("It doesn't look like bowtie_align() worked. Command tried:",
                  align_command))
  }
}

#' An S4 class to represent a gene partner in a fusion
#'
#' The PartnerGene class represents one of the genes in a fusion event.
#'
#' @slot name Character containing name of the gene.
#' @slot ensembl_id Character containing ensembl id for the gene.
#' @slot chromosome Character containing chromosome name.
#' @slot breakpoint Numeric containing the fusion breakpoint.
#' @slot strand Character containing gene strand.
#' @slot junction_sequence Biostrings::DNAString containing the sequence right
#' before/after the fusion breakpoint.
#' @slot transcripts GenomicRanges::GRangesList containing three
#' GenomicRanges::Granges() objects, one for each "transcript type". The
#' transcript types are: 1) Transcripts where the fusion breakpoint hits an
#' exon boundary, 2) transcripts where the fusion breakpoint is within an
#' exon, 3) transcripts where the fusion breakpoint is within an intron.
#'
#' @export
PartnerGene <- setClass(
  "PartnerGene",
  slots = c(
    name = "character",
    ensembl_id = "character",
    chromosome = "character",
    breakpoint = "numeric",
    strand = "character",
    junction_sequence = "DNAString",
    transcripts = "GRangesList"
  )
)

#' Show method for the PartnerGene class.
#'
#' @param object A PartnerGene object
#'
#' @return Shows information about a PartnerGene object.
setMethod(
  "show",
  "PartnerGene",
  function(object) {
    print("PartnerGene object")
    print(paste("Name:", object@name))
    print(paste("ensemblId:", object@ensembl_id))
    print(paste("Chromosome:", object@chromosome))
    print(paste("Strand:", object@strand))
    print(paste("Breakpoint:", object@breakpoint))
  }
)

#' An S4 class to represent a fusion event.
#'
#' The Fusion class represents a fusion event, holding data imported from a
#' fusion tool.
#'
#' @slot id A unique id representing a fusion event. For deFuse data this will
#' be the cluster id.
#' @slot fusion_tool Name of the fusion tool.
#' @slot genome_version Name of the genome used to map reads.
#' @slot spanning_reads_count The number of spanning reads supporting the
#' fusion.
#' @slot split_reads_count The number of split reads supporting the fusion.
#' @slot fusion_reads_alignment A Gviz::AlignmentsTrack object holding the
#' fusion reads aligned to the fusion sequence.
#' @slot gene_upstream A PartnerGene object holding information of the upstream
#' fusion partner gene.
#' @slot gene_downstream A PartnerGene object holding information of the
#' downstream fusion partner gene.
#' @slot inframe A logical value indicating whether or not the downstream
#' fusion partner gene is inframe or not. Not all fusion-finders report this.
#' @slot fusion_tool_specific_data A list that will hold fields of importance
#' for a specific fusion finder. This field is used because many fusion-finders
#' report important values that are hard to fit into a standardized format.
#' Examples of values that are added to this list is probability from deFuse
#' and EricScore from EricScript.
#'
#' @export
Fusion <- setClass(
  "Fusion",
  slots = list(
    id = "character",
    fusion_tool = "character",
    genome_version = "character",
    spanning_reads_count = "numeric",
    split_reads_count = "numeric",
    fusion_reads_alignment = "AlignmentsTrack",
    gene_upstream = "PartnerGene",
    gene_downstream = "PartnerGene",
    inframe = "logical",
    fusion_tool_specific_data = "list"
  )
)

#' Show method for the Fusion class.
#'
#' @param object A Fusion object
#'
#' @return Shows information about a Fusion object.
setMethod(
  "show",
  "Fusion",
  function(object) {
    print("Fusion object")
    print(paste("id:", object@id))
    print(paste("Fusion tool:", object@fusion_tool))
    print(paste("Genome version:", object@genome_version))
    print(
      paste(
        "Gene names: ",
        paste(
          object@gene_upstream@name,
          object@gene_downstream@name,
          sep = "-"
        ),
        sep = ""
      )
    )
    print(
      paste(
        "Chromosomes: ",
        paste(
          object@gene_upstream@chromosome,
          object@gene_downstream@chromosome,
          sep = "-"
        ),
        sep = ""
      )
    )
    print(
      paste(
        "Strands: ",
        paste(
          object@gene_upstream@strand,
          object@gene_downstream@strand,
          sep = ","
        ),
        sep = ""
      )
    )
    print(
      paste(
        "In-frame?: ",
        object@inframe,
        sep = ""
      )
    )
  }
)

#' Get the upstream fusion partner gene
#'
#' This getter retrieves the upstream PartnerGene object.
#'
#' @param x The Fusion object you wish to retrieve the upstream PartnerGene
#' object for.
#'
#' @return The upstream PartnerGene object.
#'
#' @rdname upstream_partner_gene
#'
#' @examples
#' # Load data
#' defuseData <- system.file(
#'   "extdata",
#'   "defuse_833ke_results.filtered.tsv",
#'   package="chimeraviz")
#' fusions <- import_defuse(defuseData, "hg19", 1)
#' fusion <- fusions[[1]]
#' # Get the upstream fusion partner gene
#' upstream_partner_gene(fusion)
#'
#' @export
setGeneric(
  "upstream_partner_gene",
  function(x)
    standardGeneric("upstream_partner_gene")
)
#' @rdname upstream_partner_gene
setMethod(
  "upstream_partner_gene",
  "Fusion",
  function(x)
    return(x@gene_upstream)
)

#' Get the downstream fusion partner gene
#'
#' This getter retrieves the downstream PartnerGene object.
#'
#' @param x The Fusion object you wish to retrieve the downstream PartnerGene
#' object for.
#'
#' @return The downstream PartnerGene object.
#'
#' @rdname downstream_partner_gene
#'
#' @examples
#' # Load data
#' defuseData <- system.file(
#'   "extdata",
#'   "defuse_833ke_results.filtered.tsv",
#'   package="chimeraviz")
#' fusions <- import_defuse(defuseData, "hg19", 1)
#' fusion <- fusions[[1]]
#' # Get the downstream fusion partner gene
#' downstream_partner_gene(fusion)
#'
#' @export
setGeneric(
  "downstream_partner_gene",
  function(x)
    standardGeneric("downstream_partner_gene")
)
#' @rdname downstream_partner_gene
setMethod(
  "downstream_partner_gene",
  "Fusion",
  function(x)
    return(x@gene_downstream)
)

#' Set the upstream PartnerGene object of a Fusion object
#'
#' This sets the upstream PartnerGene object of a Fusion object
#'
#' @param object The Fusion object you wish to set a new upstream PartnerGene
#' object for.
#' @param value The new PartnerGene object.
#'
#' @rdname upstream_partner_gene
#'
#' @examples
#' # Load data
#' defuseData <- system.file(
#'   "extdata",
#'   "defuse_833ke_results.filtered.tsv",
#'   package="chimeraviz")
#' fusions <- import_defuse(defuseData, "hg19", 1)
#' fusion <- fusions[[1]]
#' # Set the upstream PartnerGene object to be the same as the downstream
#' # PartnerGene object
#' upstream_partner_gene(fusion) <- downstream_partner_gene(fusion)
#'
#' @export
setGeneric(
  "upstream_partner_gene<-",
  function(object, value) {
    standardGeneric("upstream_partner_gene<-")
  }
)
#' @rdname upstream_partner_gene
setReplaceMethod(
  f = "upstream_partner_gene",
  signature = "Fusion",
  definition = function(object, value) {
    object@gene_upstream <- value
    return(object)
  }
)

#' Set the downstream PartnerGene object of a Fusion object
#'
#' This sets the downstream PartnerGene object of a Fusion object
#'
#' @param object The Fusion object you wish to set a new downstream
#' PartnerGene object for.
#' @param value The new PartnerGene object.
#'
#' @rdname downstream_partner_gene
#'
#' @examples
#' # Load data
#' defuseData <- system.file(
#'   "extdata",
#'   "defuse_833ke_results.filtered.tsv",
#'   package="chimeraviz")
#' fusions <- import_defuse(defuseData, "hg19", 1)
#' fusion <- fusions[[1]]
#' # Set the downstream PartnerGene object to be the same as the upstream
#' # PartnerGene object
#' downstream_partner_gene(fusion) <- upstream_partner_gene(fusion)
#'
#' @export
setGeneric(
  "downstream_partner_gene<-",
  function(object, value) {
    standardGeneric("downstream_partner_gene<-")
  }
)
#' @rdname downstream_partner_gene
setReplaceMethod(
  f = "downstream_partner_gene",
  signature = "Fusion",
  definition = function(object, value) {
    object@gene_downstream <- value
    return(object)
  }
)

#' Get the Ensembl ID from a PartnerGene object
#'
#' This getter retrieves the Ensembl ID from a PartnerGene object
#'
#' @param x The PartnerGene object you wish to retrieve the Ensembl ID for.
#'
#' @return The upstream fusion partner gene Ensembl ID.
#'
#' @rdname partner_gene_ensembl_id
#'
#' @examples
#' # Load data
#' defuseData <- system.file(
#'   "extdata",
#'   "defuse_833ke_results.filtered.tsv",
#'   package="chimeraviz")
#' fusions <- import_defuse(defuseData, "hg19", 1)
#' fusion <- fusions[[1]]
#' # Get the Ensembl ID from the upstream fusion partner gene
#' partner_gene_ensembl_id(upstream_partner_gene(fusion))
#'
#' @export
setGeneric(
  "partner_gene_ensembl_id",
  function(x)
    standardGeneric("partner_gene_ensembl_id")
)
#' @rdname partner_gene_ensembl_id
setMethod(
  "partner_gene_ensembl_id",
  "PartnerGene",
  function(x)
    return(x@ensembl_id)
)

#' Set the Ensembl ID of a PartnerGene object
#'
#' This sets the Ensembl ID of a PartnerGene object.
#'
#' @param object The PartnerGene object you wish to set a new Ensembl ID for.
#' @param value The new Ensembl ID.
#'
#' @rdname partner_gene_ensembl_id
#'
#' @examples
#' # Load data
#' defuseData <- system.file(
#'   "extdata",
#'   "defuse_833ke_results.filtered.tsv",
#'   package="chimeraviz")
#' fusions <- import_defuse(defuseData, "hg19", 1)
#' fusion <- fusions[[1]]
#' # Set the downstream PartnerGene object to be the same as the upstream
#' # PartnerGene object
#' partner_gene_ensembl_id(upstream_partner_gene(fusion)) <- "test"
#'
#' @export
setGeneric(
  "partner_gene_ensembl_id<-",
  function(object, value) {
    standardGeneric("partner_gene_ensembl_id<-")
  }
)
#' @rdname partner_gene_ensembl_id
setReplaceMethod(
  f = "partner_gene_ensembl_id",
  signature = "PartnerGene",
  definition = function(object, value) {
    object@ensembl_id <- value
    return(object)
  }
)

#' Get the junction sequence from a PartnerGene object
#'
#' This getter retrieves the junction sequence from a PartnerGene object
#'
#' @param x The PartnerGene object you wish to retrieve the junction sequence
#' for.
#'
#' @return The upstream fusion partner gene junction sequence.
#'
#' @rdname partner_gene_junction_sequence
#'
#' @examples
#' # Load data
#' defuseData <- system.file(
#'   "extdata",
#'   "defuse_833ke_results.filtered.tsv",
#'   package="chimeraviz")
#' fusions <- import_defuse(defuseData, "hg19", 1)
#' fusion <- fusions[[1]]
#' # Get the junction sequence from the upstream fusion partner gene
#' partner_gene_junction_sequence(upstream_partner_gene(fusion))
#'
#' @export
setGeneric(
  "partner_gene_junction_sequence",
  function(x)
    standardGeneric("partner_gene_junction_sequence")
)
#' @rdname partner_gene_junction_sequence
setMethod(
  "partner_gene_junction_sequence",
  "PartnerGene",
  function(x)
    return(x@junction_sequence)
)

#' Get the split reads count from a Fusion object
#'
#' This getter retrieves the split reads count from a Fusion object
#'
#' @param x The Fusion object you wish to retrieve the split reads count for.
#'
#' @return The Fusion split reads count.
#'
#' @rdname fusion_split_reads_count
#'
#' @examples
#' # Load data
#' defuseData <- system.file(
#'   "extdata",
#'   "defuse_833ke_results.filtered.tsv",
#'   package="chimeraviz")
#' fusions <- import_defuse(defuseData, "hg19", 1)
#' fusion <- fusions[[1]]
#' # Get the split reads count
#' fusion_split_reads_count(fusion)
#'
#' @export
setGeneric(
  "fusion_split_reads_count",
  function(x)
    standardGeneric("fusion_split_reads_count")
)
#' @rdname fusion_split_reads_count
setMethod(
  "fusion_split_reads_count",
  "Fusion",
  function(x)
    return(x@split_reads_count)
)

#' Get the spanning reads count from a Fusion object
#'
#' This getter retrieves the spanning reads count from a Fusion object
#'
#' @param x The Fusion object you wish to retrieve the spanning reads count for.
#'
#' @return The Fusion spanning reads count.
#'
#' @rdname fusion_spanning_reads_count
#'
#' @examples
#' # Load data
#' defuseData <- system.file(
#'   "extdata",
#'   "defuse_833ke_results.filtered.tsv",
#'   package="chimeraviz")
#' fusions <- import_defuse(defuseData, "hg19", 1)
#' fusion <- fusions[[1]]
#' # Get the spanning reads count
#' fusion_spanning_reads_count(fusion)
#'
#' @export
setGeneric(
  "fusion_spanning_reads_count",
  function(x)
    standardGeneric("fusion_spanning_reads_count")
)
#' @rdname fusion_spanning_reads_count
setMethod(
  "fusion_spanning_reads_count",
  "Fusion",
  function(x)
    return(x@spanning_reads_count)
)

#' Import results from a ChimPipe run into a list of Fusion objects.
#'
#' A function that imports the results from a ChimPipe run, typically from a
#' chimericJunctions_.txt file, into a list of Fusion objects.
#'
#' @param filename Filename for the ChimPipe results.
#' @param genome_version Which genome was used in mapping (hg19, hg38, etc.).
#' @param limit A limit on how many lines to read.
#'
#' @return A list of Fusion objects.
#'
#' @examples
#' chimpipefile <- system.file(
#'   "extdata",
#'   "chimericJunctions_MCF-7.txt",
#'   package="chimeraviz")
#' fusions <- import_chimpipe(chimpipefile, "hg19", 3)
#' # This should import a list of 3 fusions described in Fusion objects.

#' @importFrom data.table fread
#'
#' @export
import_chimpipe <- function(filename, genome_version, limit) {

  # Is the genome version valid?
  valid_genomes <- c("hg19", "hg38", "mm10")
  if (is.na(match(tolower(genome_version), tolower(valid_genomes)))) {
    stop("Invalid genome version given")
  }

  # If the limit is set, is the value valid?
  if (missing(limit) == FALSE) {
    if (is.numeric(limit) == FALSE || limit <= 0) {
      stop("limit must be a numeric value bigger than 0")
    }
  }

  # Try to read the fusion report
  report <- withCallingHandlers({
      col_types <- c(
        "juncCoord" = "character",
        "gnTypesA" = "character",
        "gnTypesB" = "character",
        "gnNamesA" = "character",
        "gnNamesB" = "character",
        "nbSpanningReads" = "integer",
        "nbConsistentPE" = "integer"
      )
      if (missing(limit)) {
        # Read all lines
        data.table::fread(
          input = filename,
          colClasses = col_types,
          showProgress = FALSE
        )
      } else {
          # Only read up to the limit
          data.table::fread(
            input = filename,
            colClasses = col_types,
            showProgress = FALSE,
            nrows = limit
          )
      }
    },
    error = function(cond) {
      message(paste0("Reading ", filename, " caused an error: ", cond[[1]]))
      stop(cond)
    },
    warning = function(cond) {
      # Begin Exclude Linting
      message(paste0("Reading ", filename, " caused a warning: ", cond[[1]]))
      # End Exclude Linting
    }
  )

  # Set variables
  id                   <- NA
  inframe              <- NA
  fusion_tool          <- "ChimPipe"
  spanning_reads_count <- NA
  split_reads_count    <- NA

  # List to hold all Fusion objects
  fusion_list <- vector("list", dim(report)[1])

  # Iterate through each line in the .tsv file
  for (i in 1:dim(report)[1]) {

    # Import defuse-specific fields
    fusion_tool_specific_data <- list() #TODO

    # Cluster id
    id <- as.character(i)

    # Is the downstream fusion partner in-frame?
    inframe <- NA

    parsed_junccord <- .chimpipe_parse_junccoord(report[[i, "juncCoord"]])

    # Strand
    strand_upstream <- parsed_junccord[3]
    strand_downstream <- parsed_junccord[6]

    # Number of supporting reads
    split_reads_count <- report[[i, "nbConsistentPE"]]
    spanning_reads_count <- report[[i, "nbSpanningReads"]]

    # No junction sequence is given from chimpipe
    junction_sequence_upstream <-
      Biostrings::DNAString()
    junction_sequence_downstream <-
      Biostrings::DNAString()

    # Breakpoints
    breakpoint_upstream <- tryCatch(
      as.numeric(parsed_junccord[2]),
      warning = function(w) {
        -1
      }
    )
    breakpoint_downstream <- tryCatch(
      as.numeric(parsed_junccord[5]),
      warning = function(w) {
        -1
      }
    )

    # Chromosome names
    chromosome_upstream <-
      parsed_junccord[1]
    chromosome_downstream <-
      parsed_junccord[4]

    # Gene names
    name_upstream <- report[[i, "gnNamesA"]]
    name_downstream <- report[[i, "gnNamesB"]]

    # Ensembl ids
    ensembl_id_upstream <- report[[i, "gnIdsA"]]
    ensembl_id_downstream <- report[[i, "gnIdsB"]]

    # PartnerGene objects
    gene_upstream <- new(
      Class = "PartnerGene",
      name = name_upstream,
      ensembl_id = ensembl_id_upstream,
      chromosome = chromosome_upstream,
      breakpoint = breakpoint_upstream,
      strand = strand_upstream,
      junction_sequence = junction_sequence_upstream,
      transcripts = GenomicRanges::GRangesList()
    )
    gene_downstream <- new(
      Class = "PartnerGene",
      name = name_downstream,
      ensembl_id = ensembl_id_downstream,
      chromosome = chromosome_downstream,
      breakpoint = breakpoint_downstream,
      strand = strand_downstream,
      junction_sequence = junction_sequence_downstream,
      transcripts = GenomicRanges::GRangesList()
    )

    fusion_list[[i]] <- new(
      Class = "Fusion",
      id = id,
      fusion_tool = fusion_tool,
      genome_version = genome_version,
      spanning_reads_count = spanning_reads_count,
      split_reads_count = split_reads_count,
      fusion_reads_alignment = Gviz::AlignmentsTrack(),
      gene_upstream = gene_upstream,
      gene_downstream = gene_downstream,
      inframe = inframe,
      fusion_tool_specific_data = fusion_tool_specific_data
    )
  }

  # Return the list of Fusion objects
  fusion_list
}

.chimpipe_parse_junccoord <- function(junccoord) {
  if (class(junccoord) != "character") {
    stop("'junccoord' must be a character")
  }

  # Example input: chr12_9392928_+:chr11_75115716_+

  parts <- unlist(strsplit(junccoord, split = ":"))

  if (length(parts) != 2) {
    stop(paste0("Unable to parse 'junccoord':", junccoord))
  }

  parts_gene_upstream <- parts[1]
  parts_gene_downstream <- parts[2]

  upstream <- unlist(strsplit(parts_gene_upstream, split = "_"))
  downstream <- unlist(strsplit(parts_gene_downstream, split = "_"))

  if (length(upstream) != 3) {
    stop(paste0("Unable to parse 'junccoord':", junccoord))
  }
  if (length(downstream) != 3) {
    stop(paste0("Unable to parse 'junccoord':", junccoord))
  }

  c(upstream, downstream)
}

context("Test importing ChimPipe data into a list of Fusion objects")

chimpipefile <- system.file(
  "extdata",
  "chimericJunctions_MCF-7.txt",
  package = "chimeraviz")
fusions <- import_chimpipe(chimpipefile, "hg19", 3)
fusion <- fusions[[1]]

test_that("import_chimpipe imports data into Fusion objects", {
  expect_equal(length(fusions), 3)
  expect_equal(fusion@id, "1")
  expect_equal(fusion@gene_upstream@name, "KRT86")
  expect_equal(fusion@gene_upstream@chromosome, "chr12")
})

test_that("import_chimpipe fails to import data", {
  expect_error(import_chimpipe("Non-existent-file", "hg18"))
  expect_error(import_chimpipe(chimpipefile, "Invalid-genome"))
  expect_error(import_chimpipe(chimpipefile, "hg19", "invalid limit"))
})

test_that("import_chimpipe limits work", {
  expect_equal(length(import_chimpipe(chimpipefile, "hg19", 2)), 2)
})

test_that(".chimpipe_parse_junccoord works", {
  expect_equal(length(.chimpipe_parse_junccoord("chr12_9392928_+:chr11_75115716_+")), 6)
})

#' Fetch reads from fastq files
#'
#' This function will fetch read sequences from fastq files and put them into
#' new fastq files.
#'
#' Note: This function runs (read only) bash commands on your system. Therefore
#' the function will only work on a unix system.
#'
#' @param reads List of read IDs that is to be fetched.
#'
#' @param fastq_file_in1 First fastq file to search in.
#' @param fastq_file_in2 Second fastq file to seach in.
#'
#' @param fastq_file_out1 First fastq file with results.
#' @param fastq_file_out2 Second fastq file with results.
#'
#' @return The files fastqFileOut1 and fastqFileOut2 populated with the
#' specified reads.
#'
#' @examples
#' \dontrun{
#' # fastq files that has the supporting reads
#' fastq1 <- system.file("extdata", "reads.1.fq", package="chimeraviz")
#' fastq2 <- system.file("extdata", "reads.2.fq", package="chimeraviz")
#' # Which read ids to extract
#' reads <- c(
#'   "13422259", "19375605", "29755061",
#'   "31632876", "32141428", "33857245")
#' # Extract the actual reads and put them in the tmp files "fastqFileOut1" and
#' # "fastqFileOut2"
#' fastqFileOut1 <- tempfile(pattern = "fq1", tmpdir = tempdir())
#' fastqFileOut2 <- tempfile(pattern = "fq2", tmpdir = tempdir())
#' fetch_reads_from_fastq(reads, fastq1, fastq2,
#'     fastqFileOut1, fastqFileOut2)
#' # We now have the reads supporting fusion 5267 in the two files.
#' }
#'
#' @export
fetch_reads_from_fastq <- function(
  reads,
  fastq_file_in1,
  fastq_file_in2,
  fastq_file_out1,
  fastq_file_out2
  ) {

  # Since this function use the bash function egrep to extract reads, give a
  # warning if we're running on windows
  if (Sys.info()["sysname"] == "Windows") {
    stop(paste("This function uses the bash function egrep. It looks like",
               "you're running on windows, so this function will terminate."))
  }

  if (is.vector(reads, mode = "character") == FALSE) {
    stop("reads should be a character vector of read ids")
  }

  if (file.exists(fastq_file_in1) == FALSE ||
      file.exists(fastq_file_in2) == FALSE) {
    stop("Invalid fastq input files")
  }

  # The command below will extract the sequences for ids 11, 22, and 33 from the
  # fastq file reads.fastq

  # $ egrep -A 3 '@11|22|33' reads.fastq | sed '/^--$/d'

  # By default, the -A parameter inserts a "--" between matches, so the sed part
  # above removes that

  # First build the query with the read ids
  query <- paste("@",
                 paste(reads, collapse = "|"),
                 sep = "")

  # Then create commands for each fastq file
  command1 <- paste("egrep -A 3 '",
                    query,
                    "' ",
                    shQuote(fastq_file_in1),
                    " | sed '/^--$/d'", # Exclude Linting
                    sep = "")
  command2 <- paste("egrep -A 3 '",
                    query,
                    "' ",
                    shQuote(fastq_file_in2),
                    " | sed '/^--$/d'", # Exclude Linting
                    sep = "")

  # We now have two commands:
  # egrep -A 1 '@11|22|33' reads.1.fastq | sed '/^--$/d'
  # egrep -A 1 '@11|22|33' reads.2.fastq | sed '/^--$/d'

  # Run the command and capture output
  supporting_reads_fq1 <- system(command1, intern = TRUE)
  supporting_reads_fq2 <- system(command2, intern = TRUE)

  # Write these to each their own files
  write(supporting_reads_fq1, file = fastq_file_out1)
  write(supporting_reads_fq2, file = fastq_file_out2)

  # Check file sizes
  if (file.info(fastq_file_out1)$size <= 10) {
    warning(paste("It looks like we couldn't find any of the reads in the",
                  "fastq file. Check your arguments."))
  }
}

#' Write fusion junction sequence to a fasta file
#'
#' This function will write the fusion sequence to a fasta file, using
#' Biostring::writeXStringSet() .
#'
#' @param fusion The Fusion object we want to create a fasta file from.
#' @param filename The filename to write to.
#'
#' @return Writes the fusion junction sequence to the given filename.
#'
#' @examples
#' # Import the filtered defuse results
#' defuse833keFiltered <- system.file(
#'   "extdata",
#'   "defuse_833ke_results.filtered.tsv",
#'   package="chimeraviz")
#' fusions <- import_defuse(defuse833keFiltered, "hg19", 1)
#' # Get a specific fusion
#' fusion <- get_fusion_by_id(fusions, 5267)
#' # Create temporary file to hold the fusion sequence
#' fastaFileOut <- tempfile(pattern = "fusionSequence", tmpdir = tempdir())
#' # Write fusion sequence to file
#' write_fusion_reference(fusion, fastaFileOut)
#'
#' @export
write_fusion_reference <- function(fusion, filename) {

  # Check if we got a fusion object
  if (class(fusion) != "Fusion") {
    stop("fusion argument must be an object of type Fusion")
  }

  # First put the fusion junction sequence in a DNAStringSet object
  fusion_sequence <- Biostrings::DNAStringSet(
    x = c(fusion@gene_upstream@junction_sequence,
          fusion@gene_downstream@junction_sequence))

  # Give an error if the length of the fusionSequence is 0:
  if (nchar(fusion_sequence) == 0) {
    stop(
      paste0(
        "The fusion sequence length is zero, so the fusion reference sequence",
        " cannot be written."
      )
    )
  }

  # Set sequence name to chrNA, since this is a sequence created from a fusion
  # event (i.e. not a sequence from a real chromosome). The "chrNA" name will
  # make Gviz happy.
  names(fusion_sequence) <- "chrNA"
  # Write to file
  Biostrings::writeXStringSet(fusion_sequence, filename)
}

#' Get ensembl ids for a fusion object
#'
#' This function will get the ensembl ids from the org.Hs.eg.db/org.Mm.eg.db
#' package given the gene names of the fusion event.
#'
#' @param fusion The Fusion object we want to get ensembl ids for.
#'
#' @return The Fusion object with Ensembl ids set.
#'
#' @import org.Hs.eg.db org.Mm.eg.db
#'
#' @examples
#' # Import the filtered defuse results
#' defuse833keFiltered <- system.file(
#'   "extdata",
#'   "defuse_833ke_results.filtered.tsv",
#'   package="chimeraviz")
#' fusions <- import_defuse(defuse833keFiltered, "hg19", 1)
#' # Get a specific fusion
#' fusion <- get_fusion_by_id(fusions, 5267)
#' # See the ensembl ids:
#' partner_gene_ensembl_id(upstream_partner_gene(fusion))
#' # [1] "ENSG00000180198"
#' partner_gene_ensembl_id(downstream_partner_gene(fusion))
#' # [1] "ENSG00000162639"
#' # Reset the fusion objects ensembl ids
#' partner_gene_ensembl_id(upstream_partner_gene(fusion)) <- ""
#' partner_gene_ensembl_id(downstream_partner_gene(fusion))  <- ""
#' # Get the ensembl ids
#' fusion <- get_ensembl_ids(fusion)
#' # See that we now have the same ensembl ids again:
#' partner_gene_ensembl_id(upstream_partner_gene(fusion))
#' # [1] "ENSG00000180198"
#' partner_gene_ensembl_id(downstream_partner_gene(fusion))
#' # [1] "ENSG00000162639"
#'
#' @export
get_ensembl_ids <- function(fusion) {

  # Check if we got a fusion object
  if (class(fusion) != "Fusion") {
    stop("fusion argument must be an object of type Fusion")
  }

  if (startsWith(fusion@genome_version, "hg")) {
    annotation_db <- org.Hs.eg.db # Exclude Linting
  } else if (startsWith(fusion@genome_version, "mm")) {
    annotation_db <- org.Mm.eg.db # Exclude Linting
  } else {
    stop("Unsupported genome version")
  }

  result <- AnnotationDbi::select(
    annotation_db,
    keys = c(fusion@gene_upstream@name, fusion@gene_downstream@name),
    keytype = "ALIAS",
    columns = c("ALIAS", "ENSEMBL"))

  upstream_gene_result <-
    result[which(result$ALIAS == fusion@gene_upstream@name), ]
  downstream_gene_result <-
    result[which(result$ALIAS == fusion@gene_downstream@name), ]

  # Stop execution if no results
  if (any(is.na(upstream_gene_result$ENSEMBL))) {
    stop(
      paste(
        "Could not find Ensembl id for ",
        fusion@gene_upstream@name,
        ". ",
        "If you know the id, add it manually with ",
        "fusion@gene_upstream@ensembl_id <- \"ensemblId\"",
        sep = ""
      )
    )
  }
  if (any(is.na(downstream_gene_result$ENSEMBL))) {
    stop(
      paste(
        "Could not find Ensembl id for ",
        fusion@gene_downstream@name,
        ". ",
        "If you know the id, add it manually with ",
        "fusion@gene_downstream@ensembl_id <- \"ensemblId\"",
        sep = ""
      )
    )
  }

  # Store ensembl ids in fusion object
  fusion@gene_upstream@ensembl_id <- upstream_gene_result[, 2]
  fusion@gene_downstream@ensembl_id <- downstream_gene_result[, 2]

  # Return updated fusion object
  fusion
}

#' Split GRanges object based on cds
#'
#' This function will look for ranges (exons) in the GRanges object that has the
#' coding DNA sequence starting or stopping within it. If found, these exons are
#' split, and each exon in the GRanges object will be tagged as either
#' "protein_coding", "5utr", or "3utr". The returned GRanges object will have
#' feature values set in mcols(gr)$feature reflecting this.
#'
#' @param gr The GRanges object we want to split and tag with feature info.
#'
#' @return An updated GRanges object with feature values set.
#'
#' @examples
#' # Load fusion data and choose a fusion object:
#' defuseData <- system.file(
#'   "extdata",
#'   "defuse_833ke_results.filtered.tsv",
#'   package="chimeraviz")
#' fusions <- import_defuse(defuseData, "hg19", 1)
#' fusion <- get_fusion_by_id(fusions, 5267)
#' # Create edb object
#' edbSqliteFile <- system.file(
#'   "extdata",
#'   "Homo_sapiens.GRCh37.74.sqlite",
#'   package="chimeraviz")
#' edb <- ensembldb::EnsDb(edbSqliteFile)
#' # Get all exons for all transcripts in the genes in the fusion transcript
#' allTranscripts <- ensembldb::exonsBy(
#'   edb,
#'   filter = list(
#'     AnnotationFilter::GeneIdFilter(
#'       c(
#'         partner_gene_ensembl_id(upstream_partner_gene(fusion)),
#'         partner_gene_ensembl_id(downstream_partner_gene(fusion))))),
#'   columns = c(
#'     "gene_id",
#'     "gene_name",
#'     "tx_id",
#'     "tx_cds_seq_start",
#'     "tx_cds_seq_end",
#'     "exon_id"))
#' # Extract one of the GRanges objects
#' gr <- allTranscripts[[1]]
#' # Check how many ranges there are here
#' length(gr)
#' # Should be 9 ranges
#' # Split the ranges containing the cds start/stop positions and add feature
#' # values:
#' gr <- split_on_utr_and_add_feature(gr)
#' # Check the length again
#' length(gr)
#' # Should be 11 now, as the range containing the cds_strat position and the
#' # range containing the cds_stop position has been split into separate ranges
#'
#' @importFrom S4Vectors mcols
#'
#' @export
split_on_utr_and_add_feature <- function(gr) {

  # Check if we got a valid GRanges object
  if (class(gr) != "GRanges") {
    stop("gr argument must be an object of type GRanges")
  }

  # Check that we have the tx_cds_seq_start and tx_cds_seq_end values


  # check that gr has a tx_cds_seq_start and tx_cds_seq_end mcols value

  # Get cds start and cds end
  cds_start <- S4Vectors::mcols(gr)$tx_cds_seq_start[[1]]
  cds_end <- S4Vectors::mcols(gr)$tx_cds_seq_end[[1]]

  # find exon containing start
  cds_start_exon <- cds_start > start(gr) & cds_start < end(gr)

  if (any(cds_start_exon)) {
    # Create two copies of the exon in which the cds starts
    first <- gr[cds_start_exon]
    second <- gr[cds_start_exon]
    # Update ranges for the new objects
    end(first) <- cds_start - 1
    start(second) <- cds_start
    # Remove the original range
    gr <- gr[!cds_start_exon]
    # Add new ranges
    gr <- append(gr, first)
    gr <- append(gr, second)
    # Sort again
    gr <- sort(gr)
  }

  # find exon containing end
  cds_end_exon <- cds_end > start(gr) & cds_end < end(gr)

  if (any(cds_end_exon)) {
    # Create two copies of the exon in which the cds ends
    first <- gr[cds_end_exon]
    second <- gr[cds_end_exon]
    # Update ranges for the new objects
    end(first) <- cds_end
    start(second) <- cds_end + 1
    # Remove the original range
    gr <- gr[!cds_end_exon]
    # Add new ranges
    gr <- append(gr, first)
    gr <- append(gr, second)
    # Sort again
    gr <- sort(gr)
  }

  # add features
  S4Vectors::mcols(gr)$feature <- "protein_coding"
  # 5utr:
  #   plus strand:
  #     end(gr) < cds_start # Exclude Linting
  #   minus strand:
  #     start(gr) > cds_end # Exclude Linting
  if (length(gr[as.character(strand(gr)) == "+" & end(gr) < cds_start |
                as.character(strand(gr)) == "-" & start(gr) > cds_end])) {
    S4Vectors::mcols(
      gr[as.character(strand(gr)) == "+" & end(gr) < cds_start |
           as.character(strand(gr)) == "-" & start(gr) > cds_end]
      )$feature <- "5utr"
  }
  # 3utr:
  #   plus strand:
  #     start(gr) > cds_end # Exclude Linting
  #   minus strand:
  #     end(gr) < cds_start # Exclude Linting
  if (length(gr[as.character(strand(gr)) == "+" & start(gr) > cds_end |
                as.character(strand(gr)) == "-" & end(gr) < cds_start])) {
    S4Vectors::mcols(
      gr[as.character(strand(gr)) == "+" & start(gr) > cds_end |
           as.character(strand(gr)) == "-" & end(gr) < cds_start]
      )$feature <- "3utr"
  }

  gr
}

#' Retrieves transcripts for partner genes in a Fusion object using Ensembldb
#'
#' This function will check where in the transcript (the GRanges object) the
#' fusion breakpoint is located, and return either "exonBoundary", "withinExon",
#' "withinIntron", or "intergenic".
#'
#' @param gr The GRanges object containing the transcript to be checked.
#' @param fusion The fusion object used to check the transcript.
#'
#' @return Either "exonBoundary", "withinExon", "withinIntron", or "intergenic"
#' depending on where in the transcript the breakpoint hits.
#'
#' @examples
#' # Load fusion data and choose a fusion object:
#' defuseData <- system.file(
#'   "extdata",
#'   "defuse_833ke_results.filtered.tsv",
#'   package="chimeraviz")
#' fusions <- import_defuse(defuseData, "hg19", 1)
#' fusion <- get_fusion_by_id(fusions, 5267)
#' # Create edb object
#' edbSqliteFile <- system.file(
#'   "extdata",
#'   "Homo_sapiens.GRCh37.74.sqlite",
#'   package="chimeraviz")
#' edb <- ensembldb::EnsDb(edbSqliteFile)
#' # Get all exons for all transcripts in the genes in the fusion transcript
#' allTranscripts <- ensembldb::exonsBy(
#'   edb,
#'   filter = list(
#'     AnnotationFilter::GeneIdFilter(
#'       c(
#'         partner_gene_ensembl_id(upstream_partner_gene(fusion)),
#'         partner_gene_ensembl_id(downstream_partner_gene(fusion))))),
#'   columns = c(
#'     "gene_id",
#'     "gene_name",
#'     "tx_id",
#'     "tx_cds_seq_start",
#'     "tx_cds_seq_end",
#'     "exon_id"))
#' # Extract one of the GRanges objects
#' gr <- allTranscripts[[1]]
#' # Check where in the transcript the fusion breakpoint hits
#' decide_transcript_category(gr, fusion)
#' # "exonBoundary"
#' # Check another case
#' gr <- allTranscripts[[3]]
#' decide_transcript_category(gr, fusion)
#' # "withinIntron"
#'
#' @importFrom S4Vectors mcols
#'
#' @export
decide_transcript_category <- function(gr, fusion) {

  # Check if we got a valid GRanges object
  if (class(gr) != "GRanges") {
    stop("gr argument must be an object of type GRanges")
  }

  # Check if we got a fusion object
  if (class(fusion) != "Fusion") {
    stop("fusion argument must be an object of type Fusion")
  }

  # 4 possible cases:
  #
  # 1: geneA is on the + strand
  #    Exon boundary event if breakpointA equals an exon_end position
  # 2: geneA is on the - strand
  #    Exon boundary event if breakpointA equals an exon_start position
  # 3: geneB is on the + strand
  #    Exon boundary event if breakpointB equals an exon_start position
  # 4: geneB is on the - strand
  #    Exon boundary event if breakpointB equals an exon_end position

  # Helper variables
  upstream_gene <-
    S4Vectors::mcols(gr)$gene_id[[1]] == fusion@gene_upstream@ensembl_id
  breakpoint <-
    if (all(S4Vectors::mcols(gr)$gene_id == fusion@gene_upstream@ensembl_id)) {
      fusion@gene_upstream@breakpoint
    } else {
      fusion@gene_downstream@breakpoint
    }
  exon_start_positions <- GenomicRanges::start(gr)
  exon_end_positions <- GenomicRanges::end(gr)

  # Check whether or not the breakpoint is within the transcript
  if (all(breakpoint < exon_start_positions) ||
      all(exon_end_positions < breakpoint)) {
    return("intergenic")
  }

  # Assuming only single-stranded transcripts
  if (as.character(strand(gr)[1]) == "+") {

    # To decide whether or not the breakpoint occurs at an exon boundary, we
    # have to be vary careful about both whether this is the upstream/downstrem
    # fusion partner gene
    if (upstream_gene) {
      # GeneA
      if (any(exon_end_positions == breakpoint)) {
        # exon boundary
        return("exonBoundary")
      }
    } else {
      # GeneB
      if (any(exon_start_positions == breakpoint)) {
        # exon boundary
        return("exonBoundary")
      }
    }

  } else {

    # To decide whether or not the breakpoint occurs at an exon boundary, we
    # have to be vary careful about both whether this is the upstream/downstrem
    # fusion partner gene
    if (all(upstream_gene)) {
      # GeneA
      if (any(exon_start_positions == breakpoint)) {
        # exon boundary
        return("exonBoundary")
      }
    } else {
      # GeneB
      if (any(exon_end_positions == breakpoint)) {
        # exon boundary
        return("exonBoundary")
      }
    }

  }

  # Within exon if end < breakpoint & breakpoint < start
  if (any(start(gr) < breakpoint & breakpoint < end(gr))) {
    return("withinExon")
  } else {
    return("withinIntron")
  }
}

#' Retrieves transcripts for partner genes in a Fusion object using Ensembldb
#'
#' This function will retrieve transcripts for both genes in a fusion. It will
#' check all transcripts and decide for each transcript if the fusion breakpoint
#' happens at 1) an exon boundary, 2) within an exon, or 3) within an intron.
#' This is done because fusions happening at exon boundaries are more likely to
#' produce biologically interesting gene products. The function returns an
#' updated Fusion object, where the fusion@gene_upstream@transcriptsX slots are set with
#' transcript information.
#'
#' @param fusion The fusion object to find transcripts for.
#' @param edb The edb object used to fetch data from.
#'
#' @return An updated fusion object with transcript data stored.
#'
#' @examples
#' # Load fusion data and choose a fusion object:
#' defuseData <- system.file(
#'   "extdata",
#'   "defuse_833ke_results.filtered.tsv",
#'   package="chimeraviz")
#' fusions <- import_defuse(defuseData, "hg19", 1)
#' fusion <- get_fusion_by_id(fusions, 5267)
#' # Create edb object
#' edbSqliteFile <- system.file(
#'   "extdata",
#'   "Homo_sapiens.GRCh37.74.sqlite",
#'   package="chimeraviz")
#' edb <- ensembldb::EnsDb(edbSqliteFile)
#' # Add transcripts data to fusion object
#' fusion <- get_transcripts_ensembl_db(fusion, edb)
#' # The transcripts are now accessible through fusion@gene_upstream@transcripts and
#' # fusion@gene_downstream@transcripts .
#'
#' @importFrom S4Vectors mcols
#' @importFrom ensembldb exonsBy
#' @importFrom AnnotationFilter GeneIdFilter
#'
#' @export
get_transcripts_ensembl_db <- function(fusion, edb) {

  # Check if we got a fusion object
  if (class(fusion) != "Fusion") {
    stop("fusion argument must be an object of type Fusion")
  }

  # Check if we got a valid edb
  if (class(edb) != "EnsDb") {
    stop("edb argument must be an object of type EnsDb")
  }

  # Fetch ensembl gene ids if we don't already have them
  if (
    is.na(fusion@gene_upstream@ensembl_id) ||
    is.na(fusion@gene_downstream@ensembl_id)
  ) {
    fusion <- get_ensembl_ids(fusion)
  }

  # Get all exon information
  all_transcripts <- ensembldb::exonsBy(
    edb,
    filter = list(
      AnnotationFilter::GeneIdFilter(
        c(
          fusion@gene_upstream@ensembl_id,
          fusion@gene_downstream@ensembl_id))),
    columns = c(
      "gene_id",
      "gene_name",
      "tx_id",
      "tx_cds_seq_start",
      "tx_cds_seq_end",
      "exon_id"))

  # Fail if no transcripts were found
  if (length(all_transcripts) == 0) {
    stop(paste(
      "No transcripts available for the genes ",
      fusion@gene_upstream@name,
      " and ",
      fusion@gene_downstream@name,
      ".",
      sep = ""))
  }

  # Go through each transcript in the GRangesList and
  grangeslist_upstream <- GRangesList()
  grangeslist_downstream <- GRangesList()
  for (i in seq_along(all_transcripts)) {

    # Extract the GRanges object
    gr <- all_transcripts[[i]]

    # Add $transcript as a metadata column
    S4Vectors::mcols(gr)$transcript <- names(all_transcripts[i])
    # Add $symbol as a metadata column
    S4Vectors::mcols(gr)$symbol <- names(all_transcripts[i])

    # Add utr/coding S4Vectors::mcols$feature data for each region
    if (is.na(S4Vectors::mcols(gr)$tx_cds_seq_start[[1]])) {
      # The transcript is not coding. Set all to "utr"
      S4Vectors::mcols(gr)$feature <- "utr"
    }else {
      # The transcript is coding. The function below will set
      # S4Vectors::mcols(gr)$feature = "utr5"/"protein_coding"/"utr3" for each
      # region
      gr <- split_on_utr_and_add_feature(gr)
    }

    # Create new GRangesList
    grl <- GRangesList(gr)

    # Tag each transcript with either "exonBoundary", "withinExon",
    # "withinIntron", or "intergenic", depending on where in the transcript the
    # fusion breakpoint hits
    S4Vectors::mcols(grl)$transcript_category <- decide_transcript_category(
      gr,
      fusion)

    # Set S4Vectors::mcols$transcript and name for the grl
    S4Vectors::mcols(grl)$transcript <- names(all_transcripts[i])
    names(grl) <- names(all_transcripts[i])

    # Add S4Vectors::mcols$coding to signify whether it is a coding transcript
    # at all
    S4Vectors::mcols(grl)$coding <-
      !all(is.na(S4Vectors::mcols(gr)$tx_cds_seq_start))

    # Append the GRanges object to the correct GRangesList
    if (S4Vectors::mcols(gr)$gene_id[[1]] == fusion@gene_upstream@ensembl_id) {
      grangeslist_upstream <- append(grangeslist_upstream, grl)
    } else {
      grangeslist_downstream <- append(grangeslist_downstream, grl)
    }
  }

  # Add transcripts to fusion object
  fusion@gene_upstream@transcripts <- grangeslist_upstream
  fusion@gene_downstream@transcripts <- grangeslist_downstream

  # Warn if no transcripts were found for one of the genes
  if (length(fusion@gene_upstream@transcripts) == 0) {
    warning(paste(
      "No transcripts available for the upstream gene ",
      fusion@gene_upstream@name,
      " available.",
      sep = ""))
  }
  if (length(fusion@gene_upstream@transcripts) == 0) {
    warning(paste(
      "No transcripts available for the downstream gene ",
      fusion@gene_downstream@name,
      " available.",
      sep = ""))
  }

  # In case the fusion object doesn't have the strands set (as is the case for
  # JAFFA), set the strands now:
  if (
    fusion@gene_upstream@strand == "*" || fusion@gene_downstream@strand == "*"
  ) {
    fusion@gene_upstream@strand <- as.character(
      strand(fusion@gene_upstream@transcripts[[1]][1]))
    fusion@gene_downstream@strand <- as.character(
      strand(fusion@gene_downstream@transcripts[[1]][1]))
  }

  # Return fusion object
  fusion

}

#' Add fusion reads alignment to fusion object
#'
#' This function lets you add a fusion read alignment file to a fusion object.
#' If you've mapped the reads supporting a fusion against the fusion junction
#' sequence, and have the resulting bamfile, use this function to add the
#' information (as a Gviz::GAlignmentPairs object) to the fusion object.
#'
#' @param fusion The fusion object to add a genomic alignment to.
#' @param bamfile The bam file containing the fusion reads plotted to the fusion
#' sequence.
#'
#' @return An updated fusion object with fusion@fusion_reads_alignment set.
#'
#' @examples
#' # Load data
#' defuse833ke <- system.file(
#'   "extdata",
#'   "defuse_833ke_results.filtered.tsv",
#'   package="chimeraviz")
#' fusions <- import_defuse(defuse833ke, "hg19", 1)
#' # Find the specific fusion we have aligned reads for
#' fusion <- get_fusion_by_id(fusions, 5267)
#' # Get reference to the bamfile with the alignment data
#' bamfile5267 <- system.file(
#'   "extdata",
#'   "5267readsAligned.bam",
#'   package="chimeraviz")
#' # Add the bam file of aligned fusion reads to the fusion object
#' fusion <- add_fusion_reads_alignment(fusion, bamfile5267)
#'
#' @export
add_fusion_reads_alignment <- function(fusion, bamfile) {

  # Check if we got a fusion object
  if (class(fusion) != "Fusion") {
    stop("fusion argument must be an object of type Fusion")
  }

  fusion_reads_alignment <- Gviz::AlignmentsTrack(
    bamfile,
    isPaired = TRUE,
    # Setting chromosome to chrNA because this is a fusion sequence not found in
    # any reference genome.
    chromosome = "chrNA",
    name = "Fusion Reads",
    genome = fusion@genome_version)

  # Return new fusion object, now with the fusion read alignment
  fusion@fusion_reads_alignment <- fusion_reads_alignment
  fusion
}

#' Coerce Fusion object to data.frame
#'
#' This function is used in create_fusion_report() to convert Fusion objects to a
#' data.frame-format.
#'
#' @param fusion The Fusion object to coerce.
#'
#' @return A data.frame with the fusion object.
#'
#' @seealso create_fusion_report
#'
#' @examples
#' # Load data
#' defuse833ke <- system.file(
#'   "extdata",
#'   "defuse_833ke_results.filtered.tsv",
#'   package="chimeraviz")
#' fusions <- import_defuse(defuse833ke, "hg19", 1)
#' # Find the fusion object to create a data frame from
#' fusion <- get_fusion_by_id(fusions, 5267)
#' # Create the data frame
#' dfFusion <- fusion_to_data_frame(fusion)
#'
#' @export
fusion_to_data_frame <- function(fusion) {

  # Check if we got a list of fusion objects
  if (class(fusion) != "Fusion") {
    stop("fusions argument must be a list of Fusion objects")
  }

  df <- data.frame(
    fusion@id,
    fusion@gene_upstream@name,
    fusion@gene_upstream@ensembl_id,
    fusion@gene_upstream@breakpoint,
    fusion@gene_downstream@name,
    fusion@gene_downstream@ensembl_id,
    fusion@gene_downstream@breakpoint,
    fusion@split_reads_count,
    fusion@spanning_reads_count)
  names(df) <- c(
    "id",
    "gene_upstream",
    "ensembl_upstream",
    "breakpoint_upstrea,",
    "gene_downstream",
    "ensembl_downstream",
    "breakpoint_downstream",
    "Split Reads",
    "Spanning Reads")
  df
}

#' Select which transcript to use (for plotting) for a GenePartner object
#'
#' This function takes a GenePartner object and creates a transcript data.frame
#' with transcript information, including only the transcripts given by the
#' parameter which_transcripts
#'
#' select_transcript() selects which transcript to create by this prioritization:
#'
#' 1. Exon boundary transcripts.
#' 2. Within exon transcripts.
#' 3. Within intron transcripts.
#' 4. Intergenic transcripts.
#'
#' @param gene_partner The GenePartner object to select a transcript for.
#' @param which_transcripts This character vector decides which transcripts are
#' to be plotted. Can be "exonBoundary", "withinExon", "withinIntron",
#' "intergenic", or a character vector with specific transcript ids. Default
#' value is "exonBoundary".
#'
#' @return A data.frame with transcript data.
#'
#' @examples
#' # Load data and example fusion event
#' defuse833ke <- system.file(
#'   "extdata",
#'   "defuse_833ke_results.filtered.tsv",
#'   package="chimeraviz")
#' fusions <- import_defuse(defuse833ke, "hg19", 1)
#' fusion <- get_fusion_by_id(fusions, 5267)
#' # Load edb
#' edbSqliteFile <- system.file(
#'   "extdata",
#'   "Homo_sapiens.GRCh37.74.sqlite",
#'   package="chimeraviz")
#' edb <- ensembldb::EnsDb(edbSqliteFile)
#' # Get transcripts
#' fusion <- get_transcripts_ensembl_db(fusion, edb)
#' # Select transcript
#' transcriptsA <- select_transcript(upstream_partner_gene(fusion))
#'
#' @export
select_transcript <- function(
  gene_partner,
  which_transcripts = "exonBoundary") {

  # Check if we got a PartnerGene object
  if (class(gene_partner) != "PartnerGene") {
    stop("genePartner argument must be an object of type PartnerGene")
  }

  # Does the PartnerGene have any transcripts?
  if (isEmpty(gene_partner@transcripts)) {
    stop("genePartner has no transcripts. See get_transcripts_ensembl_db()")
  }

  # Either select from one of these four categories
  # 1. Exon boundary transcripts.
  # 2. Within exon transcripts.
  # 3. Within intron transcripts.
  # 4. Intergenic transcripts.
  #
  # or select the specific transcripts given by which_transcripts.

  transcript_categories <- c(
    "exonBoundary",
    "withinExon",
    "withinIntron",
    "intergenic")

  if (which_transcripts[[1]] %in% transcript_categories) {

    message(paste0("Selecting transcripts for ", gene_partner@name, ".."))

    # If the user has chosen one of the four transcript categories, then we
    # want to check whether or not such transcripts exist. If they exist,
    # simply return them. If they don't exist, go on to try the other
    # categories. Try the wanted category first:
    transcripts_of_wanted_category <-
      gene_partner@transcripts[
        mcols(gene_partner@transcripts)$
          transcript_category == which_transcripts[[1]]
      ]
    if (length(transcripts_of_wanted_category) > 0) {
      message(paste0("..found transcripts of type ", which_transcripts[[1]]))
      return(transcripts_of_wanted_category)
    }
    # Check the remaining categories
    remaining_categories <- transcript_categories[
      transcript_categories != which_transcripts[[1]]
    ]
    for (transcript_category in remaining_categories) {
      transcripts_in_this_category <- gene_partner@transcripts[
        mcols(gene_partner@transcripts)$
          transcript_category == transcript_category
      ]
      if (length(transcripts_in_this_category) > 0) {
        message(paste0("..found transcripts of type ", transcript_category))
        return(transcripts_in_this_category)
      }
    }
  }

  # At this point the user wants specific transcripts. Get the transcripts that
  # we have for this genePartner.

  specific_transcripts <- gene_partner@transcripts[
    names(gene_partner@transcripts) %in% which_transcripts]
  if (length(specific_transcripts) > 0) {
    return(specific_transcripts)
  }

  stop("The specific transcripts could not be found")
}

# Check that there's at least one transcript that has the fusion breakpoint
# within the transcript.
.check_that_breakpoints_are_within_transcripts <- function(
  fusion,
  transcripts_upstream,
  transcripts_downstream) {
  if (class(transcripts_upstream) == "CompressedGRangesList") {
    if (!any(start(transcripts_upstream)@unlistData <
             fusion@gene_upstream@breakpoint) &
        any(fusion@gene_upstream@breakpoint <
            end(transcripts_upstream)@unlistData)) {
      stop(paste0(
        "None of the transcripts given for gene A has the fusion breakpoint ",
        "within them. This plot cannot be created with the given transcripts."))
    }
  } else if (class(transcripts_upstream) == "GRanges") {
    if (!any(start(transcripts_upstream) <
             fusion@gene_upstream@breakpoint) &
        any(fusion@gene_upstream@breakpoint <
            end(transcripts_upstream))) {
      stop(paste0(
        "None of the transcripts given for gene A has the fusion breakpoint ",
        "within them. This plot cannot be created with the given transcripts."))
    }
  } else {
    stop(paste0(
      "Unknown input to .check_that_breakpoints_are_within_transcripts."))
  }
  if (class(transcripts_downstream) == "CompressedGRangesList") {
    if (!any(start(transcripts_downstream)@unlistData <
             fusion@gene_downstream@breakpoint) &
        any(fusion@gene_downstream@breakpoint <
            end(transcripts_downstream)@unlistData)) {
      stop(paste0(
        "None of the transcripts given for gene B has the fusion breakpoint ",
        "within them. This plot cannot be created with the given transcripts."))
    }
  } else if (class(transcripts_downstream) == "GRanges") {
    if (!any(start(transcripts_downstream) <
             fusion@gene_downstream@breakpoint) &
        any(fusion@gene_downstream@breakpoint <
            end(transcripts_downstream))) {
      stop(paste0(
        "None of the transcripts given for gene B has the fusion breakpoint ",
        "within them. This plot cannot be created with the given transcripts."))
    }
  } else {
    stop(paste0(
      "Unknown input to .check_that_breakpoints_are_within_transcripts."))
  }
}

# Check that the transcripts have a breakpoint exon
.check_that_transcripts_have_breakpoint_exons <- function(
  fusion,
  transcript_upstream,
  transcript_downstream) {

  if (fusion@gene_upstream@strand == "+") {
    if (
      length(
        transcript_upstream[
          end(ranges(transcript_upstream)) == fusion@gene_upstream@breakpoint
        ]
      ) == 0
    ) {
      stop(paste(
        "The transcript for gene A doesn't have an exon boundary matching the",
        "fusion breakpoint. The plot cannot be created."))
    }
  } else {
    if (
      length(
        transcript_upstream[
          start(ranges(transcript_upstream)) == fusion@gene_upstream@breakpoint
        ]
      ) == 0
    ) {
      stop(paste(
        "The transcript for gene A doesn't have an exon boundary matching the",
        "fusion breakpoint. The plot cannot be created."))
    }
  }
  if (fusion@gene_downstream@strand == "+") {
    if (
      length(
        transcript_downstream[
          start(ranges(transcript_downstream)) ==
          fusion@gene_downstream@breakpoint
        ]
      ) == 0
    ) {
      stop(paste(
        "The transcript for gene B doesn't have an exon boundary matching the",
        "fusion breakpoint. The plot cannot be created."))
    }
  } else {
    if (
      length(
        transcript_downstream[
          end(ranges(transcript_downstream)) ==
          fusion@gene_downstream@breakpoint
        ]
      ) == 0
    ) {
      stop(paste(
        "The transcript for gene B doesn't have an exon boundary matching the",
        "fusion breakpoint. The plot cannot be created."))
    }
  }
}

#' Remove introns and shift exons leftward
#'
#' This function takes a GRanges object and moves each IRanges object within
#' next to each other starting at 1. This effectively removes the introns from
#' the GRanges object.
#'
#' @param transcript The GRanges object to remove introns from.
#'
#' @return A GRanges object with introns removed.
#'
#' @examples
#' # Create a simple GRanges object:
#' gr <- IRanges::IRanges(
#'   start = c(13, 40, 100),
#'   end = c(20, 53, 110))
#' # Downshift it and see the introns are removed:
#' down_shift(gr)
#'
#' @export
down_shift <- function(transcript) {

  # Check if we got a GRanges object
  if (!class(transcript) %in% c("GRanges", "IRanges")) {
    stop("transcript argument must be an object of type GRanges")
  }

  for (i in seq_along(transcript)) {
    if (i == 1) {
      transcript[i] <- IRanges::shift(
        transcript[i],
        shift = 1 - start(transcript[i])
      )
    } else {
      shift_down_by <- - (start(transcript[i]) - end(transcript[i - 1])) + 1
      transcript[i] <- IRanges::shift(
        transcript[i],
        shift = shift_down_by
      )
    }
  }

  transcript
}

# -----------------------------------------------------------------------------
# Functions that validate parameters passed to functions in chimeraviz

.is_fusion_valid <- function(argument_checker, fusion) {
  # Check if we got a fusion object
  checkmate::assert_class(x = fusion,
                          classes = "Fusion",
                          add = argument_checker)
  argument_checker
}

.is_edb_valid <- function(argument_checker, edb, fusion) {
  if (!is.null(edb)){
    checkmate::assert_class(x = edb,
                            classes = "EnsDb",
                            add = argument_checker)
  } else {
    if (isEmpty(fusion@gene_upstream@transcripts)) {
      argument_checker$push(
        paste0("There are no transcipts for gene A. Please provide an ",
                     "EnsDb object to the edb parameter, or see",
               "get_transcripts_ensembl_db().")
      )
    } else if (isEmpty(fusion@gene_downstream@transcripts)) {
      argument_checker$push(
        paste0("There are no transcipts for gene B. Please provide an ",
               "EnsDb object to the edb parameter, or see",
               "get_transcripts_ensembl_db().")
      )
    }
  }
  argument_checker
}

.is_bamfile_valid_allow_null <- function(argument_checker, bamfile) {
  if (!is.null(bamfile)){
    checkmate::assert_file_exists(x = bamfile,
                                  add = argument_checker)
  }
  argument_checker
}

.is_bamfile_valid <- function(argument_checker, bamfile) {
  # Check that the argument is given
  if (is.null(bamfile)) {
    argument_checker$push(
      "'bamfile' must be the path to a .BAM file."
    )
  }
  # Check that the file exists
  if (!is.null(bamfile)) {
    checkmate::assert_file_exists(x = bamfile,
                                  add = argument_checker)
  }
  argument_checker
}

.is_bedfile_valid <- function(argument_checker, bedfile) {
  # Check that the argument is given
  if (is.null(bedfile)) {
    argument_checker$push("'bedfile' must be the path to a .BED file.")
  }
  # Check that the file exists
  if (!is.null(bedfile)) {
    checkmate::assert_file_exists(x = bedfile,
                                  add = argument_checker)
  }
  argument_checker
}

.is_bedgraphfile_valid <- function(argument_checker, bedgraphfile) {
  # Check that the argument is given
  if (is.null(bedgraphfile)) {
    argument_checker$push("'bedgraphfile' must be the path to a .bedGraph file.")
  }
  # Check that the file exists
  if (!is.null(bedgraphfile)) {
    checkmate::assert_file_exists(x = bedgraphfile,
                                  add = argument_checker)
  }
  argument_checker
}

.is_bamfile_bedgraphfile_valid <- function(
  argument_checker,
  bamfile,
  bedgraphfile) {
  # Either bamfile or bedgraphfile can be given, not both
  bamfile_given <- !is.null(bamfile)
  bedgraphfile_given <- !is.null(bedgraphfile)
  if (bamfile_given && bedgraphfile_given) {
    argument_checker$push(
      "Either 'bamfile' or 'bedgraphfile' must be given, not both."
    )
  } else if (!bamfile_given && !bedgraphfile_given) {
    # It is OK to not give any of them
  } else if (bamfile_given) {
    .is_bamfile_valid(argument_checker, bamfile)
  } else {
    .is_bedgraphfile_valid(argument_checker, bedgraphfile)
  }
  argument_checker
}

.is_which_transcripts_valid <- function(
  argument_checker,
  which_transcripts,
  fusion
) {
  if (class(which_transcripts) != "character") {
    argument_checker$push(
      paste0("'which_transcripts' must be a character (or a character ",
             "vector) holding the desired transcript category or the ",
             "names of specific transcripts.")
    )
  }
  # Is which_transcripts valid?
  transcript_categories <- c(
    "exonBoundary",
    "withinExon",
    "withinIntron",
    "intergenic"
  )
  # Check if the transcript(s) given actually exist, either in
  # transcript_categories, in geneA, or in geneB
  for (i in seq_along(which_transcripts)) {
    if (
      !which_transcripts[[i]] %in% transcript_categories &&
      !which_transcripts[[i]] %in% names(fusion@gene_upstream@transcripts) &&
      !which_transcripts[[i]] %in% names(fusion@gene_downstream@transcripts)
    ) {
      argument_checker$push(
        paste0("No transcript with name ", which_transcripts[[i]],
               " was found.")
      )
    }
  }
  argument_checker
}

.is_ylim_valid <- function(argument_checker, ylim) {
  if (class(ylim) != "numeric" || length(ylim) != 2) {
    argument_checker$push(
      "'ylim' must be a numeric vector of length 2"
    )
  }
  argument_checker
}

.is_parameter_boolean <- function(
  argument_checker,
  parameter,
  parameter_name
) {
  checkmate::assert_logical(x = parameter,
                            .var.name = parameter_name,
                            add = argument_checker)
  argument_checker
}

.is_character_parameter_valid <- function(
  argument_checker,
  parameter,
  parameter_name
) {
  checkmate::assert_character(x = parameter,
                              .var.name = parameter_name,
                              add = argument_checker)
  argument_checker
}

.is_nucleotide_amount_valid <- function(
  argument_checker,
  nucleotide_amount,
  fusion
) {

  fusion_junction_sequence_length <-
    length(fusion@gene_upstream@junction_sequence) +
    length(fusion@gene_downstream@junction_sequence)

  if (class(nucleotide_amount) != "numeric" ||
      nucleotide_amount <= 0 ||
      nucleotide_amount > fusion_junction_sequence_length) {
    argument_checker$push(
      paste0(
        "'nucleotide_amount' must be a numeric bigger than or equal ",
        "to 0 and less than or equal to the fusion junction ",
        "sequence length."
      )
    )
  }

  if (fusion_junction_sequence_length == 0) {
    argument_checker$push(
      msg = paste0(
        "length of junction sequence is 0, have they been determined?"
      )
    )
  }
  argument_checker
}

# End of functions that validate parameters passed to functions in chimeraviz
# -----------------------------------------------------------------------------

.get_transcripts_if_not_there <- function(fusion, edb) {
  # Establish a new 'checkmate' object
  argument_checker <- checkmate::makeAssertCollection()
  # Check parameters
  argument_checker <- .is_fusion_valid(argument_checker, fusion)
  argument_checker <- .is_edb_valid(argument_checker, edb, fusion)
  # Return errors and warnings (if any)
  checkmate::reportAssertions(argument_checker)

  if (
    isEmpty(fusion@gene_upstream@transcripts) ||
    isEmpty(fusion@gene_downstream@transcripts)
  ) {
    message("Fetching transcripts for gene partners..")
    fusion <- get_transcripts_ensembl_db(fusion, edb)
    message("..transcripts fetched.")
  }
  fusion
}

context("Test utility functions")

# Import defuse results
defuse833ke <- system.file(
  "extdata",
  "defuse_833ke_results.filtered.tsv",
  package = "chimeraviz")
fusions <- import_defuse(defuse833ke, "hg19", 1)

# Find the fusion we have get_reads.pl output for
fusion <- get_fusion_by_id(fusions, 5267)

# The read ids we're interested in
reads <- c(
  "13422259", "19375605", "29755061",
  "31632876", "32141428", "33857245")
# fastq files that has the supporting reads
fastq1 <- system.file("extdata", "reads.1.fq", package = "chimeraviz")
fastq2 <- system.file("extdata", "reads.2.fq", package = "chimeraviz")
# Temporary files to put extracted reads into
fastq_file_out_1 <- tempfile(pattern = "fq1", tmpdir = tempdir())
fastq_file_out_2 <- tempfile(pattern = "fq2", tmpdir = tempdir())

# edb object containing transcript data for the fusion transcripts from the
# deFuse example data with cluster_id=5267 and cluster_id=11759
edb_sqlite_file <- system.file(
  "extdata",
  "Homo_sapiens.GRCh37.74.sqlite",
  package = "chimeraviz")
edb <- ensembldb::EnsDb(edb_sqlite_file)

# Get all exons for all transcripts in the genes in the fusion transcript
all_transcripts <- ensembldb::exonsBy(
  edb,
  filter = list(
    AnnotationFilter::GeneIdFilter(
      list(
        fusion@gene_upstream@ensembl_id,
        fusion@gene_downstream@ensembl_id))),
  columns = c(
    "gene_id",
    "gene_name",
    "tx_id",
    "tx_cds_seq_start",
    "tx_cds_seq_end",
    "exon_id"))

test_that("fetch_reads_from_fastq works as it should", {

  skip_on_os(os = "windows")

  fetch_reads_from_fastq(
    reads,
    fastq1,
    fastq2,
    fastq_file_out_1,
    fastq_file_out_2)

  # fastq_file_out_1 and fastq_file_out_2 should now contain the reads
  # supporting fusion with cluster_id 5267. To test this, read the files.

  new_fastq_content_1 <- readLines(fastq_file_out_1)
  new_fastq_content_2 <- readLines(fastq_file_out_2)

  # We know for a fact that there are 6 reads supporting the fusion. Since each
  # read will have four lines in the file, check if the length matches
  expect_equal(length(new_fastq_content_1), 6 * 4)
  expect_equal(length(new_fastq_content_2), 6 * 4)
})

test_that("fetch_reads_from_fastq fails when it's supposed to", {

  skip_on_os(os = "windows")

  expect_error(
    fetch_reads_from_fastq(
      c(1, 2, 3),
      "fq1",
      "fq2",
      "fq1",
      "fq2")
    )
  expect_error(
    fetch_reads_from_fastq(
      reads,
      fastq1,
      "this-is-not-an-existing-file",
      fastq_file_out_1,
      fastq_file_out_2)
    )

  expect_warning(
    fetch_reads_from_fastq(
      c("123123123", "213987987321", "123768"),
      fastq1,
      fastq2,
      fastq_file_out_1,
      fastq_file_out_2)
    )
})

test_that("write_fusion_reference writes the fusion sequence to a fasta file", {
  # Create temporary file to hold the fusion sequence
  fasta_file_out <- tempfile(pattern = "fq1", tmpdir = tempdir())
  # Write fusion sequence to file
  write_fusion_reference(fusion, fasta_file_out)
  # Read the file
  fasta_file_content <- Biostrings::readDNAStringSet(fasta_file_out)

  # The fusion sequence wasn't written directly to a file, but put into a
  # DNAStringSet object first
  fusion_sequence <- Biostrings::DNAStringSet(
    x = c(fusion@gene_upstream@junction_sequence,
          fusion@gene_downstream@junction_sequence))
  names(fusion_sequence) <- "chrNA"

  # Check if equal
  expect_equal(fusion_sequence, fasta_file_content)
})

test_that("write_fusion_reference fails when given invalid input", {
  expect_error(write_fusion_reference("not-a-fusion-object", fasta_file_out))
})

test_that(
  paste0(
    "write_fusion_reference fails when there's no fusion junction sequence ",
    "in the Fusion object"
  ), {
  # Create a Fusion object with no fusion junction sequences
  fusion_no_junc_seq <- fusion
  fusion_no_junc_seq@gene_upstream@junction_sequence <-
    Biostrings::DNAString("")
  fusion_no_junc_seq@gene_downstream@junction_sequence <-
    Biostrings::DNAString("")
  # Create temporary file to hold the fusion sequence
  fasta_file_out <- tempfile(pattern = "fq1", tmpdir = tempdir())
  # Call the function
  expect_error(write_fusion_reference(fusion_no_junc_seq, fasta_file_out))
})

test_that("add_fusion_reads_alignment works as expected", {
  # We will test this by checking the value of
  # fusion@fusion_reads_alignment@genome before and after adding the bamfile.

  # Before:
  expect_equal(is.na(fusion@fusion_reads_alignment@genome), TRUE)

  # Find alignment data
  bamfile5267 <- system.file(
    "extdata",
    "5267readsAligned.bam",
    package = "chimeraviz")
  # Add the bam file of aligned fusion reads to the fusion object
  fusion <- add_fusion_reads_alignment(fusion, bamfile5267)

  # After:
  expect_equal(fusion@fusion_reads_alignment@genome, fusion@genome_version)
})

test_that("get_transcripts_ensembl_db adds transcript data to fusion object", {
  # Add transcripts data to fusion object
  fusion <- get_transcripts_ensembl_db(fusion, edb)

  # See that we got the right amount of transcripts
  expect_equal(length(fusion@gene_upstream@transcripts), 12)
  expect_equal(length(fusion@gene_downstream@transcripts), 6)
})
test_that("get_transcripts_ensembl_db sets the strand if missing for a gene", {
  # This test exists because the JAFFA results does not contain strand
  # information

  fusion@gene_upstream@strand <- "*"
  fusion@gene_downstream@strand <- "*"
  fusion <- get_transcripts_ensembl_db(fusion, edb)
  expect_equal(fusion@gene_upstream@strand, "+")
  expect_equal(fusion@gene_downstream@strand, "-")
})

test_that("decide_transcript_category returns the correct categories", {
  expect_equal(
    decide_transcript_category(all_transcripts[[1]], fusion),
    "exonBoundary"
  )
  expect_equal(
    decide_transcript_category(all_transcripts[[3]], fusion),
    "intergenic"
  )
})

test_that("split_on_utr_and_add_feature splits ranges as it should", {
  # Check length on the GRanges object before splitting
  expect_equal(length(all_transcripts[[1]]), 9)
  # And check it after
  expect_equal(length(split_on_utr_and_add_feature(all_transcripts[[1]])), 11)

  # Check length on the GRanges object before splitting
  expect_equal(length(all_transcripts[[2]]), 8)
  # And check it after
  expect_equal(length(split_on_utr_and_add_feature(all_transcripts[[2]])), 10)
})

test_that(
  "get_ensembl_ids gets the correct Ensembl gene ids for a fusion object", {
  # Store the correct Ensembl ids temporarily
  correct_ensembl_id_upstream <- fusion@gene_upstream@ensembl_id
  correct_ensembl_id_downstream <- fusion@gene_downstream@ensembl_id

  # Reset the fusion objects ensembl ids
  fusion@gene_upstream@ensembl_id <- ""
  fusion@gene_downstream@ensembl_id <- ""

  # Get the ensembl ids
  fusion <- get_ensembl_ids(fusion)

  # Check that they are the same as the one we had originally
  expect_equal(fusion@gene_upstream@ensembl_id, correct_ensembl_id_upstream)
  expect_equal(fusion@gene_downstream@ensembl_id, correct_ensembl_id_downstream)

})

test_that("get_ensembl_ids gives error when it can't find ensembl id", {
  # Store the correct gene names temporarily
  name_upstream <- fusion@gene_upstream@name
  name_downstream <- fusion@gene_downstream@name

  # Set the geneNames[1] to a non-existant gene name
  fusion@gene_upstream@name <- "non-existant"
  fusion@gene_downstream@name <- "non-existant"

  # Check that we get a warning
  expect_error(get_ensembl_ids(fusion))

  # Restore original gene names
  fusion@gene_upstream@name <- name_upstream
  fusion@gene_downstream@name <- name_downstream
})

test_that("fusion_to_data_frame works as expected", {
  data_frame_fusion <- fusion_to_data_frame(fusion)
  expect_is(data_frame_fusion, "data.frame")
  expect_equal(
    fusion@gene_upstream@name,
    as.character(data_frame_fusion$gene_upstream)
  )

  # Expect error when given something other than a Fusion object
  expect_error(fusion_to_data_frame(42))
})

test_that("down_shift works as expected", {
  ir <- IRanges::IRanges(
    start = c(13, 40, 100),
    end = c(20, 53, 110))
  gr <- GenomicRanges::GRanges(seqnames = "chr1", ranges = ir)
  # Downshift it and see the introns are removed:
  gr_downshifted <- down_shift(gr)
  expect_is(gr_downshifted, "GRanges")
  expect_equal(IRanges::width(gr), IRanges::width(gr_downshifted))
})

test_that(".is_fusion_valid works as expected", {
  # valid case
  argument_checker <- checkmate::makeAssertCollection()
  argument_checker <- .is_fusion_valid(argument_checker, fusion)
  checkmate::reportAssertions(argument_checker)
  # error case
  argument_checker <- checkmate::makeAssertCollection()
  argument_checker <- .is_fusion_valid(argument_checker, "fusion")
  expect_error(
    checkmate::reportAssertions(argument_checker)
  )
})

test_that(".is_edb_valid works as expected", {
  # valid cases

  argument_checker <- checkmate::makeAssertCollection()
  argument_checker <- .is_edb_valid(argument_checker, edb)
  checkmate::reportAssertions(argument_checker)

  fusion@gene_upstream@transcripts[[1]] <- GRanges(1, 2)
  fusion@gene_downstream@transcripts[[1]] <- GRanges(1, 2)
  argument_checker <- checkmate::makeAssertCollection()
  argument_checker <- .is_edb_valid(argument_checker, NULL, fusion)
  checkmate::reportAssertions(argument_checker)

  # error case
  argument_checker <- checkmate::makeAssertCollection()
  argument_checker <- .is_edb_valid(argument_checker, "edb")
  expect_error(
    checkmate::reportAssertions(argument_checker)
  )
})

test_that(".is_bamfile_valid works as expected", {
  # valid case
  argument_checker <- checkmate::makeAssertCollection()
  argument_checker <- .is_bamfile_valid(
    argument_checker,
    system.file(
      "extdata",
      "fusion5267and11759reads.bam",
      package = "chimeraviz"
    )
  )
  checkmate::reportAssertions(argument_checker)

  # error cases

  argument_checker <- checkmate::makeAssertCollection()
  argument_checker <- .is_bamfile_valid(argument_checker, "edb")
  expect_error(
    checkmate::reportAssertions(argument_checker)
  )

  argument_checker <- checkmate::makeAssertCollection()
  argument_checker <- .is_bamfile_valid(argument_checker, "")
  expect_error(
    checkmate::reportAssertions(argument_checker)
  )
})

test_that(".is_bedfile_valid works as expected", {
  # valid case
  argument_checker <- checkmate::makeAssertCollection()
  argument_checker <- .is_bedfile_valid(
    argument_checker,
    system.file("extdata", "protein_domains_5267.bed", package = "chimeraviz")
  )
  checkmate::reportAssertions(argument_checker)

  # error cases

  argument_checker <- checkmate::makeAssertCollection()
  argument_checker <- .is_bedfile_valid(argument_checker, "edb")
  expect_error(
    checkmate::reportAssertions(argument_checker)
  )

  argument_checker <- checkmate::makeAssertCollection()
  argument_checker <- .is_bedfile_valid(argument_checker, "")
  expect_error(
    checkmate::reportAssertions(argument_checker)
  )
})

test_that(".is_bedgraphfile_valid works as expected", {
  # valid case
  argument_checker <- checkmate::makeAssertCollection()
  argument_checker <- .is_bedgraphfile_valid(
    argument_checker,
    system.file(
      "extdata",
      "fusion5267and11759reads.bedGraph",
      package = "chimeraviz"
    )
  )
  checkmate::reportAssertions(argument_checker)

  # error cases

  argument_checker <- checkmate::makeAssertCollection()
  argument_checker <- .is_bedgraphfile_valid(argument_checker, "edb")
  expect_error(
    checkmate::reportAssertions(argument_checker)
  )

  argument_checker <- checkmate::makeAssertCollection()
  argument_checker <- .is_bedgraphfile_valid(argument_checker, "")
  expect_error(
    checkmate::reportAssertions(argument_checker)
  )
})

test_that(".is_bamfile_bedgraphfile_valid works as expected", {
  # valid cases

  argument_checker <- checkmate::makeAssertCollection()
  argument_checker <- .is_bamfile_bedgraphfile_valid(
    argument_checker,
    NULL,
    system.file(
      "extdata",
      "fusion5267and11759reads.bedGraph",
      package = "chimeraviz"
    )
  )
  checkmate::reportAssertions(argument_checker)

  argument_checker <- checkmate::makeAssertCollection()
  argument_checker <- .is_bamfile_bedgraphfile_valid(
    argument_checker,
    system.file(
      "extdata",
      "fusion5267and11759reads.bam",
      package = "chimeraviz"
    ),
    NULL
  )
  checkmate::reportAssertions(argument_checker)

  argument_checker <- checkmate::makeAssertCollection()
  argument_checker <- .is_bamfile_bedgraphfile_valid(
    argument_checker,
    NULL,
    NULL
  )
  checkmate::reportAssertions(argument_checker)

  # error cases

  argument_checker <- checkmate::makeAssertCollection()
  argument_checker <- .is_bamfile_bedgraphfile_valid(
    argument_checker,
    system.file(
      "extdata",
      "fusion5267and11759reads.bam",
      package = "chimeraviz"
    ),
    system.file(
      "extdata",
      "fusion5267and11759reads.bedGraph",
      package = "chimeraviz"
    )
  )
  expect_error(
    checkmate::reportAssertions(argument_checker)
  )
})

test_that(".is_which_transcripts_valid works as expected", {
  # error cases

  argument_checker <- checkmate::makeAssertCollection()
  argument_checker <- .is_which_transcripts_valid(
    argument_checker,
    42,
    fusion
  )
  expect_error(
    checkmate::reportAssertions(argument_checker)
  )

  argument_checker <- checkmate::makeAssertCollection()
  argument_checker <- .is_which_transcripts_valid(
    argument_checker,
    "yolo",
    fusion
  )
  expect_error(
    checkmate::reportAssertions(argument_checker)
  )
})

test_that(".is_ylim_valid works as expected", {
  # ylim invalid length
  argument_checker <- checkmate::makeAssertCollection()
  argument_checker <- .is_ylim_valid(argument_checker, c(1))
  expect_error(
    checkmate::reportAssertions(argument_checker)
  )

  # ylim valid length
  argument_checker <- checkmate::makeAssertCollection()
  argument_checker <- .is_ylim_valid(argument_checker, c(1, 2))
  checkmate::reportAssertions(argument_checker)
})

test_that(".is_parameter_boolean is works as expected", {
  # error case
  argument_checker <- checkmate::makeAssertCollection()
  argument_checker <- .is_parameter_boolean(
    argument_checker,
    42,
    "the answer"
  )
  expect_error(
    checkmate::reportAssertions(argument_checker)
  )
})

test_that(".is_character_parameter_valid is works as expected", {
  # error case
  argument_checker <- checkmate::makeAssertCollection()
  argument_checker <- .is_character_parameter_valid(
    argument_checker,
    42,
    "the answer"
  )
  expect_error(
    checkmate::reportAssertions(argument_checker)
  )
})

test_that(".is_nucleotide_amount_valid works as expected", {
  # error cases

  argument_checker <- checkmate::makeAssertCollection()
  argument_checker <- .is_nucleotide_amount_valid(
    argument_checker,
    -1,
    fusion
  )
  expect_error(
    checkmate::reportAssertions(argument_checker)
  )

  argument_checker <- checkmate::makeAssertCollection()
  argument_checker <- .is_nucleotide_amount_valid(
    argument_checker,
    length(fusion@gene_upstream@junction_sequence) +
      length(fusion@gene_downstream@junction_sequence) + 1,
    fusion
  )
  expect_error(
    checkmate::reportAssertions(argument_checker)
  )

  # Expect error when the fusion junction sequence is empty
  fusion@gene_upstream@junction_sequence <- Biostrings::DNAString()
  fusion@gene_downstream@junction_sequence <- Biostrings::DNAString()
  argument_checker <- checkmate::makeAssertCollection()
  argument_checker <- .is_nucleotide_amount_valid(
    argument_checker,
    1,
    fusion
  )
  expect_error(
    checkmate::reportAssertions(argument_checker)
  )
})

context("Test select_transcript")

# Load data and example fusion event
defuse833ke <- system.file(
  "extdata",
  "defuse_833ke_results.filtered.tsv",
  package = "chimeraviz")
fusions <- import_defuse(defuse833ke, "hg19", 1)
fusion <- get_fusion_by_id(fusions, 5267)
# Load edb
edb_sqlite_file <- system.file(
  "extdata",
  "Homo_sapiens.GRCh37.74.sqlite",
  package = "chimeraviz")
edb <- ensembldb::EnsDb(edb_sqlite_file)
# bamfile with reads in the regions of this fusion event
bamfile5267 <- system.file(
  "extdata",
  "fusion5267and11759reads.bam",
  package = "chimeraviz")
# Get transcripts
fusion <- get_transcripts_ensembl_db(fusion, edb)

testthat::test_that(
  paste0(
    "select_transcript returns expected results with which_transcripts=",
    "\"exonBoundary\""
  ), {
    transcripts <- select_transcript(fusion@gene_upstream, "exonBoundary")
    testthat::expect_equal(length(transcripts), 4)
  }
)

testthat::test_that(
  paste0(
    "select_transcript gives exonBoundary transcripts when we ask for ",
    "withinExon transcripts, because there are no witinExon transcripts"
  )
  , {
  testthat::expect_message(
    select_transcript(fusion@gene_upstream, "withinExon"),
    "..found transcripts of type exonBoundary"
  )
})

testthat::test_that(
  paste0(
    "select_transcript gives error with which_transcripts=\"4242\" when there",
    " are no such transcripts"
  ), {
  testthat::expect_error(select_transcript(fusion@gene_upstream, "4242"))
})

testthat::test_that(
  "select_transcript returns the expected specific transcripts", {
  transcripts <- select_transcript(
    fusion@gene_upstream,
    c("ENST00000373831", "ENST00000373832")
  )
  testthat::expect_equal(length(transcripts), 2)
})

context("Test plot_fusion()")

# Load data and example fusion event
defuse833ke <- system.file(
  "extdata",
  "defuse_833ke_results.filtered.tsv",
  package = "chimeraviz")
fusions <- import_defuse(defuse833ke, "hg19", 1)
fusion <- get_fusion_by_id(fusions, 5267)
# Load edb
edb_sqlite_file <- system.file(
  "extdata",
  "Homo_sapiens.GRCh37.74.sqlite",
  package = "chimeraviz")
edb <- ensembldb::EnsDb(edb_sqlite_file)
# bamfile with reads in the regions of this fusion event
bamfile5267 <- system.file(
    "extdata",
    "fusion5267and11759reads.bam",
    package = "chimeraviz")

test_that("plot_fusion produces a png file", {
  # Temporary file to store the plot
  png_filename <- tempfile(
    pattern = "fusionPlot",
    fileext = ".png",
    tmpdir = tempdir())
  # Open device
  png(png_filename, width = 1000, height = 750)
  # Plot and expect certain message output
  expect_message(
    plot_fusion(
      fusion = fusion,
      edb = edb,
      bamfile = bamfile5267),
    "As HENMT1 is on the minus strand, the plot for this gene will be reversed")
  # Close device
  dev.off()
  # See if we actually produced a file with size > 0
  expect_equal(file.info(png_filename)$size > 0, TRUE)
})

test_that("plot_fusion works when adding transcripts in advance", {
  # Get transcripts
  fusion <- get_transcripts_ensembl_db(fusion, edb)
  # Temporary file to store the plot
  png_filename <- tempfile(
    pattern = "fusionPlot",
    fileext = ".png",
    tmpdir = tempdir())
  # Open device
  png(png_filename, width = 1000, height = 750)
  # Plot and expect certain message output
  expect_message(
    plot_fusion(
      fusion = fusion,
      bamfile = bamfile5267),
    "As HENMT1 is on the minus strand, the plot for this gene will be reversed")
  # Close device
  dev.off()
  # See if we actually produced a file with size > 0
  expect_equal(file.info(png_filename)$size > 0, TRUE)
})

test_that("select_transcript picks the correct transcript type", {
  # Expect error when we haven't added transcript to the genepartner yet
  expect_error(select_transcript(fusion@gene_upstream))
  # Get transcripts
  fusion <- get_transcripts_ensembl_db(fusion, edb)

  transcripts_upstream <- select_transcript(fusion@gene_upstream)
  # This GRangesList object should have 4 transcripts
  expect_equal(length(transcripts_upstream), 4)

  transcripts_downstream <- select_transcript(fusion@gene_downstream)
  # This GRangesList object should have 4 transcripts
  expect_equal(length(transcripts_downstream), 4)
})

#' Create a Fusion Report
#'
#' This function will create a html report with an overplot and a sortable,
#' searchable table with the fusion data.
#'
#' @param fusions A list of Fusion objects.
#' @param output_filename Output html-file filename.
#' @param quiet Parameter passed to rmarkdown::render() to toggle its output.
#'
#' @return Creates a html report with an overplot and a sortable, searchable
#' table with the fusion data.
#'
#' @examples
#' # Load data
#' defuse833ke <- system.file(
#'   "extdata",
#'   "defuse_833ke_results.filtered.tsv",
#'   package="chimeraviz")
#' fusions <- import_defuse(defuse833ke, "hg19", 3)
#' # Temporary file to store the report
#' random_filename <- paste0(
#'   paste0(sample(LETTERS, 5, replace = TRUE), collapse=''),
#'   ".png"
#' )
#' # Create report
#' create_fusion_report(fusions, random_filename)
#' # Delete the file
#' file.remove(random_filename)
#'
#' @importFrom DT datatable
#' @importFrom plyr ldply
#'
#' @export
create_fusion_report <- function(
  fusions,
  output_filename,
  quiet = TRUE
) {

  # Check if we got a list of fusion objects
  if (class(fusions[[1]]) != "Fusion") {
    stop("fusions argument must be a list of Fusion objects")
  }

  # Full path to the output file
  output_path <- paste0(getwd(), "/", output_filename)

  # Get reference to the fusion list report file
  fusion_report_rmd <- system.file(
    "reports",
    "fusionListReport.Rmd",
    package = "chimeraviz")

  # Render the report
  rmarkdown::render(
    fusion_report_rmd,
    output_file = output_path,
    params = list(fusions = fusions),
    quiet = quiet)
}

#' Scale a vector of numeric values to an interval.
#'
#' This function takes a vector of numeric values as well as an interval
#' [new_min, new_max] that the numeric values will be scaled (normalized) to.
#'
#' @param the_list A vector of numeric values.
#' @param new_min Minimum value for the new interval.
#' @param new_max Maximum value for the new interval.
#'
#' @return A data frame with fusion link data compatible with RCircos::RCircos.Link.Plot()
#'
#' # @examples # Apparently examples shouldn't be set on private functions
#' list012 <- c(0,1,2)
#' .scale_list_to_interval(list012, 1, 3)
#' # [1] 1 2 3
#'
#' @name chimeraviz-internals-scaleListToInterval
.scale_list_to_interval <- function(the_list, new_min, new_max) {
if (length(the_list) <= 1) {
    stop(paste("Invalid list. Using this function with less than two values",
                "makes no sense."))
  }
  (new_max - new_min) * (the_list - min(the_list)) /
    (max(the_list) - min(the_list)) + new_min
}

#' Create link data for RCircos from the given fusions.
#'
#' This function takes a list of Fusion objects and creates a data frame in the
#' format that RCircos::RCircos.Link.Plot() expects for link data.
#'
#' @param fusion_list A list of Fusion objects.
#' @param min_link_width The minimum link line width. Default = 1
#' @param max_link_widt The maximum link line width. Default = 10
#'
#' @return A data frame with fusion link data compatible with RCircos::RCircos.Link.Plot()
#'
#' # @examples # Apparently examples shouldn't be set on private functions
#' defuse833ke <- system.file("extdata", "defuse_833ke_results.filtered.tsv", package="chimeraviz")
#' fusions <- import_defuse(defuse833ke, "hg19", 3)
#' linkData <- chimeraviz::.fusions_to_link_data(fusions)
#' # This linkData can be used with RCircos::RCircos.Link.Plot()
#'
#' @name chimeraviz-internals-fusions_to_link_data
.fusions_to_link_data <- function(
  fusion_list,
  min_link_width = 1,
  max_link_widt = 10
) {
  chromosome <- vector(mode = "character", length = length(fusion_list))
  chrom_start <- vector(mode = "numeric", length = length(fusion_list))
  chrom_end <- vector(mode = "numeric", length = length(fusion_list))

  chromosome_1 <- vector(mode = "character", length = length(fusion_list))
  chrom_start_1 <- vector(mode = "numeric", length = length(fusion_list))
  chrom_end_1 <- vector(mode = "numeric", length = length(fusion_list))

  link_width <- vector(mode = "numeric", length = length(fusion_list))

  for (i in seq_along(fusion_list)) {
    fusion <- fusion_list[[i]]

    chromosome[[i]] <- fusion@gene_upstream@chromosome
    chrom_start[[i]] <- fusion@gene_upstream@breakpoint
    # This value shouldn't matter:
    chrom_end[[i]] <- fusion@gene_upstream@breakpoint + 1

    chromosome_1[[i]] <- fusion@gene_downstream@chromosome
    chrom_start_1[[i]] <- fusion@gene_downstream@breakpoint
    # This value shouldn't matter:
    chrom_end_1[[i]] <- fusion@gene_downstream@breakpoint + 1

    # Set link width = number of spanning+split reads supporting fusion
    link_width[[i]] <- fusion@spanning_reads_count + fusion@split_reads_count
  }

  # Normalize all link width values to the interval
  # [min_link_width, max_link_width]
  if (length(link_width) > 1) {
    link_width <-
      .scale_list_to_interval(link_width, min_link_width, max_link_widt)
  } else {
    link_width[[1]] <- max_link_widt
  }

  data.frame(chromosome,
             chrom_start,
             chrom_end,
             chromosome_1,
             chrom_start_1,
             chrom_end_1,
             link_width)
}

#' Create gene label data for RCircos from the given fusions.
#'
#' This function takes a list of Fusion objects and creates a data frame in the
#' format that RCircos.Gene.Name.Plot() expects for gene label data.
#'
#' @param fusion_list A list of Fusion objects.
#'
#' @return A data frame with fusion gene label data compatible with RCircos.Gene.Name.Plot()
#'
#' # @examples # Apparently examples shouldn't be set on private functions
#' defuse833ke <- system.file("extdata", "defuse_833ke_results.filtered.tsv", package="chimeraviz")
#' fusions <- import_defuse(defuse833ke, "hg19", 3)
#' labelData <- chimeraviz::.fusions_to_gene_label_data(fusions)
#' # This labelData can be used with RCircos.Gene.Connector.Plot() and RCircos.Gene.Name.Plot()
#'
#' @name chimeraviz-internals-fusions_to_gene_label_data
.fusions_to_gene_label_data <- function(fusion_list) {
  original_length <- length(fusion_list)
  new_length <- original_length * 2

  chromosome <- vector(mode = "character", length = new_length)
  chrom_start <- vector(mode = "numeric", length = new_length)
  chrom_end <- vector(mode = "numeric", length = new_length)
  gene <- vector(mode = "character", length = new_length)

  for (i in seq_along(fusion_list)) {
    fusion <- fusion_list[[i]]

    # We are building the list of gene names for both partner genes at once, so
    # let's put the gene1 genes in the first half of the list..
    chromosome[[i]] <- fusion@gene_upstream@chromosome
    chrom_start[[i]] <- fusion@gene_upstream@breakpoint
    # This value shouldn't matter:
    chrom_end[[i]] <- fusion@gene_upstream@breakpoint + 1
    gene[[i]] <- fusion@gene_upstream@name

    # and put the gene2 genes in the other half.
    chromosome[[i + original_length]] <- fusion@gene_downstream@chromosome
    chrom_start[[i + original_length]] <- fusion@gene_downstream@breakpoint
    # This value shouldn't matter:
    chrom_end[[i + original_length]] <- fusion@gene_downstream@breakpoint + 1
    gene[[i + original_length]] <- fusion@gene_downstream@name
  }

  data.frame(chromosome,
             chrom_start,
             chrom_end,
             gene)
}

#' Create a circle plot of the given fusions.
#'
#' This function takes a list of Fusion objects and creates a circle plot
#' indicating which chromosomes the fusion genes in the list consists of.
#'
#' Note that only a limited number of gene names can be shown in the circle plot
#' due to the limited resolution of the plot. RCircos will automatically limit
#' the number of gene names shown if there are too many.
#'
#' @param fusion_list A list of Fusion objects.
#'
#' @return Creates a circle plot.
#'
#' @examples
#' defuse833ke <- system.file(
#'   "extdata",
#'   "defuse_833ke_results.filtered.tsv",
#'   package="chimeraviz")
#' fusions <- import_defuse(defuse833ke, "hg19", 3)
#' # Temporary file to store the plot
#' pngFilename <- tempfile(
#'   pattern = "circlePlot",
#'   fileext = ".png",
#'   tmpdir = tempdir())
#' # Open device
#' png(pngFilename, width = 1000, height = 750)
#' # Plot!
#' plot_circle(fusions)
#' # Close device
#' dev.off()
#'
#' @export
plot_circle <- function(fusion_list) {

  .validate_plot_circle_params(fusion_list)

  # Read cytoband information depending on genome version
  if (fusion_list[[1]]@genome_version == "hg19") {
    cytoband_file <- system.file(
      "extdata",
      "UCSC.HG19.Human.CytoBandIdeogram.txt",
      package = "chimeraviz")
  } else if (fusion_list[[1]]@genome_version == "hg38") {
    cytoband_file <- system.file(
      "extdata",
      "UCSC.HG38.Human.CytoBandIdeogram.txt",
      package = "chimeraviz")
  } else if (fusion_list[[1]]@genome_version == "mm10") {
    cytoband_file <- system.file(
      "extdata",
      "UCSC.MM10.Mus.musculus.CytoBandIdeogram.txt",
      package = "chimeraviz"
    )
  } else {
    stop("Invalid genome version.")
  }
  cytoband <- utils::read.table(cytoband_file)
  # Set names to what RCircos expects
  names(cytoband) <- c("Chromosome", "ChromStart", "ChromEnd", "Band", "Stain")

  # We need the RCircos.Env object in the global namespace. Why? Because RCircos
  # is weird that way.
  assign("RCircos.Env", RCircos::RCircos.Env, .GlobalEnv)

  # Sort the cytoband data so that the chromosomes appear in order
  cytoband <- RCircos.Sort.Genomic.Data( # Exclude Linting
    genomic.data = cytoband,
    is.ideo = TRUE
  )

  # Initialize components
  cyto_info <- cytoband
  chr_exclude <- NULL
  tracks_inside <- 3
  tracks_outside <- 0
  RCircos::RCircos.Set.Core.Components(
    cyto_info,
    chr_exclude,
    tracks_inside,
    tracks_outside
  )

  # Open a new window for plotting
  RCircos::RCircos.Set.Plot.Area()

  # Draw chromosome ideogram
  RCircos::RCircos.Chromosome.Ideogram.Plot()

  # Create gene label data in the format RCircos requires
  gene_label_data <- .fusions_to_gene_label_data(fusion_list)

  # Draw gene names
  name_col <- 4;
  side <- "in";
  track_num <- 1;
  # Plot connectors
  RCircos.Gene.Connector.Plot( # Exclude Linting
    gene_label_data,
    track_num,
    side
  );
  track_num <- 2;
  # Plot gene names
  RCircos.Gene.Name.Plot( # Exclude Linting
    gene_label_data,
    name_col,
    track_num,
    side
  );

  # Create link data in the format RCircos requires
  link_data <- .fusions_to_link_data(fusion_list)
  # Make sure the ordering is correct.
  # Ref https://github.com/stianlagstad/chimeraviz/issues/52
  multi_mixedorder <- function(..., na.last = TRUE, decreasing = FALSE) {
    do.call(
      order,
      c(
        lapply(
          list(...),
          function(l) {
            if (is.character(l)) {
              factor(
                l,
                levels = gtools::mixedsort(unique(l))
              )
            } else {
              l
            }
          }
        ),
        list(na.last = na.last, decreasing = decreasing)
      )
    )
  }
  ordered_link_width <- link_data[
    multi_mixedorder(
      as.character(link_data$chromosome),
      link_data$chrom_start
    ),
  ]$link_width

  # Draw link data
  # Which track?
  track_num <- 3
  # Plot links
  RCircos::RCircos.Link.Plot(
    link.data = link_data,
    track.num = track_num,
    by.chromosome = TRUE,
    start.pos = NULL,
    genomic.columns = 3,
    is.sorted = FALSE,
    lineWidth = ordered_link_width)

  # When done, remove the RCircos.Env element from the global namespace
  remove("RCircos.Env", envir = .GlobalEnv)
}

.validate_plot_circle_params <- function(
  fusion_list
) {
  # Establish a new 'checkmate' object
  coll <- checkmate::makeAssertCollection()

  # Check input type and length
  # Cehck that all the items in the input list are fusion objects
  checkmate::assert_list(x = fusion_list,
                         min.len = 1,
                         types = "Fusion")

  # Validate that the first fusion object has a valid genome
  genome_of_first_fusion <- fusion_list[[1]]@genome_version
  if (!genome_of_first_fusion %in% list("hg19", "hg38", "mm10")) {
    coll$push(
      paste0(
        "Invalid input. genomeVersion must be either \"hg19\", \"hg38\" or ",
        "\"mm10\"."
      )
    )
  }

  # Validate that all the fusions have the same genome
  genome_version <- sapply(seq_along(fusion_list),
                           function(fl) fusion_list[[fl]]@genome_version)
  if (any(genome_version != genome_of_first_fusion)) {
    coll$push(
      paste0(
        "All Fusion objects in the fusion_list must have the same genome."
      )
    )
  }

  # Return errors and warnings (if any)
  checkmate::reportAssertions(coll)
}

context("Test scaling a list of integers to an interval [a, b]")

list123 <- c(1, 2, 3)
list_123_scaled_to_3 <- .scale_list_to_interval(
  list123,
  min(list123),
  max(list123)
)
list_123_scaled_to_10 <- .scale_list_to_interval(list123, 1, 10)

test_that(".scale_list_to_interval works", {
  expect_equal(list_123_scaled_to_3, list123)
  expect_equal(min(list_123_scaled_to_10), 1)
  expect_equal(max(list_123_scaled_to_10), 10)

  expect_error(.scale_list_to_interval(c(1), 1, 10))
})

#' Import results from a EricScript run into a list of Fusion objects.
#'
#' A function that imports the results from a EricScript run into a list of
#' Fusion objects.
#'
#' @param filename Filename for the EricScript results file.
#' @param genome_version Which genome was used in mapping (hg19, hg38, etc.).
#' @param limit A limit on how many lines to read.
#'
#' @return A list of Fusion objects.
#'
#' @examples
#' ericscriptData <- system.file(
#'   "extdata",
#'   "ericscript_SRR1657556.results.total.tsv",
#'   package = "chimeraviz")
#' fusions <- import_ericscript(ericscriptData, "hg19", 3)
#' # This should import a list of 3 fusions described in Fusion objects.
#'
#' @importFrom data.table fread
#'
#' @export
import_ericscript <- function(filename, genome_version, limit) {

  # Is the genome version valid?
  valid_genomes <- c("hg19", "hg38", "mm10")
  if (is.na(match(tolower(genome_version), tolower(valid_genomes)))) {
    stop("Invalid genome version given")
  }

  # If the limit is set, is the value valid?
  if (missing(limit) == FALSE) {
    if (is.numeric(limit) == FALSE || limit <= 0) {
      stop("limit must be a numeric value bigger than 0")
    }
  }

  # Try to read the fusion report
  report <- withCallingHandlers({
      col_types <- c(
        "GeneName1" = "character",
        "GeneName2" = "character",
        "chr1" = "character",
        "Breakpoint1" = "character",
        "strand1" = "character",
        "chr2" = "character",
        "Breakpoint2" = "character",
        "strand2" = "character",
        "EnsemblGene1" = "character",
        "EnsemblGene2" = "character",
        "crossingreads" = "integer",
        "spanningreads" = "integer",
        "JunctionSequence" = "character",
        "EricScore" = "numeric"
      )
      if (missing(limit)) {
        # Read all lines
        data.table::fread(
          input = filename,
          colClasses = col_types,
          showProgress = FALSE
        )
      } else {
        # Only read up to the limit
        data.table::fread(
          input = filename,
          colClasses = col_types,
          showProgress = FALSE,
          nrows = limit
        )
      }
    },
    error = function(cond) {
      message(paste0("Reading ", filename, " caused an error: ", cond[[1]]))
      stop(cond)
    },
    warning = function(cond) {
      # Begin Exclude Linting
      message(paste0("Reading ", filename, " caused a warning: ", cond[[1]]))
      # End Exclude Linting
    }
  )

  # Set variables
  id                   <- NA
  inframe              <- NA
  fusion_tool          <- "ericscript"
  spanning_reads_count <- NA
  split_reads_count    <- NA

  # List to hold all Fusion objects
  fusion_list <- vector("list", dim(report)[1])

  # Iterate through each line in the .tsv file
  for (i in 1:dim(report)[1]) {

    # Import ericscript-specific fields
    fusion_tool_specific_data <- list()
    fusion_tool_specific_data[["EricScore"]] <- report[[i, "EricScore"]]

    # Set id for this fusion. Soapfuse does not have any unique identifier
    # for each row of the results file, so just use our counter.
    id <- as.character(i)

    # Chromosome names
    chromosome_upstream <- paste("chr", report[[i, "chr1"]], sep = "")
    chromosome_downstream <- paste("chr", report[[i, "chr2"]], sep = "")

    # Breakpoints
    # EricScript doesn't always report fusion breakpoints. If missing, then set
    # breakpoint to -1
    breakpoint_upstream <- tryCatch(
      as.numeric(report[[i, "Breakpoint1"]]),
      warning = function(w) {
        -1
      }
    )
    breakpoint_downstream <- tryCatch(
      as.numeric(report[[i, "Breakpoint2"]]),
      warning = function(w) {
        -1
      }
    )

    # Strand
    strand_upstream <- report[[i, "strand1"]]
    strand_downstream <- report[[i, "strand2"]]

    # Number of supporting reads
    split_reads_count <- report[[i, "spanningreads"]]
    spanning_reads_count <- report[[i, "crossingreads"]]

    # Get the fusion sequence
    # EricScript outputs this as sequencebeforejunctionSEQUENCEAFTERJUNCTION, so
    # first locate the first upper cased letter:
    split_at <-
      regexpr(pattern = "([[:upper:]])", report[[i, "JunctionSequence"]])[1]
    # Then we can extract each piece:
    junction_sequence_upstream <- Biostrings::DNAString(
      substr(
        report[[i, "JunctionSequence"]],
        1,
        split_at - 1)
    )
    junction_sequence_downstream <- Biostrings::DNAString(
      substr(
        report[[i, "JunctionSequence"]],
        split_at,
        nchar(report[[i, "JunctionSequence"]]))
    )

    # Gene names
    name_upstream <- report[[i, "GeneName1"]]
    name_downstream <- report[[i, "GeneName2"]]

    # Ensembl ids
    ensembl_id_upstream <- report[[i, "EnsemblGene1"]]
    ensembl_id_downstream <- report[[i, "EnsemblGene2"]]

    # PartnerGene objects
    gene_upstream <- new(
      Class = "PartnerGene",
      name = name_upstream,
      ensembl_id = ensembl_id_upstream,
      chromosome = chromosome_upstream,
      breakpoint = breakpoint_upstream,
      strand = strand_upstream,
      junction_sequence = junction_sequence_upstream,
      transcripts = GenomicRanges::GRangesList()
    )
    gene_downstream <- new(
      Class = "PartnerGene",
      name = name_downstream,
      ensembl_id = ensembl_id_downstream,
      chromosome = chromosome_downstream,
      breakpoint = breakpoint_downstream,
      strand = strand_downstream,
      junction_sequence = junction_sequence_downstream,
      transcripts = GenomicRanges::GRangesList()
    )

    fusion_list[[i]] <- new(
      Class = "Fusion",
      id = id,
      fusion_tool = fusion_tool,
      genome_version = genome_version,
      spanning_reads_count = spanning_reads_count,
      split_reads_count = split_reads_count,
      fusion_reads_alignment = Gviz::AlignmentsTrack(),
      gene_upstream = gene_upstream,
      gene_downstream = gene_downstream,
      inframe = inframe,
      fusion_tool_specific_data = fusion_tool_specific_data
    )
  }

  # Return the list of Fusion objects
  fusion_list
}

context("Test importing EricScript data into a list of Fusion objects")

ericscript_data <- system.file(
  "extdata",
  "ericscript_SRR1657556.results.total.tsv",
  package = "chimeraviz")
fusions <- import_ericscript(ericscript_data, "hg19", 3)
fusion <- fusions[[1]]

test_that("import_ericscript imports data into Fusion objects", {
  expect_equal(length(fusions), 3)
  expect_equal(fusion@id, "1")
  expect_equal(fusion@gene_upstream@name, "PPIA")
  expect_equal(fusion@gene_upstream@chromosome, "chr7")
})

test_that("import_ericscript fails to import data", {
  expect_error(import_ericscript("Non-existent-file", "hg18"))
  expect_error(import_ericscript(ericscript_data, "Invalid-genome"))
  expect_error(import_ericscript(ericscript_data, "hg19", "invalid limit"))
})

test_that("import_ericscript limits work", {
  expect_equal(length(import_ericscript(ericscript_data, "hg19", 2)), 2)
})

#' Plot a fusion event with transcripts, coverage and ideograms.
#'
#' This function creates a plot with information about transcripts, coverage,
#' location and more.
#'
#' plot_fusion() will dispatch to either plot_fusion_separate() or
#' plot_fusion_together(). plot_fusion_separate() will plot the fusion gene
#' partners in separate graphs shown next to each other, while
#' plot_fusion_together() will plot the fusion gene partners in the same graph
#' with the same x-axis. plot_fusion() will dispatch to plot_fusion_together() if
#' the fusion gene partners are on the same strand, same chromosome and are
#' close together (<=50,000 bp apart).
#'
#' @param fusion The Fusion object to plot.
#' @param edb The ensembldb object that will be used to fetch data.
#' @param bamfile The bamfile with RNA-seq data.
#' @param which_transcripts This character vector decides which transcripts are
#' to be plotted. Can be "exonBoundary", "withinExon", "withinIntron",
#' "intergenic", or a character vector with specific transcript ids. Default
#' value is "exonBoundary".
#' @param ylim Limits for the coverage y-axis.
#' @param non_ucsc Boolean indicating whether or not the bamfile used has UCSC-
#' styled chromosome names (i.e. with the "chr" prefix). Setting this to true
#' lets you use a bamfile with chromosome names like "1" and "X", instead of
#' "chr1" and "chrX".
#' @param reduce_transcripts Boolean indicating whether or not to reduce all
#' transcripts into a single transcript for each partner gene.
#' @param bedgraphfile A bedGraph file to use instead of the bamfile to plot
#' coverage.
#'
#' @return Creates a fusion plot.
#'
#' @examples
#' # Load data and example fusion event
#' defuse833ke <- system.file(
#'   "extdata",
#'   "defuse_833ke_results.filtered.tsv",
#'   package="chimeraviz")
#' fusions <- import_defuse(defuse833ke, "hg19", 1)
#' fusion <- get_fusion_by_id(fusions, 5267)
#' # Load edb
#' edbSqliteFile <- system.file(
#'   "extdata",
#'   "Homo_sapiens.GRCh37.74.sqlite",
#'   package="chimeraviz")
#' edb <- ensembldb::EnsDb(edbSqliteFile)
#' # bamfile with reads in the regions of this fusion event
#' bamfile5267 <- system.file(
#'   "extdata",
#'   "fusion5267and11759reads.bam",
#'   package="chimeraviz")
#' # Temporary file to store the plot
#' pngFilename <- tempfile(
#'   pattern = "fusionPlot",
#'   fileext = ".png",
#'   tmpdir = tempdir())
#' # Open device
#' png(pngFilename, width = 1000, height = 750)
#' # Plot!
#' plot_fusion(
#'   fusion = fusion,
#'   bamfile = bamfile5267,
#'   edb = edb,
#'   non_ucsc = TRUE)
#' # Close device
#' dev.off()
#'
#' # Example using a .bedGraph file instead of a .bam file:
#' # Load data and example fusion event
#' defuse833ke <- system.file(
#'   "extdata",
#'   "defuse_833ke_results.filtered.tsv",
#'   package="chimeraviz")
#' fusions <- import_defuse(defuse833ke, "hg19", 1)
#' fusion <- get_fusion_by_id(fusions, 5267)
#' # Load edb
#' edbSqliteFile <- system.file(
#'   "extdata",
#'   "Homo_sapiens.GRCh37.74.sqlite",
#'   package="chimeraviz")
#' edb <- ensembldb::EnsDb(edbSqliteFile)
#' # bedgraphfile with coverage data from the regions of this fusion event
#' bedgraphfile <- system.file(
#'   "extdata",
#'   "fusion5267and11759reads.bedGraph",
#'   package="chimeraviz")
#' # Temporary file to store the plot
#' pngFilename <- tempfile(
#'   pattern = "fusionPlot",
#'   fileext = ".png",
#'   tmpdir = tempdir())
#' # Open device
#' png(pngFilename, width = 1000, height = 750)
#' # Plot!
#' plot_fusion(
#'   fusion = fusion,
#'   bedgraphfile = bedgraphfile,
#'   edb = edb,
#'   non_ucsc = TRUE)
#' # Close device
#' dev.off()
#'
#' @export
plot_fusion <- function(
  fusion,
  edb = NULL,
  bamfile = NULL,
  which_transcripts = "exonBoundary",
  ylim = c(0, 1000),
  non_ucsc = TRUE,
  reduce_transcripts = FALSE,
  bedgraphfile = NULL) {

  .validate_plot_fusion_params(
    fusion,
    edb,
    bamfile,
    which_transcripts,
    ylim,
    non_ucsc,
    reduce_transcripts,
    bedgraphfile
  )
  fusion <- .get_transcripts_if_not_there(fusion, edb)

  # Decide whether or not to plot the fusion on the same x-axis
  if (fusion@gene_upstream@chromosome == fusion@gene_downstream@chromosome) {
    message("Fusion is intrachromosomal..")
    if (
      as.character(fusion@gene_upstream@strand) !=
      as.character(fusion@gene_downstream@strand)
    ) {
      message("..but on different strands. Plot separate!")
      plot_fusion_separate(
        fusion = fusion,
        edb = edb,
        bamfile = bamfile,
        which_transcripts = which_transcripts,
        ylim = ylim,
        non_ucsc = non_ucsc,
        reduce_transcripts = reduce_transcripts,
        bedgraphfile = bedgraphfile)
    } else if (
      abs(
        fusion@gene_upstream@breakpoint - fusion@gene_downstream@breakpoint
      ) > 50000
    ) {
      message("..but too far apart. Plot separate!")
      plot_fusion_separate(
        fusion = fusion,
        edb = edb,
        bamfile = bamfile,
        which_transcripts = which_transcripts,
        ylim = ylim,
        non_ucsc = non_ucsc,
        reduce_transcripts = reduce_transcripts,
        bedgraphfile = bedgraphfile)
    } else {
      message("..and close together..")
      # If both genes are on the minus strand, and the location of geneA is to
      # the right of geneB, then we still want to plot them separatly, because
      # we want geneA (the downstream fusion gene partner) to be on the left
      # side of the plot.
      if (fusion@gene_upstream@strand == "-" &&
          fusion@gene_downstream@strand == "-" &&
          min(start(unlist(fusion@gene_upstream@transcripts))) <
          min(start(unlist(fusion@gene_downstream@transcripts)))) {
        message(
          paste0(
            "..but since both genes are on the minus strand the x-axis will ",
            "be flipped in the plot. And since geneA will then be located to ",
            "\" the right of\" geneB, we will still plot this separatly. Plot",
            " separate!"
          )
        )
        plot_fusion_separate(
          fusion = fusion,
          edb = edb,
          bamfile = bamfile,
          which_transcripts = which_transcripts,
          ylim = ylim,
          non_ucsc = non_ucsc,
          reduce_transcripts = reduce_transcripts,
          bedgraphfile = bedgraphfile)
      } else {
        message("Plot together!")
        plot_fusion_together(
          fusion = fusion,
          edb = edb,
          bamfile = bamfile,
          which_transcripts = which_transcripts,
          ylim = ylim,
          non_ucsc = non_ucsc,
          reduce_transcripts = reduce_transcripts,
          bedgraphfile = bedgraphfile)
      }
    }
  } else {
    message("Fusion is interchromosomal. Plot separate!")
    plot_fusion_separate(
      fusion = fusion,
      edb = edb,
      bamfile = bamfile,
      which_transcripts = which_transcripts,
      ylim = ylim,
      non_ucsc = non_ucsc,
      reduce_transcripts = reduce_transcripts,
      bedgraphfile = bedgraphfile)
  }
}

#' @name plot_fusion
#'
#' @export
plot_fusion_separate <- function(
  fusion,
  edb,
  bamfile = NULL,
  which_transcripts = "exonBoundary",
  ylim = c(0, 1000),
  non_ucsc = TRUE,
  reduce_transcripts = FALSE,
  bedgraphfile = NULL) {

  .validate_plot_fusion_params(
    fusion,
    edb,
    bamfile,
    which_transcripts,
    ylim,
    non_ucsc,
    reduce_transcripts,
    bedgraphfile
  )
  fusion <- .get_transcripts_if_not_there(fusion, edb)

  # Create tracks

  # Select transcripts
  transcripts_upstream <-
    select_transcript(fusion@gene_upstream, which_transcripts)
  transcripts_downstream <-
    select_transcript(fusion@gene_downstream, which_transcripts)

  # Check that there's at least one transcript that has the fusion breakpoint
  # within the transcript.
  .check_that_breakpoints_are_within_transcripts(
    fusion,
    transcripts_upstream,
    transcripts_downstream
  )

  # Set seqlevels to UCSC
  GenomeInfoDb::seqlevelsStyle(transcripts_upstream) <- "UCSC"
  GenomeInfoDb::seqlevelsStyle(transcripts_downstream) <- "UCSC"
  if (reduce_transcripts) {
    reduced_transcripts_upstream <- GenomicRanges::reduce(
      unlist(transcripts_upstream)
    )
    reduced_transcripts_downstream <- GenomicRanges::reduce(
      unlist(transcripts_downstream)
    )
    # Add transcript metadata column to make Gviz group correctly
    mcols(reduced_transcripts_upstream)$transcript <-
      fusion@gene_upstream@name
    mcols(reduced_transcripts_downstream)$transcript <-
      fusion@gene_downstream@name
    # Create the tracks
    tr_upstream_track <- Gviz::GeneRegionTrack(
      data.frame(reduced_transcripts_upstream),
      chromosome = fusion@gene_upstream@chromosome)
    tr_downstream_track <- Gviz::GeneRegionTrack(
      data.frame(reduced_transcripts_downstream),
      chromosome = fusion@gene_downstream@chromosome)
    # Set symbol names on the tracks
    symbol(tr_upstream_track) <- fusion@gene_upstream@name
    symbol(tr_downstream_track) <- fusion@gene_downstream@name
  } else {
    # Create the tracks
    tr_upstream_track <- Gviz::GeneRegionTrack(
      data.frame(transcripts_upstream),
      chromosome = fusion@gene_upstream@chromosome)
    tr_downstream_track <- Gviz::GeneRegionTrack(
      data.frame(transcripts_downstream),
      chromosome = fusion@gene_downstream@chromosome)
  }
  # Set display parameters
  transcript_display_params <- list(
    showTitle = FALSE, # hide left panel
    showId = TRUE, # show transcript id
    col = "transparent",
    fontcolor.group = "black",
    just.group = "below",
    min.distance = 0,
    fontsize.group = 25,
    thinBoxFeature = c(
      "5utr_excludedA",
      "5utr_includedA",
      "3utr_excludedA",
      "3utr_includedA",

      "5utr_excludedB",
      "5utr_includedB",
      "3utr_excludedB",
      "3utr_includedB"
    ))
  Gviz::displayPars(tr_upstream_track) <- transcript_display_params
  Gviz::displayPars(tr_downstream_track) <- transcript_display_params

  # Create axis track
  axis_track <- Gviz::GenomeAxisTrack()
  Gviz::displayPars(axis_track) <- list(
    add53 = TRUE,
    add35 = TRUE,
    col = "black", # line color
    fontcolor = "black",
    fontsize = 15
  )

  # Create ideogram tracks

  # Read cytoband information depending on genome version
  if (fusion@genome_version == "hg19") {
    cytoband_file <- system.file(
      "extdata",
      "UCSC.HG19.Human.CytoBandIdeogram.txt",
      package = "chimeraviz")
  } else if (fusion@genome_version == "hg38") {
    cytoband_file <- system.file(
      "extdata",
      "UCSC.HG38.Human.CytoBandIdeogram.txt",
      package = "chimeraviz")
  } else if (fusion@genome_version == "mm10") {
    cytoband_file <- system.file(
      "extdata",
      "UCSC.MM10.Mus.musculus.CytoBandIdeogram.txt",
      package = "chimeraviz")
  } else {
    stop("Unsupported genome version")
  }
  cytoband <- utils::read.table(cytoband_file)
  # Set names to what Gviz expects
  names(cytoband) <- c("chrom", "chromStart", "chromEnd", "name", "gieStain")

  ideogram_track_upstream <- Gviz::IdeogramTrack(
    genome = fusion@genome_version,
    chromosome = fusion@gene_upstream@chromosome,
    bands = cytoband)
  ideogram_track_downstream <- Gviz::IdeogramTrack(
    genome = fusion@genome_version,
    chromosome = fusion@gene_downstream@chromosome,
    bands = cytoband)
  # Set display paramters
  ideogram_display_params <- list(
    showBandId = TRUE,
    showId = TRUE,
    fontcolor = "black",
    cex = 1.5)
  Gviz::displayPars(ideogram_track_upstream) <- ideogram_display_params
  Gviz::displayPars(ideogram_track_downstream) <- ideogram_display_params

  # Create alignment track
  if (!is.null(bamfile)) {
    # We're getting coverage data from a bam file
    if (non_ucsc) {
      # If the bam file has non-ucsc chromosome names, i.e. "1" instead of
      # "chr1", then we need to use a custom import function that adds "chr" to
      # the chromosome names, to keep Gviz happy. Gviz strongly prefers having
      # the "chr" prefix.
      alignment_track <- Gviz::AlignmentsTrack(
        bamfile,
        isPaired = TRUE,
        genome = fusion@genome_version,
        name = "RNA coverage",
        ylim = ylim,
        importFunction = import_function_non_ucsc)
    } else {
      alignment_track <- Gviz::AlignmentsTrack(
        bamfile,
        isPaired = TRUE,
        genome = fusion@genome_version,
        name = "RNA coverage",
        ylim = ylim)
    }
    # Set display paramters
    Gviz::displayPars(alignment_track) <- list(
      showTitle = FALSE, # hide name of track
      background.panel = "transparent", # background color of the content panel
      background.title = "transparent", # background color for the title panels
      col.axis = "black",
      col.coverage = "black",
      col.title = "black",
      fill.coverage = "orange",
      fontsize = 15,
      lwd.coverage = 0.4,
      type = "coverage",
      cex.title = .8,
      cex.axis = .6
    )
  } else if (!is.null(bedgraphfile)) {
    # We're getting coverage data from a bedGraph file
    alignment_track <- DataTrack(
      range = bedgraphfile,
      ylim = ylim,
      genome = "hg19",
      name = "Coverage",
      type = "h",
      col = "orange",
      fill = "orange")
    # Set display parameters
    Gviz::displayPars(alignment_track) <- list(
      showTitle = FALSE, # hide name of track
      background.panel = "transparent", # background color of the content panel
      background.title = "transparent", # background color for the title panels
      cex.axis = .6,
      cex.title = .6,
      col.axis = "black",
      col.coverage = "black",
      col.title = "black",
      coverageHeight = 0.08,
      fill.coverage = "orange",
      fontsize = 15,
      lty = 1,
      lty.coverage = 1,
      lwd = 0.5
    )
  } else {
    # We're not going to plot coverage data
  }

  # Display parameters for highlight tracks
  display_pars_highlight_tracks <- list(
    fill = "lightgrey",
    col = "lightgrey",
    showTitle = FALSE
  )

  # Helper variables
  gene_upstream_at_minus_strand <- fusion@gene_upstream@strand == "-"
  gene_downstream_at_minus_strand <- fusion@gene_downstream@strand == "-"

  # Color options for exons included/excluded in fusion
  cols <- RColorBrewer::brewer.pal(4, "Paired")
  interestcolor <- list(
    "5utr_excludedA" = cols[1],
    "protein_coding_excludedA" = cols[1],
    "3utr_excludedA" = cols[1],

    "5utr_includedA" = cols[2],
    "protein_coding_includedA" = cols[2],
    "3utr_includedA" = cols[2],

    "5utr_excludedB" = cols[3],
    "protein_coding_excludedB" = cols[3],
    "3utr_excludedB" = cols[3],

    "5utr_includedB" = cols[4],
    "protein_coding_includedB" = cols[4],
    "3utr_includedB" = cols[4])

  # Set all colors to exluded
  if (reduce_transcripts) {
    Gviz::feature(tr_upstream_track) <- "protein_coding_excludedA"
    Gviz::feature(tr_downstream_track) <- "protein_coding_excludedB"
  } else {
    Gviz::feature(tr_upstream_track)[
      Gviz::feature(tr_upstream_track) == "5utr"
    ] <- "5utr_excludedA"
    Gviz::feature(tr_upstream_track)[
      Gviz::feature(tr_upstream_track) == "protein_coding"
    ] <- "protein_coding_excludedA"
    Gviz::feature(tr_upstream_track)[
      Gviz::feature(tr_upstream_track) == "3utr"
    ] <- "3utr_excludedA"

    Gviz::feature(tr_downstream_track)[
      Gviz::feature(tr_downstream_track) == "5utr"
    ] <- "5utr_excludedB"
    Gviz::feature(tr_downstream_track)[
      Gviz::feature(tr_downstream_track) == "protein_coding"
    ] <- "protein_coding_excludedB"
    Gviz::feature(tr_downstream_track)[
      Gviz::feature(tr_downstream_track) == "3utr"
    ] <- "3utr_excludedB"
  }

  if (gene_upstream_at_minus_strand) {
    message(paste("As ",
                  fusion@gene_upstream@name,
                  " is on the minus strand, the plot for this gene will be ",
                  "reversed",
                  sep = ""))
    highlight_start_upstream <- fusion@gene_upstream@breakpoint
    highlight_end_upstream <-
      max(start(tr_upstream_track) + width(tr_upstream_track) - 1)

    # Color the exons included/not included in the fusion
    Gviz::feature(tr_upstream_track)[
      start(tr_upstream_track) <= highlight_end_upstream &
      end(tr_upstream_track) >= highlight_start_upstream &
      Gviz::feature(tr_upstream_track) == "5utr_excludedA"
    ] <- "5utr_includedA"

    Gviz::feature(tr_upstream_track)[
      start(tr_upstream_track) <= highlight_end_upstream &
      end(tr_upstream_track) >= highlight_start_upstream &
      Gviz::feature(tr_upstream_track) == "protein_coding_excludedA"
    ] <- "protein_coding_includedA"

    Gviz::feature(tr_upstream_track)[
      start(tr_upstream_track) <= highlight_end_upstream &
      end(tr_upstream_track) >= highlight_start_upstream &
      Gviz::feature(tr_upstream_track) == "3utr_excludedA"
    ] <- "3utr_includedA"

  } else {
    highlight_start_upstream <- min(start(tr_upstream_track))
    highlight_end_upstream <- fusion@gene_upstream@breakpoint

    # Color the exons included/not included in the fusion
    Gviz::feature(tr_upstream_track)[
      start(tr_upstream_track) >= highlight_start_upstream &
      end(tr_upstream_track) <= highlight_end_upstream &
      Gviz::feature(tr_upstream_track) == "5utr_excludedA"
    ] <- "5utr_includedA"

    Gviz::feature(tr_upstream_track)[
      start(tr_upstream_track) >= highlight_start_upstream &
      end(tr_upstream_track) <= highlight_end_upstream &
      Gviz::feature(tr_upstream_track) == "protein_coding_excludedA"
    ] <- "protein_coding_includedA"

    Gviz::feature(tr_upstream_track)[
      start(tr_upstream_track) >= highlight_start_upstream &
      end(tr_upstream_track) <= highlight_end_upstream &
      Gviz::feature(tr_upstream_track) == "3utr_excludedA"
    ] <- "3utr_includedA"

  }

  if (gene_downstream_at_minus_strand) {
    message(paste("As ",
                  fusion@gene_downstream@name,
                  " is on the minus strand, the plot for this gene will be ",
                  "reversed",
                  sep = ""))
    highlight_start_downstream <- min(start(tr_downstream_track))
    highlight_end_downstream <- fusion@gene_downstream@breakpoint

    # Color the exons included/not included in the fusion
    Gviz::feature(tr_downstream_track)[
      start(tr_downstream_track) <= highlight_end_downstream &
      end(tr_downstream_track) >= highlight_start_downstream &
      Gviz::feature(tr_downstream_track) == "5utr_excludedB"
    ] <- "5utr_includedB"

    Gviz::feature(tr_downstream_track)[
      start(tr_downstream_track) <= highlight_end_downstream &
      end(tr_downstream_track) >= highlight_start_downstream &
      Gviz::feature(tr_downstream_track) == "protein_coding_excludedB"
    ] <- "protein_coding_includedB"

    Gviz::feature(tr_downstream_track)[
      start(tr_downstream_track) <= highlight_end_downstream &
      end(tr_downstream_track) >= highlight_start_downstream &
      Gviz::feature(tr_downstream_track) == "3utr_excludedB"
    ] <- "3utr_includedB"

  } else {
    highlight_start_downstream <- fusion@gene_downstream@breakpoint
    highlight_end_downstream <-
      max(start(tr_downstream_track) + width(tr_downstream_track) - 1)

    # Color the exons included/not included in the fusion
    Gviz::feature(tr_downstream_track)[
      start(tr_downstream_track) >= highlight_start_downstream &
      end(tr_downstream_track) <= highlight_end_downstream &
      Gviz::feature(tr_downstream_track) == "5utr_excludedB"
    ] <- "5utr_includedB"

    Gviz::feature(tr_downstream_track)[
      start(tr_downstream_track) >= highlight_start_downstream &
      end(tr_downstream_track) <= highlight_end_downstream &
      Gviz::feature(tr_downstream_track) == "protein_coding_excludedB"
    ] <- "protein_coding_includedB"

    Gviz::feature(tr_downstream_track)[
      start(tr_downstream_track) >= highlight_start_downstream &
      end(tr_downstream_track) <= highlight_end_downstream &
      Gviz::feature(tr_downstream_track) == "3utr_excludedB"
    ] <- "3utr_includedB"

  }

  # Update transcript track with colors
  Gviz::displayPars(tr_upstream_track) <- interestcolor
  Gviz::displayPars(tr_downstream_track) <- interestcolor

  # Highlight transcript tracks

  gr_track_highlight_upstream <- Gviz::HighlightTrack(
    trackList = tr_upstream_track,
    start = highlight_start_upstream,
    end = highlight_end_upstream,
    chromosome = fusion@gene_upstream@chromosome)
  Gviz::displayPars(gr_track_highlight_upstream) <-
    display_pars_highlight_tracks

  gr_track_highlight_downstream <- Gviz::HighlightTrack(
    trackList = tr_downstream_track,
    start = highlight_start_downstream,
    end = highlight_end_downstream,
    chromosome = fusion@gene_downstream@chromosome)
  Gviz::displayPars(gr_track_highlight_downstream) <-
    display_pars_highlight_tracks

  if (exists("alignment_track")) {

    # Highlight coverage tracks

    al_track_highlight_upstream <- Gviz::HighlightTrack(
      trackList = alignment_track,
      start = highlight_start_upstream,
      end = highlight_end_upstream,
      chromosome = fusion@gene_upstream@chromosome)
    Gviz::displayPars(al_track_highlight_upstream) <-
      display_pars_highlight_tracks

    al_track_highlight_downstream <- Gviz::HighlightTrack(
      trackList = alignment_track,
      start = highlight_start_downstream,
      end = highlight_end_downstream,
      chromosome = fusion@gene_downstream@chromosome)
    Gviz::displayPars(al_track_highlight_downstream) <-
      display_pars_highlight_tracks

  }

  # Do some plotting

  # Create the grid we're gonna use for the plot
  nf <- grid::grid.layout(
    nrow = 1,
    ncol = 2,
    widths = grid::unit(c(1, 1), "null"))

  # Open a new page to plot it
  grid::grid.newpage()

  # Divide the page according to the grid we created
  grid::pushViewport(grid::viewport(layout = nf))

  # How tall should the transcripts row be? This depends on how many transcripts
  # we want to show. If reduce_transcripts=TRUE, then we only want to display
  # one transcript per gene.
  if (reduce_transcripts) {
    transcripts_row_height <- 1
  } else {
    # It depends on max(amountOfTranscriptsA, amountOfTranscriptsB)
    transcripts_row_height <- max(
      length(transcripts_upstream),
      length(transcripts_downstream))
    # Set a max value
    if (transcripts_row_height > 10) {
      transcripts_row_height <- 10
    }
  }

  # If the range for the plot is too short, then GenomeAxisTrack will show
  # ticks too close to each other. This doesn't look nice. A way to solve it is
  # to extend the plot range if the transcript is too short:
  plot_range <- max(end(tr_upstream_track)) - min(start(tr_upstream_track))
  if (plot_range < 20000) {
    offset <- 2500
  } else {
    offset <- 0
  }

  if (!exists("alignment_track")) {

    # Open row 1, column 1
    grid::pushViewport(grid::viewport(layout.pos.row = 1, layout.pos.col = 1))
    # Plot transcriptsA and coverage with highlight
    res1 <- Gviz::plotTracks(
      # without this gviz create cluster_X entries in the GeneRegionTrack
      collapse = FALSE,
      c(
        ideogram_track_upstream,
        gr_track_highlight_upstream
      ),
      # 10k added so that we can see all transcript names
      from = min(start(tr_upstream_track)) - offset - 10000,
      # 10k added so that we can see all transcript names
      to = max(end(tr_upstream_track)) + offset + 10000,
      sizes = c(1, transcripts_row_height),
      add = TRUE,
      background.title = "transparent",
      chromosome = rep(fusion@gene_upstream@chromosome, 2),
      # Plot reverse if gene is at minus strand
      reverseStrand = gene_upstream_at_minus_strand)
    # Close row 1, column 1
    grid::popViewport(1)

  } else {

    # Open row 1, column 1
    grid::pushViewport(grid::viewport(layout.pos.row = 1, layout.pos.col = 1))
    # Plot transcriptsA and coverage with highlight
    res1 <- Gviz::plotTracks(
      # without this gviz create cluster_X entries in the GeneRegionTrack
      collapse = FALSE,
      c(
        ideogram_track_upstream,
        gr_track_highlight_upstream,
        al_track_highlight_upstream,
        axis_track
      ),
      # 10k added so that we can see all transcript names
      from = min(start(tr_upstream_track)) - offset - 10000,
      # 10k added so that we can see all transcript names
      to = max(end(tr_upstream_track)) + offset + 10000,
      sizes = c(1, transcripts_row_height, 2, .5),
      add = TRUE,
      background.title = "transparent",
      chromosome = rep(fusion@gene_upstream@chromosome, 4),
      # Plot reverse if gene is at minus strand
      reverseStrand = gene_upstream_at_minus_strand)
    # Close row 1, column 1
    grid::popViewport(1)

  }

  # If the range for the plot is too short, then GenomeAxisTrack will show
  # ticks too close to each other. This doesn't look nice. A way to solve it is
  # to extend the plot range if the transcript is too short:
  plot_range <- max(end(tr_downstream_track)) - min(start(tr_downstream_track))
  if (plot_range < 20000) {
    offset <- 2500
  } else {
    offset <- 0
  }

  if (!exists("alignment_track")) {

    # Open row 1, column 2
    grid::pushViewport(grid::viewport(layout.pos.row = 1, layout.pos.col = 2))
    # Plot transcriptsB and coverage with highlight
    res2 <- Gviz::plotTracks(
      # without this gviz create cluster_X entries in the GeneRegionTrack
      collapse = FALSE,
      c(
        ideogram_track_downstream,
        gr_track_highlight_downstream
      ),
      # 10k added so that we can see all transcript names
      from = min(start(tr_downstream_track)) - offset - 10000,
      # 10k added so that we can see all transcript names
      to = max(end(tr_downstream_track)) + offset + 10000,
      sizes = c(1, transcripts_row_height),
      add = TRUE,
      background.title = "transparent",
      chromosome = rep(fusion@gene_downstream@chromosome, 2),
      # Plot reverse if gene is at minus strand
      reverseStrand = gene_downstream_at_minus_strand)
    # Close row 1, column 2
    grid::popViewport(1)

  } else {

    # Open row 1, column 2
    grid::pushViewport(grid::viewport(layout.pos.row = 1, layout.pos.col = 2))
    # Plot transcriptsB and coverage with highlight
    res2 <- Gviz::plotTracks(
      # without this gviz create cluster_X entries in the GeneRegionTrack
      collapse = FALSE,
      c(
        ideogram_track_downstream,
        gr_track_highlight_downstream,
        al_track_highlight_downstream,
        axis_track
      ),
      # 10k added so that we can see all transcript names
      from = min(start(tr_downstream_track)) - offset - 10000,
      # 10k added so that we can see all transcript names
      to = max(end(tr_downstream_track)) + offset + 10000,
      sizes = c(1, transcripts_row_height, 2, .5),
      add = TRUE,
      background.title = "transparent",
      chromosome = rep(fusion@gene_downstream@chromosome, 4),
      # Plot reverse if gene is at minus strand
      reverseStrand = gene_downstream_at_minus_strand)
    # Close row 1, column 2
    grid::popViewport(1)

  }

  # Draw bezier curve if we're plotting exon boundary transcripts

  upstream_boundary_exons <- fusion@gene_upstream@transcripts[
    mcols(fusion@gene_upstream@transcripts)$transcript_category ==
      "exonBoundary"
  ]
  downstream_boundary_exons <- fusion@gene_downstream@transcripts[
    mcols(fusion@gene_downstream@transcripts)$transcript_category ==
      "exonBoundary"
  ]
  if (length(upstream_boundary_exons) > 0 &
      length(downstream_boundary_exons) > 0) {

    # We need to figure out which exons in tr_upstream_track and
    # tr_downstream_track matches the fusion breakpoints
    # fusion@gene_upstream@breakpoint and fusion@gene_downstream@breakpoint:
    if (gene_upstream_at_minus_strand) {
      exon_breakpoint_upstream <- mcols(
        tr_upstream_track[
          GenomicRanges::start(tr_upstream_track@range) ==
            fusion@gene_upstream@breakpoint
        ]@range
      )$exon
    } else {
      exon_breakpoint_upstream <- mcols(
        tr_upstream_track[
          GenomicRanges::end(tr_upstream_track@range) ==
            fusion@gene_upstream@breakpoint
        ]@range
      )$exon
    }
    if (gene_downstream_at_minus_strand) {
      exon_breakpoint_downstream <- mcols(
        tr_downstream_track[
          GenomicRanges::end(tr_downstream_track@range) ==
            fusion@gene_downstream@breakpoint
        ]@range
      )$exon
    } else {
      exon_breakpoint_downstream <- mcols(
        tr_downstream_track[
          GenomicRanges::start(tr_downstream_track@range) ==
            fusion@gene_downstream@breakpoint
        ]@range
      )$exon
    }

    # Get coordinates for the exons in the plot
    exon_coordinates1 <- Gviz::coords(res1$GeneRegionTrack)
    exon_coordinates2 <- Gviz::coords(res2$GeneRegionTrack)

    # Since we know which exon in geneA meets the breakpoint, its easy to fetch
    # the coordinates for the breakpoint exon in geneA:
    exon_upstream <- exon_coordinates1[
      exon_breakpoint_upstream,
      ,
      drop = FALSE
    ]
    # Do the same for geneB:
    exon_downstream <- exon_coordinates2[
      exon_breakpoint_downstream,
      ,
      drop = FALSE
    ]

    # The coordinates we're really after is the top right corner of exonA (or
    # top left corner if on the minus strand), and the top left (top right if on
    # minus strand) corner of exon B
    if (gene_upstream_at_minus_strand) {
      # top left corner:
      x_pos_gene_upstream <- min(exon_upstream[, 1])
      y_pos_gene_upstream <- min(exon_upstream[, 2])
    } else {
      # top right corner:
      x_pos_gene_upstream <- min(exon_upstream[, 3])
      y_pos_gene_upstream <- min(exon_upstream[, 2])
    }
    if (gene_downstream_at_minus_strand) {
      # top right corner:
      x_pos_gene_downstream <- min(exon_downstream[, 3])
      y_pos_gene_downstream <- min(exon_downstream[, 2])
    } else {
      # top left corner:
      x_pos_gene_downstream <- min(exon_downstream[, 1])
      y_pos_gene_downstream <- min(exon_downstream[, 2])
    }

    # This will keep the coordinates even though the dev.size change:
    grid::pushViewport(
      grid::viewport(
        xscale = c(
          0,
          grDevices::dev.size(units = "px")[1]
        ), # x goes from 0 to device width
        yscale = c(
          grDevices::dev.size(units = "px")[2],
          0
        ) # y goes from device height to 0
      )
    )

    # Calculate the bezier curve width. The following will have the effect of
    # setting the line width to 10 if there are 50 or more reads that support
    # the fusion. If there are between 1 and 50 reads supporting the fusion,
    # then the curve width will be scaled accordingly.
    supporting_reads <- fusion@spanning_reads_count + fusion@split_reads_count
    supporting_reads_text <- paste0(
      supporting_reads,
      " (",
      fusion@split_reads_count,
      ",",
      fusion@spanning_reads_count,
      ")")
    curve_width <- .scale_list_to_interval(
      c(supporting_reads,
        1,
        20),
      1,
      5)
    curve_width <- curve_width[1]

    # How far above the transcript should the bezier curve control points be?
    bezier_control_point_offset <- 18

    # The different transcripts will be plotted at different heights. Choose the
    # y position closest to the top of the plot. That is the lowest y value of
    # the two, since the y axis is flipped
    y_pos <- min(y_pos_gene_upstream, y_pos_gene_downstream)

    # Draw the bezier curve between the transcripts
    grid::grid.bezier(
      rep(
        c(
          x_pos_gene_upstream,
          x_pos_gene_downstream
        ),
        each = 2
      ), # x positions
      c(y_pos,
        y_pos - bezier_control_point_offset,
        y_pos - bezier_control_point_offset,
        y_pos
      ), # y positions
      default.units = "native",
      gp = grid::gpar(
        col = "red",
        lwd = curve_width))

    # Where should the number of supporting reads be plotted? Calculate position
    number_offset <- 7
    number_position_x <-
      x_pos_gene_upstream + (x_pos_gene_downstream - x_pos_gene_upstream) / 2
    number_position_y <-
      y_pos - bezier_control_point_offset - number_offset

    grid::grid.text(
      supporting_reads_text,
      x = number_position_x / grDevices::dev.size(units = "px")[1],
      y = 1 - number_position_y / grDevices::dev.size(units = "px")[2],
      vp = grid::viewport(
        xscale = c(0, grDevices::dev.size(units = "px")[1]),
        yscale = c(grDevices::dev.size(units = "px")[2], 0)),
      gp = grid::gpar(
        fontsize = 15))

  }
}

#' @name plot_fusion
#'
#' @export
plot_fusion_together <- function(
  fusion,
  edb,
  bamfile = NULL,
  which_transcripts = "exonBoundary",
  ylim = c(0, 1000),
  non_ucsc = TRUE,
  reduce_transcripts = FALSE,
  bedgraphfile = NULL) {

  .validate_plot_fusion_params(
    fusion,
    edb,
    bamfile,
    which_transcripts,
    ylim,
    non_ucsc,
    reduce_transcripts,
    bedgraphfile
  )
  fusion <- .get_transcripts_if_not_there(fusion, edb)

  # Create tracks

  # Select transcripts
  transcripts_upstream <-
    select_transcript(fusion@gene_upstream, which_transcripts)
  transcripts_downstream <-
    select_transcript(fusion@gene_downstream, which_transcripts)

  # Check that there's at least one transcript that has the fusion breakpoint
  # within the transcript.
  .check_that_breakpoints_are_within_transcripts(
    fusion,
    transcripts_upstream,
    transcripts_downstream
  )

  # Set seqlevels to UCSC
  GenomeInfoDb::seqlevelsStyle(transcripts_upstream) <- "UCSC"
  GenomeInfoDb::seqlevelsStyle(transcripts_downstream) <- "UCSC"
  if (reduce_transcripts) {
    reduced_transcripts_upstream <-
      GenomicRanges::reduce(unlist(transcripts_upstream))
    reduced_transcripts_downstream <-
      GenomicRanges::reduce(unlist(transcripts_downstream))
    # Add transcript metadata column to make Gviz group correctly
    mcols(reduced_transcripts_upstream)$transcript <-
      fusion@gene_upstream@name
    mcols(reduced_transcripts_downstream)$transcript <-
      fusion@gene_downstream@name
    # Add symbol metadata column to make Gviz group correctly
    mcols(reduced_transcripts_upstream)$symbol <- fusion@gene_upstream@name
    mcols(reduced_transcripts_downstream)$symbol <- fusion@gene_downstream@name
    # Create the track
    tr_track_both <- Gviz::GeneRegionTrack(
      rbind(
        data.frame(reduced_transcripts_upstream),
        data.frame(reduced_transcripts_downstream)
      ),
      chromosome = fusion@gene_upstream@chromosome)
  } else {
    # Create the track
    tr_track_both <- Gviz::GeneRegionTrack(
      rbind(
        data.frame(transcripts_upstream),
        data.frame(transcripts_downstream)
      ),
      chromosome = fusion@gene_upstream@chromosome)
  }
  # Set display parameters
  transcript_display_params <- list(
    showTitle = FALSE, # hide left panel
    showId = TRUE, # show transcript id
    col = "transparent",
    fontcolor.group = "black",
    just.group = "below",
    min.distance = 0,
    fontsize.group = 25,
    thinBoxFeature = c(
      "5utr_excludedA",
      "5utr_includedA",
      "3utr_excludedA",
      "3utr_includedA",

      "5utr_excludedB",
      "5utr_includedB",
      "3utr_excludedB",
      "3utr_includedB"
    ))
  Gviz::displayPars(tr_track_both) <- transcript_display_params

  # Create axis track
  axis_track <- Gviz::GenomeAxisTrack()
  Gviz::displayPars(axis_track) <- list(
    add53 = TRUE,
    add35 = TRUE,
    col = "black", # line color
    fontcolor = "black",
    fontsize = 15
  )

  # Create ideogram tracks

  # Read cytoband information depending on genome version
  if (fusion@genome_version == "hg19") {
    cytoband_file <- system.file(
      "extdata",
      "UCSC.HG19.Human.CytoBandIdeogram.txt",
      package = "chimeraviz")
  } else if (fusion@genome_version == "hg38") {
    cytoband_file <- system.file(
      "extdata",
      "UCSC.HG38.Human.CytoBandIdeogram.txt",
      package = "chimeraviz")
  } else if (fusion@genome_version == "mm10") {
    cytoband_file <- system.file(
      "extdata",
      "UCSC.MM10.Mus.musculus.CytoBandIdeogram.txt",
      package = "chimeraviz")
  } else {
    stop("Unsupported genome version")
  }
  cytoband <- utils::read.table(cytoband_file)
  # Set names to what Gviz expects
  names(cytoband) <- c("chrom", "chromStart", "chromEnd", "name", "gieStain")

  ideogram_track <- Gviz::IdeogramTrack(
    genome = fusion@genome_version,
    chromosome = fusion@gene_upstream@chromosome,
    bands = cytoband)
  # Set display paramters
  ideogram_display_params <- list(
    showBandId = TRUE,
    showId = TRUE,
    fontcolor = "black",
    cex = 1.5)
  Gviz::displayPars(ideogram_track) <- ideogram_display_params

  # Create alignment track
  if (!is.null(bamfile)) {
    # We're getting coverage data from a bam file
    if (non_ucsc) {
      # If the bam file has non-ucsc chromosome names, i.e. "1" instead of
      # "chr1", then we need to use a custom import function that adds "chr" to
      # the chromosome names, to keep Gviz happy. Gviz strongly prefers having
      # the "chr" prefix.
      alignment_track <- Gviz::AlignmentsTrack(
        bamfile,
        isPaired = TRUE,
        genome = fusion@genome_version,
        name = "RNA coverage",
        ylim = ylim,
        importFunction = import_function_non_ucsc)
    } else {
      alignment_track <- Gviz::AlignmentsTrack(
        bamfile,
        isPaired = TRUE,
        genome = fusion@genome_version,
        name = "RNA coverage",
        ylim = ylim)
    }
    # Set display paramters
    Gviz::displayPars(alignment_track) <- list(
      showTitle = FALSE, # hide name of track
      background.panel = "transparent", # background color of the content panel
      background.title = "transparent", # background color for the title panels
      col.axis = "black",
      col.coverage = "black",
      col.title = "black",
      fill.coverage = "orange",
      fontsize = 15,
      lwd.coverage = 0.4,
      type = "coverage",
      cex.title = .8,
      cex.axis = .6
    )
  } else if (!is.null(bedgraphfile)) {
    # We're getting coverage data from a bedGraph file
    alignment_track <- DataTrack(
      range = bedgraphfile,
      ylim = ylim,
      genome = "hg19",
      chromosome = "1",
      name = "Coverage",
      type = "h",
      col = "orange",
      fill = "orange")
    # Set display parameters
    Gviz::displayPars(alignment_track) <- list(
      showTitle = FALSE, # hide name of track
      background.panel = "transparent", # background color of the content panel
      background.title = "transparent", # background color for the title panels
      cex.axis = .6,
      cex.title = .6,
      col.axis = "black",
      col.coverage = "black",
      col.title = "black",
      coverageHeight = 0.08,
      fill.coverage = "orange",
      fontsize = 15,
      lty = 1,
      lty.coverage = 1,
      lwd = 0.5
    )
  } else {
    # We're not going to plot coverage data
  }


  # Display parameters for highlight tracks
  display_pars_highlight_tracks <- list(
    fill = "lightgrey",
    col = "lightgrey",
    showTitle = FALSE
  )

  # Helper variables that decide which way to plot the transcript later
  gene_upstream_at_minus_strand <- fusion@gene_upstream@strand == "-"
  gene_downstream_at_minus_strand <- fusion@gene_downstream@strand == "-"

  # Color options for exons included/excluded in fusion
  cols <- RColorBrewer::brewer.pal(4, "Paired")
  interestcolor <- list(
    "5utr_excludedA" = cols[1],
    "protein_coding_excludedA" = cols[1],
    "3utr_excludedA" = cols[1],

    "5utr_includedA" = cols[2],
    "protein_coding_includedA" = cols[2],
    "3utr_includedA" = cols[2],

    "5utr_excludedB" = cols[3],
    "protein_coding_excludedB" = cols[3],
    "3utr_excludedB" = cols[3],

    "5utr_includedB" = cols[4],
    "protein_coding_includedB" = cols[4],
    "3utr_includedB" = cols[4])
  if (reduce_transcripts) {
    gene_upstream_indexes <-
      Gviz::symbol(tr_track_both) == fusion@gene_upstream@name
    gene_downstream_indexes <-
      Gviz::symbol(tr_track_both) == fusion@gene_downstream@name
  } else {
    gene_upstream_indexes <-
      Gviz::symbol(tr_track_both) %in% names(transcripts_upstream)
    gene_downstream_indexes <-
      Gviz::symbol(tr_track_both) %in% names(transcripts_downstream)
  }

  # Set all colors to exluded
  Gviz::feature(tr_track_both)[
    Gviz::feature(tr_track_both) == "5utr" & gene_upstream_indexes
  ] <- "5utr_excludedA"
  Gviz::feature(tr_track_both)[
    Gviz::feature(tr_track_both) == "protein_coding" & gene_upstream_indexes
  ] <- "protein_coding_excludedA"
  Gviz::feature(tr_track_both)[
    Gviz::feature(tr_track_both) == "3utr" & gene_upstream_indexes
  ] <- "3utr_excludedA"

  Gviz::feature(tr_track_both)[
    Gviz::feature(tr_track_both) == "5utr" & gene_downstream_indexes
  ] <- "5utr_excludedB"
  Gviz::feature(tr_track_both)[
    Gviz::feature(tr_track_both) == "protein_coding" & gene_downstream_indexes
  ] <- "protein_coding_excludedB"
  Gviz::feature(tr_track_both)[
    Gviz::feature(tr_track_both) == "3utr" & gene_downstream_indexes
  ] <- "3utr_excludedB"

  if (gene_upstream_at_minus_strand) {
    message(paste("As ",
                  fusion@gene_upstream@name,
                  " is on the minus strand, the plot for this gene will be ",
                  "reversed",
                  sep = ""))
    highlight_start_upstream <- fusion@gene_upstream@breakpoint
    highlight_end_upstream <-
      max(
        start(
          tr_track_both[gene_upstream_indexes]
        ) + width(tr_track_both[gene_upstream_indexes]) - 1)

    # Color the exons included/not included in the fusion
    Gviz::feature(tr_track_both)[
      start(tr_track_both) <= highlight_end_upstream &
      end(tr_track_both) >= highlight_start_upstream &
      Gviz::feature(tr_track_both) == "5utr_excludedA" &
      gene_upstream_indexes
    ] <- "5utr_includedA"

    Gviz::feature(tr_track_both)[
      start(tr_track_both) <= highlight_end_upstream &
      end(tr_track_both) >= highlight_start_upstream &
      Gviz::feature(tr_track_both) == "protein_coding_excludedA" &
      gene_upstream_indexes
    ] <- "protein_coding_includedA"

    Gviz::feature(tr_track_both)[
      start(tr_track_both) <= highlight_end_upstream &
      end(tr_track_both) >= highlight_start_upstream &
      Gviz::feature(tr_track_both) == "3utr_excludedA" &
      gene_upstream_indexes
    ] <- "3utr_includedA"
  } else {
    highlight_start_upstream <- min(start(tr_track_both[gene_upstream_indexes]))
    highlight_end_upstream <- fusion@gene_upstream@breakpoint

    # Color the exons included/not included in the fusion
    Gviz::feature(tr_track_both)[
      start(tr_track_both) >= highlight_start_upstream &
      end(tr_track_both) <= highlight_end_upstream &
      Gviz::feature(tr_track_both) == "5utr_excludedA" &
      gene_upstream_indexes
    ] <- "5utr_includedA"

    Gviz::feature(tr_track_both)[
      start(tr_track_both) >= highlight_start_upstream &
      end(tr_track_both) <= highlight_end_upstream &
      Gviz::feature(tr_track_both) == "protein_coding_excludedA" &
      gene_upstream_indexes
    ] <- "protein_coding_includedA"

    Gviz::feature(tr_track_both)[
      start(tr_track_both) >= highlight_start_upstream &
      end(tr_track_both) <= highlight_end_upstream &
      Gviz::feature(tr_track_both) == "3utr_excludedA" &
      gene_upstream_indexes
    ] <- "3utr_includedA"
  }

  if (gene_downstream_at_minus_strand) {
    message(paste("As ",
                  fusion@gene_downstream@name,
                  " is on the minus strand, the plot for this gene will be ",
                  "reversed",
                  sep = ""))
    highlight_start_downstream <-
      min(start(tr_track_both[gene_downstream_indexes]))
    highlight_end_downstream <- fusion@gene_downstream@breakpoint

    # Color the exons included/not included in the fusion
    Gviz::feature(tr_track_both)[
      start(tr_track_both) <= highlight_end_downstream &
      end(tr_track_both) >= highlight_start_downstream &
      Gviz::feature(tr_track_both) == "5utr_excludedB" &
      gene_downstream_indexes
    ] <- "5utr_includedB"

    Gviz::feature(tr_track_both)[
      start(tr_track_both) <= highlight_end_downstream &
      end(tr_track_both) >= highlight_start_downstream &
      Gviz::feature(tr_track_both) == "protein_coding_excludedB" &
      gene_downstream_indexes
    ] <- "protein_coding_includedB"

    Gviz::feature(tr_track_both)[
      start(tr_track_both) <= highlight_end_downstream &
      end(tr_track_both) >= highlight_start_downstream &
      Gviz::feature(tr_track_both) == "3utr_excludedB" &
      gene_downstream_indexes
    ] <- "3utr_includedB"
  } else {
    highlight_start_downstream <- fusion@gene_downstream@breakpoint
    highlight_end_downstream <-
      max(
        start(
          tr_track_both[gene_downstream_indexes]
        ) + width(tr_track_both[gene_downstream_indexes]) - 1)

    # Color the exons included/not included in the fusion
    Gviz::feature(tr_track_both)[
      start(tr_track_both) >= highlight_start_downstream &
      end(tr_track_both) <= highlight_end_downstream &
      Gviz::feature(tr_track_both) == "5utr_excludedB" &
      gene_downstream_indexes
    ] <- "5utr_includedB"

    Gviz::feature(tr_track_both)[
      start(tr_track_both) >= highlight_start_downstream &
      end(tr_track_both) <= highlight_end_downstream &
      Gviz::feature(tr_track_both) == "protein_coding_excludedB" &
      gene_downstream_indexes
    ] <- "protein_coding_includedB"

    Gviz::feature(tr_track_both)[
      start(tr_track_both) >= highlight_start_downstream &
      end(tr_track_both) <= highlight_end_downstream &
      Gviz::feature(tr_track_both) == "3utr_excludedB" &
      gene_downstream_indexes
    ] <- "3utr_includedB"
  }

  # Update transcript track with colors
  Gviz::displayPars(tr_track_both) <- interestcolor

  # Highlight transcript track
  gr_track_highlight <- Gviz::HighlightTrack(
    trackList = tr_track_both,
    start = c(highlight_start_upstream, highlight_start_downstream),
    end = c(highlight_end_upstream, highlight_end_downstream),
    chromosome = fusion@gene_upstream@chromosome)
  Gviz::displayPars(gr_track_highlight) <- display_pars_highlight_tracks

  if (exists("alignment_track")) {

    # Highlight coverage track
    al_track_highlight <- Gviz::HighlightTrack(
      trackList = alignment_track,
      start = c(highlight_start_upstream, highlight_start_downstream),
      end = c(highlight_end_upstream, highlight_end_downstream),
      chromosome = fusion@gene_upstream@chromosome)
    Gviz::displayPars(al_track_highlight) <- display_pars_highlight_tracks

  }

  # Do some plotting

  # Open a new page to plot
  grid::grid.newpage()

  # How tall should the transcripts row be? This depends on how many transcripts
  # we want to show. If reduce_transcripts=TRUE, then we only want to display
  # one transcript per gene.
  if (reduce_transcripts) {
    transcripts_row_height <- 1
  } else {
    # It depends on max(amountOfTranscriptsA, amountOfTranscriptsB)
    transcripts_row_height <- max(
      length(transcripts_upstream),
      length(transcripts_downstream))
    # Set a max value
    if (transcripts_row_height > 10) {
      transcripts_row_height <- 10
    }
  }

  # If the range for the plot is too short, then GenomeAxisTrack will show
  # ticks too close to each other. This doesn't look nice. A way to solve it is
  # to extend the plot range if the transcript is too short:
  plot_range <- max(end(tr_track_both)) - min(start(tr_track_both))
  if (plot_range < 20000) {
    offset <- 2500
  } else {
    offset <- 0
  }

  if (!exists("alignment_track")) {

    # Plot transcripts with highlight
    res <- Gviz::plotTracks(
      # without this gviz create cluster_X entries in the GeneRegionTrack
      collapse = FALSE,
      c(ideogram_track, gr_track_highlight),
      # 10k added so that we can see all transcript names
      from = min(start(tr_track_both)) - offset - 10000,
      # 10k added so that we can see all transcript names
      to = max(end(tr_track_both)) + offset + 10000,
      sizes = c(1, transcripts_row_height, 2, 1),
      add = TRUE,
      background.title = "transparent",
      chromosome = rep(fusion@gene_upstream@chromosome, 2),
      # Plot reverse if gene is at minus strand
      reverseStrand = gene_upstream_at_minus_strand)

  } else {

    # Plot transcripts and coverage with highlight
    res <- Gviz::plotTracks(
      # without this gviz create cluster_X entries in the GeneRegionTrack
      collapse = FALSE,
      c(ideogram_track, gr_track_highlight, axis_track),
      # 10k added so that we can see all transcript names
      from = min(start(tr_track_both)) - offset - 10000,
      # 10k added so that we can see all transcript names
      to = max(end(tr_track_both)) + offset + 10000,
      sizes = c(1, transcripts_row_height, 1),
      add = TRUE,
      background.title = "transparent",
      chromosome = rep(fusion@gene_upstream@chromosome, 4),
      # Plot reverse if gene is at minus strand
      reverseStrand = gene_upstream_at_minus_strand)

  }

  # Draw bezier curve if we're plotting exon boundary transcripts

  upstream_boundary_exons <-
    fusion@gene_upstream@transcripts[
      mcols(fusion@gene_upstream@transcripts)$transcript_category ==
        "exonBoundary"
    ]
  downstream_boundary_exons <-
    fusion@gene_downstream@transcripts[
      mcols(fusion@gene_downstream@transcripts)$transcript_category ==
        "exonBoundary"
    ]
  if (length(upstream_boundary_exons) > 0 &
      length(downstream_boundary_exons) > 0) {

    # We need to figure out which exons in trTrackBoth matches the
    # fusion breakpoints fusion@gene_upstream@breakpoint and
    # fusion@gene_downstream@breakpoint:
    if (gene_upstream_at_minus_strand) {
      exon_breakpoint_upstream <-
        mcols(
          tr_track_both[
            GenomicRanges::start(tr_track_both@range) ==
              fusion@gene_upstream@breakpoint
            ]@range
        )$exon
    } else {
      exon_breakpoint_upstream <-
        mcols(
          tr_track_both[
            GenomicRanges::end(tr_track_both@range) ==
              fusion@gene_upstream@breakpoint
            ]@range
        )$exon
    }
    if (gene_downstream_at_minus_strand) {
      exon_breakpoint_downstream <-
        mcols(
          tr_track_both[
            GenomicRanges::end(tr_track_both@range) ==
              fusion@gene_downstream@breakpoint
          ]@range
        )$exon
    } else {
      exon_breakpoint_downstream <-
        mcols(
          tr_track_both[
            GenomicRanges::start(tr_track_both@range) ==
              fusion@gene_downstream@breakpoint
          ]@range
        )$exon
    }

    # Get coordinates for the exons in the plot
    exo_coordinates <- Gviz::coords(res$GeneRegionTrack)

    # Since we know which exon in geneA meets the breakpoint, its easy to fetch
    # the coordinates for the breakpoint exon in geneA:
    exon_upstream <- exo_coordinates[
      exon_breakpoint_upstream,
      ,
      drop = FALSE
    ]
    # Do the same for geneB:
    exon_downstream <- exo_coordinates[
      exon_breakpoint_downstream,
      ,
      drop = FALSE
    ]

    # The coordinates we're really after is the top right corner of exonA (or
    # top left corner if on the minus strand), and the top left (top right if on
    # minus strand) corner of exon B
    if (gene_upstream_at_minus_strand) {
      # top left corner:
      x_pos_gene_upstream <- min(exon_upstream[, 1])
      y_pos_gene_upstream <- min(exon_upstream[, 2])
    } else {
      # top right corner:
      x_pos_gene_upstream <- min(exon_upstream[, 3])
      y_pos_gene_upstream <- min(exon_upstream[, 2])
    }
    if (gene_downstream_at_minus_strand) {
      # top right corner:
      x_pos_gene_downstream <- min(exon_downstream[, 3])
      y_pos_gene_downstream <- min(exon_downstream[, 2])
    } else {
      # top left corner:
      x_pos_gene_downstream <- min(exon_downstream[, 1])
      y_pos_gene_downstream <- min(exon_downstream[, 2])
    }

    # This will keep the coordinates even though the dev.size change:
    grid::pushViewport(
      grid::viewport(
        xscale = c(
          0,
          grDevices::dev.size(units = "px")[1]
        ), # x goes from 0 to device width
        yscale = c(
          grDevices::dev.size(units = "px")[2],
          0
        ) # y goes from device height to 0
      )
    )

    # # Draw some point just to check that we have the right coordinates:
    # grid.points(topCornerExonA[1], topCornerExonA[2]) # Exclude Linting
    # grid.points(topCornerExonB[1], topCornerExonB[2]) # Exclude Linting

    # Calculate the bezier curve width. The following will have the effect of
    # setting the line width to 10 if there are 50 or more reads that support
    # the fusion. If there are between 1 and 50 reads supporting the fusion,
    # then the curve width will be scaled accordingly.
    supporting_reads <- fusion@spanning_reads_count + fusion@split_reads_count
    supporting_reads_text <- paste0(
      supporting_reads,
      " (",
      fusion@split_reads_count,
      ",",
      fusion@spanning_reads_count,
      ")")
    curve_width <- .scale_list_to_interval(
      c(supporting_reads,
        1,
        20),
      1,
      5)
    curve_width <- curve_width[1]

    # How far above the transcript should the bezier curve control points be?
    bezier_control_point_offset <- 18

    # The different transcripts will be plotted at different heights. Choose the
    # y position closest to the top of the plot. That is the lowest y value of
    # the two, since the y axis is flipped
    y_pos <- min(y_pos_gene_upstream, y_pos_gene_downstream)

    # Draw the bezier curve between the transcripts
    grid::grid.bezier(
      rep(
        c(
          x_pos_gene_upstream,
          x_pos_gene_downstream
        ),
        each = 2
      ), # x positions
      c(
        y_pos,
        y_pos - bezier_control_point_offset,
        y_pos - bezier_control_point_offset,
        y_pos
      ), # y positions
      default.units = "native",
      gp = grid::gpar(
        col = "red",
        lwd = curve_width))

    # Where should the number of supporting reads be plotted? Calculate position
    number_offset <- 7
    number_position_x <-
      x_pos_gene_upstream + (x_pos_gene_downstream - x_pos_gene_upstream) / 2
    number_position_y <-
      y_pos - bezier_control_point_offset - number_offset

    grid::grid.text(
      supporting_reads_text,
      x = number_position_x / grDevices::dev.size(units = "px")[1],
      y = 1 - number_position_y / grDevices::dev.size(units = "px")[2],
      vp = grid::viewport(
        xscale = c(0, grDevices::dev.size(units = "px")[1]),
        yscale = c(grDevices::dev.size(units = "px")[2], 0)),
      gp = grid::gpar(
        fontsize = 15))

  }
}

#' Alternative import function for Gviz::AlignmentsTrack
#'
#' This alternative import function for use with Gviz::AlignmentsTrack imports
#' a bamfile with non-UCSC chromosome names.
#'
#' @param file The bamfile.
#' @param selection Which regions to get from the bamfile.
#'
#' @return A GRanges object with coverage data for the selection.
import_function_non_ucsc <- function(file, selection) {

  ind_names <- c(sub("\\.bam$", ".bai", file), paste(file,
                                                    "bai", sep = "."))
  index <- NULL
  for (i in ind_names) {
    if (file.exists(i)) {
      index <- i
      break
    }
  }
  if (is.null(index))
    stop(
      "Unable to find index for BAM file '",
      file,
      "'. You can build an index using the following command:\n\t",
      "library(Rsamtools)\n\tindexBam(\"",
      file,
      "\")"
    )
  paired_end <- parent.env(environment())[["._isPaired"]]
  if (is.null(paired_end))
    paired_end <- TRUE
  bf <- Rsamtools::BamFile(file, index = index, asMates = paired_end)
  # Rename chromosome names from UCSD to non-UCSC
  GenomeInfoDb::seqlevels(selection) <- sub(
    "chr",
    "",
    GenomeInfoDb::seqlevels(selection))
  param <- Rsamtools::ScanBamParam(
    which = selection,
    what = Rsamtools::scanBamWhat(),
    tag = "MD",
    flag = Rsamtools::scanBamFlag(isUnmappedQuery = FALSE))
  reads <- if (
    as.character(seqnames(selection)[1]) %in%
    names(Rsamtools::scanBamHeader(bf)$targets)
  ) {
    Rsamtools::scanBam(bf, param = param)[[1]]
  } else {
    list()
  }
  md <- if (is.null(reads$tag$MD))
    rep(as.character(NA), length(reads$pos))
  else reads$tag$MD
  if (length(reads$pos)) {
    layed_seq <- GenomicAlignments::sequenceLayer(reads$seq, reads$cigar)
    region <- unlist(Rsamtools::bamWhich(param), use.names = FALSE)
    ans <- stackStrings(
      layed_seq,
      start(region),
      end(region),
      shift = reads$pos - 1L,
      Lpadding.letter = "+",
      Rpadding.letter = "+")
    names(ans) <- seq_along(reads$qname)
  }
  else {
    ans <- DNAStringSet()
  }
  return(
    GRanges(
      # Return chromosome name prefixed with "chr"
      seqnames = if (is.null(reads$rname)) {
        character()
      } else {
        paste("chr", reads$rname, sep = "")
      },
      strand = if (is.null(reads$strand)) {
        character()
      } else {
        reads$strand
      },
      ranges = IRanges(start = reads$pos, width = reads$qwidth),
      id = reads$qname, cigar = reads$cigar, mapq = reads$mapq,
      flag = reads$flag, md = md, seq = ans, isize = reads$isize,
      groupid = if (paired_end) {
        reads$groupid
      } else {
        seq_along(reads$pos)
      },
      status = if (paired_end) {
        reads$mate_status
      } else {
        rep(
          factor(
            "unmated",
            levels = c("mated", "ambiguous", "unmated")
          ),
          length(reads$pos)
        )
      }
    )
  )
}

.validate_plot_fusion_params <- function(
  fusion,
  edb = NULL,
  bamfile,
  which_transcripts = "exonBoundary",
  ylim = c(0, 1000),
  non_ucsc = TRUE,
  reduce_transcripts = FALSE,
  bedgraphfile
) {
  # Establish a new 'checkmate' object
  argument_checker <- checkmate::makeAssertCollection()

  # Check parameters
  .is_fusion_valid(argument_checker, fusion)
  .is_edb_valid(argument_checker, edb, fusion)
  .is_bamfile_bedgraphfile_valid(
    argument_checker,
    bamfile,
    bedgraphfile
  )
  .is_which_transcripts_valid(
    argument_checker,
    which_transcripts,
    fusion)
  .is_ylim_valid(argument_checker, ylim)
  .is_parameter_boolean(
    argument_checker,
    non_ucsc,
    "non_ucsc")
  .is_parameter_boolean(
    argument_checker,
    reduce_transcripts,
    "reduce_transcripts")

  # Return errors and warnings (if any)
  checkmate::reportAssertions(argument_checker)
}

#' Find a specific fusion object in a list by id
#'
#' Helper function to retrieve the Fusion object with the given id.
#'
#' @param fusion_list A list of Fusion objects.
#' @param id The id (e.g. the cluster_id from a deFuse run) we're looking for.
#'
#' @return A Fusion object.
#'
#' @examples
#' defuse833ke <- system.file(
#'   "extdata",
#'   "defuse_833ke_results.filtered.tsv",
#'   package="chimeraviz")
#' fusions <- import_defuse(defuse833ke, "hg19", 1)
#' fusion <- get_fusion_by_id(fusions, 5267)
#' # This should be the Fusion object:
#' fusion
#' # [1] "Fusion object"
#' # [1] "id: 5267"
#' # [1] "Fusion tool: defuse"
#' # [1] "Genome version: hg19"
#' # [1] "Gene names: RCC1-HENMT1"
#' # [1] "Chromosomes: chr1-chr1"
#' # [1] "Strands: +,-"
#'
#' @export
get_fusion_by_id <- function(fusion_list, id) {
  result <- Filter(function(fusion) fusion@id == id, fusion_list)
  if (length(result) == 1) {
    return(result[[1]])
  } else {
    stop("Could not find fusion.")
  }
}

#' Find fusions that includes the given gene.
#'
#' Helper function to retrieve the Fusion objects that has geneName as one of
#' the partner genes.
#'
#' Note: get_fusion_by_gene_name(fusionList, "MT") will match both MT-ND5 and
#' MT-ND4.
#'
#' @param fusion_list A list of Fusion objects.
#' @param gene_name The gene name we're looking for.
#'
#' @return A list of Fusion objects.
#'
#' @examples
#' defuse833ke <- system.file(
#'   "extdata",
#'   "defuse_833ke_results.filtered.tsv",
#'   package="chimeraviz")
#' fusions <- import_defuse(defuse833ke, "hg19", 1)
#' length(get_fusion_by_gene_name(fusions, "RCC1"))
#' # [1] 1
#'
#' @export
get_fusion_by_gene_name <- function(fusion_list, gene_name) {
  result <- Filter(
    function(fusion) {
      (grepl(gene_name, fusion@gene_upstream@name) == 1 ||
         grepl(gene_name, fusion@gene_downstream@name) == 1)
      },
    fusion_list
  )
  if (length(result) > 0) {
    return(result)
  } else {
    warning("Could not find fusions involving that gene name.")
    return(result)
  }
}

#' Find fusions that involves genes in the given chromosome.
#'
#' Helper function to retrieve the Fusion objects that involves genes in the
#' given chromosome name.
#'
#' @param fusion_list A list of Fusion objects.
#' @param chr The chromosome name we're looking for fusions in.
#'
#' @return A list of Fusion objects.
#'
#' @examples
#' defuse833ke <- system.file(
#'   "extdata",
#'   "defuse_833ke_results.filtered.tsv",
#'   package="chimeraviz")
#' fusions <- import_defuse(defuse833ke, "hg19", 1)
#' length(get_fusion_by_chromosome(fusions, "chr1"))
#' # [1] 1
#'
#' @export
get_fusion_by_chromosome <- function(fusion_list, chr) {
  result <- Filter(
    function(fusion) {
      (fusion@gene_upstream@chromosome == chr ||
         fusion@gene_downstream@chromosome == chr)
      },
    fusion_list
  )
  if (length(result) > 0) {
    return(result)
  } else {
    warning("Could not find fusions involving that chromosome.")
    return(result)
  }
}

context("Test filter functions that operate on lists of Fusion objects")

defuse833ke <- system.file(
  "extdata",
  "defuse_833ke_results.filtered.tsv",
  package = "chimeraviz")
fusions <- import_defuse(defuse833ke, "hg19", 3)

test_that("get_fusion_by_id works", {
  expect_equal(get_fusion_by_id(fusions, "58")@id, "58")
  expect_equal(get_fusion_by_id(fusions, "5267")@id, "5267")

  expect_error(get_fusion_by_id(fusions, 9999999))
})

test_that("get_fusion_by_gene_name works", {
  expect_equal(length(get_fusion_by_gene_name(fusions, "NEMF")), 1)
  expect_equal(length(get_fusion_by_gene_name(fusions, "HENMT1")), 1)

  expect_warning(get_fusion_by_gene_name(fusions, "Non-existent-gene-name"))
})

test_that("get_fusion_by_chromosome works", {
  expect_equal(length(get_fusion_by_chromosome(fusions, "chr11")), 1)
  expect_equal(length(get_fusion_by_chromosome(fusions, "chr1")), 1)

  expect_warning(get_fusion_by_chromosome(fusions, "Non-existent-chromosome"))
})

#' Import results from a JAFFA run into a list of Fusion objects.
#'
#' A function that imports the results from a JAFFA run, typically from a
#' jaffa_results.csv file, into a list of Fusion objects.
#'
#' @param filename Filename for the jaffa_results.csv file.
#' @param genome_version Which genome was used in mapping (hg19, hg38, etc.).
#' @param limit A limit on how many lines to read.
#'
#' @return A list of Fusion objects.
#'
#' @examples
#' jaffaData <- system.file(
#'   "extdata",
#'   "jaffa_results.csv",
#'   package = "chimeraviz")
#' fusions <- import_jaffa(jaffaData, "hg19", 3)
#' # This should import a list of 3 fusions described in Fusion objects.
#'
#' @importFrom data.table fread
#'
#' @export
import_jaffa <- function(filename, genome_version, limit) {

  # Is the genome version valid?
  valid_genomes <- c("hg19", "hg38", "mm10")
  if (is.na(match(tolower(genome_version), tolower(valid_genomes)))) {
    stop("Invalid genome version given")
  }

  # If the limit is set, is the value valid?
  if (missing(limit) == FALSE) {
    if (is.numeric(limit) == FALSE || limit <= 0) {
      stop("limit must be a numeric value bigger than 0")
    }
  }

  # Try to read the fusion report
  report <- withCallingHandlers({
      col_types <- c(
        "sample" = "character",
        "fusion genes" = "character",
        "chrom1" = "character",
        "base1" = "integer",
        "chrom2" = "character",
        "base2" = "integer",
        "spanning pairs" = "character",
        "spanning reads" = "integer",
        "inframe" = "character",
        "classification" = "character"
      )
      if (missing(limit)) {
        # Read all lines
        data.table::fread(
          input = filename,
          colClasses = col_types,
          showProgress = FALSE
        )
      } else {
        # Only read up to the limit
        data.table::fread(
          input = filename,
          colClasses = col_types,
          showProgress = FALSE,
          nrows = limit
        )
      }
    },
    error = function(cond) {
      message(paste0("Reading ", filename, " caused an error: ", cond[[1]]))
      stop(cond)
    },
    warning = function(cond) {
      # Begin Exclude Linting
      #message(paste0("Reading ", filename, " caused a warning: ", cond[[1]]))
      # End Exclude Linting
    }
  )

  # Set variables
  id                   <- NA
  inframe              <- NA
  fusion_tool          <- "jaffa"
  spanning_reads_count <- NA

  # List to hold all Fusion objects
  fusion_list <- vector("list", dim(report)[1])

  # Iterate through each line in the .tsv file
  for (i in 1:dim(report)[1]) {

    # Import jaffa-specific fields
    fusion_tool_specific_data <- list()
    fusion_tool_specific_data[["classification"]] <-
      report[[i, "classification"]]

    # id for this fusion
    id <- as.character(i)

    # Is the downstream fusion partner in-frame?
    if (is.na(report[[i, "inframe"]])) {
      inframe <- NA
    } else if (report[[i, "inframe"]]) {
      inframe <- TRUE
    } else {
      inframe <- FALSE
    }

    # Chromosome names
    chromosome_upstream <- report[[i, "chrom1"]]
    chromosome_downstream <- report[[i, "chrom2"]]

    # Breakpoints
    breakpoint_upstream <- report[[i, "base1"]]
    breakpoint_downstream <- report[[i, "base2"]]

    # Strand
    strand_upstream <- "*"
    strand_downstream <- "*"

    # Number of supporting reads
    split_reads_count <- as.numeric(report[[i, "spanning reads"]])
    # In the JAFFA example run output the "spanning pairs" field had a couple of
    # "-" values. This causes problems, as we need this to be numeric. If it's
    # indeed a character, just set it to 0:
    spanning_reads_count <- tryCatch(
      as.numeric(report[[i, "spanning pairs"]]),
      warning = function(w) {
        0
      }
    )

    # Not including the fusion junction sequence for JAFFA
    junction_sequence_upstream <- Biostrings::DNAString()
    junction_sequence_downstream <- Biostrings::DNAString()

    # Gene names
    gene_names <- unlist(strsplit(report[[i, "fusion genes"]], split = ":"))
    name_upstream <- gene_names[1]
    name_downstream <- gene_names[2]

    # Ensembl ids
    ensembl_id_upstream <- NA_character_
    ensembl_id_downstream <- NA_character_

    # PartnerGene objects
    gene_upstream <- new(
      Class = "PartnerGene",
      name = name_upstream,
      ensembl_id = ensembl_id_upstream,
      chromosome = chromosome_upstream,
      breakpoint = breakpoint_upstream,
      strand = strand_upstream,
      junction_sequence = junction_sequence_upstream,
      transcripts = GenomicRanges::GRangesList()
    )
    gene_downstream <- new(
      Class = "PartnerGene",
      name = name_downstream,
      ensembl_id = ensembl_id_downstream,
      chromosome = chromosome_downstream,
      breakpoint = breakpoint_downstream,
      strand = strand_downstream,
      junction_sequence = junction_sequence_downstream,
      transcripts = GenomicRanges::GRangesList()
    )

    fusion_list[[i]] <- new(
      Class = "Fusion",
      id = id,
      fusion_tool = fusion_tool,
      genome_version = genome_version,
      spanning_reads_count = spanning_reads_count,
      split_reads_count = split_reads_count,
      fusion_reads_alignment = Gviz::AlignmentsTrack(),
      gene_upstream = gene_upstream,
      gene_downstream = gene_downstream,
      inframe = inframe,
      fusion_tool_specific_data = fusion_tool_specific_data
    )
  }

  # Return the list of Fusion objects
  fusion_list
}

context("Test importing JAFFA data into a list of Fusion objects")

jaffa_data <- system.file(
  "extdata",
  "jaffa_results.csv",
  package = "chimeraviz")
fusions <- import_jaffa(jaffa_data, "hg19", 3)
fusion <- fusions[[1]]

test_that("import_jaffa imports data into Fusion objects", {
  expect_equal(length(fusions), 3)
  expect_equal(fusion@id, "1")
  expect_equal(fusion@gene_upstream@name, "BCAS4")
  expect_equal(fusion@gene_upstream@chromosome, "chr20")
  expect_equal(fusion@gene_upstream@strand, "*")
})

test_that("import_jaffa fails to import data", {
  expect_error(import_jaffa("Non-existent-file", "hg18"))
  expect_error(import_jaffa(jaffa_data, "Invalid-genome"))
  expect_error(import_jaffa(jaffa_data, "hg19", "invalid limit"))
})

test_that("import_jaffa limits work", {
  expect_equal(length(import_jaffa(jaffa_data, "hg19", 2)), 2)
})

#' Import results from a FusionMap run into a list of Fusion objects.
#'
#' A function that imports the results from a FusionMap run, typically from
#' a InputFastq.FusionReport.txt file, into a list of Fusion objects.
#'
#' @param filename Filename for the FusionMap PairedEndFusionReport.txt results
#' file.
#' @param genome_version Which genome was used in mapping (hg19, hg38, etc.).
#' @param limit A limit on how many lines to read.
#'
#' @return A list of Fusion objects.
#'
#' @examples
#' fusionmapData <- system.file(
#'   "extdata",
#'   "FusionMap_01_TestDataset_InputFastq.FusionReport.txt",
#'   package = "chimeraviz")
#' fusions <- import_fusionmap(fusionmapData, "hg19", 3)
#' # This should import a list of 3 fusions described in Fusion objects.
#'
#' @importFrom data.table fread
#'
#' @export
import_fusionmap <- function(filename, genome_version, limit) {

  # Is the genome version valid?
  valid_genomes <- c("hg19", "hg38", "mm10")
  if (is.na(match(tolower(genome_version), tolower(valid_genomes)))) {
    stop("Invalid genome version given")
  }

  # If the limit is set, is the value valid?
  if (missing(limit) == FALSE) {
    if (is.numeric(limit) == FALSE || limit <= 0) {
      stop("limit must be a numeric value bigger than 0")
    }
  }

  # Try to read the fusion report
  report <- withCallingHandlers({
      col_types <- c(
        "FusionID" = "character",
        "DatasetP2_SimulatedReads.RescuedCount" = "integer",
        "Strand" = "character",
        "Chromosome1" = "character",
        "Position1" = "integer",
        "Chromosome2" = "character",
        "Position2" = "integer",
        "KnownGene1" = "character",
        "KnownGene2" = "character",
        "FusionJunctionSequence" = "character",
        "FrameShift" = "character",
        "FrameShiftClass" = "character"
      )
      if (missing(limit)) {
        # Read all lines
        data.table::fread(
          input = filename,
          colClasses = col_types,
          showProgress = FALSE
        )
      } else {
        # Only read up to the limit
        data.table::fread(
          input = filename,
          colClasses = col_types,
          showProgress = FALSE,
          nrows = limit
        )
      }
    },
    error = function(cond) {
      message(paste0("Reading ", filename, " caused an error: ", cond[[1]]))
      stop(cond)
    },
    warning = function(cond) {
      # Begin Exclude Linting
      message(paste0("Reading ", filename, " caused a warning: ", cond[[1]]))
      # End Exclude Linting
    }
  )

  # Set variables
  id                   <- NA
  inframe              <- NA
  fusion_tool          <- "fusionmap"
  spanning_reads_count <- NA
  split_reads_count    <- NA
  junction_sequence    <- NA

  # List to hold all Fusion objects
  fusion_list <- vector("list", dim(report)[1])

  # Iterate through each line in the .tsv file
  for (i in 1:dim(report)[1]) {

    # Import fusionmap-specific fields
    fusion_tool_specific_data <- list()
    fusion_tool_specific_data[["FrameShift"]] <- report[[i, "FrameShift"]]

    # id for this fusion
    id <- report[[i, "FusionID"]]

    # Is the downstream fusion partner in-frame?
    if (report[[i, "FrameShiftClass"]] == "InFrame") {
      inframe <- TRUE
    } else {
      inframe <- FALSE
    }

    # Chromosome names
    chromosome_upstream <- paste("chr", report[[i, "Chromosome1"]], sep = "")
    chromosome_downstream <- paste("chr", report[[i, "Chromosome2"]], sep = "")

    # Breakpoints
    breakpoint_upstream <- report[[i, "Position1"]]
    breakpoint_downstream <- report[[i, "Position2"]]

    # Strand
    strands <- unlist(strsplit(report[[i, "Strand"]], split = ""))
    strand_upstream <- strands[1]
    strand_downstream <- strands[2]

    # Number of supporting reads. We only get one value from a FusionMap P
    # results file, so store that in splitReadsCount
    split_reads_count <- report[[i, 2]]
    spanning_reads_count <- 0

    # Get the fusion sequence. Split it into the part from gene1 and gene2
    junction_sequence <- strsplit(report[[i, "FusionJunctionSequence"]], "\\@")
    junction_sequence_upstream <-
      Biostrings::DNAString(junction_sequence[[1]][1])
    junction_sequence_downstream <-
      Biostrings::DNAString(junction_sequence[[1]][2])

    # Gene names
    name_upstream <- report[[i, "KnownGene1"]]
    name_downstream <- report[[i, "KnownGene2"]]

    # Ensembl ids
    ensembl_id_upstream <- NA_character_
    ensembl_id_downstream <- NA_character_

    # PartnerGene objects
    gene_upstream <- new(
      Class = "PartnerGene",
      name = name_upstream,
      ensembl_id = ensembl_id_upstream,
      chromosome = chromosome_upstream,
      breakpoint = breakpoint_upstream,
      strand = strand_upstream,
      junction_sequence = junction_sequence_upstream,
      transcripts = GenomicRanges::GRangesList()
    )
    gene_downstream <- new(
      Class = "PartnerGene",
      name = name_downstream,
      ensembl_id = ensembl_id_downstream,
      chromosome = chromosome_downstream,
      breakpoint = breakpoint_downstream,
      strand = strand_downstream,
      junction_sequence = junction_sequence_downstream,
      transcripts = GenomicRanges::GRangesList()
    )

    fusion_list[[i]] <- new(
      Class = "Fusion",
      id = id,
      fusion_tool = fusion_tool,
      genome_version = genome_version,
      spanning_reads_count = spanning_reads_count,
      split_reads_count = split_reads_count,
      fusion_reads_alignment = Gviz::AlignmentsTrack(),
      gene_upstream = gene_upstream,
      gene_downstream = gene_downstream,
      inframe = inframe,
      fusion_tool_specific_data = fusion_tool_specific_data
    )
  }

  # Return the list of Fusion objects
  fusion_list
}

context("Test importing FusionMap data into a list of Fusion objects")

fusionmap_data <- system.file(
  "extdata",
  "FusionMap_01_TestDataset_InputFastq.FusionReport.txt",
  package = "chimeraviz")
fusions <- import_fusionmap(fusionmap_data, "hg19", 3)
fusion <- fusions[[1]]

test_that("import_fusionmap imports data into Fusion objects", {
  expect_equal(length(fusions), 3)
  expect_equal(fusion@id, "FUS_11184689_446432191(++)")
  expect_equal(fusion@gene_upstream@name, "FADS3")
  expect_equal(fusion@gene_upstream@chromosome, "chr11")
})

test_that("import_fusionmap fails to import data", {
  expect_error(import_fusionmap("Non-existent-file", "hg18"))
  expect_error(import_fusionmap(fusionmap_data, "Invalid-genome"))
  expect_error(import_fusionmap(fusionmap_data, "hg19", "invalid limit"))
})

test_that("import_fusionmap limits work", {
  expect_equal(length(import_fusionmap(fusionmap_data, "hg19", 2)), 2)
})

#' Import results from a SOAPfuse run into a list of Fusion objects.
#'
#' A function that imports the results from a SOAPfuse run, typically from
#' a final.Fusion.specific.for.genes file, into a list of Fusion objects.
#'
#' @param filename Filename for the SOAPfuse
#' final-list-candidate-fusion-genes.txt results file.
#' @param genome_version Which genome was used in mapping (hg19, hg38, etc.).
#' @param limit A limit on how many lines to read.
#'
#' @return A list of Fusion objects.
#'
#' @examples
#' soapfuse833ke <- system.file(
#'   "extdata",
#'   "soapfuse_833ke_final.Fusion.specific.for.genes",
#'   package = "chimeraviz")
#' fusions <- import_soapfuse(soapfuse833ke, "hg19", 3)
#' # This should import a list of 3 fusions described in Fusion objects.
#'
#' @importFrom data.table fread
#'
#' @export
import_soapfuse <- function(filename, genome_version, limit) {

  # Is the genome version valid?
  valid_genomes <- c("hg19", "hg38", "mm10")
  if (is.na(match(tolower(genome_version), tolower(valid_genomes)))) {
    stop("Invalid genome version given")
  }

  # If the limit is set, is the value valid?
  if (missing(limit) == FALSE) {
    if (is.numeric(limit) == FALSE || limit <= 0) {
      stop("limit must be a numeric value bigger than 0")
    }
  }

  # Try to read the fusion report
  report <- withCallingHandlers({
      col_types <- c(
        "up_gene" = "character",
        "up_chr" = "character",
        "up_strand" = "character",
        "up_Genome_pos" = "integer",
        "dw_gene" = "character",
        "dw_chr" = "character",
        "dw_strand" = "character",
        "dw_Genome_pos" = "integer",
        "Span_reads_num" = "integer",
        "Junc_reads_num" = "integer",
        "down_fusion_part_frame-shift_or_not" = "character"
      )
      if (missing(limit)) {
        # Read all lines
        data.table::fread(
          input = filename,
          colClasses = col_types,
          showProgress = FALSE
        )
      } else {
        # Only read up to the limit
        data.table::fread(
          input = filename,
          colClasses = col_types,
          showProgress = FALSE,
          nrows = limit
        )
      }
    },
    error = function(cond) {
      message(paste0("Reading ", filename, " caused an error: ", cond[[1]]))
      stop(cond)
    },
    warning = function(cond) {
      # Begin Exclude Linting
      message(paste0("Reading ", filename, " caused a warning: ", cond[[1]]))
      # End Exclude Linting
    }
  )

  # Set variables
  id                   <- NA
  inframe              <- NA
  fusion_tool          <- "soapfuse"
  spanning_reads_count <- NA
  split_reads_count    <- NA

  # List to hold all Fusion objects
  fusion_list <- vector("list", dim(report)[1])

  # Iterate through each line in the .tsv file
  for (i in 1:dim(report)[1]) {

    # Import soapfuse-specific fields
    fusion_tool_specific_data <- list()

    # Set id for this fusion. Soapfuse does not have any unique identifier
    # for each row of the results file, so just use our counter.
    id <- as.character(i)

    # Is the downstream fusion partner in-frame?
    if (is.na(report[[i, "down_fusion_part_frame-shift_or_not"]])) {
      inframe <- NA
    } else if (
      report[[i, "down_fusion_part_frame-shift_or_not"]] == "inframe-shift"
    ) {
      inframe <- TRUE
    } else {
      inframe <- FALSE
    }

    # Chromosome names
    chromosome_upstream <- report[[i, "up_chr"]]
    chromosome_downstream <- report[[i, "dw_chr"]]

    # Breakpoints
    breakpoint_upstream <- report[[i, "up_Genome_pos"]]
    breakpoint_downstream <- report[[i, "dw_Genome_pos"]]

    # Strand
    strand_upstream <- report[[i, "up_strand"]]
    strand_downstream <- report[[i, "dw_strand"]]

    # Number of supporting reads
    split_reads_count <- report[[i, "Span_reads_num"]]
    spanning_reads_count <- report[[i, "Junc_reads_num"]]

    # SOAPfuse doesn't provide the fusion sequence
    junction_sequence_upstream <- Biostrings::DNAString()
    junction_sequence_downstream <- Biostrings::DNAString()

    # Gene names
    name_upstream <- report[[i, "up_gene"]]
    name_downstream <- report[[i, "dw_gene"]]

    # Ensembl ids
    ensembl_id_upstream <- NA_character_
    ensembl_id_downstream <- NA_character_

    # PartnerGene objects
    gene_upstream <- new(
      Class = "PartnerGene",
      name = name_upstream,
      ensembl_id = ensembl_id_upstream,
      chromosome = chromosome_upstream,
      breakpoint = breakpoint_upstream,
      strand = strand_upstream,
      junction_sequence = junction_sequence_upstream,
      transcripts = GenomicRanges::GRangesList()
    )
    gene_downstream <- new(
      Class = "PartnerGene",
      name = name_downstream,
      ensembl_id = ensembl_id_downstream,
      chromosome = chromosome_downstream,
      breakpoint = breakpoint_downstream,
      strand = strand_downstream,
      junction_sequence = junction_sequence_downstream,
      transcripts = GenomicRanges::GRangesList()
    )

    fusion_list[[i]] <- new(
      Class = "Fusion",
      id = id,
      fusion_tool = fusion_tool,
      genome_version = genome_version,
      spanning_reads_count = spanning_reads_count,
      split_reads_count = split_reads_count,
      fusion_reads_alignment = Gviz::AlignmentsTrack(),
      gene_upstream = gene_upstream,
      gene_downstream = gene_downstream,
      inframe = inframe,
      fusion_tool_specific_data = fusion_tool_specific_data
    )
  }

  # Return the list of Fusion objects
  fusion_list
}

context("Test importing soapfuse data into a list of Fusion objects")

soapfuse833ke <- system.file(
  "extdata",
  "soapfuse_833ke_final.Fusion.specific.for.genes",
  package = "chimeraviz")
fusions <- import_soapfuse(soapfuse833ke, "hg19", 3)
fusion <- fusions[[1]]

test_that("import_soapfuse imports data into Fusion objects", {
  expect_equal(length(fusions), 3)
  expect_equal(fusion@id, "1")
  expect_equal(fusion@gene_upstream@name, "APLP2")
  expect_equal(fusion@gene_upstream@chromosome, "chr11")
})

test_that("import_soapfuse fails to import data", {
  expect_error(import_soapfuse("Non-existent-file", "hg38"))
  expect_error(import_soapfuse(soapfuse833ke, "Invalid-genome"))
  expect_error(import_soapfuse(soapfuse833ke, "hg38", "invalid limit"))
})

test_that("import_soapfuse limits work", {
  expect_equal(length(import_soapfuse(soapfuse833ke, "hg38", 2)), 2)
})

#' Graph plot of possible fusion transcripts.
#'
#' This function takes a fusion object and a TranscriptDb object and plots a
#' graph showing the possible fusion transcripts.
#'
#' @param fusion The Fusion object to plot.
#' @param edb The edb object that will be used to fetch data.
#' @param which_transcripts This character vector decides which transcripts are
#' to be plotted. Can be "exonBoundary", "withinExon", "withinIntron",
#' "intergenic", or a character vector with specific transcript ids. Default
#' value is "exonBoundary".
#' @param rankdir Choose whether the graph should be plotted from left to right
#' ("LR"), or from top to bottom ("TB"). This parameter is given to
#' Rgraphviz::plot().
#'
#' @return Creates a fusion transcripts graph plot.
#'
#' @examples
#' # Load data and example fusion event
#' defuse833ke <- system.file(
#'   "extdata",
#'   "defuse_833ke_results.filtered.tsv",
#'   package="chimeraviz")
#' fusions <- import_defuse(defuse833ke, "hg19", 1)
#' fusion <- get_fusion_by_id(fusions, 5267)
#' # Load edb
#' edbSqliteFile <- system.file(
#'   "extdata",
#'   "Homo_sapiens.GRCh37.74.sqlite",
#'   package="chimeraviz")
#' edb <- ensembldb::EnsDb(edbSqliteFile)
#' # Temporary file to store the plot
#' pngFilename <- tempfile(
#'   pattern = "fusionPlot",
#'   fileext = ".png",
#'   tmpdir = tempdir())
#' # Open device
#' png(pngFilename, width = 500, height = 500)
#' # Plot!
#' plot_fusion_transcripts_graph(
#'   fusion = fusion,
#'   edb = edb)
#' # Close device
#' dev.off()
#'
#' @export
plot_fusion_transcripts_graph <- function(
  fusion,
  edb = NULL,
  which_transcripts = "exonBoundary",
  rankdir = "TB") {

  .validate_plot_fusion_transcripts_graph_params(
    fusion,
    edb,
    which_transcripts,
    rankdir
  )
  fusion <- .get_transcripts_if_not_there(fusion, edb)

  # Select which transcripts to use
  transcripts_upstream <-
    select_transcript(fusion@gene_upstream, which_transcripts)
  transcripts_downstream <-
    select_transcript(fusion@gene_downstream, which_transcripts)

  # Unlist
  transcripts_upstream <- unlist(transcripts_upstream)
  transcripts_downstream <- unlist(transcripts_downstream)

  # calculate fusion transcripts

  # Select the exons that are part of a possible fusion transcript. I.e. these:
  #
  # 1: geneA is on the + strand
  #    Exons with start/end positions before breakpoint
  # 2: geneA is on the - strand
  #    Exons with start/end positions after breakpoint
  # 3: geneB is on the + strand
  #    Exons with start/end positions after breakpoint
  # 4: geneB is on the - strand
  #    Exons with start/end positions before breakpoint

  if (fusion@gene_upstream@strand == "+") {
    fusion_exons_upstream <-
      transcripts_upstream[
        start(ranges(transcripts_upstream)) < fusion@gene_upstream@breakpoint &
          end(ranges(transcripts_upstream)) <= fusion@gene_upstream@breakpoint
      ]
  } else {
    fusion_exons_upstream <-
      transcripts_upstream[
        start(ranges(transcripts_upstream)) >= fusion@gene_upstream@breakpoint &
          end(ranges(transcripts_upstream)) > fusion@gene_upstream@breakpoint
      ]
  }
  if (fusion@gene_downstream@strand == "+") {
    fusion_exons_downstream <-
      transcripts_downstream[
        start(ranges(transcripts_downstream)) >=
          fusion@gene_downstream@breakpoint &
        end(ranges(transcripts_downstream)) >
          fusion@gene_downstream@breakpoint
      ]
  } else {
    fusion_exons_downstream <-
      transcripts_downstream[
        start(ranges(transcripts_downstream)) <
          fusion@gene_downstream@breakpoint &
        end(ranges(transcripts_downstream)) <=
          fusion@gene_downstream@breakpoint
      ]
  }

  # If either fusionExonsA or fusionExonsB is empty, then the breakpoint does
  # not occur inside any of the transcripts.
  if (length(fusion_exons_upstream) == 0) {
    stop(paste(
      "None of the transcripts given for gene A has the fusion breakpoint ",
      "within them. This plot cannot be created with the given transcripts."))
  }
  if (length(fusion_exons_downstream) == 0) {
    stop(paste(
      "None of the transcripts given for gene B has the fusion breakpoint ",
      "within them. This plot cannot be created with the given transcripts."))
  }

  # For each transcript in fusionExonsA, match the exons in those transcripts..

  transcript_names_upstream <-
    unique(unlist(mcols(fusion_exons_upstream)$transcript))
  transcript_names_downstream <-
    unique(unlist(mcols(fusion_exons_downstream)$transcript))
  fusion_transcripts <- GRangesList()
  for (i in seq_along(transcript_names_upstream)) {
    tr_name_upstream <- transcript_names_upstream[[i]]
    for (j in seq_along(transcript_names_downstream)) {
      tr_name_downstream <- transcript_names_downstream[[j]]

      exons_upstream <- fusion_exons_upstream[
        mcols(fusion_exons_upstream)$transcript == tr_name_upstream
      ]
      exons_downstream <- fusion_exons_downstream[
        mcols(fusion_exons_downstream)$transcript == tr_name_downstream
      ]

      # Reverse order of exons if on minus strand
      if (fusion@gene_upstream@strand == "-") {
        exons_upstream <- rev(exons_upstream)
      }
      if (fusion@gene_downstream@strand == "-") {
        exons_downstream <- rev(exons_downstream)
      }

      new <- append(
        exons_upstream,
        exons_downstream
      )
      mcols(new)$transcript <- paste("fusion", i, j, sep = "")
      fusion_transcripts <- append(
        fusion_transcripts,
        GRangesList(new)
      )
    }
  }

  fusion_transcripts <- unlist(fusion_transcripts)

  exon_names_upstream <-
    unique(
      mcols(
        fusion_transcripts[
          mcols(fusion_transcripts)$gene_id == fusion@gene_upstream@ensembl_id
        ]
      )$exon_id
    )
  exon_names_downstream <-
    unique(
      mcols(
        fusion_transcripts[
          mcols(fusion_transcripts)$gene_id == fusion@gene_downstream@ensembl_id
        ]
      )$exon_id
    )

  # These lists will hold edge information
  from <- c()
  to <- c()

  transcript_names <- sort(unique(mcols(fusion_transcripts)$transcript))
  for (i in seq_along(transcript_names)) {
    exons_in_transcript <-
      fusion_transcripts[
        mcols(fusion_transcripts)$transcript == transcript_names[i]
      ]
    for (j in 2:length(exons_in_transcript)) {
      # Add normal edge
      from <- c(from, mcols(exons_in_transcript[j - 1])$exon_id)
      to <- c(to, mcols(exons_in_transcript[j])$exon_id)
    }
  }

  # Use the edge information we've gathered and create a graph
  ft <- cbind(from, to)
  # Remove edges that go to the same node. Do this because the way
  # get_transcripts_ensembl_db is implemented it will split exons in which the
  # cds starts or ends. The two new exons from such a split will have the same
  # id, causing these edges to appear.
  ft <- ft[ft[, 1] != ft[, 2], ]
  ft[ft[, 1] == ft[, 2], ]

  r_eg <- graph::ftM2graphNEL(unique(ft))

  # Color exons
  cols <- RColorBrewer::brewer.pal(4, "Paired")
  interestcolor <- list(
    "excludedA" = cols[1],
    "includedA" = cols[2],
    "excludedB" = cols[3],
    "includedB" = cols[4])

  # Create a list of exon names with colors
  list_upstream <- rep(interestcolor[2], length(exon_names_upstream))
  names(list_upstream) <- exon_names_upstream
  list_downstream <- rep(interestcolor[4], length(exon_names_downstream))
  names(list_downstream) <- exon_names_downstream

  n_attrs <- list()
  e_attrs <- list()
  n_attrs$fillcolor <- c(
    "start" = "gray",
    list_upstream,
    list_downstream,
    "stop" = "gray")

  Rgraphviz::plot(
    r_eg,
    attrs = list(
      node = list(
        shape = "box",
        fontsize = 42),
      edge = list(),
      graph = list(rankdir = rankdir)), # left-to-right: LR, top-to-bottom: TB
    nodeAttrs = n_attrs,
    edgeAttrs = e_attrs)

}

.validate_plot_fusion_transcripts_graph_params <- function(
  fusion,
  edb,
  which_transcripts,
  rankdir
) {
  # Establish a new 'AssertCollection' object
  argument_checker <- checkmate::makeAssertCollection()

  # Check parameters
  .is_fusion_valid(argument_checker, fusion)
  .is_edb_valid(argument_checker, edb, fusion)
  .is_which_transcripts_valid(
    argument_checker,
    which_transcripts,
    fusion)

  # The "withinExon" option for which_transcripts will not work for the fusion
  # transcripts graph plot. Check if that was the wanted option
  if (which_transcripts[[1]] == "withinExon") {
    argument_checker$push(
      paste0("The \"withinExon\" option for which_transcripts will not ",
             "work for the fusion transcripts graph plot, as only ",
             "complete exons can be shown.")
    )
  }

  if (class(rankdir) != "character" || !rankdir %in% c("LR", "TB")) {
    argument_checker$push(
      "'rankdir' must be one of \"LR\" or \"TB\""
    )
  }

  # Return errors and warnings (if any)
  checkmate::reportAssertions(argument_checker)
}

---
title: "Fusion Report"
output: rmarkdown::html_document
params:
   fusions: !r list()
---

```{r, echo = FALSE, message = FALSE, fig.height = 7, fig.width = 7, fig.align='center', dev='png'}
reportFusions <- params$fusions
plot_circle(reportFusions)
```

```{r, echo = FALSE}
# with the fusion_to_data_frame function, we can use ldply() from the plyr package to create a data frame of our fusion objects:
dfFusions <- plyr::ldply(reportFusions, fusion_to_data_frame)

# with this data frame, we can use datatable() from the DT package to create an html sortable table:
DT::datatable(dfFusions, filter = 'top')
```

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plot_fusion_reads.R
\name{plot_fusion_reads}
\alias{plot_fusion_reads}
\title{Create a plot of the reads supporting the given fusion.}
\usage{
plot_fusion_reads(fusion, show_all_nucleotides = TRUE, nucleotide_amount = 10)
}
\arguments{
\item{fusion}{The Fusion object to plot.}

\item{show_all_nucleotides}{Boolean indicating whether or not to show all
nucleotides. If FALSE, then only nucleotide_amount amount of nucleotides will
be shown on each end of the fusion junction. If TRUE, then the whole fusion
junction sequence will be shown.}

\item{nucleotide_amount}{The number of nucleotides to show on each end of the
fusion junction sequence. Defaults to 10. Only applicable if
show_all_nucleotides is set to TRUE.}
}
\value{
Creates a fusion reads plot.
}
\description{
This function takes a Fusion object and plots the reads supporting the fusion
on top of the fusion sequence (fusion@junction_sequence), provided that
add_fusion_reads_alignment() has been run earlier in order to add fusion reads
alignment data to the fusion object.
}
\details{
Note that the package used for plotting, Gviz, is strict on chromosome names.
If the plot produced doesn't show the reads, the problem might be solved by
naming the fusion sequence "chrNA".
}
\examples{
# Load data
defuseData <- system.file(
  "extdata",
  "defuse_833ke_results.filtered.tsv",
  package="chimeraviz")
fusions <- import_defuse(defuseData, "hg19", 1)
# Find the specific fusion we have aligned reads for
fusion <- get_fusion_by_id(fusions, 5267)
bamfile <- system.file(
  "extdata",
  "5267readsAligned.bam",
  package="chimeraviz")
# Add the bam file of aligned fusion reads to the fusion object
fusion <- add_fusion_reads_alignment(fusion, bamfile)
# Temporary file to store the plot
pngFilename <- tempfile(
  pattern = "fusionPlot",
  fileext = ".png",
  tmpdir = tempdir())
# Calculate image size based on supporting reads and lenght of junction
# sequence.
imageWidth <- (nchar(partner_gene_junction_sequence(upstream_partner_gene(fusion))) +
  nchar(partner_gene_junction_sequence(downstream_partner_gene(fusion)))) * 15
imageHeight <- (fusion_split_reads_count(fusion)+fusion_spanning_reads_count(fusion)) * 20
# Open device
png(pngFilename, width = imageWidth, height = imageHeight)
# Now we can plot
plot_fusion_reads(fusion)
# Close device
dev.off()

}
\seealso{
add_fusion_reads_alignment
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/import_defuse.R
\name{import_defuse}
\alias{import_defuse}
\title{Import results from a deFuse run into a list of Fusion objects.}
\usage{
import_defuse(filename, genome_version, limit)
}
\arguments{
\item{filename}{Filename for the deFuse results .tsv file.}

\item{genome_version}{Which genome was used in mapping (hg19, hg38, etc.).}

\item{limit}{A limit on how many lines to read.}
}
\value{
A list of Fusion objects.
}
\description{
A function that imports the results from a deFuse run, typically from a
results.filtered.tsv file, into a list of Fusion objects.
}
\examples{
defuse833ke <- system.file(
  "extdata",
  "defuse_833ke_results.filtered.tsv",
  package="chimeraviz")
fusions <- import_defuse(defuse833ke, "hg19", 3)
# This should import a list of 3 fusions described in Fusion objects.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plot_fusion.R
\name{plot_fusion}
\alias{plot_fusion}
\alias{plot_fusion_separate}
\alias{plot_fusion_together}
\title{Plot a fusion event with transcripts, coverage and ideograms.}
\usage{
plot_fusion(
  fusion,
  edb = NULL,
  bamfile = NULL,
  which_transcripts = "exonBoundary",
  ylim = c(0, 1000),
  non_ucsc = TRUE,
  reduce_transcripts = FALSE,
  bedgraphfile = NULL
)

plot_fusion_separate(
  fusion,
  edb,
  bamfile = NULL,
  which_transcripts = "exonBoundary",
  ylim = c(0, 1000),
  non_ucsc = TRUE,
  reduce_transcripts = FALSE,
  bedgraphfile = NULL
)

plot_fusion_together(
  fusion,
  edb,
  bamfile = NULL,
  which_transcripts = "exonBoundary",
  ylim = c(0, 1000),
  non_ucsc = TRUE,
  reduce_transcripts = FALSE,
  bedgraphfile = NULL
)
}
\arguments{
\item{fusion}{The Fusion object to plot.}

\item{edb}{The ensembldb object that will be used to fetch data.}

\item{bamfile}{The bamfile with RNA-seq data.}

\item{which_transcripts}{This character vector decides which transcripts are
to be plotted. Can be "exonBoundary", "withinExon", "withinIntron",
"intergenic", or a character vector with specific transcript ids. Default
value is "exonBoundary".}

\item{ylim}{Limits for the coverage y-axis.}

\item{non_ucsc}{Boolean indicating whether or not the bamfile used has UCSC-
styled chromosome names (i.e. with the "chr" prefix). Setting this to true
lets you use a bamfile with chromosome names like "1" and "X", instead of
"chr1" and "chrX".}

\item{reduce_transcripts}{Boolean indicating whether or not to reduce all
transcripts into a single transcript for each partner gene.}

\item{bedgraphfile}{A bedGraph file to use instead of the bamfile to plot
coverage.}
}
\value{
Creates a fusion plot.
}
\description{
This function creates a plot with information about transcripts, coverage,
location and more.
}
\details{
plot_fusion() will dispatch to either plot_fusion_separate() or
plot_fusion_together(). plot_fusion_separate() will plot the fusion gene
partners in separate graphs shown next to each other, while
plot_fusion_together() will plot the fusion gene partners in the same graph
with the same x-axis. plot_fusion() will dispatch to plot_fusion_together() if
the fusion gene partners are on the same strand, same chromosome and are
close together (<=50,000 bp apart).
}
\examples{
# Load data and example fusion event
defuse833ke <- system.file(
  "extdata",
  "defuse_833ke_results.filtered.tsv",
  package="chimeraviz")
fusions <- import_defuse(defuse833ke, "hg19", 1)
fusion <- get_fusion_by_id(fusions, 5267)
# Load edb
edbSqliteFile <- system.file(
  "extdata",
  "Homo_sapiens.GRCh37.74.sqlite",
  package="chimeraviz")
edb <- ensembldb::EnsDb(edbSqliteFile)
# bamfile with reads in the regions of this fusion event
bamfile5267 <- system.file(
  "extdata",
  "fusion5267and11759reads.bam",
  package="chimeraviz")
# Temporary file to store the plot
pngFilename <- tempfile(
  pattern = "fusionPlot",
  fileext = ".png",
  tmpdir = tempdir())
# Open device
png(pngFilename, width = 1000, height = 750)
# Plot!
plot_fusion(
  fusion = fusion,
  bamfile = bamfile5267,
  edb = edb,
  non_ucsc = TRUE)
# Close device
dev.off()

# Example using a .bedGraph file instead of a .bam file:
# Load data and example fusion event
defuse833ke <- system.file(
  "extdata",
  "defuse_833ke_results.filtered.tsv",
  package="chimeraviz")
fusions <- import_defuse(defuse833ke, "hg19", 1)
fusion <- get_fusion_by_id(fusions, 5267)
# Load edb
edbSqliteFile <- system.file(
  "extdata",
  "Homo_sapiens.GRCh37.74.sqlite",
  package="chimeraviz")
edb <- ensembldb::EnsDb(edbSqliteFile)
# bedgraphfile with coverage data from the regions of this fusion event
bedgraphfile <- system.file(
  "extdata",
  "fusion5267and11759reads.bedGraph",
  package="chimeraviz")
# Temporary file to store the plot
pngFilename <- tempfile(
  pattern = "fusionPlot",
  fileext = ".png",
  tmpdir = tempdir())
# Open device
png(pngFilename, width = 1000, height = 750)
# Plot!
plot_fusion(
  fusion = fusion,
  bedgraphfile = bedgraphfile,
  edb = edb,
  non_ucsc = TRUE)
# Close device
dev.off()

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/AllClasses.R
\name{fusion_spanning_reads_count}
\alias{fusion_spanning_reads_count}
\alias{fusion_spanning_reads_count,Fusion-method}
\title{Get the spanning reads count from a Fusion object}
\usage{
fusion_spanning_reads_count(x)

\S4method{fusion_spanning_reads_count}{Fusion}(x)
}
\arguments{
\item{x}{The Fusion object you wish to retrieve the spanning reads count for.}
}
\value{
The Fusion spanning reads count.
}
\description{
This getter retrieves the spanning reads count from a Fusion object
}
\examples{
# Load data
defuseData <- system.file(
  "extdata",
  "defuse_833ke_results.filtered.tsv",
  package="chimeraviz")
fusions <- import_defuse(defuseData, "hg19", 1)
fusion <- fusions[[1]]
# Get the spanning reads count
fusion_spanning_reads_count(fusion)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/import_ericscript.R
\name{import_ericscript}
\alias{import_ericscript}
\title{Import results from a EricScript run into a list of Fusion objects.}
\usage{
import_ericscript(filename, genome_version, limit)
}
\arguments{
\item{filename}{Filename for the EricScript results file.}

\item{genome_version}{Which genome was used in mapping (hg19, hg38, etc.).}

\item{limit}{A limit on how many lines to read.}
}
\value{
A list of Fusion objects.
}
\description{
A function that imports the results from a EricScript run into a list of
Fusion objects.
}
\examples{
ericscriptData <- system.file(
  "extdata",
  "ericscript_SRR1657556.results.total.tsv",
  package = "chimeraviz")
fusions <- import_ericscript(ericscriptData, "hg19", 3)
# This should import a list of 3 fusions described in Fusion objects.

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utilities.R
\name{split_on_utr_and_add_feature}
\alias{split_on_utr_and_add_feature}
\title{Split GRanges object based on cds}
\usage{
split_on_utr_and_add_feature(gr)
}
\arguments{
\item{gr}{The GRanges object we want to split and tag with feature info.}
}
\value{
An updated GRanges object with feature values set.
}
\description{
This function will look for ranges (exons) in the GRanges object that has the
coding DNA sequence starting or stopping within it. If found, these exons are
split, and each exon in the GRanges object will be tagged as either
"protein_coding", "5utr", or "3utr". The returned GRanges object will have
feature values set in mcols(gr)$feature reflecting this.
}
\examples{
# Load fusion data and choose a fusion object:
defuseData <- system.file(
  "extdata",
  "defuse_833ke_results.filtered.tsv",
  package="chimeraviz")
fusions <- import_defuse(defuseData, "hg19", 1)
fusion <- get_fusion_by_id(fusions, 5267)
# Create edb object
edbSqliteFile <- system.file(
  "extdata",
  "Homo_sapiens.GRCh37.74.sqlite",
  package="chimeraviz")
edb <- ensembldb::EnsDb(edbSqliteFile)
# Get all exons for all transcripts in the genes in the fusion transcript
allTranscripts <- ensembldb::exonsBy(
  edb,
  filter = list(
    AnnotationFilter::GeneIdFilter(
      c(
        partner_gene_ensembl_id(upstream_partner_gene(fusion)),
        partner_gene_ensembl_id(downstream_partner_gene(fusion))))),
  columns = c(
    "gene_id",
    "gene_name",
    "tx_id",
    "tx_cds_seq_start",
    "tx_cds_seq_end",
    "exon_id"))
# Extract one of the GRanges objects
gr <- allTranscripts[[1]]
# Check how many ranges there are here
length(gr)
# Should be 9 ranges
# Split the ranges containing the cds start/stop positions and add feature
# values:
gr <- split_on_utr_and_add_feature(gr)
# Check the length again
length(gr)
# Should be 11 now, as the range containing the cds_strat position and the
# range containing the cds_stop position has been split into separate ranges

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utilities.R
\name{select_transcript}
\alias{select_transcript}
\title{Select which transcript to use (for plotting) for a GenePartner object}
\usage{
select_transcript(gene_partner, which_transcripts = "exonBoundary")
}
\arguments{
\item{gene_partner}{The GenePartner object to select a transcript for.}

\item{which_transcripts}{This character vector decides which transcripts are
to be plotted. Can be "exonBoundary", "withinExon", "withinIntron",
"intergenic", or a character vector with specific transcript ids. Default
value is "exonBoundary".}
}
\value{
A data.frame with transcript data.
}
\description{
This function takes a GenePartner object and creates a transcript data.frame
with transcript information, including only the transcripts given by the
parameter which_transcripts
}
\details{
select_transcript() selects which transcript to create by this prioritization:

1. Exon boundary transcripts.
2. Within exon transcripts.
3. Within intron transcripts.
4. Intergenic transcripts.
}
\examples{
# Load data and example fusion event
defuse833ke <- system.file(
  "extdata",
  "defuse_833ke_results.filtered.tsv",
  package="chimeraviz")
fusions <- import_defuse(defuse833ke, "hg19", 1)
fusion <- get_fusion_by_id(fusions, 5267)
# Load edb
edbSqliteFile <- system.file(
  "extdata",
  "Homo_sapiens.GRCh37.74.sqlite",
  package="chimeraviz")
edb <- ensembldb::EnsDb(edbSqliteFile)
# Get transcripts
fusion <- get_transcripts_ensembl_db(fusion, edb)
# Select transcript
transcriptsA <- select_transcript(upstream_partner_gene(fusion))

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utilities.R
\name{down_shift}
\alias{down_shift}
\title{Remove introns and shift exons leftward}
\usage{
down_shift(transcript)
}
\arguments{
\item{transcript}{The GRanges object to remove introns from.}
}
\value{
A GRanges object with introns removed.
}
\description{
This function takes a GRanges object and moves each IRanges object within
next to each other starting at 1. This effectively removes the introns from
the GRanges object.
}
\examples{
# Create a simple GRanges object:
gr <- IRanges::IRanges(
  start = c(13, 40, 100),
  end = c(20, 53, 110))
# Downshift it and see the introns are removed:
down_shift(gr)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/AllClasses.R
\name{partner_gene_junction_sequence}
\alias{partner_gene_junction_sequence}
\alias{partner_gene_junction_sequence,PartnerGene-method}
\title{Get the junction sequence from a PartnerGene object}
\usage{
partner_gene_junction_sequence(x)

\S4method{partner_gene_junction_sequence}{PartnerGene}(x)
}
\arguments{
\item{x}{The PartnerGene object you wish to retrieve the junction sequence
for.}
}
\value{
The upstream fusion partner gene junction sequence.
}
\description{
This getter retrieves the junction sequence from a PartnerGene object
}
\examples{
# Load data
defuseData <- system.file(
  "extdata",
  "defuse_833ke_results.filtered.tsv",
  package="chimeraviz")
fusions <- import_defuse(defuseData, "hg19", 1)
fusion <- fusions[[1]]
# Get the junction sequence from the upstream fusion partner gene
partner_gene_junction_sequence(upstream_partner_gene(fusion))

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/import_infusion.R
\name{import_infusion}
\alias{import_infusion}
\title{Import results from an InFusion run into a list of Fusion objects.}
\usage{
import_infusion(filename, genome_version, limit)
}
\arguments{
\item{filename}{Filename for the jaffa_results.csv file.}

\item{genome_version}{Which genome was used in mapping (hg19, hg38, etc.).}

\item{limit}{A limit on how many lines to read.}
}
\value{
A list of Fusion objects.
}
\description{
A function that imports the results from an InFusion run nto a list of Fusion
objects.
}
\examples{
infusionData <- system.file(
  "extdata",
  "infusion_fusions.txt",
  package = "chimeraviz")
fusions <- import_infusion(infusionData, "hg19", 3)
# This should import a list of 3 fusions described in Fusion objects.

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/import_chimpipe.R
\name{import_chimpipe}
\alias{import_chimpipe}
\title{Import results from a ChimPipe run into a list of Fusion objects.}
\usage{
import_chimpipe(filename, genome_version, limit)
}
\arguments{
\item{filename}{Filename for the ChimPipe results.}

\item{genome_version}{Which genome was used in mapping (hg19, hg38, etc.).}

\item{limit}{A limit on how many lines to read.}
}
\value{
A list of Fusion objects.
}
\description{
A function that imports the results from a ChimPipe run, typically from a
chimericJunctions_.txt file, into a list of Fusion objects.
}
\examples{
chimpipefile <- system.file(
  "extdata",
  "chimericJunctions_MCF-7.txt",
  package="chimeraviz")
fusions <- import_chimpipe(chimpipefile, "hg19", 3)
# This should import a list of 3 fusions described in Fusion objects.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utilities.R
\name{fusion_to_data_frame}
\alias{fusion_to_data_frame}
\title{Coerce Fusion object to data.frame}
\usage{
fusion_to_data_frame(fusion)
}
\arguments{
\item{fusion}{The Fusion object to coerce.}
}
\value{
A data.frame with the fusion object.
}
\description{
This function is used in create_fusion_report() to convert Fusion objects to a
data.frame-format.
}
\examples{
# Load data
defuse833ke <- system.file(
  "extdata",
  "defuse_833ke_results.filtered.tsv",
  package="chimeraviz")
fusions <- import_defuse(defuse833ke, "hg19", 1)
# Find the fusion object to create a data frame from
fusion <- get_fusion_by_id(fusions, 5267)
# Create the data frame
dfFusion <- fusion_to_data_frame(fusion)

}
\seealso{
create_fusion_report
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/reports.R
\name{create_fusion_report}
\alias{create_fusion_report}
\title{Create a Fusion Report}
\usage{
create_fusion_report(fusions, output_filename, quiet = TRUE)
}
\arguments{
\item{fusions}{A list of Fusion objects.}

\item{output_filename}{Output html-file filename.}

\item{quiet}{Parameter passed to rmarkdown::render() to toggle its output.}
}
\value{
Creates a html report with an overplot and a sortable, searchable
table with the fusion data.
}
\description{
This function will create a html report with an overplot and a sortable,
searchable table with the fusion data.
}
\examples{
# Load data
defuse833ke <- system.file(
  "extdata",
  "defuse_833ke_results.filtered.tsv",
  package="chimeraviz")
fusions <- import_defuse(defuse833ke, "hg19", 3)
# Temporary file to store the report
random_filename <- paste0(
  paste0(sample(LETTERS, 5, replace = TRUE), collapse=''),
  ".png"
)
# Create report
create_fusion_report(fusions, random_filename)
# Delete the file
file.remove(random_filename)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plot_circle.R
\name{chimeraviz-internals-scaleListToInterval}
\alias{chimeraviz-internals-scaleListToInterval}
\alias{.scale_list_to_interval}
\title{Scale a vector of numeric values to an interval.}
\usage{
.scale_list_to_interval(the_list, new_min, new_max)
}
\arguments{
\item{the_list}{A vector of numeric values.}

\item{new_min}{Minimum value for the new interval.}

\item{new_max}{Maximum value for the new interval.}
}
\value{
A data frame with fusion link data compatible with RCircos::RCircos.Link.Plot()

# @examples # Apparently examples shouldn't be set on private functions
list012 <- c(0,1,2)
.scale_list_to_interval(list012, 1, 3)
# [1] 1 2 3
}
\description{
This function takes a vector of numeric values as well as an interval
[new_min, new_max] that the numeric values will be scaled (normalized) to.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/extdata.R
\name{raw_fusion5267proteindomains}
\alias{raw_fusion5267proteindomains}
\title{protein_domains_5267 bed file}
\description{
Documentation for the protein_domains_5267.bed file containing protein
domains for the genes in the fusion with cluster_id=5267.
}
\section{protein_domains_5267.bed}{


This file is an excerpt from a larger file that we created by:
- downloading domain name annotation from Pfam database (PfamA version 31)
  and domain region annotation from Ensembl database through BioMart API
- switching the domain coordinates in the protein level to these in
  transcript level.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utilities.R
\name{decide_transcript_category}
\alias{decide_transcript_category}
\title{Retrieves transcripts for partner genes in a Fusion object using Ensembldb}
\usage{
decide_transcript_category(gr, fusion)
}
\arguments{
\item{gr}{The GRanges object containing the transcript to be checked.}

\item{fusion}{The fusion object used to check the transcript.}
}
\value{
Either "exonBoundary", "withinExon", "withinIntron", or "intergenic"
depending on where in the transcript the breakpoint hits.
}
\description{
This function will check where in the transcript (the GRanges object) the
fusion breakpoint is located, and return either "exonBoundary", "withinExon",
"withinIntron", or "intergenic".
}
\examples{
# Load fusion data and choose a fusion object:
defuseData <- system.file(
  "extdata",
  "defuse_833ke_results.filtered.tsv",
  package="chimeraviz")
fusions <- import_defuse(defuseData, "hg19", 1)
fusion <- get_fusion_by_id(fusions, 5267)
# Create edb object
edbSqliteFile <- system.file(
  "extdata",
  "Homo_sapiens.GRCh37.74.sqlite",
  package="chimeraviz")
edb <- ensembldb::EnsDb(edbSqliteFile)
# Get all exons for all transcripts in the genes in the fusion transcript
allTranscripts <- ensembldb::exonsBy(
  edb,
  filter = list(
    AnnotationFilter::GeneIdFilter(
      c(
        partner_gene_ensembl_id(upstream_partner_gene(fusion)),
        partner_gene_ensembl_id(downstream_partner_gene(fusion))))),
  columns = c(
    "gene_id",
    "gene_name",
    "tx_id",
    "tx_cds_seq_start",
    "tx_cds_seq_end",
    "exon_id"))
# Extract one of the GRanges objects
gr <- allTranscripts[[1]]
# Check where in the transcript the fusion breakpoint hits
decide_transcript_category(gr, fusion)
# "exonBoundary"
# Check another case
gr <- allTranscripts[[3]]
decide_transcript_category(gr, fusion)
# "withinIntron"

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/import_soapfuse.R
\name{import_soapfuse}
\alias{import_soapfuse}
\title{Import results from a SOAPfuse run into a list of Fusion objects.}
\usage{
import_soapfuse(filename, genome_version, limit)
}
\arguments{
\item{filename}{Filename for the SOAPfuse
final-list-candidate-fusion-genes.txt results file.}

\item{genome_version}{Which genome was used in mapping (hg19, hg38, etc.).}

\item{limit}{A limit on how many lines to read.}
}
\value{
A list of Fusion objects.
}
\description{
A function that imports the results from a SOAPfuse run, typically from
a final.Fusion.specific.for.genes file, into a list of Fusion objects.
}
\examples{
soapfuse833ke <- system.file(
  "extdata",
  "soapfuse_833ke_final.Fusion.specific.for.genes",
  package = "chimeraviz")
fusions <- import_soapfuse(soapfuse833ke, "hg19", 3)
# This should import a list of 3 fusions described in Fusion objects.

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/get_fusion_by.R
\name{get_fusion_by_chromosome}
\alias{get_fusion_by_chromosome}
\title{Find fusions that involves genes in the given chromosome.}
\usage{
get_fusion_by_chromosome(fusion_list, chr)
}
\arguments{
\item{fusion_list}{A list of Fusion objects.}

\item{chr}{The chromosome name we're looking for fusions in.}
}
\value{
A list of Fusion objects.
}
\description{
Helper function to retrieve the Fusion objects that involves genes in the
given chromosome name.
}
\examples{
defuse833ke <- system.file(
  "extdata",
  "defuse_833ke_results.filtered.tsv",
  package="chimeraviz")
fusions <- import_defuse(defuse833ke, "hg19", 1)
length(get_fusion_by_chromosome(fusions, "chr1"))
# [1] 1

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utilities.R
\name{fetch_reads_from_fastq}
\alias{fetch_reads_from_fastq}
\title{Fetch reads from fastq files}
\usage{
fetch_reads_from_fastq(
  reads,
  fastq_file_in1,
  fastq_file_in2,
  fastq_file_out1,
  fastq_file_out2
)
}
\arguments{
\item{reads}{List of read IDs that is to be fetched.}

\item{fastq_file_in1}{First fastq file to search in.}

\item{fastq_file_in2}{Second fastq file to seach in.}

\item{fastq_file_out1}{First fastq file with results.}

\item{fastq_file_out2}{Second fastq file with results.}
}
\value{
The files fastqFileOut1 and fastqFileOut2 populated with the
specified reads.
}
\description{
This function will fetch read sequences from fastq files and put them into
new fastq files.
}
\details{
Note: This function runs (read only) bash commands on your system. Therefore
the function will only work on a unix system.
}
\examples{
\dontrun{
# fastq files that has the supporting reads
fastq1 <- system.file("extdata", "reads.1.fq", package="chimeraviz")
fastq2 <- system.file("extdata", "reads.2.fq", package="chimeraviz")
# Which read ids to extract
reads <- c(
  "13422259", "19375605", "29755061",
  "31632876", "32141428", "33857245")
# Extract the actual reads and put them in the tmp files "fastqFileOut1" and
# "fastqFileOut2"
fastqFileOut1 <- tempfile(pattern = "fq1", tmpdir = tempdir())
fastqFileOut2 <- tempfile(pattern = "fq2", tmpdir = tempdir())
fetch_reads_from_fastq(reads, fastq1, fastq2,
    fastqFileOut1, fastqFileOut2)
# We now have the reads supporting fusion 5267 in the two files.
}

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/import_fusionmap.R
\name{import_fusionmap}
\alias{import_fusionmap}
\title{Import results from a FusionMap run into a list of Fusion objects.}
\usage{
import_fusionmap(filename, genome_version, limit)
}
\arguments{
\item{filename}{Filename for the FusionMap PairedEndFusionReport.txt results
file.}

\item{genome_version}{Which genome was used in mapping (hg19, hg38, etc.).}

\item{limit}{A limit on how many lines to read.}
}
\value{
A list of Fusion objects.
}
\description{
A function that imports the results from a FusionMap run, typically from
a InputFastq.FusionReport.txt file, into a list of Fusion objects.
}
\examples{
fusionmapData <- system.file(
  "extdata",
  "FusionMap_01_TestDataset_InputFastq.FusionReport.txt",
  package = "chimeraviz")
fusions <- import_fusionmap(fusionmapData, "hg19", 3)
# This should import a list of 3 fusions described in Fusion objects.

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utilities.R
\name{write_fusion_reference}
\alias{write_fusion_reference}
\title{Write fusion junction sequence to a fasta file}
\usage{
write_fusion_reference(fusion, filename)
}
\arguments{
\item{fusion}{The Fusion object we want to create a fasta file from.}

\item{filename}{The filename to write to.}
}
\value{
Writes the fusion junction sequence to the given filename.
}
\description{
This function will write the fusion sequence to a fasta file, using
Biostring::writeXStringSet() .
}
\examples{
# Import the filtered defuse results
defuse833keFiltered <- system.file(
  "extdata",
  "defuse_833ke_results.filtered.tsv",
  package="chimeraviz")
fusions <- import_defuse(defuse833keFiltered, "hg19", 1)
# Get a specific fusion
fusion <- get_fusion_by_id(fusions, 5267)
# Create temporary file to hold the fusion sequence
fastaFileOut <- tempfile(pattern = "fusionSequence", tmpdir = tempdir())
# Write fusion sequence to file
write_fusion_reference(fusion, fastaFileOut)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/get_fusion_by.R
\name{get_fusion_by_id}
\alias{get_fusion_by_id}
\title{Find a specific fusion object in a list by id}
\usage{
get_fusion_by_id(fusion_list, id)
}
\arguments{
\item{fusion_list}{A list of Fusion objects.}

\item{id}{The id (e.g. the cluster_id from a deFuse run) we're looking for.}
}
\value{
A Fusion object.
}
\description{
Helper function to retrieve the Fusion object with the given id.
}
\examples{
defuse833ke <- system.file(
  "extdata",
  "defuse_833ke_results.filtered.tsv",
  package="chimeraviz")
fusions <- import_defuse(defuse833ke, "hg19", 1)
fusion <- get_fusion_by_id(fusions, 5267)
# This should be the Fusion object:
fusion
# [1] "Fusion object"
# [1] "id: 5267"
# [1] "Fusion tool: defuse"
# [1] "Genome version: hg19"
# [1] "Gene names: RCC1-HENMT1"
# [1] "Chromosomes: chr1-chr1"
# [1] "Strands: +,-"

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/AllClasses.R
\name{fusion_split_reads_count}
\alias{fusion_split_reads_count}
\alias{fusion_split_reads_count,Fusion-method}
\title{Get the split reads count from a Fusion object}
\usage{
fusion_split_reads_count(x)

\S4method{fusion_split_reads_count}{Fusion}(x)
}
\arguments{
\item{x}{The Fusion object you wish to retrieve the split reads count for.}
}
\value{
The Fusion split reads count.
}
\description{
This getter retrieves the split reads count from a Fusion object
}
\examples{
# Load data
defuseData <- system.file(
  "extdata",
  "defuse_833ke_results.filtered.tsv",
  package="chimeraviz")
fusions <- import_defuse(defuseData, "hg19", 1)
fusion <- fusions[[1]]
# Get the split reads count
fusion_split_reads_count(fusion)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/AllClasses.R
\name{show,Fusion-method}
\alias{show,Fusion-method}
\title{Show method for the Fusion class.}
\usage{
\S4method{show}{Fusion}(object)
}
\arguments{
\item{object}{A Fusion object}
}
\value{
Shows information about a Fusion object.
}
\description{
Show method for the Fusion class.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utilities.R
\name{add_fusion_reads_alignment}
\alias{add_fusion_reads_alignment}
\title{Add fusion reads alignment to fusion object}
\usage{
add_fusion_reads_alignment(fusion, bamfile)
}
\arguments{
\item{fusion}{The fusion object to add a genomic alignment to.}

\item{bamfile}{The bam file containing the fusion reads plotted to the fusion
sequence.}
}
\value{
An updated fusion object with fusion@fusion_reads_alignment set.
}
\description{
This function lets you add a fusion read alignment file to a fusion object.
If you've mapped the reads supporting a fusion against the fusion junction
sequence, and have the resulting bamfile, use this function to add the
information (as a Gviz::GAlignmentPairs object) to the fusion object.
}
\examples{
# Load data
defuse833ke <- system.file(
  "extdata",
  "defuse_833ke_results.filtered.tsv",
  package="chimeraviz")
fusions <- import_defuse(defuse833ke, "hg19", 1)
# Find the specific fusion we have aligned reads for
fusion <- get_fusion_by_id(fusions, 5267)
# Get reference to the bamfile with the alignment data
bamfile5267 <- system.file(
  "extdata",
  "5267readsAligned.bam",
  package="chimeraviz")
# Add the bam file of aligned fusion reads to the fusion object
fusion <- add_fusion_reads_alignment(fusion, bamfile5267)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/import_prada.R
\name{import_prada}
\alias{import_prada}
\title{Import results from a PRADA run into a list of Fusion objects.}
\usage{
import_prada(filename, genome_version, limit)
}
\arguments{
\item{filename}{Filename for the PRADA results file.}

\item{genome_version}{Which genome was used in mapping (hg19, hg38, etc.).}

\item{limit}{A limit on how many lines to read.}
}
\value{
A list of Fusion objects.
}
\description{
A function that imports the results from a PRADA run into a list of Fusion
objects.
}
\examples{
pradaData <- system.file(
  "extdata",
  "PRADA.acc.fusion.fq.TAF.tsv",
  package = "chimeraviz")
fusions <- import_prada(pradaData, "hg19", 3)
# This should import a list of 3 fusions described in Fusion objects.

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/import_fusioncatcher.R
\name{import_fusioncatcher}
\alias{import_fusioncatcher}
\title{Import results from a Fusioncatcher run into a list of Fusion objects.}
\usage{
import_fusioncatcher(filename, genome_version, limit)
}
\arguments{
\item{filename}{Filename for the Fusioncatcher
final-list-candidate-fusion-genes.txt results file.}

\item{genome_version}{Which genome was used in mapping (hg19, hg38, etc.).}

\item{limit}{A limit on how many lines to read.}
}
\value{
A list of Fusion objects.
}
\description{
A function that imports the results from a Fusioncatcher run, typically from
a final-list-candidate-fusion-genes.txt file, into a list of Fusion objects.
}
\examples{
fusioncatcher833ke <- system.file(
  "extdata",
  "fusioncatcher_833ke_final-list-candidate-fusion-genes.txt",
  package = "chimeraviz")
fusions <- import_fusioncatcher(fusioncatcher833ke, "hg38", 3)
# This should import a list of 3 fusions described in Fusion objects.

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plot_circle.R
\name{chimeraviz-internals-fusions_to_link_data}
\alias{chimeraviz-internals-fusions_to_link_data}
\alias{.fusions_to_link_data}
\title{Create link data for RCircos from the given fusions.}
\usage{
.fusions_to_link_data(fusion_list, min_link_width = 1, max_link_widt = 10)
}
\arguments{
\item{fusion_list}{A list of Fusion objects.}

\item{min_link_width}{The minimum link line width. Default = 1}

\item{max_link_widt}{The maximum link line width. Default = 10}
}
\value{
A data frame with fusion link data compatible with RCircos::RCircos.Link.Plot()

# @examples # Apparently examples shouldn't be set on private functions
defuse833ke <- system.file("extdata", "defuse_833ke_results.filtered.tsv", package="chimeraviz")
fusions <- import_defuse(defuse833ke, "hg19", 3)
linkData <- chimeraviz::.fusions_to_link_data(fusions)
# This linkData can be used with RCircos::RCircos.Link.Plot()
}
\description{
This function takes a list of Fusion objects and creates a data frame in the
format that RCircos::RCircos.Link.Plot() expects for link data.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utilities.R
\name{get_transcripts_ensembl_db}
\alias{get_transcripts_ensembl_db}
\title{Retrieves transcripts for partner genes in a Fusion object using Ensembldb}
\usage{
get_transcripts_ensembl_db(fusion, edb)
}
\arguments{
\item{fusion}{The fusion object to find transcripts for.}

\item{edb}{The edb object used to fetch data from.}
}
\value{
An updated fusion object with transcript data stored.
}
\description{
This function will retrieve transcripts for both genes in a fusion. It will
check all transcripts and decide for each transcript if the fusion breakpoint
happens at 1) an exon boundary, 2) within an exon, or 3) within an intron.
This is done because fusions happening at exon boundaries are more likely to
produce biologically interesting gene products. The function returns an
updated Fusion object, where the fusion@gene_upstream@transcriptsX slots are set with
transcript information.
}
\examples{
# Load fusion data and choose a fusion object:
defuseData <- system.file(
  "extdata",
  "defuse_833ke_results.filtered.tsv",
  package="chimeraviz")
fusions <- import_defuse(defuseData, "hg19", 1)
fusion <- get_fusion_by_id(fusions, 5267)
# Create edb object
edbSqliteFile <- system.file(
  "extdata",
  "Homo_sapiens.GRCh37.74.sqlite",
  package="chimeraviz")
edb <- ensembldb::EnsDb(edbSqliteFile)
# Add transcripts data to fusion object
fusion <- get_transcripts_ensembl_db(fusion, edb)
# The transcripts are now accessible through fusion@gene_upstream@transcripts and
# fusion@gene_downstream@transcripts .

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/extdata.R
\name{raw_fusionmap}
\alias{raw_fusionmap}
\title{FusionMap data}
\description{
Documentation for the FusionMap example data.
}
\section{FusionMap_01_TestDataset_InputFastq.FusionReport.txt}{


This is example data provided with the FusionMap version released 2015-03-31.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/extdata.R
\name{raw_defuse}
\alias{raw_defuse}
\title{deFuse data}
\description{
Documentation for the deFuse example data.
}
\section{defuse_833ke_results.filtered.tsv}{


This file has the results from a run of deFuse-0.7.0 on the 833ke cell line.
The program was ran with the standard configuration, but with the parameter
span_count_threshold=5 instead of the standard 3. The resulting
results.filtered.tsv file was then manually filtered to only include 17
fusion events in the interest of saving computing time for tests and
examples. The original results contained 171 fusion events.
}

\section{reads_supporting_defuse_fusion_5267.*.fq}{


These two files, reads_supporting_defuse_fusion_5267.1.fq and
reads_supporting_defuse_fusion_5267.2.fq, contains the reads that support the
fusion event with cluster_id 5267.
}

\section{5267readsAligned.bam}{


The bamfile 5267readsAligned.bam and the 5267readsAligned.bam.bai index file
contains the reads supporting the fusion event with cluster_id 5267 aligned
to the fusion sequence. It is used with plot_fusion_reads().
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/extdata.R
\name{raw_jaffa}
\alias{raw_jaffa}
\title{JAFFA data}
\description{
Documentation for the JAFFA example data.
}
\section{jaffa_results.csv}{


This is example data from the described JAFFA example run documented at
https://github.com/Oshlack/JAFFA/wiki/Example .
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/extdata.R
\name{raw_cytobandhg19}
\alias{raw_cytobandhg19}
\title{Cytoband information HG19}
\description{
Cytoband information for the HG19 assembly from UCSC. Downloaded from
http://hgdownload.cse.ucsc.edu/goldenpath/hg19/database/
}
\section{UCSC.HG19.Human.CytoBandIdeogram.txt}{


This data is used with RCircos in plot_circle().
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utilities.R
\name{get_ensembl_ids}
\alias{get_ensembl_ids}
\title{Get ensembl ids for a fusion object}
\usage{
get_ensembl_ids(fusion)
}
\arguments{
\item{fusion}{The Fusion object we want to get ensembl ids for.}
}
\value{
The Fusion object with Ensembl ids set.
}
\description{
This function will get the ensembl ids from the org.Hs.eg.db/org.Mm.eg.db
package given the gene names of the fusion event.
}
\examples{
# Import the filtered defuse results
defuse833keFiltered <- system.file(
  "extdata",
  "defuse_833ke_results.filtered.tsv",
  package="chimeraviz")
fusions <- import_defuse(defuse833keFiltered, "hg19", 1)
# Get a specific fusion
fusion <- get_fusion_by_id(fusions, 5267)
# See the ensembl ids:
partner_gene_ensembl_id(upstream_partner_gene(fusion))
# [1] "ENSG00000180198"
partner_gene_ensembl_id(downstream_partner_gene(fusion))
# [1] "ENSG00000162639"
# Reset the fusion objects ensembl ids
partner_gene_ensembl_id(upstream_partner_gene(fusion)) <- ""
partner_gene_ensembl_id(downstream_partner_gene(fusion))  <- ""
# Get the ensembl ids
fusion <- get_ensembl_ids(fusion)
# See that we now have the same ensembl ids again:
partner_gene_ensembl_id(upstream_partner_gene(fusion))
# [1] "ENSG00000180198"
partner_gene_ensembl_id(downstream_partner_gene(fusion))
# [1] "ENSG00000162639"

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/extdata.R
\name{raw_Homo_sapiens.GRCh37.74}
\alias{raw_Homo_sapiens.GRCh37.74}
\title{Homo_sapiens.GRCh37.74_subset.gtf}
\description{
Homo_sapiens.GRCh37.74_subset.gtf
}
\section{Homo_sapiens.GRCh37.74_subset.gtf}{


The Homo_sapiens.GRCh37.74.gtf file is a subset version of the Ensembl
Homo_sapiens.GRCh37.74.gtf file, located here:
\url{ftp://ftp.ensembl.org/pub/release-74/gtf/homo_sapiens}. This gtf file
contains transcripts for the partner genes in two of the fusion transcripts
from the deFuse example data provided with this package: The fusion
transcript with cluster_id=5267, and the fusion transcript with
cluster_id=11759.

The file is the result of running this command:

# grep "ENST00000373831\|ENST00000373832\|ENST00000373833\|ENST00000398958\|ENST00000411533\|ENST00000419074\|ENST00000427469\|ENST00000429051\|ENST00000430407\|ENST00000434290\|ENST00000478232\|ENST00000486790\|ENST00000370031\|ENST00000370032\|ENST00000402983\|ENST00000420055\|ENST00000483729\|ENST00000493676\|ENST00000382073\|ENST00000299665\|ENST00000382064" Homo_sapiens.GRCh37.74.gtf > Homo_sapiens.GRCh37.74_subset.gtf

The transcript names given in the command above are all transcripts available
for the genes CLEC6A, CLEC4D, HENMT1, and RCC1 in Ensembl version 74.
}

\section{Homo_sapiens.GRCh37.74.sqlite}{


The Homo_sapiens.GRCh37.74.sqlite file is the sqlite database that the
Ensembldb package creates from the corresponding gtf file. It was created
using this command:

# ensDbFromGtf(
#   gtf = "Homo_sapiens.GRCh37.74_subset.gtf",
#   organism = "Homo_sapiens",
#   genomeVersion = "GRCh37",
#   version = 74)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/extdata.R
\name{raw_soapfuse}
\alias{raw_soapfuse}
\title{SOAPfuse data}
\description{
Documentation for the SOAPfuse example data.
}
\section{soapfuse_833ke_final.Fusion.specific.for.genes}{


This file has the results from a run of soapfuse-1.26 on the 833ke cell line.
The program was ran with the standard configuration file.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/AllClasses.R
\name{partner_gene_ensembl_id}
\alias{partner_gene_ensembl_id}
\alias{partner_gene_ensembl_id,PartnerGene-method}
\alias{partner_gene_ensembl_id<-}
\alias{partner_gene_ensembl_id<-,PartnerGene-method}
\title{Get the Ensembl ID from a PartnerGene object}
\usage{
partner_gene_ensembl_id(x)

\S4method{partner_gene_ensembl_id}{PartnerGene}(x)

partner_gene_ensembl_id(object) <- value

\S4method{partner_gene_ensembl_id}{PartnerGene}(object) <- value
}
\arguments{
\item{x}{The PartnerGene object you wish to retrieve the Ensembl ID for.}

\item{object}{The PartnerGene object you wish to set a new Ensembl ID for.}

\item{value}{The new Ensembl ID.}
}
\value{
The upstream fusion partner gene Ensembl ID.
}
\description{
This getter retrieves the Ensembl ID from a PartnerGene object

This sets the Ensembl ID of a PartnerGene object.
}
\examples{
# Load data
defuseData <- system.file(
  "extdata",
  "defuse_833ke_results.filtered.tsv",
  package="chimeraviz")
fusions <- import_defuse(defuseData, "hg19", 1)
fusion <- fusions[[1]]
# Get the Ensembl ID from the upstream fusion partner gene
partner_gene_ensembl_id(upstream_partner_gene(fusion))

# Load data
defuseData <- system.file(
  "extdata",
  "defuse_833ke_results.filtered.tsv",
  package="chimeraviz")
fusions <- import_defuse(defuseData, "hg19", 1)
fusion <- fusions[[1]]
# Set the downstream PartnerGene object to be the same as the upstream
# PartnerGene object
partner_gene_ensembl_id(upstream_partner_gene(fusion)) <- "test"

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/extdata.R
\name{raw_fusion5267reads}
\alias{raw_fusion5267reads}
\title{Fusion5267and11759 bamfile}
\description{
Documentation for the fusion5267and11759reads.bam file containing reads
mapped to the region where the genes in the fusions with cluster_id=5267 and
cluster_id=11759 is.
}
\section{fusion5267and11759reads.bam}{


This file is the result of running these commands:

samtools view -b original_bamfile.bam "1:28831455-28866812" "1:109189912-109205148" "12:8608225-8677832" > fusion5267and11759reads.bam
samtools index fusion5267and11759reads.bam fusion5267and11759reads.bam.bai

where we extract the reads mapping to the region where we know the fusions
with cluster_id=5267 and cluster_id=11759 from the deFuse example data is.

The original_bamfile.bam is from a study of the 833KE cell line by Andreas M. Hoff et al., documented in the paper [Identification of Novel Fusion Genes in Testicular Germ Cell Tumors](http://cancerres.aacrjournals.org/content/76/1/108.full).
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plot_fusion_transcripts_graph.R
\name{plot_fusion_transcripts_graph}
\alias{plot_fusion_transcripts_graph}
\title{Graph plot of possible fusion transcripts.}
\usage{
plot_fusion_transcripts_graph(
  fusion,
  edb = NULL,
  which_transcripts = "exonBoundary",
  rankdir = "TB"
)
}
\arguments{
\item{fusion}{The Fusion object to plot.}

\item{edb}{The edb object that will be used to fetch data.}

\item{which_transcripts}{This character vector decides which transcripts are
to be plotted. Can be "exonBoundary", "withinExon", "withinIntron",
"intergenic", or a character vector with specific transcript ids. Default
value is "exonBoundary".}

\item{rankdir}{Choose whether the graph should be plotted from left to right
("LR"), or from top to bottom ("TB"). This parameter is given to
Rgraphviz::plot().}
}
\value{
Creates a fusion transcripts graph plot.
}
\description{
This function takes a fusion object and a TranscriptDb object and plots a
graph showing the possible fusion transcripts.
}
\examples{
# Load data and example fusion event
defuse833ke <- system.file(
  "extdata",
  "defuse_833ke_results.filtered.tsv",
  package="chimeraviz")
fusions <- import_defuse(defuse833ke, "hg19", 1)
fusion <- get_fusion_by_id(fusions, 5267)
# Load edb
edbSqliteFile <- system.file(
  "extdata",
  "Homo_sapiens.GRCh37.74.sqlite",
  package="chimeraviz")
edb <- ensembldb::EnsDb(edbSqliteFile)
# Temporary file to store the plot
pngFilename <- tempfile(
  pattern = "fusionPlot",
  fileext = ".png",
  tmpdir = tempdir())
# Open device
png(pngFilename, width = 500, height = 500)
# Plot!
plot_fusion_transcripts_graph(
  fusion = fusion,
  edb = edb)
# Close device
dev.off()

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/import_oncofuse.R
\name{import_oncofuse}
\alias{import_oncofuse}
\title{Import results from a oncofuse run into a list of Fusion objects.}
\usage{
import_oncofuse(filename, genome_version, limit)
}
\arguments{
\item{filename}{Filename for the oncofuse results .tsv file.}

\item{genome_version}{Which genome was used in mapping (hg19, hg38, etc.).}

\item{limit}{A limit on how many lines to read.}
}
\value{
A list of Fusion objects.
}
\description{
A function that imports the results from a oncofuse run, typically from a
results.filtered.tsv file, into a list of Fusion objects.
}
\details{
This import function was contributed by Lavinia G, ref
https://github.com/stianlagstad/chimeraviz/issues/47#issuecomment-409773158
}
\examples{
oncofuse833ke <- system.file(
  "extdata",
  "oncofuse.outfile",
  package="chimeraviz")
fusions <- import_oncofuse(oncofuse833ke, "hg19", 3)
# This should import a list of 3 fusions described in Fusion objects.

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/get_fusion_by.R
\name{get_fusion_by_gene_name}
\alias{get_fusion_by_gene_name}
\title{Find fusions that includes the given gene.}
\usage{
get_fusion_by_gene_name(fusion_list, gene_name)
}
\arguments{
\item{fusion_list}{A list of Fusion objects.}

\item{gene_name}{The gene name we're looking for.}
}
\value{
A list of Fusion objects.
}
\description{
Helper function to retrieve the Fusion objects that has geneName as one of
the partner genes.
}
\details{
Note: get_fusion_by_gene_name(fusionList, "MT") will match both MT-ND5 and
MT-ND4.
}
\examples{
defuse833ke <- system.file(
  "extdata",
  "defuse_833ke_results.filtered.tsv",
  package="chimeraviz")
fusions <- import_defuse(defuse833ke, "hg19", 1)
length(get_fusion_by_gene_name(fusions, "RCC1"))
# [1] 1

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/AllClasses.R
\name{upstream_partner_gene}
\alias{upstream_partner_gene}
\alias{upstream_partner_gene,Fusion-method}
\alias{upstream_partner_gene<-}
\alias{upstream_partner_gene<-,Fusion-method}
\title{Get the upstream fusion partner gene}
\usage{
upstream_partner_gene(x)

\S4method{upstream_partner_gene}{Fusion}(x)

upstream_partner_gene(object) <- value

\S4method{upstream_partner_gene}{Fusion}(object) <- value
}
\arguments{
\item{x}{The Fusion object you wish to retrieve the upstream PartnerGene
object for.}

\item{object}{The Fusion object you wish to set a new upstream PartnerGene
object for.}

\item{value}{The new PartnerGene object.}
}
\value{
The upstream PartnerGene object.
}
\description{
This getter retrieves the upstream PartnerGene object.

This sets the upstream PartnerGene object of a Fusion object
}
\examples{
# Load data
defuseData <- system.file(
  "extdata",
  "defuse_833ke_results.filtered.tsv",
  package="chimeraviz")
fusions <- import_defuse(defuseData, "hg19", 1)
fusion <- fusions[[1]]
# Get the upstream fusion partner gene
upstream_partner_gene(fusion)

# Load data
defuseData <- system.file(
  "extdata",
  "defuse_833ke_results.filtered.tsv",
  package="chimeraviz")
fusions <- import_defuse(defuseData, "hg19", 1)
fusion <- fusions[[1]]
# Set the upstream PartnerGene object to be the same as the downstream
# PartnerGene object
upstream_partner_gene(fusion) <- downstream_partner_gene(fusion)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/import_jaffa.R
\name{import_jaffa}
\alias{import_jaffa}
\title{Import results from a JAFFA run into a list of Fusion objects.}
\usage{
import_jaffa(filename, genome_version, limit)
}
\arguments{
\item{filename}{Filename for the jaffa_results.csv file.}

\item{genome_version}{Which genome was used in mapping (hg19, hg38, etc.).}

\item{limit}{A limit on how many lines to read.}
}
\value{
A list of Fusion objects.
}
\description{
A function that imports the results from a JAFFA run, typically from a
jaffa_results.csv file, into a list of Fusion objects.
}
\examples{
jaffaData <- system.file(
  "extdata",
  "jaffa_results.csv",
  package = "chimeraviz")
fusions <- import_jaffa(jaffaData, "hg19", 3)
# This should import a list of 3 fusions described in Fusion objects.

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plot_fusion_transcript_with_protein_domain.R
\name{plot_fusion_transcript_with_protein_domain}
\alias{plot_fusion_transcript_with_protein_domain}
\title{Plot a specific fusion transcript with protein domain annotations}
\usage{
plot_fusion_transcript_with_protein_domain(
  fusion,
  edb = NULL,
  bamfile = NULL,
  bedfile = NULL,
  gene_upstream_transcript = "",
  gene_downstream_transcript = "",
  plot_downstream_protein_domains_if_fusion_is_out_of_frame = FALSE
)
}
\arguments{
\item{fusion}{The Fusion object to plot.}

\item{edb}{The edb object that will be used to fetch data.}

\item{bamfile}{The bamfile with RNA-seq data.}

\item{bedfile}{The bedfile with protein domain data.}

\item{gene_upstream_transcript}{The transcript id for the upstream gene.}

\item{gene_downstream_transcript}{The transcript id for the downstream gene.}

\item{plot_downstream_protein_domains_if_fusion_is_out_of_frame}{Setting this to true
makes the function plot protein domains in the downstream gene even though
the fusion is out of frame.}
}
\value{
Creates a fusion transcript plot with annotations of protein domains.
}
\description{
This function takes a fusion object, an ensembldb object, a bedfile with
protein domain data and two specific transcript ids. The function plots the
specific fusion transcript along with annotations of protein domains. If a
bamfile is specified, the fusion transcript will be plotted with coverage
information.
}
\details{
Note that the transcript database used (the edb object) must have the same
seqnames as any bamfile used. Otherwise the coverage data will be wrong.
}
\examples{
# Load data and example fusion event
defuse833ke <- system.file(
  "extdata",
  "defuse_833ke_results.filtered.tsv",
  package="chimeraviz")
fusions <- import_defuse(defuse833ke, "hg19", 1)
fusion <- get_fusion_by_id(fusions, 5267)
# Select transcripts
gene_upstream_transcript <- "ENST00000434290"
gene_downstream_transcript <- "ENST00000370031"
# Load edb
edbSqliteFile <- system.file(
  "extdata",
  "Homo_sapiens.GRCh37.74.sqlite",
  package="chimeraviz")
edb <- ensembldb::EnsDb(edbSqliteFile)
# bamfile with reads in the regions of this fusion event
bamfile5267 <- system.file(
  "extdata",
  "fusion5267and11759reads.bam",
  package="chimeraviz")
# bedfile with protein domains for the transcripts in this example
bedfile <- system.file(
  "extdata",
  "protein_domains_5267.bed",
  package="chimeraviz")
# Temporary file to store the plot
pngFilename <- tempfile(
  pattern = "fusionPlot",
  fileext = ".png",
  tmpdir = tempdir())
# Open device
png(pngFilename, width = 500, height = 500)
# Plot!
plot_fusion_transcript_with_protein_domain(
  fusion = fusion,
  edb = edb,
  bamfile = bamfile5267,
  bedfile = bedfile,
  gene_upstream_transcript = gene_upstream_transcript,
  gene_downstream_transcript = gene_downstream_transcript,
  plot_downstream_protein_domains_if_fusion_is_out_of_frame = TRUE)
# Close device
dev.off()

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/extdata.R
\name{raw_oncofuse}
\alias{raw_oncofuse}
\title{oncofuse data}
\description{
Documentation for the oncofuse example data.
}
\section{oncofuse.outfile}{


The example output from oncofuse was kindly provided by Lavinia G here:
https://github.com/stianlagstad/chimeraviz/issues/47#issuecomment-409773158
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/extdata.R
\name{raw_ericscript}
\alias{raw_ericscript}
\title{EricScript data}
\description{
Documentation for the EricScript example data.
}
\section{ericscript_SRR1657556.results.total.tsv}{


This is example data thankfully provided by EricScript author Matteo Benelli.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/import_aeron.R
\name{import_aeron}
\alias{import_aeron}
\title{Import results from an Aeron run into a list of Fusion objects.}
\usage{
import_aeron(
  filename_fusion_support,
  filename_fusion_transcript,
  genome_version,
  limit
)
}
\arguments{
\item{filename_fusion_support}{Filename for the Aeron result file
fusion_support..txt.}

\item{filename_fusion_transcript}{Filename for the Aeron result file
fusion_transcripts..txt.}

\item{genome_version}{Which genome was used in mapping (hg19, hg38, etc.).}

\item{limit}{A limit on how many lines to read.}
}
\value{
A list of Fusion objects.
}
\description{
A function that imports the results from an Aeron run into a list of Fusion
objects.
}
\details{
Note that the strands and breakpoint positions are not included in the
result files from Aeron. These have to be retrieved manually, using the
ensembl identifiers (which are included in the result files, and will be
available in the Fusion objects after importing).
}
\examples{
aeronfusionsupportfile <- system.file(
  "extdata",
  "aeron_fusion_support.txt",
  package="chimeraviz")
aeronfusiontranscriptfile <- system.file(
  "extdata",
  "aeron_fusion_transcripts.fa",
  package="chimeraviz")
fusions <- import_aeron(
  aeronfusionsupportfile,
  aeronfusiontranscriptfile,
  "hg19",
  3)
# This should import a list of 3 fusions described in Fusion objects.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/chimeraviz.R
\docType{package}
\name{chimeraviz}
\alias{chimeraviz}
\title{chimeraviz: A package for working with and visualizing fusion genes.}
\description{
chimeraviz manages data from fusion gene finders and provides useful
visualization tools.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/import_squid.R
\name{import_squid}
\alias{import_squid}
\title{Import results from a SQUID run into a list of Fusion objects.}
\usage{
import_squid(filename, genome_version, limit)
}
\arguments{
\item{filename}{Filename for the SQUID results.}

\item{genome_version}{Which genome was used in mapping (hg19, hg38, etc.).}

\item{limit}{A limit on how many lines to read.}
}
\value{
A list of Fusion objects.
}
\description{
A function that imports the results from a SQUID run into a list of Fusion
objects.
}
\examples{
squidfile <- system.file(
  "extdata",
  "squid_hcc1954_sv.txt",
  package="chimeraviz")
fusions <- import_squid(squidfile, "hg19", 3)
# This should import a list of 3 fusions described in Fusion objects.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plot_fusion.R
\name{import_function_non_ucsc}
\alias{import_function_non_ucsc}
\title{Alternative import function for Gviz::AlignmentsTrack}
\usage{
import_function_non_ucsc(file, selection)
}
\arguments{
\item{file}{The bamfile.}

\item{selection}{Which regions to get from the bamfile.}
}
\value{
A GRanges object with coverage data for the selection.
}
\description{
This alternative import function for use with Gviz::AlignmentsTrack imports
a bamfile with non-UCSC chromosome names.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/AllClasses.R
\docType{class}
\name{PartnerGene-class}
\alias{PartnerGene-class}
\alias{PartnerGene}
\title{An S4 class to represent a gene partner in a fusion}
\description{
The PartnerGene class represents one of the genes in a fusion event.
}
\section{Slots}{

\describe{
\item{\code{name}}{Character containing name of the gene.}

\item{\code{ensembl_id}}{Character containing ensembl id for the gene.}

\item{\code{chromosome}}{Character containing chromosome name.}

\item{\code{breakpoint}}{Numeric containing the fusion breakpoint.}

\item{\code{strand}}{Character containing gene strand.}

\item{\code{junction_sequence}}{Biostrings::DNAString containing the sequence right
before/after the fusion breakpoint.}

\item{\code{transcripts}}{GenomicRanges::GRangesList containing three
GenomicRanges::Granges() objects, one for each "transcript type". The
transcript types are: 1) Transcripts where the fusion breakpoint hits an
exon boundary, 2) transcripts where the fusion breakpoint is within an
exon, 3) transcripts where the fusion breakpoint is within an intron.}
}}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/extdata.R
\name{raw_starfusion}
\alias{raw_starfusion}
\title{STAR-Fusion data}
\description{
Documentation for the STAR-Fusion example data.
}
\section{star-fusion.fusion_candidates.final.abridged.txt}{


This example data was retrieved from the STAR-Fusion github page June 2.nd
2017.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plot_fusion_transcript.R
\name{plot_fusion_transcript}
\alias{plot_fusion_transcript}
\title{Plot possible fusion transcripts based on annotation.}
\usage{
plot_fusion_transcript(
  fusion,
  edb = NULL,
  bamfile = NULL,
  which_transcripts = "exonBoundary",
  bedgraphfile = NULL
)
}
\arguments{
\item{fusion}{The Fusion object to plot.}

\item{edb}{The edb object that will be used to fetch data.}

\item{bamfile}{The bamfile with RNA-seq data.}

\item{which_transcripts}{This character vector decides which transcripts are
to be plotted. Can be "exonBoundary", "withinExon", "withinIntron",
"intergenic", or a character vector with specific transcript ids. Default
value is "exonBoundary".}

\item{bedgraphfile}{A bedGraph file to use instead of the bamfile to plot
coverage.}
}
\value{
Creates a fusion transcript plot.
}
\description{
This function takes a fusion object and an ensembldb object and plots the
reduced version of the fusion transcript. This transcript consist of the
"mashed together" version of all possible fusion transcripts based on known
annotations. If a bamfile is specified, the fusion transcript will be
plotted with coverage information.
}
\details{
Note that the transcript database used (the edb object) must have the same
seqnames as any bamfile used. Otherwise the coverage data will be wrong.
}
\examples{
# Load data and example fusion event
defuse833ke <- system.file(
  "extdata",
  "defuse_833ke_results.filtered.tsv",
  package="chimeraviz")
fusions <- import_defuse(defuse833ke, "hg19", 1)
fusion <- get_fusion_by_id(fusions, 5267)
# Load edb
edbSqliteFile <- system.file(
  "extdata",
  "Homo_sapiens.GRCh37.74.sqlite",
  package="chimeraviz")
edb <- ensembldb::EnsDb(edbSqliteFile)
# bamfile with reads in the regions of this fusion event
bamfile5267 <- system.file(
  "extdata",
  "fusion5267and11759reads.bam",
  package="chimeraviz")
# Temporary file to store the plot
pngFilename <- tempfile(
  pattern = "fusionPlot",
  fileext = ".png",
  tmpdir = tempdir())
# Open device
png(pngFilename, width = 500, height = 500)
# Plot!
plot_fusion_transcript(
  fusion = fusion,
  bamfile = bamfile5267,
  edb = edb)
# Close device
dev.off()

# Example using a .bedGraph file instead of a .bam file:
# Load data and example fusion event
defuse833ke <- system.file(
  "extdata",
  "defuse_833ke_results.filtered.tsv",
  package="chimeraviz")
fusions <- import_defuse(defuse833ke, "hg19", 1)
fusion <- get_fusion_by_id(fusions, 5267)
# Load edb
edbSqliteFile <- system.file(
  "extdata",
  "Homo_sapiens.GRCh37.74.sqlite",
  package="chimeraviz")
edb <- ensembldb::EnsDb(edbSqliteFile)
# bedgraphfile with coverage data from the regions of this fusion event
bedgraphfile <- system.file(
  "extdata",
  "fusion5267and11759reads.bedGraph",
  package="chimeraviz")
# Temporary file to store the plot
pngFilename <- tempfile(
  pattern = "fusionPlot",
  fileext = ".png",
  tmpdir = tempdir())
# Open device
png(pngFilename, width = 500, height = 500)
# Plot!
plot_fusion_transcript(
  fusion = fusion,
  bamfile = bamfile5267,
  edb = edb)
# Close device
dev.off()

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/extdata.R
\name{raw_fusion5267readsBedGraph}
\alias{raw_fusion5267readsBedGraph}
\title{Fusion5267and11759 bedGraph file}
\description{
Documentation for the fusion5267and11759reads.bedGraph file containing read
count data from the regions of the fusion event with cluster_id=5267.
}
\section{fusion5267and11759reads.bedGraph}{


This file is the result of running this command:

bedtools genomecov -ibam fusion5267and11759reads.bam -bg > fusion5267and11759reads.bam.bedGraph

fusion5267and11759reads.bam has its own documentation entry for how it was
created.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/extdata.R
\name{raw_chimpipe}
\alias{raw_chimpipe}
\title{ChimPipe data}
\description{
Documentation for the ChimPipe example data.

Documentation for the SQUID example data.
}
\section{chimericJunctions_MCF-7.txt}{


This is example data from the ChimPipe tutorial all-in-one package located at
https://chimpipe.readthedocs.io/en/latest/tutorial.html#download-all-in-one-package
It was downloaded March 10th 2020
}

\section{squid_hcc1954_sv.txt}{


This is example data for SQUID provided on GitHub here:
https://github.com/Kingsford-Group/squid/issues/20#issuecomment-598888472
It was downloaded March 17th 2020
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/extdata.R
\name{raw_infusion}
\alias{raw_infusion}
\title{InFusion data}
\description{
Documentation for the InFusion example data.
}
\section{infusion_fusions.txt}{


This is example data from the InFusion getting started page located at
https://bitbucket.org/kokonech/infusion/wiki/Getting%20Started .
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plot_circle.R
\name{plot_circle}
\alias{plot_circle}
\title{Create a circle plot of the given fusions.}
\usage{
plot_circle(fusion_list)
}
\arguments{
\item{fusion_list}{A list of Fusion objects.}
}
\value{
Creates a circle plot.
}
\description{
This function takes a list of Fusion objects and creates a circle plot
indicating which chromosomes the fusion genes in the list consists of.
}
\details{
Note that only a limited number of gene names can be shown in the circle plot
due to the limited resolution of the plot. RCircos will automatically limit
the number of gene names shown if there are too many.
}
\examples{
defuse833ke <- system.file(
  "extdata",
  "defuse_833ke_results.filtered.tsv",
  package="chimeraviz")
fusions <- import_defuse(defuse833ke, "hg19", 3)
# Temporary file to store the plot
pngFilename <- tempfile(
  pattern = "circlePlot",
  fileext = ".png",
  tmpdir = tempdir())
# Open device
png(pngFilename, width = 1000, height = 750)
# Plot!
plot_circle(fusions)
# Close device
dev.off()

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/extdata.R
\name{raw_fusioncatcher}
\alias{raw_fusioncatcher}
\title{Fusioncatcher data}
\description{
Documentation for the Fusioncatcher example data.
}
\section{fusioncatcher_833ke_final-list-candidate-fusion-genes.txt}{


This file has the results from a run of Fusioncatcher-0.99.3e on the 833ke
cell line. The program was ran with the standard configuration file and with
the parameters "-p 8 -z --keep-preliminary".
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/AllClasses.R
\docType{class}
\name{Fusion-class}
\alias{Fusion-class}
\alias{Fusion}
\title{An S4 class to represent a fusion event.}
\description{
The Fusion class represents a fusion event, holding data imported from a
fusion tool.
}
\section{Slots}{

\describe{
\item{\code{id}}{A unique id representing a fusion event. For deFuse data this will
be the cluster id.}

\item{\code{fusion_tool}}{Name of the fusion tool.}

\item{\code{genome_version}}{Name of the genome used to map reads.}

\item{\code{spanning_reads_count}}{The number of spanning reads supporting the
fusion.}

\item{\code{split_reads_count}}{The number of split reads supporting the fusion.}

\item{\code{fusion_reads_alignment}}{A Gviz::AlignmentsTrack object holding the
fusion reads aligned to the fusion sequence.}

\item{\code{gene_upstream}}{A PartnerGene object holding information of the upstream
fusion partner gene.}

\item{\code{gene_downstream}}{A PartnerGene object holding information of the
downstream fusion partner gene.}

\item{\code{inframe}}{A logical value indicating whether or not the downstream
fusion partner gene is inframe or not. Not all fusion-finders report this.}

\item{\code{fusion_tool_specific_data}}{A list that will hold fields of importance
for a specific fusion finder. This field is used because many fusion-finders
report important values that are hard to fit into a standardized format.
Examples of values that are added to this list is probability from deFuse
and EricScore from EricScript.}
}}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/extdata.R
\name{raw_prada}
\alias{raw_prada}
\title{PRADA data}
\description{
Documentation for the PRADA example data.
}
\section{PRADA.acc.fusion.fq.TAF.tsv}{


This is example data thankfully provided by PRADA authors Siyuan Zheng and
Roeland Verhaak.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/extdata.R
\name{raw_cytobandhg38}
\alias{raw_cytobandhg38}
\title{Cytoband information HG138}
\description{
Cytoband information for the HG38 assembly from UCSC. Downloaded from
http://hgdownload.cse.ucsc.edu/goldenpath/hg38database/
}
\details{
All _alt or _random entries has been manually removed, as has the chrM entry.
}
\section{UCSC.HG38.Human.CytoBandIdeogram.txt}{


This data is used with RCircos in plot_circle().
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/import_starfusion.R
\name{import_starfusion}
\alias{import_starfusion}
\title{Import results from a STAR-Fusion run into a list of Fusion objects.}
\usage{
import_starfusion(filename, genome_version, limit)
}
\arguments{
\item{filename}{Filename for the STAR-Fusion
star-fusion.fusion_candidates.final.abridged results file.}

\item{genome_version}{Which genome was used in mapping (hg19, hg38, etc.).}

\item{limit}{A limit on how many lines to read.}
}
\value{
A list of Fusion objects.
}
\description{
A function that imports the results from a STAR-Fusion run, typically from
a star-fusion.fusion_candidates.final.abridged file, into a list of Fusion
objects.
}
\examples{
starfusionData <- system.file(
  "extdata",
  "star-fusion.fusion_candidates.final.abridged.txt",
  package = "chimeraviz")
fusions <- import_starfusion(starfusionData, "hg19", 3)
# This should import a list of 3 fusions described in Fusion objects.

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plot_circle.R
\name{chimeraviz-internals-fusions_to_gene_label_data}
\alias{chimeraviz-internals-fusions_to_gene_label_data}
\alias{.fusions_to_gene_label_data}
\title{Create gene label data for RCircos from the given fusions.}
\usage{
.fusions_to_gene_label_data(fusion_list)
}
\arguments{
\item{fusion_list}{A list of Fusion objects.}
}
\value{
A data frame with fusion gene label data compatible with RCircos.Gene.Name.Plot()

# @examples # Apparently examples shouldn't be set on private functions
defuse833ke <- system.file("extdata", "defuse_833ke_results.filtered.tsv", package="chimeraviz")
fusions <- import_defuse(defuse833ke, "hg19", 3)
labelData <- chimeraviz::.fusions_to_gene_label_data(fusions)
# This labelData can be used with RCircos.Gene.Connector.Plot() and RCircos.Gene.Name.Plot()
}
\description{
This function takes a list of Fusion objects and creates a data frame in the
format that RCircos.Gene.Name.Plot() expects for gene label data.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/AllClasses.R
\name{show,PartnerGene-method}
\alias{show,PartnerGene-method}
\title{Show method for the PartnerGene class.}
\usage{
\S4method{show}{PartnerGene}(object)
}
\arguments{
\item{object}{A PartnerGene object}
}
\value{
Shows information about a PartnerGene object.
}
\description{
Show method for the PartnerGene class.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/AllClasses.R
\name{downstream_partner_gene}
\alias{downstream_partner_gene}
\alias{downstream_partner_gene,Fusion-method}
\alias{downstream_partner_gene<-}
\alias{downstream_partner_gene<-,Fusion-method}
\title{Get the downstream fusion partner gene}
\usage{
downstream_partner_gene(x)

\S4method{downstream_partner_gene}{Fusion}(x)

downstream_partner_gene(object) <- value

\S4method{downstream_partner_gene}{Fusion}(object) <- value
}
\arguments{
\item{x}{The Fusion object you wish to retrieve the downstream PartnerGene
object for.}

\item{object}{The Fusion object you wish to set a new downstream
PartnerGene object for.}

\item{value}{The new PartnerGene object.}
}
\value{
The downstream PartnerGene object.
}
\description{
This getter retrieves the downstream PartnerGene object.

This sets the downstream PartnerGene object of a Fusion object
}
\examples{
# Load data
defuseData <- system.file(
  "extdata",
  "defuse_833ke_results.filtered.tsv",
  package="chimeraviz")
fusions <- import_defuse(defuseData, "hg19", 1)
fusion <- fusions[[1]]
# Get the downstream fusion partner gene
downstream_partner_gene(fusion)

# Load data
defuseData <- system.file(
  "extdata",
  "defuse_833ke_results.filtered.tsv",
  package="chimeraviz")
fusions <- import_defuse(defuseData, "hg19", 1)
fusion <- fusions[[1]]
# Set the downstream PartnerGene object to be the same as the upstream
# PartnerGene object
downstream_partner_gene(fusion) <- upstream_partner_gene(fusion)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plot_transcripts.R
\name{plot_transcripts}
\alias{plot_transcripts}
\title{Plot transcripts for each partner gene in a fusion event.}
\usage{
plot_transcripts(
  fusion,
  edb = NULL,
  bamfile = NULL,
  which_transcripts = "exonBoundary",
  non_ucsc = TRUE,
  ylim = c(0, 1000),
  reduce_transcripts = FALSE,
  bedgraphfile = NULL
)
}
\arguments{
\item{fusion}{The Fusion object to plot.}

\item{edb}{The edb object that will be used to fetch data.}

\item{bamfile}{The bamfile with RNA-seq data.}

\item{which_transcripts}{This character vector decides which transcripts are
to be plotted. Can be "exonBoundary", "withinExon", "withinIntron",
"intergenic", or a character vector with specific transcript ids. Default
value is "exonBoundary".}

\item{non_ucsc}{Boolean indicating whether or not the bamfile used has UCSC-
styled chromosome names (i.e. with the "chr" prefix). Setting this to true
lets you use a bamfile with chromosome names like "1" and "X", instead of
"chr1" and "chrX".}

\item{ylim}{Limits for the coverage y-axis.}

\item{reduce_transcripts}{Boolean indicating whether or not to reduce all
transcripts into a single transcript for each partner gene.}

\item{bedgraphfile}{A bedGraph file to use instead of the bamfile to plot
coverage.}
}
\value{
Creates a fusion transcripts plot.
}
\description{
This function takes a fusion object and an ensembldb object and plots
transcripts for both genes, showing which parts of each genes are included
in the fusion event. If the bamfile parameter is set, then the coverage is
plotted beneath the transcripts.
}
\examples{
# Load data and example fusion event
defuse833ke <- system.file(
  "extdata",
  "defuse_833ke_results.filtered.tsv",
  package="chimeraviz")
fusions <- import_defuse(defuse833ke, "hg19", 1)
fusion <- get_fusion_by_id(fusions, 5267)
# Load edb
edbSqliteFile <- system.file(
  "extdata",
  "Homo_sapiens.GRCh37.74.sqlite",
  package="chimeraviz")
edb <- ensembldb::EnsDb(edbSqliteFile)
# bamfile with reads in the regions of this fusion event
bamfile5267 <- system.file(
  "extdata",
  "fusion5267and11759reads.bam",
  package="chimeraviz")
# Temporary file to store the plot
pngFilename <- tempfile(
  pattern = "fusionPlot",
  fileext = ".png",
  tmpdir = tempdir())
# Open device
png(pngFilename, width = 500, height = 500)
# Plot!
plot_transcripts(
  fusion = fusion,
  edb = edb,
  bamfile = bamfile5267,
  non_ucsc = TRUE)
# Close device
dev.off()

# Example using a .bedGraph file instead of a .bam file:
# Load data and example fusion event
defuse833ke <- system.file(
  "extdata",
  "defuse_833ke_results.filtered.tsv",
  package="chimeraviz")
fusions <- import_defuse(defuse833ke, "hg19", 1)
fusion <- get_fusion_by_id(fusions, 5267)
# Load edb
edbSqliteFile <- system.file(
  "extdata",
  "Homo_sapiens.GRCh37.74.sqlite",
  package="chimeraviz")
edb <- ensembldb::EnsDb(edbSqliteFile)
# bedgraphfile with coverage data from the regions of this fusion event
bedgraphfile <- system.file(
  "extdata",
  "fusion5267and11759reads.bedGraph",
  package="chimeraviz")
# Temporary file to store the plot
pngFilename <- tempfile(
  pattern = "fusionPlot",
  fileext = ".png",
  tmpdir = tempdir())
# Open device
png(pngFilename, width = 500, height = 500)
# Plot!
plot_transcripts(
  fusion = fusion,
  edb = edb,
  bedgraphfile = bedgraphfile,
  non_ucsc = TRUE)
# Close device
dev.off()

}
